<?php
/**
 * @package    Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * Theme_functions
 * 
 * Общие шаблонные функции
 */
class Theme_functions
{
	/**
	 * @var object основной объект системы
	 */
	private $diafan;

	/**
	 * @var object представление в пользовательской части
	 */
	private $view;

	/**
	 * Конструктор класса. Определяет свойства класса
	 * 
	 * @return void
	 */
	public function __construct(&$view)
	{
		$this->diafan = &$view->diafan;
		$this->view   = $view;
	}

	/**
	 * Шаблонная функция: выводит основной контент страницы: заголовка (если не запрещен его вывод в настройке странице «Не показывать заголовок»), текста страницы и прикрепленного модуля. Заменяет три тега: show_h1, show_text, show_module.
	 * 
	 * @return void
	 */
	public function show_body()
	{		
		if ($name = $this->show_h1(array(), false))
		{
			echo '<h1>'.$name.'</h1>';
		}

		$this->show_text();
		$this->show_module();
	}

	/**
	 * Шаблонная функция: выводит заголовок страницы, если не запрещен его вывод в настройке странице «Не показывать заголовок». 
	 * 
	 * @return void
	 */
	public function show_h1($attributes, $print = true)
	{		
		if (! $this->diafan->title_no_show)
		{
			if ($this->diafan->titlemodule)
			{
				$name = $this->diafan->titlemodule;
				if ($this->diafan->edit_meta)
				{
					$name = $this->diafan->_useradmin->get($name, 'name', $this->diafan->edit_meta["id"], $this->diafan->edit_meta["table"], _LANG);
				}
			}
			else
			{
				$name = $this->diafan->_useradmin->get($this->diafan->name, 'name', $this->diafan->cid, 'site', _LANG);
			}
			if($print)
			{
				echo $name;
			}
			else
			{
				return $name;
			}
		}
	}

	/**
	 * Шаблонная функция: выводит текст страницы. 
	 * 
	 * @return void
	 */
	public function show_text()
	{
		if (! $this->diafan->show && ! $this->diafan->cat && ! $this->diafan->step && empty($_GET["action"]))
		{
			$text = $this->diafan->_route->replace_id_to_link($this->diafan->text);
			$text = $this->diafan->_useradmin->get($text, 'text', $this->diafan->cid, 'site', _LANG);
			if($this->diafan->configmodules("keywords", "site") && in_array('keywords', $this->diafan->installed_modules))
			{
				$text = $this->diafan->_keywords->get($text);
			}
			$this->view->get_function_in_theme($text);
		}
	}

	/**
	 * Шаблонная функция: выводит контент модуля. 
	 * 
	 * @return void
	 */
	public function show_module()
	{
		$this->view->module->show_module();
	}

	/**
	 * Шаблонная функция: выводит навигацию по сайту «Хлебные крошки».
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * separator - разделитель ссылок в навигации
	 * 
	 * @return void
	 */
	public function show_path($attributes)
	{
		if ($this->diafan->id == 1 && ! $this->diafan->path)
		{
			return;
		}
		$attributes = $this->view->get_attributes($attributes, 'separator');

		if ($this->diafan->cid == 1 && ! $this->diafan->cat && ! $this->diafan->show)
			return;

		$separator = $attributes["separator"] ? str_replace('src="/', 'src="'.BASE_PATH, $attributes["separator"]) : '/';

		if ($this->diafan->parent_id)
		{
			$cache_meta = array(
					"name"     => "page",
					"id"       => $this->diafan->cid,
					"lang_id"  => _LANG
				);
			$page = $this->diafan->_cache->get($cache_meta, 'site');
			if (! isset($page["path"]))
			{
				$page["path"] = array();
				$parents = $this->diafan->get_parents($this->diafan->cid, 'site');
				$rparents = array();
				$result = DB::query("SELECT id, [name], parent_id FROM {site} WHERE id IN (%h)", implode(',', $parents));
				while ($row = DB::fetch_array($result))
				{
					$rparents[$row["parent_id"]] = $row;
				}
				DB::free_result($result);
				$i = 0;
				while(! empty($rparents[$i]))
				{
					$row = $rparents[$i];
					unset($rparents[$i]);
					$i = $row["id"];
					$row["link"] = $this->diafan->_route->link($row["id"]);
					$page["path"][] = $row;
				}
				//сохранение кеша
				$this->diafan->_cache->save($page, $cache_meta, 'site');
			}
		}
		echo '<div class="path" xmlns:v="http://rdf.data-vocabulary.org/#">';

		if ($this->diafan->cid != 1)
		{
			echo  '<span typeof="v:Breadcrumb"><a href="'.BASE_PATH_HREF.'" rel="v:url" property="v:title">'.$this->diafan->_('Главная').'</a> '.$separator.' </span>';
		}
		if ($this->diafan->parent_id)
		{
			foreach ($page["path"] as $row)
			{
				echo '<span typeof="v:Breadcrumb"><a href="'.BASE_PATH_HREF.$row["link"].'" rel="v:url" property="v:title">'.$this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'site', _LANG).'</a> '.$separator.' </span>';
			}
		}
		if ($this->diafan->path)
		{
			foreach ($this->diafan->path as $path)
			{
				if ($this->diafan->path[0] == $path)
				{
					$path["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $this->diafan->cid, 'site', _LANG);
				}
				echo '<span typeof="v:Breadcrumb"><a href="'.BASE_PATH_HREF.$path["link"].'" rel="v:url" property="v:title">'.$path["name"].'</a> '.$separator.' </span>';
			}
		}
		echo '</div>';
	}

	/**
	 * Шаблонная функция: выводит период функционирования сайта в годах
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * year - начало отсчета (по умолчанию текущий год)
	 * 
	 * @return void
	 */
	public function show_year($attributes)
	{
		$attributes = $this->view->get_attributes($attributes, 'year');
		
		$year = preg_replace('/[^0-9]+/', '', $attributes["year"]);

		echo ($year ? $year : date("Y")).(date("Y") != $year && $year ? ' - '.date("Y").' '.$this->diafan->_('гг.') : ' '.$this->diafan->_('г.'));
	}

	/**
	 * Шаблонная функция: выводит заголовок. Используется между тегами `<title></title>` в шапке сайта.
	 * 
	 * @return void
	 */
	public function show_title()
	{
		if ($this->diafan->titlemodule_meta)
		{
			echo $this->diafan->titlemodule_meta;
			return;
		}
		if($this->diafan->page > 1)
		{
			$page = $this->diafan->_(' — Страница %d', false, $this->diafan->page);
		}
		else
		{
			$page = '';
		}
		if($this->diafan->configmodules('title_tpl', 'site'))
		{
			if($this->diafan->parent_id && ! $this->diafan->parent_name
			   && strpos($this->diafan->configmodules("title_tpl", 'site'), '%parent') !== false)
			{
				$this->diafan->parent_name = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $this->diafan->parent_id);
			}
			$this->diafan->title_meta = str_replace(
				array('%title', '%name', '%parent'),
				array($this->diafan->title_meta, $this->diafan->name, $this->diafan->parent_name),
				$this->diafan->configmodules("title_tpl", 'site')
			);
			echo $this->diafan->title_meta.$page;
			return;
		}
		if ($this->diafan->title_meta)
		{
			echo ($this->diafan->titlemodule ? $this->diafan->titlemodule.' — ' : '').$this->diafan->title_meta.$page;
			return;
		}
		echo ($this->diafan->titlemodule ? $this->diafan->titlemodule.' — ' : '').$this->diafan->name.$page.(TITLE ? ' — '.TITLE : '');
	}

	/**
	 * Шаблонная функция: выводит мета-тег description страницы.
	 * 
	 * @return void
	 */
	public function show_description()
	{
		if ($this->diafan->edit_meta)
		{
			$link = $this->diafan->_useradmin->get_meta('descr', $this->diafan->edit_meta["id"], $this->diafan->edit_meta["table"]);
		}
		else
		{
			$link = $this->diafan->_useradmin->get_meta('descr', $this->diafan->cid, 'site');
		}
		if($this->diafan->configmodules('descr_tpl', 'site'))
		{
			if($this->diafan->parent_id && ! $this->diafan->parent_name
			   && strpos($this->diafan->configmodules("descr_tpl", 'site'), '%parent') !== false)
			{
				$this->diafan->parent_name = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $this->diafan->parent_id);
			}
			$this->diafan->descr = str_replace(
				array('%descr', '%name', '%parent'),
				array($this->diafan->descr, $this->diafan->name, $this->diafan->parent_name),
				$this->diafan->configmodules("descr_tpl", 'site')
			);
		}
		echo str_replace('"', '&quot;', $this->diafan->descr).($link ? '" useradmin="'.$link : '');
	}

	/**
	 * Шаблонная функция: выводит ключевые слова страницы. Используется для мета-тега keywords.
	 * 
	 * @return void
	 */
	public function show_keywords()
	{
		if ($this->diafan->edit_meta)
		{
			$link = $this->diafan->_useradmin->get_meta('keywords', $this->diafan->edit_meta["id"], $this->diafan->edit_meta["table"]);
		}
		else
		{
			$link = $this->diafan->_useradmin->get_meta('keywords', $this->diafan->cid, 'site');
		}
		if($this->diafan->configmodules('keywords_tpl', 'site'))
		{
			if($this->diafan->parent_id && ! $this->diafan->parent_name
			   && strpos($this->diafan->configmodules("keywords_tpl", 'site'), '%parent') !== false)
			{
				$this->diafan->parent_name = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $this->diafan->parent_id);
			}
			$this->diafan->keywords = str_replace(
				array('%keywords', '%name', '%parent'),
				array($this->diafan->keywords, $this->diafan->name, $this->diafan->parent_name),
				$this->diafan->configmodules("keywords_tpl", 'site')
			);
		}
		echo str_replace('"', '&quot;', $this->diafan->keywords).($link ? '" useradmin="'.$link : '');
	}

	/**
	 * Шаблонная функция: выводит JS код запрещающий копирование контента на сайте.
	 * 
	 * @return void
	 */
	public function show_protect()
	{
		if (! $this->diafan->configmodules('protect', 'site'))
			return;
		echo ' oncopy="return false"';
	}

	/**
	 * Шаблонная функция: подключает JS файлы, обеспечивающие работу diafan.CMS.
	 * 
	 * @return void
	 */
	public function show_js()
	{
		echo '<!--[if lt IE 9]><script src="http://yandex.st/jquery/1.10.2/jquery.min.js"></script><![endif]-->
		<!--[if gte IE 9]><!-->
			<script type="text/javascript" src="http://yandex.st/jquery/2.0.3/jquery.min.js" charset="UTF-8"><</script><!--<![endif]-->

		<script type="text/javascript" src="http://yandex.st/jquery/form/3.14/jquery.form.min.js" charset="UTF-8"></script>';

		echo '<script type="text/javascript" src="http://yandex.st/jquery-ui/1.10.3/jquery-ui.min.js" charset="UTF-8"></script>
		<link href="'.BASE_PATH.'css/custom-theme/jquery-ui-1.8.18.custom.css" rel="stylesheet" type="text/css">

		<script type="text/javascript" src="'.BASE_PATH.'js/timepicker.js" charset="UTF-8"></script>

		<script type="text/javascript" src="'.BASE_PATH.'js/user-func.js" charset="UTF-8"></script>';

		if ($this->diafan->_user->useradmin)
		{
			echo '<script type="text/javascript" src="http://yandex.st/jquery/cookie/1.0/jquery.cookie.min.js" charset="UTF-8"></script>';
			echo '
			<script type="text/javascript" src="'.BASE_PATH.'modules/useradmin/useradmin.js" charset="UTF-8"></script>
			<script type="text/javascript">
				var useradmin_path = "'.BASE_PATH.ADMIN_FOLDER.'/";
				var base_path = "'.BASE_PATH.'";
			</script>
			<link href="'.BASE_PATH.'modules/useradmin/useradmin.css" rel="stylesheet" type="text/css">';
		}

		if (! IS_MOBILE && ($this->diafan->configmodules('use_animation') || $this->diafan->configmodules('use_animation', 'site') || $this->diafan->_user->useradmin))
		{
			echo '
			<script src="'.BASE_PATH.'js/jquery.prettyPhoto.js" type="text/javascript" charset="UTF-8"></script>
			<link rel="stylesheet" href="'.BASE_PATH.'css/prettyPhoto.css" type="text/css" media="screen"'
			.' title="prettyPhoto main stylesheet" charset="utf-8">';
		}

		echo $this->diafan->js;

		echo "\n".'<meta name="robots" content="';
		if($this->diafan->noindex)
		{
			echo 'noindex';
		}
		else
		{
			echo 'all';
		}
		echo '">';
		
		if($this->diafan->configmodules('type', 'captcha') == 'reCAPTCHA')
		{
			echo '<script type="text/javascript" src="http://www.google.com/recaptcha/api/js/recaptcha_ajax.js"></script>
			<script type="text/javascript">
			var create_recaptcha = function (element_id)
			{
				$(".recaptcha_show").show();
				$("#"+element_id).next(".recaptcha_show").hide();
				Recaptcha.create("'.$this->diafan->configmodules('recaptcha_public_key', 'captcha').'", element_id, {theme : "clean"});
			}
			</script>
';
		}
		if($this->diafan->configmodules("redhelper_login", "consultant"))
		{
			echo '<meta http-equiv="X-UA-Compatible" content="IE=8" />
			<meta http-equiv="X-UA-Compatible" content="IE=EmulateIE8" /> ';
		}
		if($this->diafan->canonical)
		{
			echo '<link href="'.BASE_PATH_HREF.$this->diafan->canonical.'" rel="canonical" />';
		}
	}

	/**
	 * Шаблонная функция: выводит ссылку на страницу сайта. Если текущая страница соответствует адресу, на которую ведет ссылка, то ссылка становится неактивной. Шаблонная функция аналогична стандартной HTML-конструкции `<a href=""></a>`, но ликвидирует ссылки, которые никуда не ведут.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * rewrite - псевдоссылка страницы, на которую ведет ссылка, например: news
	 * img - адрес изображения, использующегося в качестве ссылки, например: img/logo.gif
	 * img_act - адрес изображения на текущей странице, например: img/home_act.gif
	 * width - ширина изображения
	 * height - высота изображения
	 * alt - альтернативный тег для изображения или текст ссылки
	 * 
	 * @return void
	 */
	public function show_href($a)
	{
		if(in_array($this->diafan->theme, array('404.php', '503.php', '403.php')))
		{
			$this->diafan->rewrite = $this->diafan->theme;
		}
		$a = $this->view->get_attributes($a, 'rewrite', 'img', 'img_act', 'width', 'height', 'alt', 'alt'._LANG);
		
		$rewrite = $a["rewrite"];
		if ($rewrite && $this->diafan->rewrite != $rewrite)
		{
			if(! DB::query_result("SELECT s.id FROM {site} AS s INNER JOIN {rewrite} AS r ON s.id=r.site_id AND r.module_name='site' WHERE r.rewrite='%h' AND r.trash='0' AND s.[act]='1' AND s.trash='0'", $rewrite))
			{
				return;
			}
		}

		$img = ($this->diafan->rewrite != $rewrite || ! $a["img_act"]
		       ? preg_replace('/[^A-Za-z0-9-_\/\.]+/', '', $a["img"])
		       : preg_replace('/[^A-Za-z0-9-_\/\.]+/', '', $a["img_act"])
		       );
		
		$width = preg_replace('/[^0-9]+/', '', $a["width"]);
		$height = preg_replace('/[^0-9]+/', '', $a["height"]);
		
		$name = ($a["alt"] == "title"
			? TITLE
			: ($a["alt"] == "url"
			  ? BASE_PATH
			  : $this->diafan->_($a["alt"], false)
			  )
			);
		
		if ($this->diafan->rewrite != $rewrite || $this->diafan->show || $this->diafan->cat)
		{
			echo '<a href="'.BASE_PATH_HREF.($rewrite ? $rewrite.ROUTE_END : '').'" title="'.$name.'">';
		}
		if ($img)
		{
			echo '<img src="'.BASE_PATH.$img.'" alt="'.$name.'"'.($width ? ' width="'.$width.'"' : '').($height ? ' height="'.$height.'"' : '').'>';
		}
		else
		{
			echo $name;
		}
		if ($this->diafan->rewrite != $rewrite || $this->diafan->show || $this->diafan->cat)
		{
			echo '</a>';
		}
	}

	/**
	 * Шаблонная функция: выводит ссылки на социальные сети.
	 * 
	 * @return void
	 */
	public function show_social_links()
	{
		echo '
		<br><br>
		<script type="text/javascript" src="//yandex.st/share/share.js" charset="utf-8"></script>
		<div class="yashare-auto-init" data-yashareL10n="ru" data-yashareType="button" data-yashareQuickServices="yaru,vkontakte,facebook,twitter,odnoklassniki,moimir"></div> ';
	}

	/**
	 * Шаблонная функция: подключает файл-блок шаблона.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * file - имя файла из папки *themes/blocks*
	 * @return void
	 */
	public function show_include($attributes)
	{
		$attributes = str_replace('/[^a-z_0-9]+/', '', $this->view->get_attributes($attributes, 'file'));

		$inc = file_get_contents(ABSOLUTE_PATH.'themes/blocks/'.$attributes["file"].'.php');

		$this->view->get_function_in_theme($inc);
	}
}