<?php
/**
 * @package    Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(__FILE__)).'/includes/404.php');
}

define('INSTALL_DEMO', false);

require_once(ABSOLUTE_PATH.'includes/core.php');

/**
 * Installation
 *
 * Установка diafan.CMS
 */
class Installation extends Core
{
	/**
	 * @var integer код последнего обновления
	 */
	private $date_update = 1400495613;

	/**
	 * @var string номер минорной версии diafan.CMS
	 */
	private $version = '5.3';

	/**
	 * @var array устанавливаемые модули
	 */
	private $modules = array('core', 'users', 'messages', 'news', 'clauses', 'feedback', 'faq', 'shop', 'ab', 'files', 'search', 'bs', 'comments', 'rating', 'tags', 'photo', 'votes', 'subscribtion', 'forum', 'map', 'keywords', 'mistakes', 'consultant', 'webeffector', 'filemanager', 'captcha');

	/**
	 * @var array шаги установки
	 */
	private $steps = array(
		'step1' => 'Проверка системы',
		'step2' => 'Лицензия',
		'step3' => 'Параметры',
		'step4' => 'Администратор сайта',
		'step5' => 'Модули',
		'step6' => 'Окончание'
	);

	/**
	 * @var object данные для шаблона
	 */
	private $view;

	/**
	 * Инициализация установки
	 *
	 * @return void
	 */
	public function init()
	{
		date_default_timezone_set('Europe/Moscow');
		header('Content-Type: text/html; charset=utf-8');

		if (empty($_GET["rewrite"]) || ! preg_match('/^install/', $_GET["rewrite"]))
		{
			header("Cache-Control: no-cache");
			header('Location: http://'.$_SERVER["HTTP_HOST"].str_replace(! empty($_GET["rewrite"]) ? $_GET["rewrite"].'/' : '', '', $_SERVER["REQUEST_URI"])."install/", true, 302);
			exit;
		}
		$_GET["rewrite"] = preg_replace('/^install.php[\/]*/', '', $_GET["rewrite"]);
		$_GET["rewrite"] = preg_replace('/^install[\/]*/', '', $_GET["rewrite"]);

		define('IS_ADMIN', false);

		include(ABSOLUTE_PATH."config.php");
		if (! defined("MOD_DEVELOPER"))
		{
			define("MOD_DEVELOPER", false);
		}
		if (! defined("REVATIVE_PATH"))
		{
			$dir_url_path = @substr(preg_replace("/\/install.*/", "", $_SERVER['REQUEST_URI']), 1);
			define('REVATIVE_PATH', $dir_url_path);
		}
		define('BASE_PATH', "http://".$_SERVER['SERVER_NAME'].'/'.(REVATIVE_PATH ? REVATIVE_PATH.'/' : ''));
		
		
		include_once ABSOLUTE_PATH.'includes/customization.php';
		include_once ABSOLUTE_PATH.'includes/files.php';
		include(ABSOLUTE_PATH.'includes/developer.php');
		
		global $dev;
		$dev = new Dev;
		$dev->init();
		try
		{
			$this->view = new stdClass();
			$this->route();
			$this->template();
		}
		catch (Exception $e)
		{
			$dev->exception($e);
		}
	}

	/**
	 * Подключение шага установки
	 *
	 * @return void
	 */
	private function route()
	{
		if(function_exists('set_time_limit'))
		{
			set_time_limit(0);
		}
		if (empty($_GET["rewrite"]))
		{
			$_GET["rewrite"] = "step1";
		}
		elseif(! in_array($_GET["rewrite"], array_keys($this->steps)))
		{
			include_once ABSOLUTE_PATH.'includes/404.php';
		}

		session_name('SESS'.md5($_SERVER['HTTP_HOST']));
		session_start();
		
		switch ($_GET["rewrite"])
		{
			case 'step3':
				$this->step3();
				break;

			case 'step4':
				$this->step4();
				break;
	
			case 'step5':
				$this->step5();
				break;
	
			case 'step6':
				$this->step6();
				break;
		}
	}

	/**
	 * Действия на третьем шаге установки
	 *
	 * @return void
	 */
	private function step3()
	{
		$this->view->site_name = (! empty($_SESSION["install_name"]) ? str_replace('"', '&quot;', $_SESSION["install_name"]) : '');
		$this->view->db_host = (! empty($_SESSION["install_db_host"]) ? str_replace('"', '&quot;', $_SESSION["install_db_host"]) : 'localhost');
		$this->view->db_user = (! empty($_SESSION["install_db_user"]) ? str_replace('"', '&quot;', $_SESSION["install_db_user"]) : '');
		$this->view->db_pass = (! empty($_SESSION["install_db_pass"]) ? str_replace('"', '&quot;', $_SESSION["install_db_pass"]) : '');
		$this->view->db_name = (! empty($_SESSION["install_db_name"]) ? str_replace('"', '&quot;', $_SESSION["install_db_name"]) : '');
		$this->view->db_prefix = (! empty($_SESSION["install_db_prefix"]) ? str_replace('"', '&quot;', $_SESSION["install_db_prefix"]) : 'diafan_');
	}

	/**
	 * Действия на четвертом шаге установки
	 *
	 * @return void
	 */
	private function step4()
	{
		if(! empty($_POST["form"]))
		{
			$_SESSION["install_name"] = $this->get_param($_POST, "name", '', 1);
			$_SESSION["install_db_host"] = $this->get_param($_POST, "db_host", '', 1);
			$_SESSION["install_db_user"] = $this->get_param($_POST, "db_user", '', 1);
			$_SESSION["install_db_pass"] = $this->get_param($_POST, "db_pass", '', 1);
			$_SESSION["install_db_name"] = $this->get_param($_POST, "db_name", '', 1);
			$_SESSION["install_db_prefix"] = $this->get_param($_POST, "db_prefix", '', 1);
	
			if (empty($_POST['db_host']) || empty($_POST['db_user']) || empty($_POST['db_name']))
			{
				$this->error('Вами введены неверные данные о БД MySQL или не заполнены необходимые поля формы.');
			}
			$sheme = function_exists('extension_loaded') && extension_loaded('mysqli') ? 'mysqli' : 'mysql';
	
			$db_url = $sheme.'://'.$this->get_param($_POST, "db_user", '', 1);
			if($_POST["db_pass"])
			{
				$db_url .= ":".$this->get_param($_POST, "db_pass", '', 1);
			}
			$db_url .= "@".$this->get_param($_POST, "db_host", '', 1)
			."/".$this->get_param($_POST, "db_name", '', 1);
	
			require_once (ABSOLUTE_PATH.'includes/database.php');
			if(! defined('DB_PREFIX'))
			{
				define('DB_PREFIX', '');
			}
			if(! defined('DB_CHARSET'))
			{
				define('DB_CHARSET', 'utf8');
			}
	
			if (! DB::connect($db_url, true))
			{
				$this->error('Введены неверные параметры подключения к базе данных MySQL!');
			}
	
			$db = $this->get_param($_POST, "db_name", '', 1);
			$result = DB::query("SHOW TABLES FROM `".$db."`");
			while ($row = DB::fetch_array($result))
			{
				if ($row["Tables_in_".$db] == $this->get_param($_POST, "db_prefix", '', 1)."users")
				{
					$this->error('Префикс занят!');
					break;
				}
			}
			DB::free_result($result);
			DB::close();
	
			$new_values = array(
					'TIT1' => str_replace('\\"', '&quot;', $this->get_param($_POST, "name", '', 1)),
					'DB_URL' => $db_url,
					'DB_PREFIX' => str_replace('"', '&quot;', $this->get_param($_POST, "db_prefix", '', 1)),
					'DB_CHARSET' => 'utf8',
					'LANGUAGE_BASE' => 'ru',
					'VERSION_CMS' => $this->version,
					'DATE_UPDATE' => $this->date_update,
					'MOD_DEVELOPER' => false,
					'MOD_DEVELOPER_CACHE' => false,
					'MOD_DEVELOPER_PROFILING' => false,
					'ROUTE_END' => '/',
					'ROUTE_AUTO_MODULE' => true,
					'USERADMIN' => true,
					'MOBILE_VERSION' => false,
					'ADMIN_FOLDER' => 'admin',
					'USERFILES' => 'userfiles',
				);
			include_once(ABSOLUTE_PATH.'includes/config.php');
			Config::save($new_values, array(0 => array('id' => 1)), INSTALL_DEMO);
			if(INSTALL_DEMO)
			{
				$this->redirect(BASE_PATH.'install/step6/');
			}
		}
		$this->view->fio = (! empty($_SESSION["install_admin_fio"]) ? str_replace('"', '&quot;', $_SESSION["install_admin_fio"]) : '');
		$this->view->admin_name = (! empty($_SESSION["install_admin_name"]) ? str_replace('"', '&quot;', $_SESSION["install_admin_name"]) : '');
		$this->view->mail = (! empty($_SESSION["install_admin_mail"]) ? str_replace('"', '&quot;', $_SESSION["install_admin_mail"]) : '');
		$this->view->folder = (! empty($_SESSION["install_admin_folder"]) ? str_replace('"', '&quot;', $_SESSION["install_admin_folder"]) : 'admin');
		$this->view->userfiles = (! empty($_SESSION["install_userfiles"]) ? str_replace('"', '&quot;', $_SESSION["install_userfiles"]) : 'userfiles');
	}

	/**
	 * Действия на пятом шаге установки
	 *
	 * @return void
	 */
	private function step5()
	{
		if(! empty($_POST["form"]))
		{
			$userfiles = $this->get_param($_POST, "userfiles", 'userfiles', 1);
			$_SESSION["install_admin_mail"] = $this->get_param($_POST, "mail", '', 1);
			$_SESSION["install_admin_name"] = $this->get_param($_POST, "name", '', 1);
			$_SESSION["install_admin_pass"] = $this->get_param($_POST, "pass", '', 1);
			$_SESSION["install_admin_fio"] = $this->get_param($_POST, "fio", '', 1);
			$_SESSION["install_admin_folder"] = $this->get_param($_POST, "folder", 'admin', 1);
			$_SESSION["install_userfiles"] = $userfiles;
	
			if(! Files_inc::is_writable($userfiles))
			{
				$_SESSION["install_userfiles"] = '';
				$this->error('Папка '.$userfiles.' должна быть доступна на запись!');
			}

			include_once(ABSOLUTE_PATH.'includes/validate.php');
			if ($mes = Validate::password($_POST["pass"], true))
			{
				$this->error($mes);
			}
			if ($mes = Validate::mail($_POST['mail']))
			{
				$this->error($mes);
			}
	
			if (empty($_POST['fio']) || empty($_POST['name']) || empty($_POST['pass'])
			   || empty($_POST['mail']) || empty($_POST['folder']))
			{
				$this->error('Заполните, пожалуйста, все поля.');
			}

			include_once (ABSOLUTE_PATH.'includes/validate.php');
	
			$new_values = array(
					'EMAIL_CONFIG' => ! Validate::mail($_POST["mail"]) ? $this->get_param($_POST, "mail", '', 1) : '',
					'ADMIN_FOLDER' => $this->get_param($_POST, "folder", '', 1),
					'USERFILES' => $userfiles,
				);
			include_once(ABSOLUTE_PATH.'includes/config.php');
			Config::save($new_values, array(0 => array('id' => 1)));
		}

		include_once(ABSOLUTE_PATH.'includes/install.php');
		foreach ($this->modules as $module)
		{
			if ($module == "core")
			continue;

			if (file_exists(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.install.php'))
			{
				include_once(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.install.php');
				$this->view->modules[] = array("name" => $module, "title" => $title);
			}
		}
	}

	/**
	 * Действия на шестом шаге установки
	 *
	 * @return void
	 */
	private function step6()
	{
		if(INSTALL_DEMO  || !empty($_POST["example_yes"]))
		{
			if(defined('USERFILES'))
			{
				$userfiles = USERFILES;
			}
			else
			{
				$userfiles = 'userfiles';
			}
			if(! is_dir(ABSOLUTE_PATH.$userfiles.'/demofiles'))
			{
				if(! file_exists(ABSOLUTE_PATH.$userfiles.'/demofiles.zip'))
				{
					Files_inc::copy_file('http://cms.diafan.ru/demofiles.zip', 'demofiles.zip', $userfiles);
				}
				if(! class_exists('ZipArchive'))
				{
					$this->error('На сервере не установлено расширение для распоковки ZIP-архивов. Распокуйте содержимое архива '.$userfiles.'/demofiles.zip в папку '.$userfiles.'/demofiles.');
				}
				$zip = new ZipArchive;
				if ($zip->open(ABSOLUTE_PATH.$userfiles.'/demofiles.zip') === true)
				{
					$zip->extractTo($userfiles.'/demofiles');
					$zip->close();
				}
			}
		}
		
		setcookie('useradmin', false, mktime(0, 0, 0, 1, 1, 2000), '/');

		include_once ABSOLUTE_PATH.'plugins/encoding.php';
		include_once(ABSOLUTE_PATH."includes/database.php");
		DB::connect();

		if(INSTALL_DEMO)
		{
			include_once(ABSOLUTE_PATH.'includes/diafan.php');
			include_once(ABSOLUTE_PATH.'includes/demo.php');
			$demo = new Demo($this);
			$demo->install();
		}
		else
		{
			include_once(ABSOLUTE_PATH.'includes/install.php');
			$langs = array(1);
			if(! empty($_POST["lang_yes"]))
			{
				$langs[] = 2;
			}
			$install = new Install($langs);
			foreach ($this->modules as $module)
			{
				if (empty($_POST[$module]) && $module != 'core')
					continue;
	
				$install->install($module, !empty($_POST["example_yes"]));
			}

			include_once ABSOLUTE_PATH.'includes/cache.php';
			$cache = new Cache();
			$cache->delete("", array());
			$this->view->admin_name = $_SESSION["install_admin_name"];
			$this->view->admin_pass = $_SESSION["install_admin_pass"];
		}

		DB::close();
		$_SESSION = array();
		Files_inc::delete_file("installation/install.php");
	}

	/**
	 * Подключение шаблона установки
	 *
	 * @return void
	 */
	private function template()
	{
		$this->view->version = $this->version;
		$this->view->steps = $this->steps;
		$this->view->rewrite = $_GET["rewrite"];
		$this->view->name = $this->steps[$_GET["rewrite"]];

		include_once(ABSOLUTE_PATH.'adm/themes/install/install.view.php');
	}

	/**
	 * Подключение шаблона контентной области
	 *
	 * @return void
	 */
	private function view($name)
	{
		include_once(ABSOLUTE_PATH.'adm/themes/install/install.view.'.$name.'.php');
	}

	/**
	 * Обработка ошибок
	 *
	 * @param string $error текст ошибки
	 * @return void
	 */
	private function error($error)
	{
		if(preg_match('/step([0-9]+)$/', $_GET["rewrite"], $m))
		{
			$step = intval($m[1]) - 1;
		}
		echo '<script>alert("'.$error.'"); document.location.href="'.BASE_PATH.'install/'.($step > 1 ? 'step'.$step.'/' : '').'";</script>';
		exit;
	}
}

$install = new Installation();
$install->init();
exit;