<?php
/**
 * Обработка запроса на добавление объявления
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Ab_ajax
 */
class Ab_ajax extends Ajax
{
	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (! empty($_POST['module']) && $_POST['module'] == 'ab' && ! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'add':
					return $this->add();

				case 'edit':
					return $this->edit();

				case 'upload_image':
					return $this->upload_image();

				case 'delete_image':
					return $this->delete_image();
			}
		}
		return false;
	}

	/**
	 * Добавление объявления
	 * 
	 * @return boolean
	 */
	private function add()
	{
		if (! $this->site_id = DB::query_result(
					"SELECT s.id FROM {site} AS s"
					.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
					." WHERE s.id=%d AND s.module_name='ab' AND s.trash='0'"
					." AND (s.access='0'"
					.($this->diafan->_user->role_id ? " OR s.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
					.") LIMIT 1", $_POST["site_id"]
			))
		{
			return false;
		}
		$use_cat = $this->diafan->configmodules('cat', "ab", $this->site_id);
		if (empty($_POST["cat_id"]))
		{
			if($use_cat)
			{
				return false;
			}
			else
			{
				$cat_id = 0;
			}
		}
		else
		{
			$cat_id = $_POST["cat_id"];
		}
		if ($this->diafan->configmodules('only_user', "ab", $this->site_id) && ! $this->diafan->_user->id)
		{
			return false;
		}
		$this->module = "ab";
		$this->tag = "ab";

		$this->check_captcha();

		$this->validate();

		Customization::inc('modules/ab/ab.model.php');
		$model = new Ab_model($this->diafan);
		$params = $model->get_param_form($this->site_id, $cat_id, $use_cat);

		$this->empty_required_field(array("params" => $params));

		if ($this->send_errors())
			return true;
		
		$fields = array('created', 'user_id', '[act]', 'site_id', 'cat_id');
		$query = array('%d', '%d', "'%d'", '%d', '%d');
		$values = array(
			time(),
			$this->diafan->_user->id,
			$this->diafan->configmodules('premoderation', "ab", $this->site_id) ? 0 : 1,
			$this->site_id,
			$cat_id
		);
		if($this->diafan->configmodules('form_name', "ab", $this->site_id))
		{
			$fields[] = '[name]';
			$query[] = "'%s'";
			$values[] = nl2br(htmlspecialchars($_POST["name"]));
		}
		if($this->diafan->configmodules('form_anons', "ab", $this->site_id))
		{
			$fields[] = '[anons]';
			$query[] = "'%s'";
			$values[] = nl2br(htmlspecialchars($_POST["anons"]));
		}
		if($this->diafan->configmodules('form_text', "ab", $this->site_id))
		{
			$fields[] = '[text]';
			$query[] = "'%s'";
			$values[] = nl2br(htmlspecialchars($_POST["text"]));
		}
		if($this->diafan->configmodules('form_date_finish', "ab", $this->site_id))
		{
			$fields[] = 'date_finish';
			$query[] = "%d";
			$values[] = $this->diafan->unixdate($_POST["date_finish"]);
		}

		$save = DB::query("INSERT INTO {ab} (".implode(",", $fields).") VALUES (".implode(",", $query).")", $values);

		if($cat_id)
		{
			DB::query("INSERT INTO {ab_category_rel} (element_id, cat_id) VALUES (%d, %d)", $save, $cat_id);
		}

		if(! empty($_POST["tmpcode"]))
		{
			DB::query("UPDATE {images} SET element_id=%d, tmpcode='' WHERE module_name='ab' AND element_id=0 AND tmpcode='%s'", $save, $_POST["tmpcode"]);
		}

		if(! $this->diafan->configmodules('premoderation', "ab", $this->site_id))
		{
			$this->diafan->_cache->delete('', 'ab');
		}
		if(ROUTE_AUTO_MODULE && ! empty($_POST["name"]))
		{
			Customization::inc('adm/includes/save_functions.php');
			$save_functions = new Save_functions_admin($this->diafan);
			$rewrite = $save_functions->generate_rewrite($_POST["name"]);
			if($cat_id)
			{
				$rewrite_parent = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='ab' AND cat_id=%d LIMIT 1", $cat_id);
			}
			if(empty($rewrite_parent))
			{
				$rewrite_parent = DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='site' AND site_id=%d LIMIT 1", $this->site_id);
			}
			if(! empty($rewrite_parent))
			{
				$rewrite = $rewrite_parent.'/'.$rewrite;
			}
			if (DB::query_result("SELECT COUNT(*) FROM {rewrite} WHERE rewrite='%h' AND trash='0'", $rewrite))
			{
				$rewrite .= $save;
			}
			DB::query("INSERT INTO {rewrite} (rewrite, module_name, element_id, site_id) VALUES ('%h', 'ab', %d, %d)", $rewrite, $save, $this->site_id);
		}
		// если объявления сразу активируется на сайте, сохраняет ссылку на карте сайта
		if(! $this->diafan->configmodules('premoderation', "ab", $this->site_id))
		{
			$ab_row = array(
				"module_name" => 'ab',
				"id"          => $save,
				"site_id"     => $this->site_id,
				"cat_id"      => $cat_id,
			);
			foreach ($this->diafan->languages as $l)
			{
				$ab_row["act".$l["id"]] = $l["id"] == _LANG ? true : false;
			}
			$this->diafan->_map->index_element($ab_row);
		}

		$this->insert_values(array("id" => $save, "table" => "ab", "params" => $params, "multilang" => true));

		if ($this->send_errors())
			return true;

		$this->send_mail();
		$this->send_sms();

		$mes = $this->diafan->configmodules('add_message', 'ab', $this->site_id, _LANG);
		$this->result["errors"][0] = $mes ? $mes : ' ';
		$this->result["success"] = true;
		$this->result["form_hide"]= true;
		return $this->send_errors();
	}

	/**
	 * Редактирование объявления
	 * 
	 * @return boolean
	 */
	private function edit()
	{
		if(empty($_POST["id"]) || ! $this->diafan->_user->id)
		{
			return false;
		}
		$row = DB::query_fetch_array("SELECT id, [name], [anons], [text], cat_id, date_finish, [act]  FROM {ab}"
		." WHERE id=%d AND trash='0' AND site_id=%d AND user_id=%d LIMIT 1",
		$_POST["id"], $this->diafan->cid, $this->diafan->_user->id);
		if(! $row)
		{
			return false;
		}
		$use_cat = $this->diafan->configmodules('cat');
		if (empty($_POST["cat_id"]))
		{
			if($use_cat)
			{
				return false;
			}
			else
			{
				$cat_id = 0;
			}
		}
		else
		{
			$cat_id = $_POST["cat_id"];
		}

		if (! $this->diafan->_user->checked)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();

		$this->module = "ab";
		$this->tag = "ab";

		$this->validate();

		Customization::inc('modules/ab/ab.model.php');
		$model = new Ab_model($this->diafan);
		$params = $model->get_param_form($this->diafan->cid, $cat_id, $use_cat);

		$this->empty_required_field(array("params" => $params));

		if ($this->send_errors())
			return true;
		
		$fields = array('created=%d', "[act]='%d'", 'cat_id=%d');
		$values = array(
			time(),
			$this->diafan->configmodules('premoderation', "ab", $this->site_id) ? 0 : $row["act"],
			$cat_id
		);
		if($this->diafan->configmodules('form_name', "ab", $this->site_id))
		{
			$fields[] = "[name]='%s'";
			$values[] = nl2br(htmlspecialchars($_POST["name"]));
		}
		if($this->diafan->configmodules('form_anons', "ab", $this->site_id))
		{
			$fields[] = "[anons]='%s'";
			$values[] = nl2br(htmlspecialchars($_POST["anons"]));
		}
		if($this->diafan->configmodules('form_text', "ab", $this->site_id))
		{
			$fields[] = "[text]='%s'";
			$values[] = nl2br(htmlspecialchars($_POST["text"]));
		}
		if($this->diafan->configmodules('form_date_finish', "ab", $this->site_id))
		{
			$fields[] = 'date_finish=%d';
			$values[] = $this->diafan->unixdate($_POST["date_finish"]);
		}
		$values[] = $_POST["id"];

		$save = DB::query("UPDATE {ab} SET ".implode(",", $fields)." WHERE id=%d", $values);

		if($cat_id != $row["cat_id"])
		{
			DB::query("DELETE FROM {ab_category_rel} WHERE element_id=%d", $_POST["id"]);
			DB::query("INSERT INTO {ab_category_rel} (element_id, cat_id) VALUES (%d, %d)", $_POST["id"], $cat_id);
		}
		if(! $this->diafan->configmodules('premoderation', "ab", $this->site_id))
		{
			$this->diafan->_cache->delete('', 'ab');
		}
		
		$this->update_values(array("id" => $_POST["id"], "table" => "ab", "params" => $params, "multilang" => true));

		if ($this->send_errors())
			return true;

		$this->result["errors"][0] = $this->diafan->_('Изменения сохранены.');
		return $this->send_errors();
	}

	/**
	 * Загружает изображение
	 *
	 * @return boolean
	 */
	private function upload_image()
	{
		$element_id = 0;
		$tmpcode = '';
		$param_id = '';
		if(! empty($_POST["images_param_id"]))
		{
			$param_id = $this->diafan->get_param($_POST, "images_param_id", '', 2);
		}
		else
		{
			if(! $this->diafan->configmodules("images") || ! $this->diafan->configmodules('form_images'))
			{
				return false;
			}
		}
		if(! empty($_POST["id"]))
		{
			if(! $this->diafan->_user->id)
			{
				return false;
			}
			$user_id = DB::query_result("SELECT user_id FROM {ab} WHERE id=%d", $_POST["id"]);
			if($user_id != $this->diafan->_user->id)
			{
				return false;
			}
			$element_id = $this->diafan->get_param($_POST, "id", '', 2);

			if (! $this->diafan->_user->checked)
			{
				$this->result["errors"][0] = 'ERROR';
				return $this->send_errors();
			}
			$this->result["hash"] = $this->diafan->_user->get_hash();
		}
		else
		{
			if ($this->diafan->configmodules('only_user', "ab", $this->site_id) && ! $this->diafan->_user->id)
			{
				return false;
			}
			if(empty($_POST["tmpcode"]))
			{
				return false;
			}
			$tmpcode = $_POST["tmpcode"];
		}
		$this->result["success"] = true;
		if (! empty($_FILES['images'.$param_id]) && $_FILES['images'.$param_id]['tmp_name'] != '' && $_FILES['images'.$param_id]['name'] != '')
		{
			try
			{
				$this->diafan->_images->upload($element_id, "ab", $this->site_id, $_FILES['images'.$param_id]['tmp_name'], $_FILES['images'.$param_id]['name'], false, false, $param_id, $tmpcode);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = ($param_id ? 'p'.$param_id : 'images');
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
			if($param_id)
			{
				$image_tag = 'large';
			}
			else
			{
				$image_tag = 'medium';
			}
			$images = $this->diafan->_images->get($image_tag, $element_id, "ab", $this->site_id, '', $param_id, false, 0, '', $tmpcode);
			$this->result["data"] = $this->diafan->_tpl->get('images', "ab", $images);
		}
		return $this->send_errors();
	}

	/**
	 * Удаляет изображение
	 *
	 * @return boolean
	 */
	private function delete_image()
	{
		if(empty($_POST["id"]))
		{
			return false;
		}
		if(! empty($_POST["element_id"]))
		{
			if(! $this->diafan->_user->id)
			{
				return false;
			}
			$user_id = DB::query_result("SELECT user_id FROM {ab} WHERE id=%d", $_POST["element_id"]);
			if($user_id != $this->diafan->_user->id)
			{
				return false;
			}
			$where = "element_id=%d";
			$id = $_POST["element_id"];

			if (! $this->diafan->_user->checked)
			{
				$this->result["errors"][0] = 'ERROR';
				return $this->send_errors();
			}
			$this->result["hash"] = $this->diafan->_user->get_hash();
		}
		else
		{
			if ($this->diafan->configmodules('only_user', "ab", $this->site_id) && ! $this->diafan->_user->id)
			{
				return false;
			}
			if(empty($_POST["tmpcode"]))
			{
				return false;
			}
			$where = "tmpcode='%s'";
			$id = $_POST["tmpcode"];
		}
		$row = DB::query_fetch_array("SELECT * FROM {images} WHERE module_name='ab' AND id=%d AND ".$where, $_POST["id"], $id);
		if(! $row)
		{
			return false;
		}
		$this->result["success"] = true;

		$this->diafan->_images->delete($row["element_id"], "ab", $row["id"]);
		return $this->send_errors();
	}

	/**
	 * Проверяет валидность введенных данных
	 * 
	 * @return void
	 */
	private function validate()
	{
		Customization::inc('includes/validate.php');
		if($this->diafan->configmodules('form_name', "ab", $this->site_id))
		{
			if(empty($_POST["name"]))
			{
				$mes = 'Пожалуйста, введите заголовок.';
			}
			else
			{
				$mes = Validate::text($_POST["name"]);
			}
			if ($mes)
			{
				$this->result["errors"]["name"] = $this->diafan->_($mes);
			}
		}
		if($this->diafan->configmodules('form_anons', "ab", $this->site_id))
		{
			if(empty($_POST["anons"]))
			{
				$mes = 'Пожалуйста, введите краткое содержание.';
			}
			else
			{
				$mes = Validate::text($_POST["anons"]);
			}
			if ($mes)
			{
				$this->result["errors"]["anons"] = $this->diafan->_($mes);
			}
		}
		if($this->diafan->configmodules('form_text', "ab", $this->site_id))
		{
			if(empty($_POST["text"]))
			{
				$mes = 'Пожалуйста, введите полное содержание.';
			}
			else
			{
				$mes = Validate::text($_POST["text"]);
			}
			if ($mes)
			{
				$this->result["errors"]["text"] = $this->diafan->_($mes);
			}
		}
		if($this->diafan->configmodules('form_date_finish', "ab", $this->site_id))
		{
			$mes = '';
			if(! empty($_POST["date_finish"]))
			{
				$mes = Validate::date($_POST["date_finish"]);
			}
			if ($mes)
			{
				$this->result["errors"]["date_finish"] = $this->diafan->_($mes);
			}
		}
	}

	/**
	 * Уведомление администратора по e-mail
	 * 
	 * @return void
	 */
	private function send_mail()
	{
		if (! $this->diafan->configmodules("sendmailadmin", 'ab', $this->site_id))
			return false;

		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules("subject_admin", 'ab', $this->site_id)
		);
		$mes = '';
		if($this->diafan->configmodules('form_name', "ab", $this->site_id))
		{
			$mes = $this->diafan->_('Заголовок', false).': '.nl2br(htmlspecialchars($_POST["name"]));
		}
		if($this->diafan->configmodules('form_anons', "ab", $this->site_id))
		{
			$mes .= ($mes ? '<br>' : '').$this->diafan->_('Краткое содержание', false).': '.nl2br(htmlspecialchars($_POST["anons"]));
		}
		if($this->diafan->configmodules('form_text', "ab", $this->site_id))
		{
			$mes .= ($mes ? '<br>' : '').$this->diafan->_('Полное содержание', false).': '.nl2br(htmlspecialchars($_POST["text"]));
		}
		if($this->diafan->configmodules('form_date_finish', "ab", $this->site_id))
		{
			$mes .= ($mes ? '<br>' : '').$this->diafan->_('Опубликовать на сайте до', false).': '.nl2br(htmlspecialchars($_POST["date_finish"]));
		}

		$mes .= ($this->message_admin_param ? '<br>'.$this->message_admin_param : '');

		$message = str_replace(
			array('%title', '%url', '%message'),
			array(
				TITLE,
				BASE_URL,
				$mes
			),
			$this->diafan->configmodules("message_admin", 'ab', $this->site_id)
		);

		$to   = $this->diafan->configmodules("emailconfadmin", 'ab', $this->site_id)
		        ? $this->diafan->configmodules("email_admin", 'ab', $this->site_id)
		        : EMAIL_CONFIG;
		$from = $this->diafan->configmodules("emailconf", 'ab', $this->site_id)
		        ? $this->diafan->configmodules("email", 'ab', $this->site_id)
		        : '';

		include_once ABSOLUTE_PATH.'includes/mail.php';
		send_mail($to, $subject, $message, $from);
	}

	/**
	 * Уведомляет администратора по SMS
	 * 
	 * @return void
	 */
	private function send_sms()
	{
		if (! $this->diafan->configmodules("sendsmsadmin", 'ab', $this->site_id))
			return false;
			
		$message = $this->diafan->configmodules("sms_message_admin", 'ab', $this->site_id);

		$to   = $this->diafan->configmodules("sms_admin", 'ab', $this->site_id);

		include_once ABSOLUTE_PATH.'includes/sms.php';
		Sms::send($message, $to);
	}
}