<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Ab
 */
class Ab extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat', 'year', 'month', 'day', 'param', 'edit');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;

		if ($this->diafan->show)
		{
			$this->result = $this->model->id();
		}
		elseif ($this->diafan->edit)
		{
			$this->result = $this->model->edit();
		}
		elseif ($this->diafan->param)
		{
			$this->result = $this->model->list_param();
		}
		elseif(isset($_GET["action"]))
		{
			switch($_GET["action"])
			{
				case 'search':
					$this->result = $this->model->list_search();
					break;
				case 'my':
					$this->result = $this->model->list_my();
					break;
				case 'delete':
					$this->result = $this->model->delete();
					break;
				case 'block':
					$this->result = $this->model->block();
					break;
				default:
					include(ABSOLUTE_PATH.'includes/404.php');
			}
		}
		elseif (! $this->diafan->configmodules("cat"))
		{
			$this->result = $this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->result = $this->model->first_page();
		}
		else
		{
			$this->result = $this->model->list_category();
		}
		$this->get_global_variables();
		$this->result["form"] = $this->model->form();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		if (! empty($this->result["err"]))
		{
			echo $this->result["err"];
			return;
		}
		$this->diafan->_tpl->get($this->view, 'ab', $this->result);
	}

	/**
	 * Шаблонная функция: выводит последние объявления на всех страницах, кроме страницы объявлений, когда выводится список тех же объявлений, что и в функции.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых объявлений (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - категории объявлений, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. По умолчанию категория не учитывается, выводятся все объявления
	 * sort - сортировка объявлений: **date** – по дате (по умолчанию), **rand** – в случайном порядке
	 * only_ab - выводить блок только на странице, к которой прикреплен модуль «Объявления»
	 * images - количество изображений, прикрепленных к объявления
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * param - значения дополнительных характеристик
	 * template - шаблон тега (файл modules/ab/views/ab.view.show_block_**template**.php; по умолчанию шаблон modules/ab/views/ab.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_ab', 'count', 'cat_id', 'site_id', 'images', 'images_variation', 'sort', 'param', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);
		$site_ids = explode(",", $attributes["site_id"]);
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';
		$param   = $attributes["param"];
		$sort    = in_array($attributes["sort"], array("date", "rand")) ? $attributes["sort"] : "date";
		$only_ab = $attributes["only_ab"];

		if ($only_ab && ($this->diafan->module != "ab" || ! in_array($this->diafan->cid, $site_ids)))
			return;

		$result = $this->model->show_block($count, $cat_ids, $site_ids, $sort, $images, $images_variation, $param);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'ab', $result))
		{
			$this->diafan->_tpl->get('show_block', 'ab', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице объявления выводит похожие объявления. По умолчанию связи между объявлениями являются односторонними, это можно изменить, отметив опцию «В блоке похожих объявлений связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых объявлений (по умолчанию 3)
	 * images - количество изображений, прикрепленных к объявления
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/ab/views/ab.view.show_block_rel_**template**.php; по умолчанию шаблон modules/ab/views/ab.view.show_block_rel.php)
	 *  
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "ab" || ! $this->diafan->show)
			return false;

		$result = $this->model->show_block_rel($count, $images, $images_variation);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'ab', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'ab', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит форму поиска объявлений. Если для категорий прикреплены дополнительные характеристики, то поиск по ним производится только на странице категории.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * only_ab - выводить форму поиска только на странице модуля
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы. Если выбрано несколько страниц сайта, то в форме поиска появляется выпадающих список по выбранным страницам
	 * cat_id - категории объявлений, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. Можно указать значение **current**, тогда поиск будет осуществляться по текущей (открытой) категории или по всем категориям, если ни одна категория не открыта. Если выбрано несколько категорий, то в форме поиска появится выпадающий список категорий, который будет подгружать прикрепленные к категориям характеристики. Можно указать значение **all**, тогда поиск будет осуществлятся по всем категориям объявлений и в форме будут участвовать только общие характеристики
	 * ajax - подгружать результаты поиска без перезагрузки страницы. Результаты подгружаются только если открыта страница со списком объявлений, иначе поиск работает обычным образом
	 * template - шаблон тега (файл modules/ab/views/ab.view.show_search_**template**.php; по умолчанию шаблон modules/ab/views/ab.view.show_search.php)
	 * 
	 * @return void
	 */
	public function show_search($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_ab', 'cat_id', 'site_id', 'template', 'ajax');

		$only_ab = $attributes["only_ab"];
		$cat_ids   = $attributes["cat_id"] === 'current' || $attributes["cat_id"] === 'all' ? $attributes["cat_id"] : explode(",", $attributes["cat_id"]);
		$site_ids  = explode(",", $attributes["site_id"]);
		$ajax     = $attributes["ajax"] == "true" ? true : false;

		if ($cat_ids === 'current')
		{
			if($this->diafan->cat && $this->diafan->module == "ab" && (count($site_ids) == 1 && $site_ids[0] == 0 || in_array($this->diafan->cid, $site_ids)))
			{
				$cat_ids  = array($this->diafan->cat);
				$site_ids = array($this->diafan->cid);
			}
			else
			{
				$cat_ids = array();
			}
		}

		if ($only_ab && ($this->diafan->module != "ab" || $site_ids && ! in_array($this->diafan->cid, $site_ids)))
			return;

		$result = $this->model->show_search($cat_ids, $site_ids, $ajax);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_search_'.$attributes["template"], 'ab', $result))
		{
			$this->diafan->_tpl->get('show_search', 'ab', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит форму добавления сообщения. Для правильной работы тега должна существовать страница, к которой прикреплен модуль Объявления.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * only_ab - выводить форму поиска только на странице модуля
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы. Если задано несколько страниц, то в форме появляется выпадающий список «Раздел»
	 * cat_id - категории объявлений, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. Можно указать значение **current**, тогда форма будет добавлять объявление в текущую (открытую) категорию или выводить поле «Категория», если ни одна категория не открыта. Если задано несколько категорий, то в форме появляется выпадающий список «Категория»
	 * template - шаблон тега (файл modules/ab/views/feedback.view.form_**template**.php; по умолчанию шаблон modules/ab/views/ab.view.form.php)
	 * 
	 * @return boolean
	 */
	public function show_form($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_ab', 'site_id', 'cat_id', 'template');

		$only_ab = $attributes["only_ab"];
		$cat_ids   = $attributes["cat_id"] === 'current' ? $attributes["cat_id"] : explode(",", $attributes["cat_id"]);
		$site_ids  = explode(",", $attributes["site_id"]);


		if ($only_ab && ($this->diafan->module != "ab" || $site_ids && ! in_array($this->diafan->cid, $site_ids)))
			return;

		if ($cat_ids === 'current')
		{
			if($this->diafan->cat && $this->diafan->module == "ab" && (count($site_ids) == 1 && $site_ids[0] == 0 || in_array($this->diafan->cid, $site_ids)))
			{
				$cat_ids  = array($this->diafan->cat);
				$site_ids = array($this->diafan->cid);
			}
			else
			{
				$cat_ids = array();
			}
		}

		$result = $this->model->form($site_ids, $cat_ids, true);
		if ($result)
		{
			if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_form_'.$attributes["template"], 'ab', $result))
			{
				$this->diafan->_tpl->get('form', 'ab', $result);
			}
		}
		return true;
	}
}