<?php
/**
 * Подключение для работы с прикрепленными файлами
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * Attachments_inc
 */
class Attachments_inc extends Diafan
{
	/*
	 * Отдает информацию о прикрепленных файлах
	 *
	 * @param integer $element_id номер элемента, к которому прикрепляется файл
	 * @param string $module_name название модуля
	 * @param integer $param_id номер параметра, к которому прикреплен файл
	 * @return array
	*/
	public function get($element_id, $module_name, $param_id = 0)
	{
		$this->prepare($element_id, $module_name, $param_id);
		if(! empty($this->cache["prepare"]))
		{
			$values = array();
			$where = array();
			foreach($this->cache["prepare"] as $pr_module_name => $array)
			{
				$w = array();
				$values[] = $pr_module_name;
				foreach($array as $pr_param_id => $arr)
				{
					$values[] = $pr_param_id;
					foreach($arr as $pr_element_id => $a)
					{
						$this->cache["attachments"][$pr_module_name][$pr_param_id][$pr_element_id] = array();
						$values[] = $pr_element_id;
						$v_arr[] = '%d';
					}
					$w[] = " param_id=%d AND element_id".(count($arr) > 1 ? " IN (".implode(",", $v_arr).")" : "=%d");
				}
				$where[] = "module_name='%h' AND ".(count($w) > 1 ? "(".implode(" OR ", $w).")" : $w[0]);
			}

			$result = DB::query("SELECT * FROM {attachments} WHERE ".implode(" OR ", $where), $values);
			while ($row = DB::fetch_array($result))
			{
				if ($row["is_image"])
				{
					$row["link"] = BASE_PATH.USERFILES.'/'.$module_name.'/imgs/'.$row["name"];
					$row["link_preview"] = BASE_PATH.USERFILES.'/'.$module_name.'/imgs/small/'.$row["name"];
					list($row["width"], $row["height"]) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/imgs/'.$row["name"]);
				}
				else
				{
					$row["link"] = BASE_PATH.'attachments/get/'.$row["id"]."/".$row["name"];
				}
				$row["size"] = $this->diafan->convert($row["size"]);
				$this->cache["attachments"][$row["module_name"]]['p'.$row["param_id"]]['e'.$row["element_id"]][] = $row;
			}
			DB::free_result($result);
			unset($this->cache["prepare"]);
		}
		return $this->cache["attachments"][$module_name]['p'.$param_id]['e'.$element_id];
	}
	
	/**
	 * Запоминает данные элемента, которому нужно вывести информацию о прикрепленных файлах
	 *
	 * @param integer $element_id номер элемента, к которому прикрепляется файл
	 * @param string $module_name название модуля
	 * @param integer $param_id номер параметра, к которому прикреплен файл
	 * @return void
	 */
	public function prepare($element_id, $module_name, $param_id = 0)
	{
		if(isset($this->cache["attachments"][$module_name]['p'.$param_id]['e'.$element_id]))
		{
			return;
		}
		if(! isset($this->cache["prepare"][$module_name]['p'.$param_id]['e'.$element_id]))
		{
			$this->cache["prepare"][$module_name]['p'.$param_id]['e'.$element_id] = $element_id;
		}
	}

	/**
	 * Сохраняет добавленные файлы
	 * 
	 * @param integer $element_id номер элемента
	 * @param string $module_name название модуля
	 * @param array $config конфигурация
	 * @return boolean 
	 */
	public function save($element_id, $module_name, $config = array())
	{
		if(! empty($config["type"]) && $config["type"] == 'configmodules')
		{
			if(! $this->diafan->configmodules('attachments', $module_name, $config["site_id"]))
			{
				return false;
			}
			$config = array(
				'max_count_attachments' => $this->diafan->configmodules("max_count_attachments", $module_name, $config["site_id"]),
				"attachment_extensions" => $this->diafan->configmodules("attachment_extensions", $module_name, $config["site_id"]),
				"recognize_image" => $this->diafan->configmodules("recognize_image", $module_name, $config["site_id"]),
				"attachments_access_admin" => $this->diafan->configmodules("attachments_access_admin", $module_name, $config["site_id"]),
				"attach_big_width" => $this->diafan->configmodules("attach_big_width", $module_name, $config["site_id"]),
				"attach_big_height" => $this->diafan->configmodules("attach_big_height", $module_name, $config["site_id"]),
				"attach_big_quality" => $this->diafan->configmodules("attach_big_quality", $module_name, $config["site_id"]),
				"attach_medium_width" => $this->diafan->configmodules("attach_medium_width", $module_name, $config["site_id"]),
				"attach_medium_height" => $this->diafan->configmodules("attach_medium_height", $module_name, $config["site_id"]),
				"attach_medium_quality" => $this->diafan->configmodules("attach_medium_quality", $module_name, $config["site_id"]),
			);
		}
		$result = false;
		$name = (! empty($config["prefix"]) ? $config["prefix"] : '').'attachments'.(! empty($config["param_id"]) ? $config["param_id"] : '');

		if (! empty($_FILES[$name]))
		{
			foreach ($_FILES[$name]['tmp_name'] as $n => $dummy)
			{
				if ($config["max_count_attachments"] && $n > $config["max_count_attachments"])
					break;

				if($this->upload($_FILES[$name], $module_name, $element_id, $n, $config))
				{
					$result = true;
				}
			}
		}
		return $result;
	}

	/**
	 * Загружает файлы
	 *
	 * @param array $file загружаемый файл/файлы
	 * @param string $module_name название модуля
	 * @param integer $element_id номер элемента, к которому прикрепляется файл
	 * @param integer|boolean false $n номер файла в массиве файлов, если предан массив
	 * @param array $config конфигурация
	 * @return boolean
	 */
	public function upload($file, $module_name, $element_id, $n = false, $config = array())
	{
		if(! empty($config["type"]) && $config["type"] == 'configmodules')
		{
			if(! $this->diafan->configmodules('attachments', $module_name, $config["site_id"]))
				return false; 

			$config = array(
				"attachment_extensions" => $this->diafan->configmodules("attachment_extensions", $module_name, $config["site_id"]),
				"recognize_image" => $this->diafan->configmodules("recognize_image", $module_name, $config["site_id"]),
				"attachments_access_admin" => $this->diafan->configmodules("attachments_access_admin", $module_name, $config["site_id"]),
				"attach_big_width" => $this->diafan->configmodules("attach_big_width", $module_name, $config["site_id"]),
				"attach_big_height" => $this->diafan->configmodules("attach_big_height", $module_name, $config["site_id"]),
				"attach_big_quality" => $this->diafan->configmodules("attach_big_quality", $module_name, $config["site_id"]),
				"attach_medium_width" => $this->diafan->configmodules("attach_medium_width", $module_name, $config["site_id"]),
				"attach_medium_height" => $this->diafan->configmodules("attach_medium_height", $module_name, $config["site_id"]),
				"attach_medium_quality" => $this->diafan->configmodules("attach_medium_quality", $module_name, $config["site_id"]),
			);
		}
		if ($n !== false)
		{
			$name     = $file['name'][$n];
			$tmp_name = $file['tmp_name'][$n];
			$type     = $file['type'][$n];
		}
		else
		{
			$name     = $file['name'];
			$tmp_name = $file['tmp_name'];
			$type     = $file['type'];
		}
		$size = filesize($tmp_name);
		if ($name == '')
			return false;

		$new_name = strtolower($this->diafan->translit($name));
		$extension = substr(strrchr($new_name, '.'), 1);
		$new_name = substr($new_name, 0, -(strlen($extension) + 1));
		if (strlen($new_name) + strlen($extension) > 49)
		{
			$new_name = substr($new_name, 0, 49 - strlen($extension));
		}
		$attachment_extensions_array = explode(',', str_replace(' ', '', strtolower($config["attachment_extensions"])));
		if (! in_array($extension, $attachment_extensions_array))
		{
			throw new Exception($this->diafan->_('Вы не можете отправить файл %s. Доступны только следующие типы файлов:', false, $name).$config["attachment_extensions"]
			.(IS_ADMIN ? '. '.$this->diafan->_('Новые типы файлов добавляются в настройках модуля.', false) : ''));
		}

		if (! is_uploaded_file($tmp_name) || ! file_exists($tmp_name))
		{
			throw new Exception($this->diafan->_('Извините, не удалось загрузить файл: %s', false, $name));
		}
		if ($config["recognize_image"] && in_array($extension, array('jpg', 'jpeg', 'jpe', 'gif', 'png')))
		{
			$is_image = 1;
		}
		else
		{
			$is_image = 0;
		}
		$new_name = $new_name.'.'.$extension;

		$id = DB::query("INSERT INTO {attachments} (name, module_name, element_id, extension, size, is_image, access_admin, param_id) VALUES ('%s', '%s', %d, '%s', %d, '%d', '%d', %d)",
		          $new_name, $module_name, $element_id, $type, $size, $is_image, $config["attachments_access_admin"], ! empty($config["param_id"]) ? $config["param_id"] : 0);
		if ($is_image)
		{
			$new_name = $id.'_'.$new_name;
			DB::query("UPDATE {attachments} SET name='%h' WHERE id=%d", $new_name, $id);
		}
		
		try
		{
			Files_inc::create_dir($module_name, USERFILES);

			if ($is_image)
			{
				Files_inc::create_dir('imgs', USERFILES.'/'.$module_name);
				Files_inc::create_dir('small', USERFILES.'/'.$module_name.'/imgs');
	
				Customization::inc('includes/image.php');
				list($width, $height) = getimagesize($tmp_name);
				if ($width > $config["attach_big_width"] && $height > $config["attach_big_height"]
					&& ! Image::resize($tmp_name, $config["attach_big_width"], $config["attach_big_height"], $config["attach_big_quality"]))
				{
					throw new Exception($this->diafan->_('Извините, не удалось загрузить файл: %s', false, $name));
				}
				Files_inc::copy_file($tmp_name, $new_name, USERFILES."/".$module_name."/imgs");
	
				if ($width > $config["attach_medium_width"] && $height > $config["attach_medium_height"]
					&& ! Image::resize($tmp_name, $config["attach_medium_width"], $config["attach_medium_height"], $config["attach_medium_quality"]))
				{
					throw new Exception($this->diafan->_('Извините, не удалось загрузить файл: %s', false, $name));
				}
				Files_inc::upload_file($tmp_name, $new_name, USERFILES."/".$module_name."/imgs/small");
			}
			else
			{
				Files_inc::create_dir('files', USERFILES.'/'.$module_name, true);
				Files_inc::upload_file($tmp_name, $id, USERFILES."/".$module_name."/files");
			}
		}
		catch(Exception $e)
		{
			if(IS_ADMIN || MOD_DEVELOPER)
			{
				throw new Exception($e->getMessage());
			}
			else
			{
				throw new Exception($this->diafan->_('Извините, не удалось загрузить файл: %s', false, $name));
			}
		}
		return true;
	}

	/*
	 * Удаляет прикрепленные файлы/файл
	 *
	 * @param integer $element_id номер элемента, к которому прикреплен файл
	 * @param string $module_name название модуля
	 * @param integer $attachment_id номер файла
	 * @param integer $param_id номер параметра, к которому прикреплен файл
	 * @return boolean void
	*/
	public function delete($element_id, $module_name, $attachment_id = 0, $param_id = 0)
	{
		if ($attachment_id)
		{
			$result = DB::query("SELECT * FROM {attachments} WHERE id=%d AND element_id=%d AND module_name='%h'".($param_id ? " AND param_id=%d" : ""), $attachment_id, $element_id, $module_name, $param_id);
		}
		else
		{
			$result = DB::query("SELECT * FROM {attachments} WHERE element_id=%d AND module_name='%h'".($param_id ? " AND param_id=%d" : ""), $element_id, $module_name, $param_id);
		}
		while ($row = DB::fetch_array($result))
		{
			DB::query("DELETE FROM {attachments} WHERE id='%d'", $row["id"]);

			if ($row["is_image"])
			{
				Files_inc::delete_file(USERFILES.'/'.$module_name.'/imgs/'.$row["name"]);
				Files_inc::delete_file(USERFILES.'/'.$module_name.'/imgs/small/'.$row["name"]);
			}
			else
			{
				Files_inc::delete_file(USERFILES.'/'.$module_name.'/files/'.$row["id"]);
			}
		}
		DB::free_result($result);
	}
}