<?php
/**
 * Подключение для работы с каптчей
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Captcha_inc
 */
class Captcha_inc extends Model
{
	/**
	 * Выводит каптчу
	 * 
	 * @param string $modules метка каптчи
	 * @param string $error ошибка ввода кода, если запрос передан не через Ajax
	 * @param boolean $is_update каптча генерируется для обновления
	 * @return string
	 */
	public function get($modules = "modules", $error = "", $is_update = false)
	{
		switch($this->diafan->configmodules('type', 'captcha'))
		{
			case 'reCAPTCHA':
				if(isset($_POST["recaptcha_challenge_field"]))
				{
					unset($_POST["recaptcha_challenge_field"]);
					return "recaptcha";
				}
				else
				{
					$result["public_key"] = $this->diafan->configmodules('recaptcha_public_key', 'captcha');
					$result["error"] = $error;
					$result["modules"] = $modules;
					return $this->diafan->_tpl->get('get_recaptcha', 'captcha', $result);
				}

			case 'qa':
				return $this->get_qa();

			default:
				return $this->diafan->_tpl->get('get', 'captcha', array("modules" => $modules, "error" => $error, "update" => $is_update));
		}
	}

	private function get_qa($error = "", $is_update = false)
	{
		$tpl_result = array("error" => $error, "update" => $is_update, "text" => '', "answers" => array());
		
		$capcha = DB::query_fetch_array("SELECT [name], id FROM {captcha} WHERE trash='0' ORDER BY RAND () LIMIT 1");
		if($capcha)
		{
			$_SESSION["captcha_id"] = $capcha["id"];
			$tpl_result["text"] = $capcha["name"];
			$result = DB::query("SELECT [text], id FROM {captcha_answers} WHERE captcha_id=%d ORDER BY RAND()", $capcha["id"]);
			while($row = DB::fetch_array($result))
			{
				$tpl_result["answers"][] = $row;
			}
			DB::free_result($result);
		}
		return $this->diafan->_tpl->get('get_qa', 'captcha', $tpl_result);
	}

	/**
	 * Проверяет правильность ввода каптчи
	 * 
	 * @param string $modules метка каптчи
	 * @return string|boolean false
	 */
	public function error($modules = "modules")
	{
		switch($this->diafan->configmodules('type', 'captcha'))
		{
			case 'reCAPTCHA':
				return $this->error_recaptcha();

			case 'qa':
				return $this->error_qa();

			default:
				return $this->error_captcha($modules);
		}
	}
	
	private function error_recaptcha()
	{	
		if(empty($_POST["recaptcha_challenge_field"]))
		{
			$_POST["recaptcha_challenge_field"] = '';
		}
		if(empty($_POST["recaptcha_response_field"]))
		{
			$_POST["recaptcha_response_field"] = '';
		}
		$fp = fsockopen('www.google.com', 80);
		if($fp)
		{
			$param = "privatekey=".urlencode($this->diafan->configmodules('recaptcha_private_key', 'captcha'))."&"
			."remoteip=".urlencode($_SERVER['REMOTE_ADDR'])."&"
			."challenge=".urlencode($_POST["recaptcha_challenge_field"])."&"
			."response=".urlencode($_POST["recaptcha_response_field"]);
			$size = strlen($param);

			fputs($fp, "POST http://www.google.com/recaptcha/api/verify HTTP/1.1\r\n"
			."Host: www.google.com\r\n"
			."Content-type: application/x-www-form-urlencoded\r\n"
			."Content-Length: ".$size."\r\n"
			."Connection: Close\r\n\r\n"
			.$param);

			$result = false;
			$resultstr = '';
			while(!feof($fp))
			{
				$response = fgets($fp);
				if($result)
				{
					$resultstr .= $response;
				}
				if(strpos($response, "Connection: close") !== false)
				{
					$result = true;
				}
			}
			fclose($fp);
			if(strpos($resultstr, 'true') !== false && strpos($resultstr, 'success'))
			{
				return false;
			}
			else
			{
				if(MOD_DEVELOPER && strpos($resultstr, 'invalid-site-private-key') !== false)
				{
					return $this->diafan->_('Проверьте Rrivate Key для сервиса reCAPTCHA.', false);
				}
				else
				{
					return $this->diafan->_('Неправильно введен защитный код.', false);
				}
			}
		}
		else
		{
			return $this->diafan->_('Невозможно подключиться к северу reCAPTCHA.', false);
		}

		return false;
	}
	
	private function error_captcha($modules)
	{
		//Защитный код не введен
		if (empty($_POST['captcha']) || empty($_POST['captchaint']))
			return $this->diafan->_('Введите защитный код', false);

		//В сессии не записан код с данным идентификатором captchaint
		if (! isset($_SESSION['captcha'][$modules][$_POST['captchaint']])
		||
		//код из сессии не соответствует введенному. регистр не учитывается
		strtoupper($_SESSION['captcha'][$modules][$_POST['captchaint']]) != strtoupper($_POST['captcha']))
			return $this->diafan->_('Неправильно введен защитный код.', false);

		//очищаем из сессии запись с данным идентификатором
		unset($_SESSION['captcha'][$modules][$_POST['captchaint']]);
		return false;
	}
	
	private function error_qa()
	{
		if(empty($_SESSION["captcha_id"]) || empty($_POST["captcha_answer_id"]))
		{
			return $this->diafan->_('Выберите правильный ответ.', false);
		}
		if(! DB::query_result("SELECT COUNT(*) FROM {captcha_answers} WHERE trash='0' AND captcha_id=%d AND id=%d AND is_right='1'", $_SESSION["captcha_id"], $_POST["captcha_answer_id"]))
		{
			return $this->diafan->_('Ответ не верный.', false);
		}
		return false;
	}
}