<?php
/**
 * Модель модуля «Корзина товаров, оформление заказа»
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

class Cart_model extends Model
{
	/**
	 * Генерирует данные для формы оформления заказов
	 * 
	 * @return array
	 */
	public function form()
	{
		$this->result["error"] = $this->get_error("cart");
		$this->result["error_table"] = $this->get_error("cart", "table");

		$this->form_table();
		$this->form_registration();
		$this->form_param();
		$this->form_payments();
		$this->yandex_fast_order();

		return $this->result;
	}

	/**
	 * Генерирует таблицу купленных товаров
	 * 
	 * @return boolean true
	 */
	public function form_table()
	{
		// корзина
		$cart = $this->diafan->_cart->get();
		if (! $cart)
		{
			$this->result["shop_link"] = $this->diafan->_route->module('shop', true);
			return $this->result;
		}
		$this->result["currency"] = $this->diafan->configmodules("currency", "shop");
		$this->result["summ"] = 0;
		$this->result["count"] = 0;
		$this->result["discount"] = false;
		$k = 0;
		foreach ($cart as $good_id => $array)
		{
			if (!$row = DB::query_fetch_array("SELECT id, [name], article, cat_id, site_id FROM {shop} WHERE [act]='1' AND id = %d AND trash='0' LIMIT 1", $good_id))
			{
				continue;
			}
			$link = $this->diafan->_route->link($row["site_id"], "shop", $row["cat_id"], $row["id"]);
			$img = $this->diafan->_images->get('medium', $good_id, 'shop', $row["site_id"], $row["name"]);
			foreach ($array as $param => $c)
			{
				$this->result["rows"][$k]["name"] = $row["name"];
				$this->result["rows"][$k]["article"] = $row["article"];
				$this->result["rows"][$k]["link"] = $link;
				$query = array();
				$params = unserialize($param);
				if($row["cat_id"])
				{
					if (empty($select_cats[$row["cat_id"]]))
					{
						$select_cats[$row["cat_id"]] = array(
								"name" => DB::query_result("SELECT [name] FROM {shop_category} WHERE id=%d LIMIT 1", $row["cat_id"]),
								"link" => $this->diafan->_route->link($row["site_id"], "shop", $row["cat_id"])
							);
					}
					$this->result["rows"][$k]["cat"]["name"] = $select_cats[$row["cat_id"]]["name"];
					$this->result["rows"][$k]["cat"]["link"] = $select_cats[$row["cat_id"]]["link"];
				}
				foreach ($params as $id => $value)
				{
					$query[] = 'p'.$id.'='.$value;
					if (empty($param_names[$id]))
					{
						$param_names[$id] = DB::query_result("SELECT [name] FROM {shop_param} WHERE id=%d LIMIT 1", $id);
					}
					if (empty($select_names[$id][$value]))
					{
						$select_names[$id][$value] =
							DB::query_result("SELECT [name] FROM {shop_param_select} WHERE param_id=%d AND id=%d LIMIT 1", $id, $value);
					}

					$this->result["rows"][$k]["name"] .= ', '.$param_names[$id].': '.$select_names[$id][$value];
				}
				$row_price = $this->diafan->_shop->price_get($good_id, $params);

				$price = $row_price["price"];

				$this->result["rows"][$k]["link"] .= !empty($query) ? '?'.implode('&', $query) : '';
				$this->result["rows"][$k]["count"] = $c["count"];
				if ($img)
				{
					if($price_image_rel = DB::query_result("SELECT image_id FROM {shop_price_image_rel} WHERE price_id=%d LIMIT 1", $row_price["price_id"]))
					{
						foreach($img as $i)
						{
							if($i["id"] == $price_image_rel)
							{
								$this->result["rows"][$k]["img"] = $i;
							}
						}
					}
					if(empty($this->result["rows"][$k]["img"]))
					{
						$this->result["rows"][$k]["img"] = $img[0];
					}
				}
				$this->result["rows"][$k]["id"] = $row["id"].'_'.str_replace(array('{',':',';','}',' ','"',"'"), '', $param);
				$this->result["rows"][$k]["price"] = $this->diafan->_shop->price_format($price);
				$this->result["rows"][$k]["old_price"] = $this->diafan->_shop->price_format($row_price["old_price"]);
				$this->result["rows"][$k]["summ"] = $this->diafan->_shop->price_format($price * $c["count"]);
				$this->result["rows"][$k]["discount"] = 0;
				if($row_price["discount_id"])
				{
					if(! isset($cache["discount"][$row_price["discount_id"]]))
					{
						$cache["discount"][$row_price["discount_id"]] = DB::query_fetch_array("SELECT discount, deduction FROM {shop_discount} WHERE id=%d LIMIT 1", $row_price["discount_id"]);
					}
					$discount = $cache["discount"][$row_price["discount_id"]];
					$this->result["discount"] = true;
					if(! empty($discount["deduction"]))
					{
						$this->result["rows"][$k]["discount"] = $discount["deduction"].' '.$this->diafan->configmodules("currency", "shop");
					}
					else
					{
						$this->result["rows"][$k]["discount"] = $discount["discount"].' %';
					}
					if(! empty($discount["deduction"]))
					{
						$this->result["rows"][$k]["discount_summ"] = $discount["deduction"];
					}
					else
					{
						$this->result["rows"][$k]["discount_summ"] = $this->diafan->_shop->price_format($row_price["price"] / (100 - $this->result["rows"][$k]["discount"])*$this->result["rows"][$k]["discount"]);
					}
				}

				$this->result["summ"] += $price * $c["count"];
				$this->result["count"] += $c["count"];
				$k++;
			}
		}
		if(! $this->result["count"])
		{
			return $this->result;
		}
		$this->result["summ_goods"] = $this->result["summ"];
		
		$order_summ = 0;
		if($this->diafan->_user->id)
		{
			$order_summ = DB::query_result("SELECT SUM(summ) FROM {shop_order} WHERE user_id=%d AND (status='1' OR status='3')", $this->diafan->_user->id);
		}

		//скидка на общую сумму заказа
		$person_discount_ids = $this->diafan->_shop->price_get_person_discounts();
		$result = DB::query("SELECT id, discount, amount, deduction, threshold, threshold_cumulative FROM"
			." {shop_discount} WHERE act='1' AND trash='0' AND (threshold_cumulative>0 OR threshold>0)"
			." AND role_id".($this->diafan->_user->role_id ? ' IN (0, '.$this->diafan->_user->role_id.')' : '=0')
			." AND (person='0'".($person_discount_ids ? " OR id IN(".implode(",", $person_discount_ids).")" : "").")"
			." AND date_start<=%d AND (date_finish=0 OR date_finish>=%d) ORDER BY threshold_cumulative ASC, threshold ASC", time(), time()
		);
		while ($row = DB::fetch_array($result))
		{
			if($row["threshold"] && $row["threshold"] <= $this->result["summ_goods"]  || $row["threshold_cumulative"] && $row["threshold_cumulative"] <= $order_summ)
			{
				if($row['deduction'])
				{
					if($row['deduction'] < $this->result["summ_goods"])
					{
						$row["discount_summ"] = $row["deduction"];
					}
					else
					{
						$row["discount_summ"] = 0;
					}
				}
				else
				{
					$row["discount_summ"] = $this->result["summ_goods"] * $row["discount"] / 100;
				}
				if(empty($this->result["discount_total"]) || $this->result["discount_total"]["discount_summ"] < $row["discount_summ"])
				{
					$this->result["discount_total"] = $row;
				}
			}
			elseif($row["threshold"])
			{
				if($row['deduction'])
				{
					$row["discount_summ"] = $row["threshold"] - $row["deduction"];
				}
				else
				{
					$row["discount_summ"] = $row["threshold"] * $row["discount"] / 100;
				}
				if(empty($this->result["discount_next"]) || $this->result["discount_next"]["discount_summ"] <= $row["discount_summ"] && $this->result["discount_next"]["threshold"] >= $row["threshold"])
				{
					if($row["deduction"])
					{
						$row["discount"] = $row['deduction'].' '.$this->diafan->configmodules('currency',  'shop');
					}
					else
					{
						$row["discount"] .= '%';
					}
					$row["summ"] = $this->diafan->_shop->price_format($row["threshold"] - $this->result["summ_goods"]).' '.$this->diafan->configmodules('currency',  'shop');
					$this->result["discount_next"] = $row;
				}
			}
		}
		DB::free_result($result);
		if(! empty($this->result["discount_total"]))
		{
			$this->result["old_summ_goods"] = $this->diafan->_shop->price_format($this->result["summ_goods"]);
			$this->result["summ_goods"] = $this->result["summ_goods"] - $this->result["discount_total"]["discount_summ"];
			$this->result["summ"] = $this->result["summ"] - $this->result["discount_total"]["discount_summ"];
			if($this->result["discount_total"]["deduction"])
			{
				$this->result["discount_total"]["deduction"] = $this->diafan->_shop->price_format($this->result["discount_total"]["deduction"]);
				$this->result["discount_total"]["discount"] = $this->result["discount_total"]["deduction"].' '.$this->diafan->configmodules('currency',  'shop');
			}
			else
			{
				$this->result["discount_total"]["discount"] .= '%';
			}
			$this->result["discount"] = true;
		}

		// дополнительно
		$this->result["additional_cost"] = array();
		$this->result["cart_additional_cost"] = ! empty($_SESSION["cart_additional_cost"]) ? $_SESSION["cart_additional_cost"] : array();
		$result = DB::query("SELECT id, [name], price, percent, [text], amount, required FROM {shop_additional_cost} WHERE [act]='1' AND trash='0' ORDER by sort DESC");
		while ($row = DB::fetch_array($result))
		{
			$row["summ"] = $row['price'];
			if($row['percent'])
			{
				$row["summ"] = $this->result["summ_goods"] * $row['percent'] / 100;
			}
			if (! empty($row['amount']))
			{
				if ($row['amount'] < $this->result["summ_goods"])
				{
					$row["summ"] = 0;
				}
			}
			if (in_array($row["id"], $this->result["cart_additional_cost"]) || $row["required"])
			{
				$this->result["summ"] += $row['summ'];
			}
			$this->result["additional_cost"][] = $row;
		}
		DB::free_result($result);

		// способы доставки
		$this->result["delivery"] = array();
		$result = DB::query("SELECT id, [name], [text] FROM {shop_delivery} WHERE [act]='1' AND trash='0' ORDER BY sort DESC");
		while ($row = DB::fetch_array($result))
		{
			$row['price'] = 0;
			$row["thresholds"] = array();
			$result2 = DB::query("SELECT price, amount FROM {shop_delivery_thresholds} WHERE delivery_id=%d ORDER BY amount ASC", $row["id"]);
			while ($row2 = DB::fetch_array($result2))
			{
				if($row2['amount'] <= $this->result["summ"])
				{
					$row['price'] = $row2["price"];
				}
				$row["thresholds"][] = $row2;
			}
			DB::free_result($result2);

			if (empty($_SESSION["cart_delivery"]) && empty($this->result["delivery"]))
			{
				$_SESSION["cart_delivery"] = $row['id'];
			}
			if (! empty($_SESSION["cart_delivery"]) && $row['id'] == $_SESSION["cart_delivery"])
			{
				$this->result["summ"] += $row['price'];
			}
			$this->result["delivery"][] = $row;
		}
		DB::free_result($result);
		$this->result["cart_delivery"] = ! empty($_SESSION["cart_delivery"]) ? $_SESSION["cart_delivery"] : 0;
		$this->result["summ"] = $this->diafan->_shop->price_format($this->result["summ"]);
		$this->result["summ_goods"] = ! empty($this->result["summ_goods"]) ? $this->diafan->_shop->price_format($this->result["summ_goods"]) : 0;

		return $this->result;
	}

	/**
	 * Генерирует форму регистрации и авторизации
	 * 
	 * @return boolean true
	 */
	private function form_registration()
	{
		$this->result["show_auth"] = true;
		if ($this->diafan->_user->id || ! DB::query_result("SELECT id FROM {site} WHERE module_name='registration' AND [act]='1' AND trash='0' LIMIT 1"))
		{
			$this->result["show_auth"] = false;
		}
		else
		{
			Customization::inc('modules/registration/registration.model.php');
			$reg = new Registration_model($this->diafan);
			$this->result["registration"] = $reg->form();
			$this->result["registration"]["action"] = BASE_PATH_HREF . $this->diafan->_route->module("registration", true);
			$show_login = array("error" => $this->diafan->_user->errauth ? $this->diafan->_('Неверный логин или пароль.', false) : '', "action" => '', "user" => '', 'hide' => true);

			$this->result["show_login"] = $show_login;
		}
		return true;
	}

	/**
	 * Генерирует поля формы, созданные в конструкторе
	 *
	 * @param boolean $one_click сокращенная форма для быстрого заказа 
	 * @return boolean true
	 */
	private function form_param($one_click = false)
	{
		$where = "show_in_form".($one_click ? "_one_click" : '')."='1'";
		$this->result["rows_param"] = $this->get_params(array("module" => "shop", "table" => "shop_order", "fields" => "info", "where" => $where));

		$multiple = array();
		foreach($this->result["rows_param"] as $row)
		{
			$this->result['error_p'.$row["id"]] = $this->get_error("cart", 'p'.$row["id"]);
			if ($row["type"] == "multiple")
			{
				$multiple[] = $row["id"];
			}
			if(! $row["info"])
			{
					continue;
			}
			switch($row["info"])
			{
				case 'name':
					$this->result["user"]['p'.$row["id"]] = $this->diafan->_user->fio;
					break;

				case 'phone':
					$this->result["user"]['p'.$row["id"]] = $this->diafan->_user->phone;
					break;

				case 'email':
					$this->result["user"]['p'.$row["id"]] = $this->diafan->_user->mail;
					break;
			}
		}

		// данные о пользователе
		if ($this->diafan->_user->id)
		{
			$result = DB::query("SELECT param_id, value FROM {shop_order_param_user} WHERE trash='0' AND user_id=%d", $this->diafan->_user->id);
			while($row = DB::fetch_array($result))
			{
				if(empty($row["value"]))
					continue;

				if (in_array($row["param_id"], $multiple))
				{
					$this->result["user"]['p'.$row["param_id"]][] = $row["value"];
				}
				else
				{
					$this->result["user"]['p'.$row["param_id"]] = $row["value"];
				}
			}
			DB::free_result($result);
			$max_order_id = DB::query_result("SELECT MAX(id) FROM {shop_order} WHERE user_id=%d AND trash='0'", $this->diafan->_user->id);
			$result = DB::query("SELECT value, param_id FROM {shop_order_param_element} WHERE trash='0' AND element_id=%d", $max_order_id);
			while ($row = DB::fetch_array($result))
			{
				if(! empty($this->result["user"]['p'.$row["param_id"]]))
					continue;

				if (in_array($row["param_id"], $multiple))
				{
					$this->result["user"]['p'.$row["param_id"]][] = $row["value"];
				}
				else
				{
					$this->result["user"]['p'.$row["param_id"]] = $row["value"];
				}
			}
			DB::free_result($result);
		}
		
		if($this->diafan->configmodules('subscribe_in_order', 'subscribtion') && empty($this->diafan->_user->id))
		{
			$this->result['subscribe_in_order'] = true;
		}
		return true;
	}

	/**
	 * Генерирует список методов оплаты
	 * 
	 * @return boolean true
	 */
	private function form_payments()
	{
		$result = DB::query("SELECT id, [name], [text] FROM {shop_payment} WHERE [act]='1' AND trash='0' ORDER BY sort DESC");
		while($row = DB::fetch_array($result))
		{
			$this->result["payments"][] = $row;
		}
		DB::free_result($result);
	}
	
	private function yandex_fast_order()
	{
		if(! $this->diafan->configmodules('yandex_fast_order', 'shop'))
		{
			return;
		}
		$this->result["yandex_fast_order"] = true;
		$this->result["yandex_fast_order_link"] =  'http://market.yandex.ru/addresses.xml?callback='
		.urlencode(BASE_PATH_HREF.$this->diafan->_route->current_link().'?yandex_fast_order=true')
		.'&size=mini';
		if(! empty($_POST["operation_id"]))
		{
			foreach($this->result["rows_param"] as $i => $row)
			{
				if(! $row["info"])
				{
						continue;
				}
				switch($row["info"])
				{
					case 'address':
						if(! empty($_POST["street"]) || ! empty($_POST["building"]) || ! empty($_POST["suite"]) || ! empty($_POST["flat"]) || ! empty($_POST["entrance"]) || ! empty($_POST["intercom"]) || ! empty($_POST["city"]) || ! empty($_POST["country"]) || ! empty($_POST["zip"]) || ! empty($_POST["metro"]))
						{
							$this->result["user"]['p'.$row["id"]] = '';
							if(! empty($_POST["zip"]))
							{
								$this->result["user"]['p'.$row["id"]] = $_POST["zip"];
							}
							if(! empty($_POST["country"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$_POST["country"];
							}
							if(! empty($_POST["city"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$_POST["city"];
							}
							if(! empty($_POST["metro"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$this->diafan->_('станция метро', false).' '.$_POST["metro"];
							}
							if(! empty($_POST["street"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$_POST["street"];
							}
							if(! empty($_POST["suite"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$_POST["suite"];
							}
							if(! empty($_POST["building"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$this->diafan->_('дом', false).' '.$_POST["building"];
							}
							if(! empty($_POST["suite"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$this->diafan->_('корпус', false).' '.$_POST["suite"];
							}
							if(! empty($_POST["flat"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$this->diafan->_('кв.', false).' '.$_POST["flat"];
							}
							if(! empty($_POST["entrance"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$this->diafan->_('этаж', false).' '.$_POST["entrance"];
							}
							if(! empty($_POST["intercom"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? "\n" : '').$this->diafan->_('домофон', false).' '.$_POST["intercom"];
							}
						}
						break;
					case 'name':
						if(! empty($_POST["firstname"]) || ! empty($_POST["lastname"]) || ! empty($_POST["fathersname"]))
						{
							$this->result["user"]['p'.$row["id"]] = '';
							if(! empty($_POST["firstname"]))
							{
								$this->result["user"]['p'.$row["id"]] = $_POST["firstname"];
							}
							if(! empty($_POST["fathersname"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? ' ' : '').$_POST["fathersname"];
							}
							if(! empty($_POST["lastname"]))
							{
								$this->result["user"]['p'.$row["id"]] = ($this->result["user"]['p'.$row["id"]] ? ' ' : '').$_POST["lastname"];
							}
						}
						break;
					default:
						if(! empty($_POST[$row["info"]]))
						{
							$this->result["user"]['p'.$row["id"]] = $_POST[$row["info"]];
						}
						break;
					}
			}
		}
	}

	/**
	 * Генерирует данные для втого шага в оформлении заказа: оплата
	 * 
	 * @return array|boolean
	 */
	public function payment()
	{
		$row = DB::query_fetch_array("SELECT payment_id, summ FROM {shop_order} WHERE id=%d LIMIT 1", $this->diafan->show);
		
		$method = DB::query_fetch_array("SELECT payment, [text], params FROM {shop_payment} WHERE id=%d LIMIT 1",$row['payment_id']);
		if($method["payment"])
		{
			if(! DB::query_result("SELECT id FROM {shop_pay_history} WHERE order_id=%d LIMIT 1", $this->diafan->show))
			{
				DB::query("INSERT INTO {shop_pay_history} (created, status, order_id, payment, summ, user_id) VALUES (%d, 'request_pay', %d, '%s', %f, %d)", time(), $this->diafan->show, $method["payment"], $row["summ"], $this->diafan->_user->id);
			}
			$params = unserialize($method["params"]);
			$params["text"] = $this->diafan->configmodules('mes', 'shop').'<br><br>'.$method["text"];
			$params["summ"] = $row['summ'];
			$params["user_id"] = $this->diafan->_user->id;
			$params["order_id"] = $this->diafan->show;
			$params["desc_order"] = str_replace('%order', $params["order_id"], $this->diafan->configmodules("desc_order", "shop"));
			Customization::inc('modules/cart/payment/'.$method["payment"].'/cart.payment.'.$method["payment"].'.model.php');
			$class = 'Cart_payment_'.$method["payment"].'_model';
			$payment_class = new $class($this->diafan);
			$this->result = $payment_class->get($params);
			$this->result["method"] = $method["payment"];
		}
		else
		{
			return $this->diafan->configmodules('mes', 'shop').'<br><br>'.$method["text"];
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для третьего шага в офрмлении заказа: результат оплаты
	 * 
	 * @return array
	 */
	public function result()
	{
		if ($this->diafan->step == 3)
		{
			if($this->diafan->configmodules('order_redirect', 'shop'))
			{
				if(preg_match("/^([0-9]+)$/", $this->diafan->configmodules('order_redirect', 'shop')))
				{
					$this->result["redirect"] = BASE_PATH_HREF.$this->diafan->_route->link($this->diafan->configmodules('order_redirect', 'shop'));
				}
				else
				{
					$this->result["redirect"] = BASE_PATH_HREF.$this->diafan->configmodules('order_redirect', 'shop');
				}
			}
			$this->result["text"] = $this->diafan->configmodules("payment_success_text", "shop");
		}
		else
		{
			$this->result["text"] = $this->diafan->configmodules("payment_fail_text", "shop");
		}
		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: выводит информацию о заказанных товарах
	 * 
	 * @return array
	 */
	public function show_block()
	{
		$result["count"] = $this->diafan->_cart->get_count();
		$result["summ"] = $this->diafan->_shop->price_format($this->diafan->_cart->get_summ());

		$result["cart_link"] = $this->diafan->_route->module("cart", true);
		$result["link"] = BASE_PATH_HREF.$result["cart_link"].'?'.rand(0, 999999);
		$result["currency"] = $this->diafan->configmodules("currency", "shop");

		$result["wishlist_link"] = $this->diafan->_route->module("wishlist", true);
		if($result["wishlist_link"])
		{
			$result["wishlist_link"] = BASE_PATH_HREF.$result["wishlist_link"];
		}
		return $result;
	}

	/**
	 * Генерирует данные для формы быстрого заказа 
	 * 
	 * @return array
	 */
	public function one_click()
	{
		$this->result["error"] = $this->get_error("cart");
		$this->result["error_table"] = $this->get_error("cart", "table");

		$this->form_param(true);

		return $this->result;
	}
}