<?php
/**
 * Карта ссылок для модуля «Меню на сайте»
 * 
 * Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Clauses_admin_menu
 */
class Clauses_admin_menu extends Diafan
{
	/**
	 * Получает количество элементов, которые можно вывести в меню для страницы сайта
	 * 
	 * @param integer $site_id номер страницы сайта
	 * @return boolean integer
	 */
	public function count($site_id)
	{
		if ($this->diafan->configmodules("cat", 'clauses', $site_id))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {clauses_category} WHERE [act]='1' AND trash='0' AND site_id=%d", $site_id);
		}
		else
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {clauses} WHERE [act]='1' AND trash='0' AND site_id=%d", $site_id);
		}
		return $count;
	}

	/**
	 * Получает список элементов, которые можно вывести в меню для страницы сайта
	 * 
	 * @param integer $site_id номер страницы сайта
	 * @param integer $parent_id родитель
	 * @return array
	 */
	public function list_($site_id, $parent_id)
	{
		$rows = array();
		if ($this->diafan->configmodules("cat", 'clauses', $site_id))
		{
			$result = DB::query("SELECT id, [name], count_children FROM {clauses_category} WHERE [act]='1' AND trash='0' AND parent_id=%d AND site_id=%d ORDER BY sort ASC", $parent_id, $site_id);
			while ($row = DB::fetch_array($result))
			{
				$new = array();
				$new["cat_id"] = $row["id"];
				$new["element_id"] = 0;
				$new["name"] = $row["name"];
				if (! $new["count"] = $row["count_children"])
				{
					$new["count"] = DB::query_result("SELECT COUNT(DISTINCT n.id) FROM {clauses} as n INNER JOIN {clauses_category_rel} as c ON c.element_id=n.id WHERE c.cat_id=%d AND n.[act]='1' AND n.trash='0'", $row["id"]);
				}
				$rows[] = $new;
			}
			DB::free_result($result);
			if ($parent_id)
			{
				$result = DB::query("SELECT n.id, n.[name], n.cat_id FROM {clauses} as n INNER JOIN {clauses_category_rel} as c ON c.element_id=n.id WHERE c.cat_id=%d AND n.[act]='1' AND n.trash='0' GROUP BY n.id ORDER BY n.sort DESC", $parent_id);
				if ($rows && DB::num_rows($result))
				{
					$rows[] = array("hr" => true);
				}
				while ($row = DB::fetch_array($result))
				{
					$new = array();
					$new["count"] = 0;
					$new["cat_id"] = $row["cat_id"];
					$new["element_id"] = $row["id"];
					$new["name"] = $row["name"];
					$rows[] = $new;
				}
				DB::free_result($result);
			}
		}
		else
		{
			$result = DB::query("SELECT id, [name] FROM {clauses} WHERE [act]='1' AND trash='0' AND site_id=%d ORDER BY sort DESC", $site_id);
			if ($rows && DB::num_rows($result))
			{
				$rows[] = array("hr" => true);
			}
			while ($row = DB::fetch_array($result))
			{
				$new = array();
				$new["count"] = 0;
				$new["cat_id"] = 0;
				$new["element_id"] = $row["id"];
				$new["name"] = $row["name"];
				$rows[] = $new;
			}
			DB::free_result($result);
		}
		return $rows;
	}
}