<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Clauses
 */
class Clauses extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;
		if ($this->diafan->show)
		{
			$this->result = $this->model->id();
		}
		elseif (! $this->diafan->configmodules("cat"))
		{
			$this->result = $this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->result = $this->model->first_page();
		}
		else
		{
			$this->result = $this->model->list_category();
		}
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		$this->diafan->_tpl->get($this->view, 'clauses', $this->result);
	}

	/**
	 * Шаблонная функция: выводит последние статьи на всех страницах, кроме страницы статей, когда выводится список тех же статей, что и в функции.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых статей (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - категории статей, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. По умолчанию категория не учитывается, выводятся все статьи
	 * images - количество изображений, прикрепленных к статье
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * sort - сортировка статей: по умолчанию как на странице модуля, **date** – по дате, **rand** – в случайном порядке, **keywords** – статьи, похожие по названию для текущей страницы
	 * template - шаблон тега (файл modules/clauses/views/clauses.view.show_block_**template**.php; по умолчанию шаблон modules/clauses/views/clauses.view.show_block.php)
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'cat_id', 'images', 'images_variation', 'sort', 'site_id', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);
		$site_ids = explode(",", $attributes["site_id"]);
		$sort    = $attributes["sort"] == "date" || $attributes["sort"] == "rand" || $attributes["sort"] == "keywords" ? $attributes["sort"] : "";
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		$result = $this->model->show_block($count, $cat_ids, $site_ids, $images, $images_variation, $sort);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'clauses', $result))
		{
			$this->diafan->_tpl->get('show_block', 'clauses', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице статьи выводит похожие статьи. По умолчанию связи между статьями являются односторонними, это можно изменить, отметив опцию «В блоке похожих статей связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых статей (по умолчанию 3)
	 * images - количество изображений, прикрепленных к статье
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/clauses/views/clauses.view.show_block_rel_**template**.php; по умолчанию шаблон modules/clauses/views/clauses.view.show_block_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "clauses" || ! $this->diafan->show)
			return;

		$result = $this->model->show_block_rel($count, $images, $images_variation);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'clauses', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'clauses', $result);
		}
	}
}