<?php
/**
 * Конструктор комментариев
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Comments_admin_param
 */
class Comments_admin_param extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'comments_param';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'name' => array(
				'type' => 'text',
				'name' => 'Название',
				'multilang' => true,
			),
			'module_name' => array(
				'type' => 'select',
				'name' => 'Модуль',
				'help' => 'Возможность ограничить применением поля комментариями, прикрепленными к выбранному модулю.',
			),
			'type' => array(
				'type' => 'select',
				'name' => 'Тип',
			),
			'max_count_attachments' => array(
				'type' => 'none',
				'name' => 'Максимальное количество добавляемых файлов',
				'help' => 'Количество добавляемых файлов. Если значение равно нулю, то форма добавления файлов не выводится. Параметр выводится, если тип характеристики задан как «файлы».',
				'no_save' => true,
			),
			'attachment_extensions' => array(
				'type' => 'none',
				'name' => 'Доступные типы файлов (через запятую)',
				'help' => 'Параметр выводится, если тип характеристики задан как «файлы».',
				'no_save' => true,
			),
			'recognize_image' => array(
				'type' => 'none',
				'name' => 'Распознавать изображения',
				'help' => 'Позволяет прикрепленные файлы в формате JPEG, GIF, PNG отображать как изображения. Параметр выводится, если тип характеристики задан как «файлы».',
				'no_save' => true,
			),
			'attach_big' => array(
				'type' => 'none',
				'name' => 'Размер для большого изображения',
				'help' => 'Размер изображения, отображаемый в пользовательской части сайта при увеличении изображения предпросмотра. Параметр выводится, если тип характеристики задан как «файлы» и отмечена опция «Распознавать изображения».',
				'no_save' => true,
			),
			'attach_medium' => array(
				'type' => 'none',
				'name' => 'Размер для маленького изображения',
				'help' => 'Размер изображения предпросмотра. Параметр выводится, если тип характеристики задан как «файлы» и отмечена опция «Распознавать изображения».',
				'no_save' => true,
			),
			'attach_use_animation' => array(
				'type' => 'none',
				'name' => 'Использовать анимацию при увеличении изображений',
				'help' => 'Параметр добавляет JavaScript код, позволяющий включить анимацию при увеличении изображений. Параметр выводится, если отмечена опция «Распознавать изображения». Параметр выводится, если тип характеристики задан как «файлы» и отмечена опция «Распознавать изображения».',
				'no_save' => true,
			),
			'upload_max_filesize' => array(
				'type' => 'none',
				'name' => 'Максимальный размер загружаемых файлов',
				'help' => 'Параметр показывает максимально допустимый размер загружаемых файлов, установленный в настройках хостинга. Параметр выводится, если тип характеристики задан как «файлы».',
				'no_save' => true,
			),
			'images_variations' => array(
				'type' => 'none',
				'name' => 'Генерировать размеры изображений',
				'help' => 'Размеры изображений, заданные в модуле «Изобаражения». Параметр выводится, если тип характеристики задан как «изображение».',
				'no_save' => true,
			),
			'required' => array(
				'type' => 'checkbox',
				'name' => 'Обязательно для заполнения',
			),
			'show_in_list' => array(
				'type' => 'checkbox',
				'name' => 'Выводить в списке',
				'help' => 'Выводит значение поля для комментариев на сайте в списке комментариев',
			),
			'show_in_form_auth' => array(
				'type' => 'checkbox',
				'name' => 'Выводить в форме для авторизованных пользователей',
			),
			'show_in_form_no_auth' => array(
				'type' => 'checkbox',
				'name' => 'Выводить в форме для неавторизованных пользователей',
			),
			'sort' => array(
				'type' => 'function',
				'name' => 'Сортировка: установить перед',
				'help' => 'Редактирование порядка следования поля в форме',
			),
			'param_select' => array(
				'type' => 'function',
				'help' => 'Появляется для полей с типом «галочка», «выпадающий список» и «список с выбором нескольких значений»',
			),
			'text' => array(
				'type' => 'editor',
				'name' => 'Описание',
				'multilang' => true,
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'del', // удалить
		'order', // сортируется
		'trash', // использовать корзину
	);

	/**
	 * @var array значения списков
	 */
	public $select_arr = array(
		'type' => array(
			'text' => 'строка',
			'numtext' => 'число',
			'date' => 'дата',
			'datetime' => 'дата и время',
			'textarea' => 'текстовое поле',
			'checkbox' => 'галочка',
			'select' => 'выпадающий список',
			'multiple' => 'список с выбором нескольких значений',
			'email' => 'электронный ящик',
			'title' => 'заголовок группы характеристик',
			'attachments' => 'файлы',
			'images' => 'изображения',
		),
	);

	/**
	 * @var array выводить в списке содержание полей:
	 */
	public $config_other_row = array (
		'type' => array('type' => 'select', 'class' => 'param_type'),
		'module_name' => array('type' => 'select', 'class' => 'parent_name'),
	);

	/**
	 * Подготавливает конфигурацию модуля
	 * @return void
	 */
	public function prepare_config()
	{
		$this->diafan->select_arr("module_name", '-', $this->diafan->_('Все'));

		$result = DB::query("SELECT DISTINCT e.name, o.module_name FROM {config} AS o INNER JOIN {adminsite} AS e ON o.module_name = e.rewrite WHERE o.name = 'comments' AND o.value = '1' AND e.parent_id = '0'");
		while ($row = DB::fetch_array($result))
		{
			$this->diafan->select_arr("module_name", $row['module_name'], $row['name']);
		}
		DB::free_result($result);
	}

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить поле');
	}

	/**
	 * Выводит список полей формы
	 * @return void
	 */
	public function show()
	{
		$this->diafan->list_row();
	}

	/**
	 * Определяет строку с GET переменными
	 *
	 * @return void
	 */
	public function set_get_nav()
	{
		$get_nav_params = $this->diafan->get_nav_params;
		$get_nav_params["module_name"] = '';

		if (!empty( $_GET["module_name"] ))
		{
			$this->diafan->where .= " AND module_name='".($_GET["module_name"] != 'all' ? addslashes(str_replace("%", "%%", $_GET["module_name"])) : ''). "'";
			$this->diafan->get_nav .= ($this->diafan->get_nav ? '&amp;' : '?').'module_name='.urlencode($_GET["module_name"]);
			$get_nav_params["module_name"] = htmlspecialchars($_GET["module_name"]);
		}
		$this->diafan->get_nav_params = $get_nav_params;
	}

	/**
	 * Поиск
	 *
	 * @return string
	 */
	public function show_search()
	{
		if($this->diafan->edit)
			return '';

		$this->upload();
		$html = '<script type="text/javascript" src="'.BASE_PATH.'modules/comments/admin/comments.admin.param.js"></script>

		<td class="name" width="40%"></td>
		<td class="param_type"></td>
		<td class="param_name"><select rel="'.$this->diafan->get_admin_url('cat', 'site').'" class="inpselect" name="module_name" id="select_module"><option value=""></option>';
		foreach($this->diafan->select_arr("module_name") as $k => $v)
		{
			if(! $k)
			{
				$k = 'all';
			}
			$html .= '<option value="'.$k.'"'.($this->diafan->get_nav_params["module_name"] == $k ? ' selected' : '').'>'.$v.'</option>';
		}
		$html .= '</select></td>';

		return $html;
	}
}