<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Faq
 */
class Faq extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;
		
		if ($this->diafan->show)
		{
			$this->result = $this->model->id();
		}
		elseif (! $this->diafan->configmodules("cat"))
		{
			$this->result = $this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->result = $this->model->first_page();
		}
		else
		{
			$this->result = $this->model->list_category();
		}
		$this->get_global_variables();
		$this->result["form"] = $this->model->form();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		$this->diafan->_tpl->get($this->view, 'faq', $this->result);
	}

	/**
	 * Шаблонная функция: выводит последние вопросы. На странице вопросов, когда выводится список аналогичных вопросов, блок не выводится.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых вопросов (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - категории вопросов, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. По умолчанию категория не учитывается, выводятся все вопросы
	 * often - часто задаваемые вопросы (если often=1, выводятся вопросы с пометкой «Часто задаваемый вопрос»)
	 * sort - сортировка вопросов: **date** – по дате (по умолчанию), **rand** – в случайном порядке
	 * template - шаблон тега (файл modules/faq/views/faq.view.show_block_**template**.php; по умолчанию шаблон modules/faq/views/faq.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'cat_id', 'sort', 'site_id', 'often', 'template');
		
		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);
		$site_ids = explode(",", $attributes["site_id"]);
		$sort    = $attributes["sort"] == "date" || $attributes["sort"] == "rand" ? $attributes["sort"] : "date";
		$often   = $attributes["often"] ? true : false;

		$result = $this->model->show_block($count, $cat_ids, $site_ids, $often, $sort);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'faq', $result))
		{
			$this->diafan->_tpl->get('show_block', 'faq', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице вопроса выводит похожие вопросы. По умолчанию связи между вопросами являются односторонними, это можно изменить, отметив опцию «В блоке похожих вопросов связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых вопросов (по умолчанию 3)
	 * template - шаблон тега (файл modules/faq/views/faq.view.show_block_rel_**template**.php; по умолчанию шаблон modules/faq/views/faq.view.show_block_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;

		if ($this->diafan->module != "faq" || ! $this->diafan->show)
			return;

		$result = $this->model->show_block_rel($count);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'faq', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'faq', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит форму добавления вопроса. Для правильной работы тега должна существовать страница, к которой прикреплен модуль Вопрос-Ответ.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * site_id - страница, к которой прикреплен модуль, по умолчанию выбирается одна страница
	 * cat_id - категория вопросов (id категории, по умолчанию выбирается одна категория), если в настройках модуля отмечено «Использовать категории»
	 * template - шаблон тега (файл modules/faq/views/faq.view.form_**template**.php; по умолчанию шаблон modules/faq/views/faq.view.form.php)
	 * 
	 * @return void
	 */
	public function show_form($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'cat_id', 'site_id', 'template');

		$cat_id  = intval($attributes["cat_id"]);
		$site_id = intval($attributes["site_id"]);

		$result = $this->model->form($site_id, $cat_id, true);
		if (! empty($result))
		{
			if (! $attributes["template"] || ! $this->diafan->_tpl->get('form_'.$attributes["template"], 'faq', $result))
			{
				$this->diafan->_tpl->get('form', 'faq', $result);
			}
		}
	}
}