<?php
/**
 * Обработка Ajax-запроса
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Feedback_ajax
 */
class Feedback_ajax extends Ajax
{
	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (! empty($_POST['module']) && $_POST['module'] == 'feedback')
		{
			switch($_POST["action"])
			{
				case 'add':
					return $this->add();

				case 'upload_image':
					return $this->upload_image();

				case 'delete_image':
					return $this->delete_image();
			}
		}
		return false;
	}
	
	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	private function add()
	{
		if(empty($_POST["site_id"]))
		{
			return false;
		}
		if (! $this->site_id = DB::query_result(
					"SELECT s.id FROM {site} AS s"
					.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
					." WHERE s.id=%d AND s.module_name='feedback' AND s.trash='0'"
					." AND (s.access='0'"
					.($this->diafan->_user->role_id ? " OR s.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
					.") LIMIT 1", $_POST["site_id"]
			))
		{
			return false;
		}
		$this->module = "feedback";
		$this->tag = "feedback".$this->site_id;

		$this->check_captcha();
		$params = $this->get_params(array("module" => "feedback", "where" => "site_id=".$this->site_id));
		$this->empty_required_field(array("params" => $params));

		if ($this->send_errors())
			return true;

		$save = DB::query("INSERT INTO {feedback} (created, site_id, lang_id) VALUES (%d, %d, %d)", time(), $this->site_id, _LANG);

		if(! empty($_POST["tmpcode"]))
		{
			DB::query("UPDATE {images} SET element_id=%d, tmpcode='' WHERE module_name='feedback' AND element_id=0 AND tmpcode='%s'", $save, $_POST["tmpcode"]);
		}

		$this->insert_values(array("id" => $save, "table" => "feedback", "params" => $params));

		if ($this->send_errors())
			return true;

		$this->send_mail();
		$this->send_sms();

		$mes = $this->diafan->configmodules('add_message', 'feedback', $this->site_id, _LANG);
		$this->result["errors"][0] = $mes ? $mes : ' ';
		$this->result["success"] = true;
		$this->result["form_hide"]= true;
		return $this->send_errors();
	}

	/**
	 * Загружает изображение
	 *
	 * @return boolean
	 */
	private function upload_image()
	{
		if(empty($_POST["tmpcode"]))
		{
			return false;
		}
		if(empty($_POST["images_param_id"]))
		{
			return false;
		}
		$param_id = $this->diafan->get_param($_POST, "images_param_id", '', 2);

		$this->result["success"] = true;
		if (! empty($_FILES['images'.$param_id]) && $_FILES['images'.$param_id]['tmp_name'] != '' && $_FILES['images'.$param_id]['name'] != '')
		{
			try
			{
				$this->diafan->_images->upload(0, "feedback", 0, $_FILES['images'.$param_id]['tmp_name'], $_FILES['images'.$param_id]['name'], false, false, $param_id, $_POST["tmpcode"]);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = 'p'.$param_id;
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
			$images = $this->diafan->_images->get('large', 0, "feedback", 0, '', $param_id, false, 0, '', $_POST["tmpcode"]);
			$this->result["data"] = $this->diafan->_tpl->get('images', "feedback", $images);
		}
		return $this->send_errors();
	}

	/**
	 * Удаляет изображение
	 *
	 * @return boolean
	 */
	private function delete_image()
	{
		if(empty($_POST["id"]))
		{
			return false;
		}
		if(empty($_POST["tmpcode"]))
		{
			return false;
		}
		$row = DB::query_fetch_array("SELECT * FROM {images} WHERE module_name='feedback' AND id=%d AND tmpcode='%s'", $_POST["id"], $_POST["tmpcode"]);
		if(! $row)
		{
			return false;
		}
		$this->diafan->_images->delete(0, "feedback", $row["id"]);
		return true;
	}

	/**
	 * Уведомление администратора по e-mail
	 * 
	 * @return boolean
	 */
	private function send_mail()
	{
		if (! $this->diafan->configmodules("sendmailadmin", 'feedback', $this->site_id))
			return false;

		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules("subject_admin", 'feedback', $this->site_id)
		);

		$message = str_replace(
			array('%title', '%url', '%message'),
			array(
				TITLE,
				BASE_URL,
				$this->message_admin_param
			),
			$this->diafan->configmodules("message_admin", 'feedback', $this->site_id)
		);

		$to   = $this->diafan->configmodules("emailconfadmin", 'feedback', $this->site_id)
		        ? $this->diafan->configmodules("email_admin", 'feedback', $this->site_id)
		        : EMAIL_CONFIG;
		$from = $this->diafan->configmodules("emailconf", 'feedback', $this->site_id)
		        ? $this->diafan->configmodules("email", 'feedback', $this->site_id)
		        : '';

		include_once ABSOLUTE_PATH.'includes/mail.php';
		send_mail($to, $subject, $message, $from);

		return true;
	}

	/**
	 * Уведомляет администратора по sms
	 * 
	 * @return void
	 */
	private function send_sms()
	{
		if (! $this->diafan->configmodules("sendsmsadmin", 'feedback', $this->site_id))
			return false;
			
		$message = $this->diafan->configmodules("sms_message_admin", 'feedback', $this->site_id);

		$to   = $this->diafan->configmodules("sms_admin", 'feedback', $this->site_id);

		include_once ABSOLUTE_PATH.'includes/sms.php';
		Sms::send($message, $to);
	}
}