<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Files
 */
class Files extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;

		if ($this->diafan->show)
		{
			$this->result = $this->model->id();
		}
		elseif (! $this->diafan->configmodules("cat"))
		{
			$this->result = $this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->result = $this->model->first_page();
		}
		else
		{
			$this->result = $this->model->list_category();
		}
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		$this->diafan->_tpl->get($this->view, 'files', $this->result);
	}

	/**
	 * Шаблонная функция: выводит несколько файлов из файлового архива.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых файлов (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - категории файлов, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. По умолчанию категория не учитывается, выводятся все файлы
	 * images - количество изображений, прикрепленных к файлу
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * sort - сортировка файлов: по умолчанию как на странице модуля, **rand** – в случайном порядке, **date** – по дате
	 * template - шаблон тега (файл modules/files/views/files.view.show_block_**template**.php; по умолчанию шаблон modules/files/views/files.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'cat_id', 'images', 'images_variation', 'sort', 'site_id', 'template');

		$count      = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids    = explode(",", $attributes["cat_id"]);
		$site_ids   = explode(",", $attributes["site_id"]);
		$sort       = $attributes["sort"] == "date" || $attributes["sort"] == "rand" ? $attributes["sort"] : "";
		$images      = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';
		$result = $this->model->show_block($count, $cat_ids, $site_ids, $images, $images_variation, $sort);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'files', $result))
		{
			$this->diafan->_tpl->get('show_block', 'files', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице файлы выводит похожие файлы. По умолчанию связи между файлами являются односторонними, это можно изменить, отметив опцию «В блоке похожих файлов связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых файлов (по умолчанию 3)
	 * images - количество изображений, прикрепленных к файлу
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/files/views/files.view.show_block_rel_**template**.php; по умолчанию шаблон modules/files/views/files.view.show_block_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "files" || ! $this->diafan->show)
			return;

		$result = $this->model->show_block_rel($count, $images, $images_variation);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'files', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'files', $result);
		}
	}
}