<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Forum
 */
class Forum extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('add', 'show', 'cat', 'edit', 'page');

		if ($this->diafan->add)
		{
			$this->result = $this->model->add();	
		}
		elseif ($this->diafan->show)
		{
			$this->result = $this->model->id();	
		}
		elseif ($this->diafan->edit)
		{
			$this->result = $this->model->edit();	
		}
		elseif (! empty($_GET["block_id"]) || ! empty($_GET["unblock_id"]))
		{
			return $this->model->block_id();	
		}
		elseif (! empty($_GET["prior_id"]))
		{
			return $this->model->prior_id();	
		}
		elseif (! empty($_GET["close_id"]))
		{
			return $this->model->close_id();	
		}
		elseif (! empty($_GET["searchword"]))
		{
			$this->result = $this->model->list_search();	
		}
		elseif (! empty($_GET["new"]) && $this->diafan->_user->id)
		{
			$this->result = $this->model->list_new();	
		}
		elseif ($this->diafan->cat)
		{
			$this->result = $this->model->list_category();	
		}
		else
		{
			$this->result = $this->model->first_page();	
		}
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		if ($this->diafan->add || $this->diafan->edit)
		{
			$this->diafan->_tpl->get('edit', 'forum', $this->result);
		}
		elseif ($this->diafan->show)
		{
			$this->diafan->_tpl->get('id', 'forum', $this->result);
		}
		elseif (! empty($_GET["new"]) && $this->diafan->_user->id)
		{
			$this->diafan->_tpl->get('list_new', 'forum', $this->result);
		}
		elseif (! empty($_GET["searchword"]))
		{
			$this->diafan->_tpl->get('list_search', 'forum', $this->result);
		}
		elseif ($this->diafan->cat)
		{
			$this->diafan->_tpl->get('list_category', 'forum', $this->result);
		}
		else
		{
			$this->diafan->_tpl->get('first_page', 'forum', $this->result);
		}
	}

	/**
	 * Шаблонная функция: выводит последние темы.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых тем (по умолчанию 3)
	 * cat_id - категория или раздел форума (id категории, по умолчанию выводятся все темы без учета категории)
	 * template - шаблон тега (файл modules/forum/views/forum.view.show_block_**template**.php; по умолчанию шаблон modules/forum/views/forum.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'template', 'cat_id');

		$count  = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_id = intval($attributes["cat_id"]);

		$result = $this->model->show_block($count, $cat_id);
		if(! $result)
			return;

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'forum', $result))
		{
			$this->diafan->_tpl->get('show_block', 'forum', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице темы форума выводит блок похожих тем. Похожие темы подбираются по совпаданию слов в заголовках.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых тем
	 * cat_id - раздел форума
	 * template - шаблон тега (файл modules/forum/views/forum.view.show_block_rel_**template**.php; по умолчанию шаблон modules/forum/views/forum.view.show_block_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'template', 'cat_id');

		$count  = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_id = $attributes["cat_id"] == "all" ? "all" : "current";

		if ($this->diafan->module != "forum" || ! $this->diafan->show)
			return;

		$result = $this->model->show_block_rel($count, $cat_id);
		if(! $result)
			return;

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'forum', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'forum', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит блок сообщений.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых сообщений
	 * cat_id - раздел форума
	 * template - шаблон тега (файл modules/forum/views/forum.view.show_block_messages_**template**.php; по умолчанию шаблон modules/forum/views/forum.view.show_block_messages.php)
	 * 
	 * @return void
	 */
	public function show_block_messages($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'template', 'cat_id');

		$count  = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);

		Customization::inc('modules/forum/message/message.inc.php');
		$model = new Forum_message_inc($this->diafan);
		$result = $model->show_block_messages($count, $cat_ids);
		if(! $result)
			return;

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_messages_'.$attributes["template"], 'forum', $result))
		{
			$this->diafan->_tpl->get('show_block_messages', 'forum', $result);
		}
	}
}