<?php
/**
 * Работа с сообщениями в модулей «Форум»
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(dirname(__FILE__)))).'/includes/404.php';
}

/**
 * Forum_message_inc
 */
class Forum_message_inc extends Model
{
	/**
	 * @var object основной объект системы
	 */
	protected $diafan;

	/**
	 * @var array сгенерированные в моделе данные, передаваемые в шаблон
	 */
	protected $result;

	/**
	 * @var boolean пользователь является модератором
	 */
    private $moderator;

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct(&$diafan)
	{
		$this->diafan = &$diafan;
                
        $this->moderator = $this->diafan->_user->roles('moderator', 'forum', '', 'site');
	}

	/**
	 * Генерирует список сообщений и форму добавления сообщений
	 * 
	 * @param integer $element_id номер категории форума
	 * @return string
	 */
	public function show($theme)
	{
		$this->result["close"] = $theme["close"];
		$this->result["hash"] = $this->diafan->_user->get_hash();
		$user_news = array();
		if ($this->diafan->_user->id)
		{
			$result = DB::query("SELECT element_id FROM {forum_show} WHERE table_name='forum' AND user_id=%d LIMIT 1", $this->diafan->_user->id);
			while ($row = DB::fetch_array($result))
			{
				$user_news[] = $row["element_id"];
			}
			DB::free_result($result);
		}

		////navigation//
		$this->diafan->_paginator->page    = $this->diafan->page;
		$this->diafan->_paginator->nastr   = $this->diafan->configmodules('nastr');
		$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
		$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
		$this->diafan->_paginator->nen = DB::query_result(
			"SELECT COUNT(*) FROM {forum} WHERE cat_id=%d AND trash='0' AND parent_id=0"
			.(! $this->moderator ? " AND act='1'" : ''), $theme["id"]
		);
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->diafan->_paginator->get());
		////navigation///

		$result_sql = DB::query_range(
			"SELECT * FROM {forum} WHERE cat_id=%d AND trash='0' AND parent_id=0"
			.(! $this->moderator ? " AND act='1'" : '')
			." ORDER BY created ASC", $theme["id"],
			$this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
		);
		$rows = array();
		$parents = array();
		while ($row = DB::fetch_array($result_sql))
		{
			$this->prepare_author($row["user_update"]);
			$this->prepare_author($row["author"]);
			if ($this->diafan->configmodules("attachments"))
			{
				$this->diafan->_attachments->prepare($row["id"], "forum");
			}
			$rows[0][] = $row;
			$parents[] = $row["id"];
		}
		DB::free_result($result_sql);
		if($parents)
		{
			$result = DB::query("SELECT f.* FROM {forum} AS f"
				." INNER JOIN {forum_parents} AS p ON f.id=p.element_id"
				." WHERE f.cat_id=%d AND f.trash='0' AND p.parent_id IN (%s)"
				.(! $this->moderator ? " AND f.act='1'" : '')
				." GROUP BY f.id ORDER BY f.created ASC", $theme["id"], implode(",", $parents));
			while ($row = DB::fetch_array($result))
			{
				$this->prepare_author($row["user_update"]);
				$this->prepare_author($row["author"]);
				if ($this->diafan->configmodules("attachments"))
				{
					$this->diafan->_attachments->prepare($row["id"], "forum");
				}
				$rows[$row["parent_id"]][] = $row;
			}
			DB::free_result($result);
		}
		foreach($rows as $parent_id => $rows_parent)
		{
			foreach($rows_parent as $i => $row)
			{
				$row["access_edit_delete"] = $this->diafan->_user->id
					&& ($row["author"] == $this->diafan->_user->id || $this->moderator)
					&& ! $this->result["close"] ? true : false;
				$row["access_block"]       = $this->moderator ? true : false;
				$row["created"] = $this->format_date($row["created"]);
				if ($row["date_update"])
				{
					$row["date_update"] = $this->format_date($row["date_update"]);
				}
				if($row["user_update"] == $row["author"] || ! $row["user_update"])
				{
					$row["user_update"] = 0;
				}
				else
				{
					$row["user_update"] = $this->get_author($row["user_update"]);
				}
				if($row["author"])
				{
					$row["author"] = $this->get_author($row["author"]);
				}
				else
				{
					$row["author"] = $row["name"];
				}
				$row["attachments"] = array();
				if ($this->diafan->configmodules("attachments"))
				{
					$row["attachments"]["rows"] = $this->diafan->_attachments->get($row["id"], 'forum');
					$row["attachments"]["access"] = $row["access_edit_delete"];
					$row["attachments"]["use_animation"] = $this->diafan->configmodules("use_animation", "forum");
				}
				if ($row["show"] = in_array($row["id"], $user_news))
				{
					$user_news_id[] = $row["id"];
				}
				$row["form"] = $this->get_form($row["id"]);
				$row["hash"] = $this->result["hash"];
				$rows[$parent_id][$i] = $row;
			}
		}
		$this->result["form"] = $this->get_form();

		if ($this->diafan->_user->id)
		{
			if(! empty($user_news_id))
			{
				DB::query("DELETE FROM {forum_show} WHERE user_id=%d AND element_id IN (%s) AND table_name='forum'", implode(",", $user_news_id));
			}
			DB::query("DELETE FROM {forum_show} WHERE user_id=%d AND element_id=%d AND table_name='forum_category'", $this->diafan->_user->id, $theme["id"]);
		}
		$this->result["rows"] = $this->build_tree($rows);

		$text = $this->diafan->_tpl->get('get', 'forum_message', $this->result);

		return $text;
	}

	/**
	 * Формирует дерево сообщений из полученного массива
	 *
	 * @param array $rows все сообщения
	 * @param integer $parent_id номер текущего сообщения-родителя
	 * @param integer $level уровень
	 * @return string
	 */
	private function build_tree($rows, $parent_id = 0, $level = 1)
	{
		$result = array();
		$count_level = $this->diafan->configmodules("count_level", "forum");

		if($count_level && $level > $count_level)
			return $result;

		if (! empty($rows[$parent_id]))
		{
			foreach ($rows[$parent_id] as $row)
			{
				$row["children"] = $this->build_tree($rows, $row["id"], $level+1);
				if($level == $count_level)
				{
					$row["form"] = false;
				}
				$result[] = $row;
			}
		}
		return $result;
	}
	
	/*
	 * Формирует данные для формы
	 * 
	 * @param integer $count количество сообщений
	 * @param integer $parent_id номер сообщения-родителя
	 * @return array
	 */
	public function get_form($parent_id = 0)
	{
		if ($this->diafan->configmodules('only_user', 'forum') && ! $this->diafan->_user->id || $this->result["close"])
		{
			return false;
		}
		$form["field_name"] = $this->diafan->_user->id ? false : true;
		$form["parent_id"] = $parent_id;
		$form["captcha"] = '';
		if ($this->diafan->configmodules('captcha', 'forum'))
		{
			$form["captcha"] = $this->diafan->_captcha->get("forum_message".$parent_id, $this->get_error("forum_message".$parent_id, "captcha"));
		}
		$form["premoderation"]     = $this->diafan->configmodules('premoderation_message', "forum") && ! $this->moderator;
		$form["hash"]              = $this->result["hash"] ? $this->result["hash"] : $this->diafan->_user->get_hash();
		$form["error"]             = $this->get_error("forum_messages".$parent_id);
		$form["error_message"]     = $this->get_error("forum_messages".$parent_id, 'message');
		$form["error_name"]        = $this->get_error("forum_messages".$parent_id, 'name');
		$form["error_attachments"] = $this->get_error("forum_messages".$parent_id, 'attachments');
		if ($this->diafan->configmodules("attachments", "forum"))
		{
			$form["add_attachments"]       = true;
			$form["max_count_attachments"] = $this->diafan->configmodules("max_count_attachments", "forum");
			$form["attachment_extensions"] = $this->diafan->configmodules("attachment_extensions", "forum");
		}
		else
		{
			$form["add_attachments"] = false;
		}
		return $form;
	}

	/**
	 * Запоминает номер категории, для которой нужно вывести информацию о сообщениях
	 * 
	 * @param integer $element_id номер категории форума
	 * @return void
	 */
	public function prepare_info($element_id = 0)
	{
		if (! $element_id)
		{
			return;
		}
		if(isset($this->cache["info"][$element_id]))
		{
			return;
		}
		if(empty($this->cache["prepare_info"]) || ! in_array($element_id, $this->cache["prepare_info"]))
		{
			$this->cache["prepare_info"][] = $element_id;
		}
	}

	/**
	 * Генерирует данные о сообщениях в категории форума
	 * 
	 * @param integer $element_id номер категории форума
	 * @return integer
	 */
	public function info($element_id = 0)
	{
		if (!$element_id)
		{
			$element_id = $this->diafan->show;
		}
		$this->prepare_info($element_id);
		if(! empty($this->cache["prepare_info"]))
		{
			foreach($this->cache["prepare_info"] as $id)
			{
				$this->cache["info"][$id]["count"] = 0;
				$this->cache["info"][$id]["count_new"] = 0;
				$this->cache["info"][$id]["last_user"] = 0;
			}
			
			$result = DB::query("SELECT COUNT(id) as count, cat_id FROM {forum} WHERE cat_id IN (%s) AND trash='0'"
				.(! $this->moderator ? " AND act='1'" : '')." GROUP BY cat_id", implode(",", $this->cache["prepare_info"]));
			while($row = DB::fetch_array($result))
			{
				$this->cache["info"][$row["cat_id"]]["count"] = $row["count"];
			}
			DB::free_result($result);

			$result = DB::query("SELECT COUNT(m.id) as count, m.cat_id FROM {forum} as m"
				." INNER JOIN {forum_show} as n ON n.element_id=m.id"
				." WHERE n.user_id=%d AND n.table_name='forum' AND m.trash='0'"
				." AND m.cat_id IN (%s)"
				.(! $this->moderator ? " AND m.act='1'" : '')." GROUP BY m.cat_id",
				$this->diafan->_user->id,
				implode(",", $this->cache["prepare_info"]));
			while($row = DB::fetch_array($result))
			{
				$this->cache["info"][$row["cat_id"]]["count_new"] = $row["count"];
			}
			DB::free_result($result);

			foreach($this->cache["prepare_info"] as $id)
			{
				$row = DB::query_fetch_array("SELECT created, author, name FROM {forum} WHERE cat_id=%d AND trash='0'"
				.(! $this->moderator ? " AND act='1'" : '')." ORDER BY created DESC LIMIT 1", $id);
				if($row)
				{
					$row["created"] = $this->format_date($row["created"]);
					$this->prepare_author($row["author"]);
					$this->cache["info"][$id]["last_user"] = $row;
				}
			}
			unset($this->cache["prepare_info"]);
		}

		return $this->cache["info"][$element_id];
	}

	/**
	 * Удаляет сообщения, прикрепленные к категории форума
	 *
	 * @param integer $element_id номер категории форума
	 * @return boolean true
	 */
	public function delete($element_id = 0)
	{
		if (! $element_id)
		{
			$element_id = $this->diafan->del;
		}

		$result = DB::query("SELECT id FROM {forum} WHERE cat_id=%d", $element_id);
		while ($row = DB::fetch_array($result))
		{
			$this->diafan->attachments->delete($row["id"], "forum");
			DB::query("DELETE FROM {forum_show} WHERE element_id=%d AND table_name='forum'", $row["id"]);
		}
		DB::free_result($result);
		DB::query("DELETE FROM {forum} WHERE cat_id='%d'", $element_id);

		return true;
	}

	/**
	 * Шаблонная функция: блок сообщений 
	 *
	 * @param integer $count количество сообщений
	 * @param array $cat_ids категории
	 * @return array
	 */
	public function show_block_messages($count, $cat_ids)
	{
		if(! $site_id = $this->diafan->_route->id_module("forum", 0, false))
		{
			return false;
		}

		$cats = array();
		if($cat_ids)
		{
			foreach($cat_ids as $i => $v)
			{
				$v = preg_replace('/[^0-9]+/', '', $v);
				if(empty($v))
				{
					unset($cat_ids[$i]);
				}
				else
				{
					$cat_ids[$i] = $v;
				}
			}
			if($cat_ids)
			{
				$result = DB::query("SELECT id, parent_id FROM {forum_category} WHERE trash='0' AND act='1' AND id IN(%s) LIMIT 1", implode(",", $cat_ids));
				// если указан раздел первого уровня, то ищем во всех категориях
				if($row = DB::fetch_array($result))
				{
					DB::free_result($result);
					$result = DB::query("SELECT id FROM {forum_category} WHERE parent_id=%d AND trash='0' AND act='1'", $cat_id);
					while($row = DB::fetch_array($result))
					{
						$cats[] = $row["id"];
					}
					DB::free_result($result);
				}
				else
				{
					$cats[] = $cat_id;
				}
			}
		}

		$this->result["rows"] = array();
		$rows = array();
		$result = DB::query("SELECT * FROM {forum} WHERE ".($cats ? "cat_id IN (".implode(",", $cats).") AND" : "")
			." trash='0' AND act='1' ORDER BY created DESC LIMIT %d", $count);
		while ($row = DB::fetch_array($result))
		{
			$this->diafan->_route->prepare("forum", $row["cat_id"]);
			if ($this->diafan->configmodules("attachments"))
			{
				$this->diafan->_attachments->prepare($row["id"], "forum");
			}
			$this->prepare_author($row["author"]);
			$this->prepare_author($row["user_update"]);
			$rows[] = $row;
		}
		DB::free_result($result);
		foreach($rows as $row)
		{
			$row["created"] = $this->format_date($row["created"]);
			if ($row["date_update"])
			{
				$row["date_update"] = $this->format_date($row["date_update"]);
			}
			if($row["user_update"] == $row["author"] || ! $row["user_update"])
			{
				$row["user_update"] = 0;
			}
			else
			{
				$row["user_update"] = $this->get_author($row["user_update"]);
			}
			if($row["author"])
			{
				$row["author"] = $this->get_author($row["author"]);
			}
			else
			{
				$row["author"] = $row["name"];
			}

			$row["attachments"] = array();
			if ($this->diafan->configmodules("attachments"))
			{
				$row["attachments"]["rows"] = $this->diafan->_attachments->get($row["id"], 'forum');
				$row["attachments"]["access"] = false;
				$row["attachments"]["use_animation"] = $this->diafan->configmodules("use_animation", "forum");
			}
			$row["link"] = $this->diafan->_route->link($site_id, "forum", 0, $row["cat_id"]).'#'.$row["id"];
			$this->result["rows"][] = $row;
		}
		return $this->result;
	}
}