<?php
/**
 * Обработка Ajax-запросов при работе с изображениями в административной части
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Images_admin_ajax
 */
class Images_admin_ajax extends Frame_admin
{
	/**
	 * @var array результаты, передаваемы Ajaxом
	 */
	private $result;

	/**
	 * Вызывает обработку Ajax-запросов
	 * 
	 * @return void
	 */
	public function ajax()
	{
		if (! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'upload':
					$this->upload();
					break;

				case 'show':
					return $this->show();
			}
		}
		
		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_user->checked)
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$this->result["hash"] = $this->diafan->_user->get_hash();

		if (! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'upload_links':
					$this->upload_links();
					break;

				case 'delete':
					$this->delete();
					break;

				case 'edit':
					$this->edit();
					break;

				case 'save':
					$this->save();
					break;

				case 'up':
				case 'down':
					$this->sort();
					break;

				case 'selectarea':
					$this->selectarea();
					break;

				case 'resize':
					$this->resize();
					break;
			}
		}
	}

	/**
	 * Загружает изображение
	 * 
	 * @return void
	 */
	private function upload()
	{
		if (! $this->diafan->_user->roles('edit', $this->diafan->rewrite))
		{
			$this->result["error"] = $this->diafan->_('Извините, у вас нет прав на сохранение.');
			$this->send_json();
		}

		if (! isset($_FILES["images"]) || ! is_array($_FILES["images"]))
		{
			$this->result["errors"]["image"] = $this->diafan->_('Вы забыли добавить файл для загрузки');
			$this->send_json();
		}
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;
		$module_tag = $this->diafan->config('category') ? $this->diafan->module.'cat' : $this->diafan->table;
		$param_id = $this->diafan->get_param($_POST, "param_id", 0, 2);

		$this->result["id"] = $this->get_id_element();
		if($this->diafan->variable('images', 'count') == 1)
		{
			$this->diafan->_images->delete($this->result["id"], $module_tag);
		}

		Customization::inc('modules/images/admin/images.admin.view.php');
		$images_view = new Images_admin_view($this->diafan);

		foreach($_FILES["images"]["name"] as $i => $name)
		{
			if(! empty($_POST['name']) &&  $_POST['name'] != 'undefined')
			{
				$new_name = preg_replace('/[^A-Za-z0-9-_]+/', '', strtolower($this->diafan->translit(substr($_POST['name'], 0, 50))));
			}
			else
			{
				$extension = strrchr($name, '.');
				$new_name = preg_replace('/[^A-Za-z0-9-_]+/', '', strtolower($this->diafan->translit(substr(str_replace($extension, '', $name), 0, 50))));
			}
			$site_id = $this->diafan->get_param($_POST, 'site_id', 0, 2);
			$tmpcode = (! empty($_POST["tmpcode"]) ? $_POST["tmpcode"] : '');
	
			try
			{
				$result = $this->diafan->_images->upload($this->result["id"], $module_name, $site_id, $_FILES["images"]['tmp_name'][$i], $new_name, $this->diafan->config('category'), true, $param_id, $tmpcode);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = 'image';
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
	
			if ($result && is_array($result))
			{
				foreach($result as $r)
				{
					$this->result["selectarea"][] = $images_view->selectarea($r);
				}
			}
		}
		$this->result["success"] = true;

		$this->send_json();
	}

	/**
	 * Подгружает все изображения
	 * 
	 * @return void
	 */
	private function show()
	{
		if (! $this->diafan->_user->roles('edit', $this->diafan->rewrite))
		{
			$this->result["error"] = $this->diafan->_('Извините, у вас нет прав на сохранение.');
			$this->send_json();
		}
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;
		$module_tag = $this->diafan->config('category') ? $this->diafan->module.'cat' : $this->diafan->table;
		$param_id = $this->diafan->get_param($_POST, "param_id", 0, 2);

		$this->result["id"] = $this->get_id_element();
		Customization::inc('modules/images/admin/images.admin.view.php');
		$images_view = new Images_admin_view($this->diafan);
		$this->diafan->_cache->delete("", $module_name);

		$this->result["data"]   = $images_view->show($this->result["id"], $param_id);
		$this->result["target"] = ".images_container".$param_id;
		$this->result["success"] = true;

		$this->send_json();
	}

	/**
	 * Загружает изображения по ссылкам
	 * 
	 * @return void
	 */
	private function upload_links()
	{
		if (! $this->diafan->_user->roles('edit', $this->diafan->rewrite))
		{
			$this->result["error"] = $this->diafan->_('Извините, у вас нет прав на сохранение.');
			$this->send_json();
		}

		if (empty($_POST["links"]))
		{
			$this->result["errors"]["image"] = $this->diafan->_('Добавьте ссылки для загрузки.');
			$this->send_json();
		}
		$links = explode("\n", $_POST["links"]);
		foreach($links as $i => $v)
		{
			$links[$i] = trim($links[$i]);
			if(! $links[$i])
			{
				unset($links[$i]);
			}
		}
		if(empty($links))
		{
			$this->result["errors"]["image"] = $this->diafan->_('Добавьте ссылки для загрузки.');
			$this->send_json();
		}
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;
		$module_tag = $this->diafan->config('category') ? $this->diafan->module.'cat' : $this->diafan->table;
		$param_id = $this->diafan->get_param($_POST, "param_id", 0, 2);

		$this->result["id"] = $this->get_id_element();
		if($this->diafan->variable('images', 'count') == 1)
		{
			$this->diafan->_images->delete($this->result["id"], $module_tag);
		}

		if(! empty($_POST['name']) &&  $_POST['name'] != 'undefined')
		{
			$name = preg_replace('/[^A-Za-z0-9-_]+/', '', strtolower($this->diafan->translit(substr($_POST['name'], 0, 50))));
		}
		$site_id = $this->diafan->get_param($_POST, 'site_id', 0, 2);
		$tmpcode = (! empty($_POST["tmpcode"]) ? $_POST["tmpcode"] : '');

		Customization::inc('modules/images/admin/images.admin.view.php');
		$images_view = new Images_admin_view($this->diafan);

		try
		{
			foreach($links as $link)
			{
				if(! empty($name))
				{
					$new_name = $name;
				}
				else
				{
					$extension = substr(strrchr($link, '.'), 1);
					$url_arr = explode("/", $link);
					$new_name = $url_arr[count($url_arr) - 1];
					$new_name = preg_replace('/[^A-Za-z0-9-_]+/', '', strtolower($this->diafan->translit(str_replace($extension, '', $new_name))));
				}

				$result = $this->diafan->_images->upload($this->result["id"], $module_name, $site_id, $link, $new_name, $this->diafan->config('category'), true, $param_id, $tmpcode);
				if ($result && is_array($result))
				{
					foreach($result as $r)
					{
						$this->result["selectarea"][] = $images_view->selectarea($r);
					}
				}
			}
		}
		catch(Exception $e)
		{
			Dev::$exception_field = 'images'.$param_id;
			Dev::$exception_result = $this->result;
			throw new Exception($e->getMessage());
		}

		$this->diafan->_cache->delete("", $module_name);

		$this->result["data"]   = $images_view->show($this->result["id"], $param_id);
		$this->result["target"] = ".images_container".$param_id;
		$this->result["success"] = true;

		$this->send_json();
	}

	/**
	 * Сохраняет пользовательское выделение изображения
	 * 
	 * @return void
	 */
	private function selectarea()
	{
		if (! $this->diafan->_user->roles('edit', $this->diafan->rewrite))
		{
			$this->result["error"] = $this->diafan->_('Извините, у вас нет прав на сохранение.');
			$this->send_json();
		}
		if(empty($_POST["id"]) || empty($_POST["variation_id"]))
			return false;

		if(! isset($_POST["x1"]) || ! isset($_POST["x2"]) || ! isset($_POST["y1"]) || ! isset($_POST["y2"]))
			return false;

		if($_POST["x1"] == $_POST["x2"] || $_POST["y1"] == $_POST["y2"])
			return false;
		
		$variation = DB::query_fetch_array("SELECT * FROM {images_variations} WHERE id=%d LIMIT 1", $_POST["variation_id"]);
		$actions = unserialize($variation["param"]);
		
		$row = DB::query_fetch_array("SELECT name, folder_num FROM {images} WHERE id=%d LIMIT 1", $_POST["id"]);

		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;
		$path = ABSOLUTE_PATH.USERFILES."/".$module_name."/".$variation["folder"]."/".($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"];

		Customization::inc("includes/image.php");
		Image::crop($path, $_POST["x2"] - $_POST["x1"], $_POST["y2"] - $_POST["y1"], $variation["quality"], 'top', $_POST["y1"], 'left', $_POST["x1"]);
		$this->diafan->_images->get_variation_image($row["name"], $module_name, $variation, $row["folder_num"], false, true);

		$this->send_json();
	}

	/**
	 * Удаляет изображение
	 * 
	 * @return void
	 */
	private function delete()
	{
		if (! $this->diafan->_user->roles('del', $this->diafan->rewrite))
		{
			$this->result["error"] = $this->diafan->_('Извините, у вас нет прав на удаление.');
			$this->send_json();
		}
		if (empty($_POST["element_id"]) && empty($_POST["tmpcode"]))
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}
		$element_id = (! empty($_POST["element_id"]) ? $_POST["element_id"] : 0);
		$tmpcode = (! empty($_POST["tmpcode"]) ? $_POST["tmpcode"] : '');
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;
		$module_tag = $this->diafan->config('category') ? $this->diafan->module.'cat' : $this->diafan->table;

		$del_image_ids = array();
		if(! empty($_POST['image_ids']))
		{
			foreach($_POST['image_ids'] as $image_id)
			{
				$del_image_ids[] = intval($image_id);
			}
		}
		else
		{
			$del_image_ids = array(intval($_POST['image_id']));
		}
		foreach($del_image_ids as $image_id)
		{
			if (! empty($image_id))
			{
				$row = DB::query_fetch_array("SELECT id, module_name, element_id, param_id FROM {images} WHERE id=%d and module_name='%s' LIMIT 1", $image_id, $module_tag);
				if ($row)
				{
					$this->diafan->_images->delete($row["element_id"], $row["module_name"], $row["id"]);
					$this->diafan->_cache->delete("", $module_name);
				}
			}
		}
		Customization::inc('modules/images/admin/images.admin.view.php');
		$images_view = new Images_admin_view($this->diafan);

		$this->result["data"]   = $images_view->show($element_id, (! empty($row) ? $row["param_id"] : ''));
		$this->result["target"] = ".images_container".(! empty($row) ? $row["param_id"] : '');

		$this->send_json();
	}

	/**
	 * Редактирует изображение
	 * 
	 * @return void
	 */
	private function edit()
	{
		if (empty($_POST["element_id"]) && empty($_POST["tmpcode"]))
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}
		$element_id = (! empty($_POST["element_id"]) ? $_POST["element_id"] : 0);
		$tmpcode = (! empty($_POST["tmpcode"]) ? $_POST["tmpcode"] : '');
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;

		$image_id = intval($_POST['image_id']);
		$row = DB::query_fetch_array("SELECT id, [alt], [title], name, folder_num FROM {images} WHERE id=%d AND element_id=%d AND tmpcode='%s' LIMIT 1", $image_id, $element_id, $tmpcode);
		if (! $row)
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$this->diafan->_cache->delete("", $module_name);

		Customization::inc('modules/images/admin/images.admin.view.php');
		$images_view = new Images_admin_view($this->diafan);

		$this->result["data"] = $images_view->edit_attribute($row);
		$this->result["target"] = ".images_actions[image_id=".$image_id."]";

		$this->send_json();
	}

	/**
	 * Сохраняет данные об изображении
	 * 
	 * @return void
	 */
	private function save()
	{
		if (empty($_POST["element_id"]) && empty($_POST["tmpcode"]))
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}
		$element_id = (! empty($_POST["element_id"]) ? $_POST["element_id"] : 0);
		$tmpcode = (! empty($_POST["tmpcode"]) ? $_POST["tmpcode"] : '');
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;

		$image_id = intval($_POST['image_id']);
		$row_image = DB::query_fetch_array("SELECT id, [alt], [title], element_id, param_id FROM {images} WHERE id=%d AND element_id=%d AND tmpcode='%s' LIMIT 1", $image_id, $element_id, $tmpcode);
		if($row_image["alt"] != $_POST["alt"] || $row_image["title"] != $_POST["title"])
		{
			DB::query("UPDATE {images} SET [alt]='%h', [title]='%h' WHERE id=%d AND element_id=%d AND tmpcode='%s'", $_POST["alt"], $_POST["title"], $image_id, $element_id, $tmpcode);
	
			$this->diafan->_cache->delete("", $module_name);
		}

		Customization::inc('modules/images/admin/images.admin.view.php');
		$images_view = new Images_admin_view($this->diafan);

		$this->result["data"]   = $images_view->show($row_image["element_id"], $row_image["param_id"]);
		$this->result["target"] = ".images_container".$row_image["param_id"];

		$this->send_json();
	}

	/**
	 * Сортирует изображения
	 * 
	 * @return void
	 */
	private function sort()
	{
		if (empty($_POST["element_id"]) && empty($_POST["tmpcode"]))
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}
		$element_id = (! empty($_POST["element_id"]) ? $_POST["element_id"] : 0);
		$tmpcode = (! empty($_POST["tmpcode"]) ? $_POST["tmpcode"] : '');
		$module_name = $this->diafan->config('category') ? $this->diafan->module : $this->diafan->table;
		$module_tag = $this->diafan->config('category') ? $this->diafan->module.'cat' : $this->diafan->table;

		$image_id = intval($_POST['image_id']);
		if (! empty($image_id))
		{
			$row_image = DB::query_fetch_array("SELECT sort, param_id FROM {images} WHERE id=%d AND element_id=%d AND tmpcode='%s' LIMIT 1", $image_id, $element_id, $tmpcode);
			$oldsort = $row_image["sort"];
			$row = DB::query_fetch_array("SELECT id, sort FROM {images} WHERE element_id=%d AND tmpcode='%s' AND module_name='%s' AND id<>%d AND param_id=%d AND"
				  .($_POST["action"] == "up" ? " sort>=%d ORDER BY sort ASC" : " sort<=%d ORDER BY sort DESC")." LIMIT 1",
				  $element_id, $tmpcode, $module_tag, $image_id, $row_image["param_id"], $oldsort);
			if ($row)
			{
				DB::query("UPDATE {images} SET sort=%d WHERE id=%d", $oldsort, $row["id"]);
				DB::query("UPDATE {images} SET sort=%d WHERE id=%d", $row["sort"], $image_id);
			}
			$this->diafan->_cache->delete("", $module_name);
	
			Customization::inc('modules/images/admin/images.admin.view.php');
			$images_view = new Images_admin_view($this->diafan);
	
			$this->result["data"]   = $images_view->show($element_id, $row_image["param_id"]);
			$this->result["target"] = ".images_container".$row_image["param_id"];
		}

		$this->send_json();
	}

	/**
	 * Изменяет размер всех загруженных изображений
	 * 
	 * @return void
	 */
	private function resize()
	{
		if (! $this->diafan->_user->roles('edit', $this->diafan->rewrite))
		{
			$this->result["error"] = $this->diafan->_('Извините, у вас нет прав на сохранение.');
			$this->send_json();
		}
		if(function_exists('set_time_limit'))
		{
			set_time_limit(0);
		}

		$images_variations = array();
		$result = DB::query("SELECT * FROM {images_variations} WHERE trash='0' ORDER BY id DESC");
		while($row = DB::fetch_array($result))
		{
			$images_variations[$row["id"]] = $row;
		}
		DB::free_result($result);

		// не давать перегенерировать, если раздел задан как "Все" и для разных разделов заданых разные настройки
		if($this->diafan->config('config') && ! $this->diafan->site && $this->diafan->config('element_site'))
		{
			$result = DB::query("SELECT id FROM {site} WHERE module_name='%s' AND trash='0'", $this->diafan->module);
			while($row = DB::fetch_array($result))
			{
				if($this->diafan->configmodules('images_variations', $this->diafan->module) != $this->diafan->configmodules('images_variations', $this->diafan->module, $row["id"]))
				{
					DB::free_result($result);
					$this->result["error"] = $this->diafan->_('Выберите раздел сайта, так как для разных разделов заданы разные настройки.');
					return $this->send_json();
				}
			}
			DB::free_result($result);
		}

		if(! empty($_POST["images_variation_id"]))
		{
			$param_id = ! $this->diafan->config('config') ? $this->diafan->edit : 0;
			if($this->diafan->config('config'))
			{
				$table = $this->diafan->module;
			}
			else
			{
				$table = str_replace('_param', '', $this->diafan->table);
			}
			$variations = array();
			foreach($_POST["images_variation_id"] as $i => $id)
			{
				if(! empty($_POST["images_variation_name"][$i]))
				{
					$variations[] = $id;
				}
			}
			$count = 0;

			$result = DB::query("SELECT i.name, i.element_id, i.id, i.module_name, i.folder_num FROM {images} AS i"
			.($this->diafan->site ? " INNER JOIN {".$table."} AS m ON i.element_id=m.id AND m.site_id=".$this->diafan->site : '')
			." WHERE i.module_name='%s' AND i.trash='0' AND i.param_id=%d LIMIT %d, 30", $table, $param_id, (! empty($_POST["resize_count"]) ? $_POST["resize_count"] : 0));
			while ($row = DB::fetch_array($result))
			{
				if(! file_exists(ABSOLUTE_PATH.USERFILES."/original/".($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]))
				{
					$this->diafan->_images->delete($row["element_id"], $row["module_name"], $row["id"]);
					continue;
				}
				foreach($images_variations as $images_variation)
				{
					Files_inc::delete_file(USERFILES."/".$table.'/'.$images_variation['folder'].'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
					if(in_array($images_variation['id'], $variations))
					{
						$this->diafan->_images->get_variation_image($row["name"], $table, $images_variation, $row["folder_num"]);
					}
				}
				$count++;
			}
			DB::free_result($result);
		}

		if($this->diafan->config('config') && ! empty($_POST["images_variation_cat_id"]))
		{
			$variations = array();
			foreach($_POST["images_variation_cat_id"] as $i => $id)
			{
				if(! empty($_POST["images_variation_cat_name"][$i]))
				{
					$variations[] = $id;
				}
			}

			$result = DB::query("SELECT i.name, i.element_id, i.id, i.module_name, i.folder_num FROM {images} AS i"
			.($this->diafan->site ? " INNER JOIN {".$this->diafan->module."_category} AS m ON i.element_id=m.id AND m.site_id=".$this->diafan->site : '')
			." WHERE i.module_name='%s' LIMIT %d, 30", $this->diafan->module."cat", (! empty($_POST["resize_count"]) ? $_POST["resize_count"] : 0));
			while ($row = DB::fetch_array($result))
			{
				if(! file_exists(ABSOLUTE_PATH.USERFILES."/original/".($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]))
				{
					$this->diafan->_images->delete($row["element_id"], $row["module_name"], $row["id"]);
					continue;
				}
				foreach($images_variations as $images_variation)
				{
					Files_inc::delete_file(USERFILES."/".$this->diafan->module.'/'.$images_variation['folder'].'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
					if(in_array($images_variation['id'], $variations))
					{
						$this->diafan->_images->get_variation_image($row["name"], $this->diafan->module, $images_variation, $row["folder_num"]);
					}
				}
				$count++;
			}
			DB::free_result($result);
		}
		if($count)
		{
			$this->result["error"] = 'next';
		}
		else
		{
			$this->result["error"] = $this->diafan->_('Размер изображений успешно изменен');
		}

		$this->send_json();
	}

	/**
	 * Отдает ответ Ajax
	 * 
	 * @return void
	 */
	private function send_json()
	{
		if ($this->result)
		{
			include_once ABSOLUTE_PATH.'plugins/json.php';
			echo to_json($this->result);
			exit;
		}
	}

	/**
	 * Возвращает номер элемента, к которому подключается фотография или тэг
	 * 
	 * @return integer
	 */
	private function get_id_element()
	{
		if (! empty($_REQUEST["id"]))
		{
			return intval($_REQUEST["id"]);
		}
		else
		{
			return 0;
		}
	}
}
