<?php
/**
 * Подключение для работы с прикрепленными изображениями
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Images_inc
 */
class Images_inc extends Diafan
{
	/**
	 * var array максимальный размер и качество оригинала изображения
	 */
	private $original = array(3000, 3000, 90);

	/**
	 * var array размер и качество изображения, выводимого в административной части
	 */
	private $small = array(50, 50, 70);

	/**
	 * var integer максимальное количество файлов в папке 
	 */
	private $max_files_in_folder = 10000;

	/**
	 * Получает изображения, прикрепленные к элементу модуля
	 *
	 * @param string $variation размер изображения, указанный в настройках модуля
	 * @param integer $element_id номер элемента, к которому прикреплены изображения
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $site_id страница сайта, к которой прикреплен элемент
	 * @param string $alt альтернативный текст получаемых изображений
	 * @param integer $param_id номер параметра, к которому прикреплено изображение
	 * @param boolean $is_category изображения прикреплены к категории
	 * @param integer $count количество изображений
	 * @param string $link_to_variation размер изображения, на который ведет ссылка
	 * @param string $tmpcode временный идентификационный код элемента, к которому прикреплены изображения
	 * @return array
	 */
	public function get($variation, $element_id, $module_name, $site_id, $alt, $param_id = 0, $is_category = false, $count = 0, $link_to = '', $tmpcode = '')
	{
		if(! $variation)
		{
			return array();
		}
		list($config_module_name, ) = explode("_", $module_name);
		if($tmpcode || is_array($count))
		{
			$rows = array();
			$result = DB::query(
					"SELECT id, name, [alt], [title], folder_num FROM {images}"
					." WHERE module_name='%s' AND element_id=%d AND param_id=%d AND trash='0' AND tmpcode='%s'"
					.(is_array($count) ? " AND id IN (".implode(",", $count).")" : "")
					." ORDER BY sort DESC"
					.($count && ! is_array($count) ? " LIMIT ".$count : ''),
					$module_name.($is_category ? 'cat' : ''),
					$element_id, $param_id, $tmpcode
				);
			while ($row = DB::fetch_array($result))
			{
				$rows[] = $row;
			}
			DB::free_result($result);
		}
		else
		{
			$this->prepare($element_id, $module_name, $param_id, $is_category);
			if(! empty($this->cache["prepare"]))
			{
				$where = array();
				$values = array();
				foreach($this->cache["prepare"] as $pr_module_name => $array)
				{
					$w = array();
					$values[] = $pr_module_name;
					foreach($array as $pr_param_id => $arr)
					{
						$values[] = $pr_param_id;
						foreach($arr as $pr_element_id => $a)
						{
							$this->cache["images"][$pr_module_name][$pr_param_id][$pr_element_id] = array();
							$values[] = $a;
							$v_arr[] = '%d';
						}
						$w[] = " param_id=%d AND element_id".(count($arr) > 1 ? " IN (".implode(",", $v_arr).")" : "=%d");
					}
					$where[] = "module_name='%h' AND ".(count($w) > 1 ? "(".implode(" OR ", $w).")" : $w[0]);
				}
				$result = DB::query(
						"SELECT id, name, [alt], [title], folder_num, module_name, param_id, element_id FROM {images}"
						." WHERE trash='0' AND ".(count($where) > 1 ? "(".implode(" OR ", $where).")" : $where[0])
						." ORDER BY sort DESC", $values
					);
				while ($row = DB::fetch_array($result))
				{
					$this->cache["images"][$row["module_name"]]['p'.$row["param_id"]]['e'.$row["element_id"]][] = $row;
				}
				DB::free_result($result);
				unset($this->cache["prepare"]);
			}
			$rows = $this->cache["images"][$module_name.($is_category ? 'cat' : '')]['p'.$param_id]['e'.$element_id];
		}
		if(empty($rows))
		{
			return array();
		}
		$this->get_variations();
		
		if($param_id)
		{
			if(! isset($this->cache["param_config"][$module_name][$param_id]))
			{
				$this->cache["param_config"][$module_name][$param_id] = unserialize(DB::query_result("SELECT config FROM {%s_param} WHERE id=%d LIMIT 1", $module_name, $param_id));
			}
			$module_variations = $this->cache["param_config"][$module_name][$param_id];
		}
		else
		{
			$module_variations = unserialize($this->diafan->configmodules('images_variations'.($is_category ? '_cat' : ''), $config_module_name, $site_id));
		}
		if(! $module_variations)
			return array();

		$variation_folder = '';
		$link_to_variation_folder = '';
		foreach($module_variations as $module_variation)
		{
			if($module_variation['name'] == $variation && ! empty($this->cache["images_variations"][$module_variation['id']]))
			{
				$variation_folder = $this->cache["images_variations"][$module_variation['id']]["folder"];
			}
			if($link_to && $module_variation['name'] == $link_to && ! empty($this->cache["images_variations"][$module_variation['id']]))
			{
				$link_to_variation_folder = $this->cache["images_variations"][$module_variation['id']]["folder"];
			}
		}
		if(! $variation_folder)
		{
			return array();
		}

		$images = array();
		foreach($rows as $row)
		{
			if(! is_array($count) && $count && count($images) == $count)
			{
				break;
			}
			if(! file_exists(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$variation_folder.'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]))
			{
				continue;
			}
			if ($link_to_variation_folder)
			{
				$img["link"] = USERFILES.'/'.$module_name.'/'.$link_to_variation_folder.'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"];

				if (! $this->diafan->configmodules('use_animation', $config_module_name))
				{
					list($img["link_width"], $img["link_height"]) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$link_to_variation_folder.'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
					$img["type"] = "large_image";
				}
				else
				{
					$img["type"] = "animation";
				}
			}
			else
			{
				$img["type"] = "link";
				$img["link"] = $link_to;
			}
			$img["id"] = $row["id"];
			list($img["width"], $img["height"]) = getimagesize(ABSOLUTE_PATH.USERFILES.'/'.$module_name.'/'.$variation_folder.'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
			$img["alt"] = $row["alt"] ? $row["alt"] : $alt;
			$img["title"] = $row["title"] ? $row["title"] : $alt;
			$img["src"] = BASE_PATH.USERFILES.'/'.$module_name.'/'.$variation_folder.'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"];
			$images[] = $img;
		}
		return $images;
	}

	/**
	 * Запоминает данные об элементах, которым нужно вывести прикрепленные изображения
	 *
	 * @param integer $element_id номер элемента, к которому прикреплены изображения
	 * @param string $module_name название модуля, по умолчанию модуль, прикрепленный к текущей странице
	 * @param integer $param_id номер параметра, к которому прикреплено изображение
	 * @param boolean $is_category изображения прикреплены к категории
	 * @return array
	 */
	public function prepare($element_id, $module_name, $param_id = 0, $is_category = false)
	{
		if(isset($this->cache["images"][$module_name.($is_category ? 'cat' : '')]['p'.$param_id]['e'.$element_id]))
		{
			return;
		}
		if(! isset($this->cache["prepare"][$module_name.($is_category ? 'cat' : '')]['p'.$param_id]['e'.$element_id]))
		{
			$this->cache["prepare"][$module_name.($is_category ? 'cat' : '')]['p'.$param_id]['e'.$element_id] = $element_id;
		}
	}

	/*
	 * Удаляет прикрепленные изображения
	 *
	 * @param integer $element_id номер элемента, к которому прикреплены изображения
	 * @param string $module_name название модуля
	 * @param integer $image_id номер изображения
	 * @return void
	*/
	public function delete($element_id, $module_name, $image_id = 0)
	{
		$this->get_variations();
		if ($image_id)
		{
			$result = DB::query("SELECT * FROM {images} WHERE id=%d AND element_id=%d AND module_name='%h'", $image_id, $element_id, $module_name);
		}
		else
		{
			$result = DB::query("SELECT * FROM {images} WHERE element_id=%d AND module_name='%h'", $element_id, $module_name);
		}
		$module_name = str_replace("cat", "", $module_name);
		while ($row = DB::fetch_array($result))
		{
			DB::query("DELETE FROM {images} WHERE id='%d'", $row["id"]);
			Files_inc::delete_file(USERFILES.'/original/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
			Files_inc::delete_file(USERFILES.'/small/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
			foreach($this->cache["images_variations"] as $variation)
			{
				Files_inc::delete_file(USERFILES.'/'.$module_name.'/'.$variation['folder'].'/'.($row["folder_num"] ? $row["folder_num"].'/' : '').$row["name"]);
			}
		}
		DB::free_result($result);
	}

	/*
	 * Удаляет прикрепленные изображения
	 *
	 * @param integer $element_id номер элемента, к которому прикреплены изображения
	 * @param string $module_name название модуля
	 * @param integer $site_id страница сайта
	 * @param sting $tmp_name расположение файла
	 * @param string $new_name название файла без расширения
	 * @param boolean $is_category изображения прикрепляются к категории
	 * @param boolean $handle ручная загрузка изображений по одной
	 * @param integer $param_id
	 * @param string $tmpcode временный код для прикрепления изображений к еще не созданному  элементу
	 * @param integer $sort номер для сортировки
	 * @return mixed
	*/
	public function upload($element_id, $module_name, $site_id, $tmp_name, $new_name, $is_category = false, $handle = false, $param_id = 0, $tmpcode = '', $sort = 0)
	{
		$folder_num = 0;
		list($config_module_name, ) = explode("_", $module_name);
		$max_id = DB::query_result("SELECT MAX(id) FROM {images} LIMIT 1");
		if($max_id > $this->max_files_in_folder)
		{
			$folder_num = ceil($max_id / $this->max_files_in_folder);
		}
		Files_inc::create_dir('original', USERFILES, true);

		if($folder_num)
		{
			Files_inc::create_dir($folder_num, USERFILES.'/original', true);
		}

		$config_cat = $is_category ? '_cat' : '';
		if($param_id)
		{
			$config_images_variations = DB::query_result("SELECT config FROM {%s_param} WHERE id=%d LIMIT 1", $module_name, $param_id);
		}
		else
		{
			$config_images_variations = $this->diafan->configmodules("images_variations".$config_cat, $config_module_name, $site_id);
		}

		if (! $config_images_variations)
		{
			if($param_id)
			{
				throw new Exception($this->diafan->_('В настройках поля не заданы размеры изображения.'));
			}
			else
			{
				throw new Exception($this->diafan->_('В настройках модуля не заданы размеры изображения.'));
			}
		}

		$tmp = time().mt_rand(0, 9999);
		Files_inc::copy_file($tmp_name, $tmp, USERFILES."/original".($folder_num ? '/'.$folder_num : ''));
		$tmp_name = USERFILES."/original".($folder_num ? '/'.$folder_num : '')."/".$tmp;

		$info = @getimagesize(ABSOLUTE_PATH.$tmp_name);
		if ($info == false)
		{
			Files_inc::delete_file($tmp_name);
			throw new Exception($this->diafan->_('Не верный формат файла. Изображения загружаются только в форматах  GIF, JPEG, PNG.'));
		}

		Customization::inc("includes/image.php");

		$mimes = array(
			'image/gif' => 'gif',
			'image/jpeg' => 'jpg',
			'image/png' => 'png',
			'image/pjpeg' => 'jpg',
			'image/x-png'=> 'png'
		);
		if(empty($info['mime']) || ! in_array($info['mime'], array_keys($mimes)))
		{
			Files_inc::delete_file($tmp_name);
			throw new Exception($this->diafan->_('Не верный формат файла. Изображения загружаются только в форматах  GIF, JPEG, PNG.'));
		}
		$extension = $mimes[$info['mime']];

		$new_name .= '.'.$extension;
		$rand = mt_rand(0, 999999);

		$id = DB::query(
				"INSERT INTO {images} (module_name, element_id, param_id, name, tmpcode, created, folder_num, sort) VALUES ('%s', %d, %d, '%s', '%s', %d, %d, %d)",
				$module_name.($is_category ? "cat" : ""),
				$element_id,
				$param_id,
				$rand,
				$tmpcode,
				time(),
				$folder_num,
				$sort
			);
		$new_name = $id.'_'.$new_name;
		DB::query("UPDATE {images} SET name='%h'".(! $sort ? ", sort=id" : "")." WHERE id=%d", $new_name, $id);

		Files_inc::rename_file($new_name, $tmp, USERFILES."/original".($folder_num ? '/'.$folder_num : ''));

		// уменьшает оригинал до размеров максимальных размеров
		if ($info[0] > $this->original[0] || $info[1] > $this->original[1])
		{
			if (! Image::resize(ABSOLUTE_PATH.USERFILES."/original/".($folder_num ? $folder_num.'/' : '').$new_name, $this->original[0], $this->original[1], $this->original[2]))
			{
				$this->delete($element_id, $module_name.($is_category ? "cat" : ""), $id);
				throw new Exception($this->diafan->_('Не верный формат файла. Изображения загружаются только в форматах  GIF, JPEG, PNG.'));
			}
		}
		
		Files_inc::create_dir('small', USERFILES);
		if($folder_num)
		{
			Files_inc::create_dir($folder_num, USERFILES.'/small');
		}

		// добавляет вариант изображения, выводимый в административной части
		Files_inc::copy_file(ABSOLUTE_PATH.USERFILES."/original/".($folder_num ? $folder_num.'/' : '').$new_name, $new_name, USERFILES."/small".($folder_num ? '/'.$folder_num : ''));
		Image::resize(ABSOLUTE_PATH.USERFILES."/small/".($folder_num ? $folder_num.'/' : '').$new_name, $this->small[0], $this->small[1], $this->small[2]);

		Files_inc::create_dir($module_name, USERFILES);
		$this->get_variations();

		$module_images_variations = unserialize($config_images_variations);
		foreach($module_images_variations as $module_variation)
		{
			if(empty($this->cache["images_variations"][$module_variation["id"]]))
				continue;

			$variation = $this->cache["images_variations"][$module_variation["id"]];

			$action = $this->get_variation_image($new_name, $module_name, $variation, $folder_num, $handle);
			if($action && $action["name"] == 'selectarea')
			{
				$path = BASE_PATH.USERFILES."/".$module_name."/".$variation["folder"]."/".($folder_num ? $folder_num.'/' : '').$new_name;
				$selectarea[] = array("id" => $id, "variant_id" => $module_variation["id"], "path" => $path, "width" => $action["width"], "height" => $action["height"]);
			}
		}
		if(! empty($selectarea))
		{
			return $selectarea;
		}
		else
		{
			return false;
		}
	}
	
	public function get_variation_image($file_name, $module_name, $variation, $folder_num, $handle = false, $after_selectarea = false)
	{
		Customization::inc("includes/image.php");

		Files_inc::create_dir($variation["folder"], USERFILES.'/'.$module_name);
		if($folder_num)
		{
			Files_inc::create_dir($folder_num, USERFILES.'/'.$module_name.'/'.$variation["folder"]);
		}

		$path = ABSOLUTE_PATH.USERFILES."/".$module_name."/".$variation["folder"]."/".($folder_num ? $folder_num.'/' : '').$file_name;
		if(! $after_selectarea)
		{
			Files_inc::copy_file(ABSOLUTE_PATH.USERFILES."/original/".($folder_num ? $folder_num.'/' : '').$file_name, $file_name, USERFILES."/".$module_name."/".$variation["folder"]."/".($folder_num ? $folder_num.'/' : ''));
		}

		$actions = unserialize($variation["param"]);
		foreach($actions as $action)
		{
			if($after_selectarea)
			{
				if($action["name"] == 'selectarea')
				{
					$after_selectarea = false;
				}
				continue;
			}
			switch($action["name"])
			{
				case 'resize':
					Image::resize($path, $action["width"], $action["height"], $variation["quality"], $action["max"]);
					break;

				case 'selectarea':
					if($handle)
					{
						return $action;
					}
					break;

				case 'crop':
					Image::crop($path, $action["width"], $action["height"], $variation["quality"], $action["vertical"], $action["vertical_px"], $action["horizontal"], $action["horizontal_px"]);
					break;

				case 'wb':
					Image::wb($path, $variation["quality"]);
					break;

				case 'watermark':
					Image::watermark($path, ABSOLUTE_PATH.USERFILES."/watermark/".$action["file"], $variation["quality"], $action["vertical"], $action["vertical_px"], $action["horizontal"], $action["horizontal_px"]);
					break;
			}
		}
	}
	
	/**
	 * Выбирает все размеры изображений
	 *
	 * @return void
	 */
	private function get_variations()
	{
		if(! isset($this->cache["images_variations"]))
		{
			$this->cache["images_variations"] = array();
			$result = DB::query("SELECT * FROM {images_variations} WHERE trash='0' ORDER BY id DESC");
			while($row = DB::fetch_array($result))
			{
				$this->cache["images_variations"][$row["id"]] = $row;
			}
			DB::free_result($result);
		}
	}
}