<?php
/**
 * Подключение модуля к административной части других модулей
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Map_admin_inc
 */
class Map_admin_inc extends Diafan
{
	/**
	 * Добавление элемента в поисковый индекс.
	 * 
	 * @return void
	 */
	public function save()
	{
		// если есть поле "Не показывать на карте сайта" и оно отмечено
		if ($this->diafan->is_variable('map_no_show') && ! empty($_POST["map_no_show"]))
		{
			// и элемент проиденксирован (не новые и поле не было отмечено раньше)
			if(! $this->diafan->savenew && empty($this->diafan->oldrow["map_no_show"]))
			{
				// удаляем из индекса
				$this->diafan->_map->delete($this->diafan->save, $this->diafan->table);
			}
			return;
		}

		if ($this->diafan->is_variable('rewrite') && ! $this->diafan->is_save_rewrite)
		{
			$this->diafan->get_rewrite();
		}

		if (! $this->diafan->save_site_id)
		{
			$this->diafan->get_site_id();
		}

		$row = array(
				"module_name" => $this->diafan->module,
				"table" => $this->diafan->table,
				"id"    => $this->diafan->save,
				"site_id" => $this->diafan->save_site_id,
				"changefreq" => ! empty($_POST["changefreq"]) ? $_POST["changefreq"] : '',
				"priority" => ! empty($_POST["priority"]) ? $_POST["priority"] : '',
				"is_save" => true,
			);
		if($this->diafan->module == 'site')
		{
			$row["type"] = 'site';
		}
		elseif($this->diafan->config('category'))
		{
			$row["type"] = 'cat';
		}
		else
		{
			$row["cat_id"] = ! empty($_POST["cat_id"]) ? $_POST["cat_id"] : 0;
		}
		if($this->diafan->is_variable('act'))
		{
			if($this->diafan->variable_multilang('act'))
			{
				foreach ($this->diafan->_languages->all as $l)
				{
					if($l["id"] == _LANG)
					{
						$row["act".$l["id"]] = ! empty($_POST["act"]) ? 1 : 0;
					}
					else
					{
						$row["act".$l["id"]] = ! empty($this->diafan->oldrow["act".$l["id"]]) ? 1 : 0;
					}
				}
			}
			else
			{
				$row["act"] = ! empty($_POST["act"]) ? 1 : 0;
			}
		}
		$this->diafan->_map->index_element($row);
	}

	/**
	* Удаление индекса для выбранных модулей
	* 
	* @param array $table_name таблица
	* @param integer $element_id идентификатор элемента
	* @param integer $trash_id идентификатор элемента
	* @param bool $is_category категория или нет
	* @return void
	*/
	public function delete($table_name, $element_id, $trash_id = 0, $is_category = false)
	{
		$table_name .= ($is_category ? '_category' : '');
		$this->diafan->del_or_trash_where("map_index", "table_name='".$table_name."' AND element_id=".$element_id, $trash_id);
	}

	/**
	* Создание/удаление индекса карты сайта при активации/деактивации/удалении элементов
	* 
	* @param array $table_name таблица
	* @param array $element_ids номера элементов
	* @param boolean $act активация/деактивация
	* @return void
	*/
	public function act($table_name, $element_ids, $act)
	{
		// только при активации/деактивации из  списка, при сохранении индексируем функцией save
		if($this->diafan->save)
		{
			return;
		}

		$module_name = $this->diafan->module;

		if ($module_name != 'site' && ! file_exists('modules/'.$module_name.'/'.$module_name.'.sitemap.php'))
		{
			return;
		}

		if($act)
		{
			if($element_ids)
			{
				if($table_name == 'site')
				{
					$rows = DB::query_fetch_all("SELECT * FROM {site} WHERE id IN (%s) AND map_no_show='0' AND trash='0'", implode(",", $element_ids));
					if($rows)
					{
						foreach ($rows as &$row)
						{
							$row["module_name"] = $module_name;
							$row["table"] = $table_name;
							$row["site_id"] = $row["id"];
							$row["cat_id"] = 0;
							$row["element_id"] = 0;
							$row["type"] = 'site';
						}
						$this->diafan->_map->index_elements($rows);
					}
				}
				else
				{
					Customization::inc('modules/'.$module_name.'/'.$module_name.'.sitemap.php');
					$class_name = ucfirst($module_name).'_sitemap';
					$class = new $class_name($this->diafan);
					if(is_callable(array(&$class, 'config')))
					{
						if($this->diafan->config('element_site'))
						{
							if($this->diafan->site)
							{
								$site_ids = array($this->diafan->site);
							}
							else
							{
								$site_result = DB::query_result("SELECT GROUP_CONCAT(DISTINCT(site_id), ',') FROM {%s} WHERE id IN (%s)", $table_name, implode(",", $element_ids));
								$site_ids = explode(',', $site_result);
							}
						}
						else
						{
							$site_ids = 0;
						}
						foreach ($site_ids as $site_id)
						{
							$config = call_user_func_array(array(&$class, 'config'), array($site_id));
							if(! empty($config["table"]))
							{
								if($config["table"] == $table_name)
								{
									$table_in_config = true;
								}
							}
							elseif(! empty($config["type"]))
							{
								if($module_name == $table_name && in_array('element', $config["type"])
								   || $module_name.'_category' == $table_name && in_array('category', $config["type"]))
								{
									$table_in_config = true;
								}
							}
							else
							{
								if($module_name == $table_name)
								{
									$table_in_config = true;
								}
							}
							if(empty($table_in_config))
							{
								return;
							}
							$where = '';
							if(! empty($config["where"]))
							{
								if(! is_array($config["where"]))
								{
									$where = $config["where"];
								}
								elseif($module_name == $table_name)
								{
									$where =  ! empty($config['where']['element']) ? $config['where']['element'] : '';
								}
								elseif($module_name.'_category')
								{
									$where =  ! empty($config['where']['category']) ? $config['where']['category'] : '';
								}
							}
		
							$rows = DB::query_fetch_all("SELECT * FROM {%s} WHERE id IN (%s) ".$where, $table_name, implode(",", $element_ids));
							if($rows)
							{
								foreach ($rows as &$row)
								{
									$row["module_name"] = $module_name;
									$row["table"] = $table_name;
									$row["site_id"] = ! empty($row["site_id"]) ? $row["site_id"] : 0;
									if($module_name.'_category' == $table_name)
									{
										$row["cat_id"] = $row["id"];
										$row["element_id"] = 0;
										$row["type"] = 'cat';
									}
									else
									{
										$row["cat_id"] = ! empty($row["cat_id"]) && $this->diafan->configmodules("cat",  $module_name, $row["site_id"]) ? $row["cat_id"] : 0;
										$row["element_id"] = $row["id"];
										$row["type"] = 'element';
									}
								}
								$this->diafan->_map->index_elements($rows);
							}
						}
					}
				}
			}
		}
		else
		{
			$this->diafan->_map->delete($element_ids, $table_name);
		}
	}
	/**
	 * Редактирование поля "Подключить комментарии" для настроек модуля
	 * 
	 * @return void
	 */
	public function edit_config()
	{
		if($this->diafan->configmodules("module_".$this->diafan->module."_index", "map"))
		{
			$this->diafan->_map->index_module($this->diafan->module);
		}
	}

	/**
	 * Сохранение настроек конфигурации модулей
	 * 
	 * @return void
	 */
	public function save_config()
	{
		$fields = array('page_show', 'cat');

		foreach ($fields as $field)
		{
			$value = ! empty($_POST[$field]) ? $_POST[$field] : 0;
			if($this->diafan->configmodules($field, $this->diafan->module, $this->diafan->site, _LANG) != $value)
			{
				$this->diafan->configmodules($field, $this->diafan->module, $this->diafan->site, _LANG, $value);
				$edit = true;
			}
		}
		if(! empty($edit))
		{
			$this->diafan->configmodules("module_".$this->diafan->module."_index", "map", 0, _LANG, true);
		}
	}
}