<?php
/**
 * Подключение модуля
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Map_inc
 */
class Map_inc extends Model
{
	/**
	 * @var array языковые версии сайта
	 */
	public $lang;

	/**
	 * @var array ЧПУ
	 */
	public $rewrites;

	/**
	 * @var string
	 */
	public $type;

	/**
	 * @var integer максимальное количество ссылок, индексируемых за один проход скрипта
	 */
	private $max_insert = 500;

	/**
	 * @var integer
	 */
	private $current_insert = 0;

	/**
	 * @var string таблица, проиндексированная за предыдущий проход скрипта
	 */
	private $last_table = '';
	
	/**
	 * @var integer страница сайта, проиндексированная за предыдущий проход скрипта
	 */
	private $last_site_id = 0;

	/**
	 * Индексирует весь сайт
	 * 
	 * @return void
	 */
	public function index_all()
	{
		$this->type = 'all';

		$this->set_languages();
		$this->set_rewrites();

		$this->last_site_id = $this->diafan->configmodules($this->type."_current_index_site", "map");
		$this->last_table = $this->diafan->configmodules($this->type."_current_index_module_table", "map");
		if(! $this->last_site_id)
		{
			DB::query("TRUNCATE TABLE {map_index}");
		}

		$rows = DB::query_fetch_all("SELECT * FROM {site} WHERE trash='0' AND access='0' AND map_no_show='0'".($this->last_site_id ? " AND id>=%d" : "")." ORDER BY id ASC LIMIT ".$this->max_insert, $this->last_site_id);

		foreach ($rows as $row)
		{
			if($this->last_site_id != $row["id"])
			{
				foreach ($this->lang as $l)
				{
					if (! $row["act".$l["id"]])
						continue;
	
					if($row["id"] == 1)
					{
						$rewrite = '';
					}
					else
					{
						if(empty($this->rewrites["site"][$row["id"]]))
						{
							continue;
						}
						$rewrite = $this->rewrites["site"][$row["id"]];
					}
					if(ROUTE_END && $rewrite)
					{
						$rewrite .= ROUTE_END;
					}
					$this->insert('site', $row["id"], $l["name"].$rewrite, $row["timeedit"], $row["changefreq"], $row["priority"]);
				}
				$this->check_max($row["id"], 'site', 0);
			}
			$this->index_site_module($row);
		}
		$this->diafan->configmodules($this->type."_current_index_module_table", "map", 0, _LANG, '');
		$this->diafan->configmodules($this->type."_current_index_module_element", "map", 0, _LANG, '');
		$this->diafan->configmodules($this->type."_current_index_site", "map", 0, _LANG, '');
		$this->diafan->configmodules("full_index", "map", 0, _LANG, true);
	}

	/**
	 * Индексирует модуль
	 * 
	 * @return void
	 */
	public function index_module($module_name)
	{
		$this->type = 'module_'.$module_name;

		$this->set_languages();
		$this->set_rewrites($module_name);

		$this->last_site_id = $this->diafan->configmodules($this->type."_current_index_site", "map");
		$this->last_table = $this->diafan->configmodules($this->type."_current_index_module_table", "map");
		if(! $this->last_site_id)
		{
			DB::query("DELETE FROM {map_index} WHERE table_name LIKE '".$module_name."%'");
		}

		$rows = DB::query_fetch_all("SELECT * FROM {site} WHERE trash='0' AND access='0' AND map_no_show='0' AND module_name='%s'".($this->last_site_id ? " AND id>=%d" : "")." ORDER BY id ASC LIMIT ".$this->max_insert, $module_name, $this->last_site_id);

		foreach ($rows as $row)
		{
			if($this->last_site_id != $row["id"])
			{
				foreach ($this->lang as $l)
				{
					if (! $row["act".$l["id"]])
						continue;
	
					if($row["id"] != 1 && empty($this->rewrites["site"][$row["id"]]))
					{
						continue;
					}
				}
				$this->check_max($row["id"], 'site', 0);
			}
			$this->index_site_module($row);
		}
		$this->diafan->configmodules($this->type."_current_index_module_table", "map", 0, _LANG, '');
		$this->diafan->configmodules($this->type."_current_index_module_element", "map", 0, _LANG, '');
		$this->diafan->configmodules($this->type."_current_index_site", "map", 0, _LANG, '');
		$this->diafan->configmodules($this->type."_index", "map", 0, _LANG, '');
	}

	/**
	 * Индексирует группу элементов
	 * 
	 * @return void
	 */
	public function index_elements($rows)
	{
		$this->set_languages();
		$this->set_rewrites($rows[0]["module_name"], $rows);
		foreach ($rows as $row)
		{
			$ids[] = $row["id"];
			$table = $row["table"];
		}
		if(! empty($ids))
		{
			$this->delete($ids, $table);
		}
		foreach ($rows as $row)
		{
			$row["link"] = $this->get_url($row["module_name"], $row["site_id"], $row["cat_id"], $row["element_id"], 0);
			foreach ($this->lang as $l)
			{
				if (isset($row["act".$l["id"]]) && ! $row["act".$l["id"]] || isset($row["act"]) && empty($row["act"]))
					continue;
				$this->insert($row["table"], $row["id"], $l["name"].$row["link"], $row["timeedit"], $row["changefreq"], $row["priority"]);
			}
		}
	}

	/**
	 * Индексирует один элемент
	 * 
	 * @return void
	 */
	public function index_element($row)
	{
		$this->set_languages();
		$this->set_rewrites($row["module_name"], array($row));
		
		if(empty($row["type"]))
		{
			$row["type"] = 'element';
		}
		switch($row["type"])
		{
			case 'cat':
				$row["element_id"] = 0;
				$row["cat_id"] = $row["id"];
				$row["param_id"] = 0;
				$row["table"] = $row["module_name"].'_category';
				break;

			case 'site':
				$row["site_id"] = $row["id"];
				$row["element_id"] = 0;
				$row["cat_id"] = 0;
				$row["param_id"] = 0;
				$row["table"] = 'site';
				break;

			case 'param':
				$row["element_id"] = 0;
				$row["cat_id"] = 0;
				$row["param_id"] = $row["id"];
				$row["table"] = $row["module_name"].'_param';
				break;

			default:
				$row["element_id"] = $row["id"];
				if(empty($row["cat_id"]))
				{
					$row["cat_id"] = 0;
				}
				$row["param_id"] = 0;
				if(empty($row["table"]))
				{
					$row["table"] = $row["module_name"];
				}
				break;
		}

		$this->delete($row["id"], $row["table"]);

		if(empty($row["changefreq"]))
		{
			$row["changefreq"] = '';
		}
		if(empty($row["priority"]))
		{
			$row["priority"] = '';
		}
		if(empty($row["timeedit"]))
		{
			$row["timeedit"] = time();
		}
		if(! $row["changefreq"] && ! $row["priority"])
		{
			if($row["type"] == 'element' && $row["cat_id"])
			{
				$parent = DB::query_fetch_array("SELECT * FROM {%s} WHERE id=%d", $row["module_name"].'_category', $row["cat_id"]);
				if($parent)
				{
					$row["changefreq"] = (! empty($parent["changefreq"]) ? $parent["changefreq"] : '');
					$row["priority"] = (! empty($parent["priority"]) ? $parent["priority"] : '');
				}
			}
		}
		if($row["type"] != 'site')
		{
			$parent = DB::query_fetch_array("SELECT changefreq, priority FROM {site} WHERE id=%d", $row["site_id"]);
			if($parent)
			{
				$row["changefreq"] = (! empty($parent["changefreq"]) ? $parent["changefreq"] : '');
				$row["priority"] = (! empty($parent["priority"]) ? $parent["priority"] : '');
			}
		}

		$row["link"] = $this->get_url($row["module_name"], $row["site_id"], $row["cat_id"], $row["element_id"], $row["param_id"]);

		foreach ($this->lang as $l)
		{
			if (isset($row["act".$l["id"]]) && ! $row["act".$l["id"]] || isset($row["act"]) && empty($row["act"]))
				continue;
			$this->insert($row["table"], $row["id"], $l["name"].$row["link"], $row["timeedit"], $row["changefreq"], $row["priority"]);
		}
	}

	/**
	 * Удаляет один или несколько элементов
	 *
	 * @param integer|array номер одного или нескольких элементов
	 * @param string $table_name таблица элемента
	 * @return void
	 */
	public function delete($element_ids, $table_name)
	{
		if(is_array($element_ids))
		{
			$where = " IN (%s)";
			$value = preg_replace('/[^0-9,]+/', '', implode(",", $element_ids));
		}
		else
		{
			$where = "=%d";
			$value = $element_ids;
		}
		DB::query("DELETE FROM {map_index} WHERE table_name='%s' AND element_id".$where, $table_name, $value);
	}

	/**
	 * Добавляет в базу данных элемент
	 *
	 * @param string $table таблица элемента
	 * @param integer номер элемента
	 * @param string $url адрес (без домена, ЧПУ на конце)
	 * @param integer $timeedit время редактирования
	 * @param string $changefreq частота редактирования
	 * @param float $priority приоритет
	 * @return void
	 */
	private function insert($table, $element, $url, $timeedit, $changefreq, $priority)
	{
		DB::query("INSERT INTO {map_index} (url, table_name, element_id, timeedit, changefreq, priority) VALUES ('%s', '%s', %d, %d, '%s', '%h')", $url, $table, $element, $timeedit, $changefreq, $priority);

		$this->current_insert++;
	}

	/**
	 * Записываем все языки сайта
	 * 
	 * @return void
	 */
	private function set_languages()
	{
		if(! empty($this->lang))
			return;

		foreach ($this->diafan->languages as $row)
		{
			$this->lang[] = array("name" => (! $row["base_site"] ? $row["shortname"].'/' : ''), "id" => $row["id"]);
		}
	}

	/**
	 * Записываем все ЧПУ
	 * 
	 * @return void
	 */
	private function set_rewrites($module_name = '', $rows = array())
	{
		$where = '';
		if($module_name)
		{
			$where = "module_name='site' OR module_name='".$module_name."'";
		}
		if($rows)
		{
			$ids = array();
			$cat_ids = array();
			$type = '';
			foreach ($rows as $row)
			{
				if(empty($row["type"]))
				{
					$row["type"] = 'element';
				}
				$ids[] = $row["id"];
				if($row["type"] == 'category' && $row["cat_id"])
				{
					$cat_ids[] = $row["cat_id"];
				}
				$type = $row["type"];
			}
			if($ids)
			{
				if($cat_ids)
				{
					$where .= ' AND ('.$type.'_id IN ('.implode(",", $ids).') OR cat_id IN ('.implode(",", $cat_ids).'))';
				}
				else
				{
					$where .= ' AND '.$type.'_id IN ('.implode(",", $ids).')';
				}
			}
		}
		if($where)
		{
			$where = " AND (".$where.")";
		}
		$rows = DB::query_fetch_all("SELECT * FROM {rewrite} WHERE trash='0'".$where);
		foreach ($rows as $row)
		{
			if($row["module_name"] == 'site')
			{
				$this->rewrites[$row["module_name"]][$row["site_id"]] = $row["rewrite"];
			}
			else
			{
				if(! empty($row["param_id"]))
				{
					$this->rewrites[$row["module_name"]]["params"][$row["param_id"]] = $row["rewrite"];
				}
				elseif(! empty($row["element_id"]))
				{
					$this->rewrites[$row["module_name"]]["elements"][$row["element_id"]] = $row["rewrite"];
				}
				else
				{
					$this->rewrites[$row["module_name"]]["cats"][$row["cat_id"]] = $row["rewrite"];
				}
			}
		}
	}

	/**
	 * Индексирует страницы модуля, прикрепленного к странице сайта
	 *
	 * @param array $row данные о странице сайта
	 * @return void
	 */
	private function index_site_module($row)
	{
		if ($row["module_name"] && file_exists(ABSOLUTE_PATH.'modules/'.$row["module_name"].'/'.$row["module_name"].'.sitemap.php'))
		{
			Customization::inc('modules/'.$row["module_name"].'/'.$row["module_name"].'.sitemap.php');
			$class_name = ucfirst($row["module_name"]).'_sitemap';
			$class = new $class_name($this->diafan);
			if(is_callable(array(&$class, 'config')))
			{
				$config = call_user_func_array(array(&$class, 'config'), array($row["id"]));
				if(is_array($config))
				{
					if(! empty($config["type"]))
					{
						if($this->last_site_id != $row["id"] || $this->last_table != $row["module_name"]."_category" && $this->last_table != $row["module_name"]."_param_select")
						{
							if(in_array('element', $config["type"]))
							{
								$where =  ! empty($config['where']['element']) ? $config['where']['element'] : '';
								$this->elements($row["module_name"], (! empty($config["table"]) ? $config["table"] : $row["module_name"]), $row["id"], $row["timeedit"], $row["changefreq"], $row["priority"], $where);
							}
						}
						if($this->last_site_id != $row["id"] || $this->last_table != $row["module_name"]."_param_select")
						{
							if(in_array('category', $config["type"]))
							{
								$where =  ! empty($config['where']['category']) ? $config['where']['category'] : '';
								$this->categories($row["module_name"], $row["id"], $row["timeedit"], $row["changefreq"], $row["priority"], $where);
							}
						}
						if(in_array('param', $config["type"]))
						{
							$where =  ! empty($config['where']['param']) ? $config['where']['param'] : '';
							$this->categories($row["module_name"], $row["id"], $row["timeedit"], $row["changefreq"], $row["priority"], $where);
						}
					}
				}
			}
		}
	}
	
	/**
	 * Индексирует категорий модулей
	 * 
	 * @param string $module название модуля
	 * @param integer $site_id страница сайта
	 * @param integer $timeedit время редактирования страницы сайта
	 * @param string $changefreq частота изменения страницы сайта
	 * @param float $priority приоритет страницы сайта
	 * @param string $where дополнительное условие отображения на сайте
	 * @return void
	 */
	private function categories($module, $site_id, $timeedit, $changefreq, $priority, $where)
	{
		if($this->last_table == $module."_category")
		{
			$where .= " AND id>".$this->diafan->configmodules($this->type."_current_index_module_element", "map");
		}

		$rows = DB::query_fetch_all("SELECT * FROM {".$module."_category} WHERE trash='0' ". $where." ORDER BY id ASC LIMIT ".$this->max_insert);
		foreach ($rows as $row)
		{
			$row["link"] = $this->get_url($module, $site_id, $row["id"], 0, 0);

			if(! empty($row["timeedit"]) && $row["timeedit"] > $timeedit)
			{
				$timeedit = $row["timeedit"];
			}
			if(! empty($row["changefreq"]))
			{
				$changefreq = $row["changefreq"];
			}
			if(! empty($row["priority"]))
			{
				$priority = $row["priority"];
			}
			foreach ($this->lang as $l)
			{
				if (isset($row["act".$l["id"]]) && ! $row["act".$l["id"]] || isset($row["act"]) && ! $row["act"])
					continue;
				$this->insert($module."_category", $row["id"], $l["name"].$row["link"], $timeedit, $changefreq, $priority);
			}
			$this->check_max($site_id, $module."_category", $row["id"]);
		}
	}
	
	/**
	 * Индексирует элементы модулей
	 * 
	 * @param string $module название модуля
	 * @param string $table название таблицы
	 * @param integer $site_id страница сайта
	 * @param integer $timeedit время редактирования страницы сайта
	 * @param string $changefreq частота изменения страницы сайта
	 * @param float $priority приоритет страницы сайта
	 * @param string $where дополнительное условие отображения на сайте
	 * @return void
	 */
	private function elements($module, $table, $site_id, $timeedit, $changefreq, $priority, $where)
	{
		$cat = $this->diafan->configmodules("cat", $module, $site_id);
		if($this->last_table == $table)
		{
			$where .= " AND id>".$this->diafan->configmodules($this->type."_current_index_module_element", "map");
		}

		$rows = DB::query_fetch_all("SELECT * FROM {".$table."} WHERE trash='0' ". $where." ORDER BY id ASC LIMIT ".$this->max_insert);
		foreach ($rows as  $row)
		{
			$row["link"] = $this->get_url($module, $site_id, ($cat ? $row["cat_id"] : 0), $row["id"], 0);

			if(! empty($row["timeedit"]) && $row["timeedit"] > $timeedit)
			{
				$timeedit = $row["timeedit"];
			}
			if(! empty($row["changefreq"]))
			{
				$changefreq = $row["changefreq"];
			}
			if(! empty($row["priority"]))
			{
				$priority = $row["priority"];
			}

			foreach ($this->lang as $l)
			{
				if (isset($row["act".$l["id"]]) && ! $row["act".$l["id"]] || isset($row["act"]) && ! $row["act"])
					continue;

				$this->insert($table, $row["id"], $l["name"].$row["link"], $timeedit, $changefreq, $priority);
			}
			$this->check_max($site_id, $module, $row["id"]);
			
		}
	}
	
	/**
	 * Индексирует дополнительные поля модуля, имеющие отдельные страницы
	 * 
	 * @param string $module название модуля
	 * @param integer $site_id страница сайта
	 * @param integer $timeedit время редактирования страницы сайта
	 * @param string $changefreq частота изменения страницы сайта
	 * @param float $priority приоритет страницы сайта
	 * @param string $where дополнительное условие отображения на сайте
	 * @return void
	 */
	public function params($module, $site_id, $timeedit, $changefreq, $priority, $where)
	{
		if($this->last_table == $table)
		{
			$where .= " AND id>".$this->diafan->configmodules($this->type."_current_index_module_element", "map");
		}

		$rows = DB::query_fetch_all("SELECT * FROM {".$module."_param_select} WHERE trash='0' ". $where." ORDER BY id ASC LIMIT ".$this->max_insert);
		foreach ($rows as  $row)
		{
			$row["link"] = $this->get_url($module, $site_id, 0, 0, $row["id"]);

			if(! empty($row["timeedit"]) && $row["timeedit"] > $timeedit)
			{
				$timeedit = $row["timeedit"];
			}

			foreach ($this->lang as $l)
			{
				if (isset($row["act".$l["id"]]) && ! $row["act".$l["id"]] || isset($row["act"]) && ! $row["act"])
					continue;

				$this->insert($module.'_param_select', $row["id"], $l["name"].$row["link"], $timeedit, $changefreq, $priority);
			}
			$this->check_max($site_id, $module.'_param_select', $row["id"]);
		}
	}
	
	/**
	 * Проверяет достижение максимума интексируемых ссылок за один проход скрипта
	 * 
	 * @param integer $site_id текущая страница сайта
	 * @param string $table текущая таблица
	 * @param string $element номер текущего проиндексированного элемента
	 * @return void
	 */
	private function check_max($site, $table, $element)
	{
		if($this->current_insert + 1 > $this->max_insert)
		{
			$this->diafan->configmodules($this->type."_current_index_module_table", "map", 0, _LANG, $table);
			$this->diafan->configmodules($this->type."_current_index_module_element", "map", 0, _LANG, $element);
			$this->diafan->configmodules($this->type."_current_index_site", "map", 0, _LANG, $site);
			if($this->type == 'all')
			{
				echo '<meta http-equiv="Refresh" content="0; url='.BASE_PATH.'sitemap.xml">';
			}
			else
			{
				echo '<meta http-equiv="Refresh" content="0; url='.URL.'">';
			}
			exit;
		}
	}
	
	/**
	 * Формирует ссылку
	 * 
	 * @param string $module модуль
	 * @param integer $site_id страница сайта
	 * @param integer $cat_id категория
	 * @param integer $element_id элемент
	 * @param integer $param_id номер параметра
	 * @return void
	 */
	private function get_url($module, $site_id, $cat_id, $element_id, $param_id)
	{
		if($module == 'site')
		{
			return ($this->rewrites["site"][$site_id] ? $this->rewrites["site"][$site_id].'ROUTE_END' : '');
		}
		if($element_id)
		{
			if(! empty($this->rewrites[$module]["elements"][$element_id]))
			{
				return $this->rewrites[$module]["elements"][$element_id].'ROUTE_END';
			}
			elseif($cat_id && ! empty($this->rewrites[$module]["cats"][$cat_id]))
			{
				return $this->rewrites[$module]["cats"][$cat_id].'/show'.$element_id.'/';
			}
			else
			{
				return ($this->rewrites["site"][$site_id] ? $this->rewrites["site"][$site_id].'/' : '').($cat_id ? 'cat'.$cat_id.'/' : '').'show'.$element_id.'/';
			}
		}
		if($param_id)
		{
			if(! empty($this->rewrites[$module]["params"][$param_id]))
			{
				return $this->rewrites[$module]["params"][$param_id].'ROUTE_END';
			}
			else
			{
				return ($this->rewrites["site"][$site_id] ? $this->rewrites["site"][$site_id].'/' : '').'param'.$param_id.'/';
			}
		}
		if($cat_id)
		{
			if(! empty($this->rewrites[$module]["cats"][$cat_id]))
			{
				return $this->rewrites[$module]["cats"][$cat_id].'ROUTE_END';
			}
			else
			{
				return ($this->rewrites["site"][$site_id] ? $this->rewrites["site"][$site_id].'/' : '').'cat'.$cat_id.'/';
			}
		}
	}
}