<?php
/**
 * Обработка Ajax-запросов при работе с меню в административной части
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Menu_admin_ajax
 */
class Menu_admin_ajax extends Frame_admin
{
	/**
	 * @var array результаты, передаваемы Ajaxом
	 */
	private $result;

	/**
	 * Вызывает обработку Ajax-запросов
	 * 
	 * @return void
	 */
	public function ajax()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_user->checked)
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}

		$this->result["hash"] = $this->diafan->_user->get_hash();

		if (! empty($_POST["action"]))
		{
			switch($_POST["action"])
			{
				case 'list_site_id':
					$this->list_site_id();
					break;

				case 'list_module':
					$this->list_module();
					break;

				case 'group_menu':
					$this->group_menu();
					break;
			}
		}
	}

	/**
	 * Подгружает список модулей
	 * 
	 * @return void
	 */
	private function list_site_id()
	{
		if (! $_POST["parent_id"])
		{
			$list = '<div class="menu_list menu_list_first"><h2>'.$this->diafan->_('Страницы сайта').'</h2>';
		}
		else
		{
			$list = '<div class="menu_list">';
		}
		
		$result = DB::query("SELECT id, [name], module_name, count_children FROM {site} WHERE [act]='1' AND trash='0' AND parent_id='%d' ORDER BY sort DESC", $_POST["parent_id"]);
		while ($row = DB::fetch_array($result))
		{
			$list .= '<p site_id="'.$row["id"].'" module_name="site">';
			if ($row["count_children"])
			{
				$list .= '<a href="#" class="plus menu_plus">+</a>';
			}
			else
			{
				$list .= '&nbsp;&nbsp;';
			}
			$list .= '&nbsp;<a href="'.BASE_PATH.$this->diafan->_route->link($row["id"]).'" class="menu_select">'.$row["name"].'</a>';
			if ($row["module_name"] && file_exists(ABSOLUTE_PATH.'modules/'.$row["module_name"].'/admin/'.$row["module_name"].'.admin.menu.php'))
			{
				Customization::inc('modules/'.$row["module_name"].'/admin/'.$row["module_name"].'.admin.menu.php');
				
				$class_name  = ucfirst($row["module_name"]).'_admin_menu';
				$class = new $class_name($this->diafan);
				$count = $class->count($row["id"]);
				if ($count)
				{
					$list .= ' <a href="#" class="menu_select_module plus" module_name="'.$row["module_name"].'">m</a>';
				}
			}
			$list .= '</p>';
		}
		DB::free_result($result);
		$list .= '</div>';

		$this->result["data"] = $list;

		$this->send_json();
	}

	/**
	 * Подгружает список ссылок для меню на элементы модуля
	 * 
	 * @return void
	 */
	private function list_module()
	{
		if (empty($_POST["module_name"]) || empty($_POST["site_id"]))
		{
			$this->result["error"] = 'ERROR';
			$this->send_json();
		}
		$module_name = $this->diafan->get_param($_POST, "module_name", "", 1);
		$parent_id   = $this->diafan->get_param($_POST, "parent_id", 0, 2);
		$site_id     = $this->diafan->get_param($_POST, "site_id", 0, 2);

		$list = '';
		if (! $parent_id)
		{
			$name = $this->diafan->_(DB::query_result("SELECT title FROM {modules} WHERE name='%h'", $module_name));
			if(! $name)
			{
				$name = $module_name;
			}
			$list .= '<h2>'.$name.'</h2>';
		}
		else
		{
			$list = '<div class="menu_list">';
		}

		if (file_exists(ABSOLUTE_PATH.'modules/'.$module_name.'/admin/'.$module_name.'.admin.menu.php'))
		{
			Customization::inc('modules/'.$module_name.'/admin/'.$module_name.'.admin.menu.php');
			$class_name  = ucfirst($module_name).'_admin_menu';
			$class = new $class_name($this->diafan);
			$rows = $class->list_($site_id, $parent_id);
			foreach ($rows as $row)
			{
				if (! empty($row["hr"]))
				{
					$list .= '<p><hr></p>';
					continue;
				}
				$list .= '<p site_id="'.$site_id.'" module_name="'.$module_name.'" cat_id="'.$row["cat_id"].'" element_id="'.$row["element_id"].'">';
				if ($row["count"])
				{
					$list .= '<a href="#" class="plus menu_plus">+</a>';
				}
				else
				{
					$list .= '&nbsp;&nbsp;';
				}
				$link = BASE_PATH.$this->diafan->_route->link($site_id, $module_name, $row["cat_id"], $row['element_id'], (! empty($row['param_id']) ? $row['param_id'] : ''));
				$list .= '&nbsp;<a href="'.$link.'" class="menu_select">'.($row["name"] ? $row["name"] : ($row["cat_id"] ? $row["cat_id"] : $row["element_id"])).'</a>';
				$list .= '</p>';
			}
		}
		if ($parent_id)
		{
			$list .= '</div>';
		}

		$this->result["data"] = $list;
		$this->send_json();
	}

	/**
	 * Сохраняет ссылки на элементы в меню при групповом выделение в списке
	 * 
	 * @return void
	 */
	private function group_menu()
	{
		Customization::inc('modules/menu/admin/menu.admin.inc.php');

		$prows = array();
		$rows = array();
		$_POST['ids'] = array_reverse($_POST['ids']);
		foreach($_POST['ids'] as $id)
		{
			$row = DB::query_fetch_array("SELECT * FROM {%h} WHERE id=%d LIMIT 1", $this->diafan->table, $id);
			if(! $row)
				continue;
			$module = str_replace('_category', '', $this->diafan->table);
			$save = array(
				"id" => $row["id"],
				"module" => $module, 
				"is_category" => strpos($this->diafan->table, '_category') !== false,
				"site_id" => ! empty($row["site_id"]) ? $row["site_id"] : 0,
				"is_new" => false,
				"parent_id" => ! empty($row["parent_id"]) ? $row["parent_id"] : '',
				"name" => htmlspecialchars_decode($row["name"._LANG]),
				"old_name" => $row["name"._LANG],
				"access" => $row["access"],
				"old_access" => $row["access"],
				"sort" => ! empty($row["sort"]) ? $row["sort"] : 0,
				"act" => $row["act"._LANG],
				"cat_id" => ! empty($row["cat_id"]) ? $row["cat_id"] : 0,
				"date_start" => ! empty($row["date_start"]) ? $row["date_start"] : 0,
				"old_date_start" => ! empty($row["date_start"]) ? $row["date_start"] : 0,
				"date_finish" => ! empty($row["date_finish"]) ? $row["date_finish"] : 0,
				"old_date_finish" => ! empty($row["date_finish"]) ? $row["date_finish"] : 0
			);
			$menu_cat_ids = ! empty($_POST["menu_cat_ids"]) ? $_POST["menu_cat_ids"] : array();
			$menu_inc = new Menu_admin_inc($this->diafan);
			$menu_inc->save_menu($save, $menu_cat_ids);
		}

		$this->result["redirect"] = URL;
		$this->send_json();
	}
	

	/**
	 * Отправляет результат
	 *
	 * @return boolean
	 */
	private function send_json()
	{
		if (! empty($this->result["errors"]) || ! empty($this->result["data"]) || ! empty($this->result["redirect"]))
		{
			include_once ABSOLUTE_PATH.'plugins/json.php';
			echo to_json($this->result);
			return true;
		}
		return false;
	}
}