<?php
/**
 * Редактирование пунктов меню
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Menu_admin
 */
class Menu_admin extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'menu';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'name' => array(
				'type' => 'text',
				'name' => 'Название пункта меню',
				'help' => 'Это название выводится как пункт меню на сайте.',
				'multilang' => true,
			),
			'act' => array(
				'type' => 'checkbox',
				'name' => 'Опубликовать на сайте',
				'help' => 'Показывать ли пункт меню на сайте или временно скрыть.',
				'default' => true,
				'multilang' => true,
			),
			'module_name' => array(
				'type' => 'function',
				'name' => 'Ссылка',
				'help' => 'Ссылка на страницу сайта.',
				
			),
			'attributes' => array(
				'type' => 'text',
				'name' => 'Атрибуты ссылки',
				'help' => 'HTML-код, выводимый в виде атрибутов для тега ссылки `<a></a>` (Веб-мастеру и программисту).',
			),
			'cat_id' => array(
				'type' => 'function',
				'name' => 'Меню',
				'help' => 'Выбор меню, к которому относится текущий пункт.',
			),
			'target_blank' => array(
				'type' => 'checkbox',
				'name' => 'Открывать в новом окне',
				'help' => 'Если отмечена, клик пользователя по пункту меню на сайте откроет ссылку в новом окне.',
			),
			'parent_id' => array(
				'type' => 'function',
				'name' => 'Вложенность: принадлежит',
				'help' => 'Перемещение текущего пункта меню и всех его подпунктов в принадлежность другому пункту меню (администратору сайта).'
			),
			'sort' => array(
				'type' => 'function',
				'name' => 'Сортировка: установить перед',
				'help' => 'Изменить положение текущего пункта меню среди других пунктов (администратору сайта).'
			),
			'access' => array(
				'type' => 'none',
				'name' => 'Доступ',
				'hide' => true,
			),
			'images' => array(
				'type' => 'module',
				'name' => 'Изображение',
				'help' => 'Назначить текущему пункту меню изображение.',
				'count' => 1,
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'del', // удалить
		'act', // показать/скрыть
		'parent', // содержит вложенности
		'trash', // использовать корзину
		'order', // сортируется
		'order_asc', // сортируется от старого к новому
		'element', // используются группы
		'category_flat', // категори не содержат вложенности
		'category_no_empty', // категория всегда выбрана
	);

	/**
	 * Подготавливает конфигурацию модуля
	 * @return void
	 */
	public function prepare_config()
	{
		if ($this->diafan->edit)
		{
			$this->diafan->variable_unset('access');
		}
	}

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить пункт меню');
	}

	/**
	 * Выводит списко ответов
	 * @return void
	 */
	public function show()
	{
		if (! $this->diafan->cat)
		{
			$this->diafan->cat = 1;
		}

		$this->diafan->list_row();
	}

	/**
	 * Редактирование поля "Ссылка"
	 * @return void
	 */
	public function edit_variable_module_name()
	{
		if (! $this->diafan->addnew)
		{
			if($this->diafan->values['othurl'])
			{
				$link = $this->diafan->values['othurl'];
			}
			else
			{
				if($this->diafan->values['module_name'] == 'site')
				{
					$link = BASE_PATH.$this->diafan->_route->link($this->diafan->values['site_id']);
				}
				else
				{
					$link = BASE_PATH.$this->diafan->_route->link($this->diafan->values['site_id'], $this->diafan->values['module_name'], $this->diafan->values['module_cat_id'], $this->diafan->values['element_id'], $this->diafan->values['param_id']);
				}
			}
			
			
		}
		echo '
		<script type="text/javascript" src="'.BASE_PATH.'modules/menu/admin/menu.admin.js"></script>
		<tr id="module_name">
			<td class="td_first">'.$this->diafan->variable_name().'</td>  
			<td><input type="text" size="40" name="'.$this->diafan->key.'" value="'.(! empty($link) ? $link : '').'">
			<a href="javascript:void(0)"><img src="'.BASE_PATH.'adm/img/edit.gif" width="12" height="14" alt="" title="'.$this->diafan->_('Редактировать').'" class="menu_check"></a>
			<div id="inline" class="hide"></div>
			<span class="help_img"><img title="Адрес ссылки, куда ведет текущий пункт меню. Заполняется автоматически при отметке галки «Показывать в меню» у элемента на сайте. Возможно указать вручную, как на внутреннюю страницу сайта, так и на любой другой сайт." src="'.BASE_PATH.'adm/img/quest.gif"></span>
			</td>
		</tr>';
	}

	/**
	 * Сохранение поля "Элемент модуля"
	 * @return void
	 */
	public function save_variable_module_name()
	{
		if(empty($_POST['module_name']))
		{
			return TRUE;
		}	    
		
		$link = preg_replace('/'.str_replace('/', '\\/', BASE_PATH).'/', '', $_POST['module_name']);
		$link = preg_replace('/^\//', '', $link);
		if(ROUTE_END == '/')
		{
			$link = preg_replace('/\/$/', '', $link);
		}
		if($row = $this->diafan->_route->search($link))	
		{
			$this->diafan->set_query("module_name='%s'");
			$this->diafan->set_value($row['module_name']);
			
			$this->diafan->set_query("module_cat_id=%d");
			$this->diafan->set_value($row['cat_id']);
			   
			$this->diafan->set_query("element_id=%d");
			$this->diafan->set_value($row['element_id']);
			   
			$this->diafan->set_query("param_id=%d");
			$this->diafan->set_value($row['param_id']);
			
			$this->diafan->set_query("site_id=%d");
			$this->diafan->set_value($row['site_id']);
			
			$this->diafan->set_query("othurl='%s'");
			$this->diafan->set_value('');
		}
		else
		{
			$this->diafan->set_query("othurl='%s'");
			$this->diafan->set_value($_POST['module_name']);
		}   
	}

	/**
	 * Сохранение поля "Доступ, период показа"
	 * @return void
	 */
	public function save_variable_access()
	{
		if ($_POST["element_id"])
		{
			$element = DB::query_fetch_array("SELECT access, date_start, date_finish FROM {%h} WHERE id=%d LIMIT 1", $_POST["module_name"], $_POST["element_id"]);
		}
		elseif ($_POST["module_cat_id"])
		{
			$element = DB::query_fetch_array("SELECT access FROM {%h_category} WHERE id=%d LIMIT 1", $_POST["module_name"], $_POST["module_cat_id"]);
		}
		else
		{
			$element = DB::query_fetch_array("SELECT access, date_start, date_finish FROM {site} WHERE id=%d LIMIT 1", $_POST["site_id"]);
		}

		$this->diafan->set_query("access='%d'");
		$this->diafan->set_value($element["access"]);
		if(! empty($element["date_start"]))
		{
			$this->diafan->set_query("date_start=%d");
			$this->diafan->set_value($element["date_start"]);
		}
		if(! empty($element["date_finish"]))
		{
			$this->diafan->set_query("date_finish=%d");
			$this->diafan->set_value($element["date_finish"]);
		}
	}

	/**
	 * Сохранение поля "Родитель"
	 * 
	 * @return void
	 */
	public function save_variable_parent_id()
	{
		if (!$this->diafan->savenew && $_POST["cat_id"] != $this->diafan->oldrow["cat_id"])
		{
			if ($_POST["parent_id"] && DB::query_result("SELECT cat_id FROM {menu} WHERE id=%d LIMIT 1", $_POST["parent_id"]) != $_POST["cat_id"])
			{
				$_POST["parent_id"] = 0;
			}
		}
		parent::__call('save_variable_parent_id', array());
	}

	/**
	 * Сохранение поля "Атрибуты ссылки"
	 * @return void
	 */
	public function save_variable_attributes()
	{
		$this->diafan->set_query("attributes='%s'");
		$this->diafan->set_value($_POST["attributes"]);
	}
}