<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * News
 */
class News extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat', 'year', 'month', 'day');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;
		
		if ($this->diafan->show)
		{
			$this->result = $this->model->id();
		}
		elseif (! $this->diafan->configmodules("cat") || $this->diafan->year || $this->diafan->month)
		{
			$this->result = $this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->result = $this->model->first_page();
		}
		else
		{
			$this->result = $this->model->list_category();
		}
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		$this->diafan->_tpl->get($this->view, 'news', $this->result);
	}

	/**
	 * Шаблонная функция: выводит последние новости на всех страницах, кроме страницы новостей, когда выводится список тех же новостей, что и в функции.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых новостей (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - категории новостей, если в настройках модуля отмечено «Использовать категории»
	 * sort - сортировка новостей: **date** – по дате (по умолчанию), **keywords** – новости, похожие по названию для текущей страницы
	 * images - количество изображений, прикрепленных к новости
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/news/views/news.view.show_block_**template**.php; по умолчанию шаблон modules/news/views/news.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'cat_id', 'images', 'images_variation', 'sort', 'site_id', 'template');
		
		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);
		$site_ids = explode(",", $attributes["site_id"]);
		$sort    = $attributes["sort"] == "date" || $attributes["sort"] == "keywords" ? $attributes["sort"] : "";
		$images   = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		$result = $this->model->show_block($count, $cat_ids, $site_ids, $images, $images_variation, $sort);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'news', $result))
		{
			$this->diafan->_tpl->get('show_block', 'news', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице новости выводит похожие новости. По умолчанию связи между новостями являются односторонними, это можно изменить, отметив опцию «В блоке похожих новостей связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых новостей (по умолчанию 3)
	 * images - количество изображений, прикрепленных к новости
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/news/views/news.view.show_block_rel_**template**.php; по умолчанию шаблон modules/news/views/news.view.show_block_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "news" || ! $this->diafan->show)
			return;

		$result = $this->model->show_block_rel($count, $images, $images_variation);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'news', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'news', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит календарь со ссылками на новости за период. Периоды отображаются в виде ссылок на месяцы, только если имеются новости, соответствующие этим периодам.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * site_id - страница, к которой прикреплен модуль, по умолчанию выбирается одна страница
	 * cat_id - категория новостей (id категории, по умолчанию учитываются все новости), если в настройках модуля отмечено «Использовать категории»
	 * only_news - выводить блок только на странице «Новости»
	 * detail - детализация (**day** – дни, **month** – месяца (по умолчанию), **year** – годы)
	 * template - шаблон тега (файл modules/news/views/news.view.show_calendar_**template**.php или modules/news/views/news.view.show_calendar_day_**template**.php для детализации по дням; по умолчанию шаблон modules/news/views/news.view.show_calendar.php или modules/news/views/news.view.show_calendar_day.php для детализации по дням)
	 * 
	 * @return void
	 */
	public function show_calendar($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_news', 'detail', 'site_id', 'cat_id', 'template');

		$cat_id  = $attributes["cat_id"];
		$site_id = $attributes["site_id"];
		$detail = in_array($attributes["detail"], array("day", "month", "year")) ? $attributes["detail"] : "month";
		$template = $attributes["template"];

		if ($attributes["only_news"] && ($site_id && $this->diafan->cid != $site_id || $this->diafan->module != "news"))
		{
			return false;
		}

		$result = $this->model->show_calendar($detail, $site_id, $cat_id, $template);
		if (! $result)
		{
			return;
		}

		if (! $attributes["template"] || ! $text = $this->diafan->_tpl->get('show_calendar_'.($detail == "day" ? '_day' : '').$attributes["template"], 'news', $result))
		{
			$text = $this->diafan->_tpl->get('show_calendar'.($detail == "day" ? '_day' : ''), 'news', $result);
		}
		if($detail == "day")
		{
			echo $text;
		}
	}
}