<?php
/**
 * Модель
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Photo_model
 */
class Photo_model extends Model
{
	/**
	 * Генерирует данные для списка всех фотографий без деления на категории
	 * 
	 * @return array
	 */
	public function list_()
	{
		if ($this->diafan->cat)
		{
			include ABSOLUTE_PATH.'includes/404.php';
		}
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		$cache_meta = array(
				"name"     => "list",
				"lang_id"  => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid,
				"role_id"  => $this->diafan->_user->role_id ? $this->diafan->_user->role_id : 0,
				"time"     => $time
			);

		//кеширование
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			////navigation//
			$this->diafan->_paginator->page    = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
			$this->diafan->_paginator->nen = $this->list_query_count($time);
			$links = $this->diafan->_paginator->get();
			////navigation///

			$result = $this->list_query($time);
			
			$this->result["rows"] = $this->get_elements($result);

			$this->result["paginator"] = $links;
			
			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}

		for($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->prepare_data_element($this->result["rows"][$i]);
		}
		for($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->format_data_element($this->result["rows"][$i]);
		}
		$this->theme_view();
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Получает из базы данных общее количество элементов, если не используются категории
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @return integer
	 */
	private function list_query_count($time)
	{
		$count = DB::query_result(
			"SELECT COUNT(DISTINCT e.id) FROM {photo} AS e"
			.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
			." WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
			." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
			." AND (e.access='0'"
			.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
			.")", 
			$this->diafan->cid, $time, $time
		);
		return $count;
	}

	/**
	 * Получает из базы данных элементы на одной странице, если не используются категории
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @return resource
	 */
	private function list_query($time)
	{
		$result = DB::query_range(
			"SELECT e.id, e.[name], e.[anons], e.timeedit, e.cat_id, e.site_id FROM {photo} AS e"
			.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
			." WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
			." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
			." AND (e.access='0'"
			.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
			.")"
			." GROUP BY e.id ORDER BY e.sort DESC, e.id DESC",
			$this->diafan->cid, $time, $time,
			$this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
		);
		return $result;
	}

	/**
	 * Генерирует данные для первой страницы фотогалереи
	 * 
	 * @return array
	 */
	public function first_page()
	{
		if ($this->diafan->page)
		{
			include ABSOLUTE_PATH.'includes/404.php';
		}
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "first_page",
				"lang_id" => _LANG,
				"site_id"  => $this->diafan->cid,
				"role_id" => $this->diafan->_user->role_id ? $this->diafan->_user->role_id : 0,
				"time"     => $time
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$this->result = array();

			$result = $this->first_page_cats_query();
			while ($row = DB::fetch_array($result))
			{
				$this->diafan->_route->prepare("photo", $row["id"], "cat");
				if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
				{
					$this->diafan->_images->prepare($row["id"], $this->diafan->module, 0, true);
				}
				$rows[] = $row;
			}
			DB::free_result($result);
			if(! empty($rows))
			{
				foreach($rows as $i => $row)
				{
					$cat = array();
					$cat["name"] = $row["name"];
					$cat["anons"] = $row["anons"];
					$cat["id"] = $row["id"];
	
					if (empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
					{
						$this->result["timeedit"] = $row["timeedit"];
					}
	
					$cat["children"] = $this->get_children_category($row["id"], $time);
	
					if ($this->diafan->configmodules("children_elements"))
					{
						$cat_ids = $this->diafan->get_children($row["id"], "photo_category");
						$cat_ids[] = $row["id"];
					}
					else
					{
						$cat_ids = array($row["id"]);
					}
	
					$cat["rows"] = array();
					if($this->diafan->configmodules("count_list"))
					{
						$result_elements = $this->first_page_elements_query($time, $cat_ids);
						$cat["rows"] = $this->get_elements($result_elements);
					}
	
					$cat["link_all"] = $this->diafan->_route->link($row["site_id"], 'photo', $row["id"]);
	
					if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
					{
						$cat["img"] =
						$this->diafan->_images->get(
								'medium', $row["id"], $this->diafan->module,
								$row["site_id"], $row["name"], 0, true,
								$this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
								$cat["link_all"]
							);
					}
					$this->result["categories"][] = $cat;
				}
			}

			//сохранение кеша
			$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
		}
		for($i = 0; $i < count($this->result["categories"]); $i++)
		{
			$this->prepare_data_category($this->result["categories"][$i]);
		}
		for($i = 0; $i < count($this->result["categories"]); $i++)
		{
			$this->format_data_category($this->result["categories"][$i]);
		}
		$this->theme_view_first_page();
		return $this->result;
	}

	/**
	 * Получает из базы данных категории верхнего уровня для первой странице модуля, если категории используются
	 * 
	 * @return resource
	 */
	private function first_page_cats_query()
	{
		$result = DB::query(
		"SELECT c.id, c.[name], c.[anons], c.timeedit, c.site_id FROM {photo_category} AS c"
		.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='photo'" : "")
		." WHERE c.[act]='1' AND c.parent_id=0 AND c.trash='0' AND c.site_id=%d"
		." AND (c.access='0'"
		.($this->diafan->_user->role_id ? " OR c.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		.")"
		." GROUP BY c.id ORDER by c.sort ASC, c.id ASC",
		$this->diafan->cid
		);
		return $result;
	}


	/**
	 * Получает из базы данных элементы для первой страницы модуля, если категории используются
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @param array $cat_ids номера категорий, элементы из которых выбираются
	 * @return resource
	 */
	private function first_page_elements_query($time, $cat_ids)
	{
		$result = DB::query_range(
		"SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id FROM {photo} AS e"
		." INNER JOIN {photo_category_rel} AS r ON e.id=r.element_id"
		.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
		." WHERE r.cat_id IN (%s) AND e.[act]='1' AND e.trash='0'"
		." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
		." AND (e.access='0'"
		.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		.")"
		." GROUP BY e.id ORDER BY e.sort DESC, e.id DESC",
		implode(',', $cat_ids), $time, $time,
		0, $this->diafan->configmodules("count_list")
		);
		return $result;
	}

	/**
	 * Генерирует данные для списка фотографий в категории
	 * 
	 * @return array
	 */
	public function list_category()
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "list",
				"cat_id"   => $this->diafan->cat,
				"lang_id" => _LANG,
				"page"     => $this->diafan->page > 1 ? $this->diafan->page : 1,
				"site_id"  => $this->diafan->cid,
				"role_id" => $this->diafan->_user->role_id ? $this->diafan->_user->role_id : 0,
				"time"     => $time
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$result = $this->list_category_query();

			if (! $row = DB::fetch_array($result))
			{
				include_once ABSOLUTE_PATH.'includes/404.php';
			}
			DB::free_result($result);
			if (empty($row) || (!empty($row['access']) && !$this->access(0, $row['id'])))
			{
				include_once(ABSOLUTE_PATH.'includes/403.php');
			}
			$this->result = $row;

			$this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'photo_category'));
			$this->result["path"] = $this->get_path('photo');

			if ($this->diafan->configmodules("images_cat"))
			{
				$this->diafan->_images->prepare($row["id"], $this->diafan->module, 0, true);
			}

			$this->result["children"] = $this->get_children_category($row["id"], $time);

			if ($this->diafan->configmodules("images_cat"))
			{
				$this->result["img"] = $this->diafan->_images->get(
						'medium', $row["id"], $this->diafan->module,
						$this->diafan->cid, $row["name"], 0, true, 0, 'large'
					);
			}

			if ($this->diafan->configmodules("children_elements"))
			{
				$cat_ids = $this->diafan->get_children($this->diafan->cat, "photo_category");
				$cat_ids[] = $this->diafan->cat;
			}
			else
			{
				$cat_ids = array($this->diafan->cat);
			}

			////navigation//
			$this->diafan->_paginator->page = $this->diafan->page;
			$this->diafan->_paginator->navlink = $this->diafan->_route->current_link(array("page", "dpage"));
			$this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("dpage", array("page" => "%d"));
			$this->diafan->_paginator->nen = $this->list_category_elements_query_count($time, $cat_ids);
			$this->result["paginator"] = $this->diafan->_paginator->get();
			////navigation///

			$result_elements = $this->list_category_elements_query($time, $cat_ids);
			$this->result["rows"] = $this->get_elements($result_elements);

			$this->meta_cat($row);
			$this->theme_view_cat($row);
			
			$this->list_category_previous_next($row["sort"], $row["parent_id"]);

			if($row["act"])
			{
				//сохранение кеша
				$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
			}
		}

		$this->prepare_data_category($this->result);
		$this->format_data_category($this->result);

		if($this->result["anons_plus"])
		{
			$this->result["text"] = $this->result["anons"].$this->result["text"];
		}

		$this->result["comments"] = $this->diafan->_comments->get(0, '', 0, true);

		if (! empty($this->result["previous"]["text"]))
		{
			$this->result["previous"]["text"] =
				$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'photo_category', _LANG);
		}
		if (! empty($this->result["next"]["text"]))
		{
			$this->result["next"]["text"] =
				$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'photo_category', _LANG);
		}
		foreach ($this->result["path"] as $k => $path)
		{
			if ($k == 0)
				continue;
			$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'photo_category', _LANG);
		}
		$this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		return $this->result;
	}

	/**
	 * Получает из базы данных данные о текущей категории для списка элементов в категории
	 * 
	 * @return resource
	 */
	private function list_category_query()
	{
		if($this->diafan->page > 1)
		{
			$fields = ", '' AS text";
		}
		else
		{
			$fields = ", [text]";
		}
		foreach($this->diafan->languages as $l)
		{
			$fields .= ', act'.$l["id"];
		}
		$result = DB::query("SELECT id, [name], [anons], [anons_plus] ".$fields.", timeedit, [descr], [keywords], sort, parent_id, [title_meta], access, theme, view, [act] FROM {photo_category}"
		." WHERE id=%d AND trash='0' AND site_id=%d"
		.(! $this->is_admin() ? " AND [act]='1'" : '')
		." ORDER BY sort ASC, id ASC",
		$this->diafan->cat, $this->diafan->cid);
		return $result;
	}

	/**
	 * Получает из базы данных количество элементов в категории
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @param array $cat_ids номера категорий, элементы из которых выбираются
	 * @return integer
	 */
	private function list_category_elements_query_count($time, $cat_ids)
	{
		$count = DB::query_result(
		"SELECT COUNT(DISTINCT e.id) FROM {photo} AS e"
		.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
		." INNER JOIN {photo_category_rel} AS r ON e.id=r.element_id AND r.cat_id IN (%s)"
		." WHERE e.[act]='1' AND e.trash='0'"
		." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
		." AND (e.access='0'"
		.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		.")",
		implode(',', $cat_ids), $time, $time
		);
		return $count;
	}

	/**
	 * Получает из базы данных элементы для списка элементов в категории
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @param array $cat_ids номера категорий, элементы из которых выбираются
	 * @return resource
	 */
	private function list_category_elements_query($time, $cat_ids)
	{
		$result = DB::query_range(
		"SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id FROM {photo} AS e"
		.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
		." INNER JOIN {photo_category_rel} AS r ON e.id=r.element_id AND r.cat_id IN (%s)"
		." WHERE e.[act]='1' AND e.trash='0'"
		." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
		." AND (e.access='0'"
		.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		.")"
		." GROUP BY e.id ORDER BY e.sort DESC, e.id DESC",
		implode(',', $cat_ids),  $time, $time,
		$this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
		);
		return $result;
	}

	/**
	 * Формирует ссылки на предыдущую и следующую категории
	 * 
	 * @param integer $sort номер для сортировки текущей категории
	 * @param integer $parent_id номер категории-родителя
	 * @return void
	 */
	private function list_category_previous_next($sort, $parent_id)
	{
		$previous = DB::query_fetch_array(
		"SELECT c.[name], c.id FROM {photo_category} AS c"
		. ($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='photo'" : "")
		. " WHERE c.[act]='1' AND c.trash='0' AND c.site_id=%d"
		. " AND (c.sort<%d OR c.sort=%d AND c.id<%d) AND c.parent_id=%d"
		. " AND (c.access='0'"
		. ($this->diafan->_user->role_id ? " OR c.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		. ")"
		. " ORDER BY c.sort DESC, c.id DESC LIMIT 1", $this->diafan->cid, $sort, $sort, $this->diafan->cat, $parent_id);
		if ($previous)
		{
			$this->result["previous"]["text"] = $previous["name"];
			$this->result["previous"]["id"]   = $previous["id"];
			$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "photo", $previous["id"]);
		}
		$next = DB::query_fetch_array(
		"SELECT c.[name], c.id FROM {photo_category} AS c"
		. ($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='photo'" : "")
		. " WHERE c.[act]='1' AND c.trash='0' AND c.site_id=%d"
		. " AND (c.sort>%d OR c.sort=%d AND c.id>%d) AND c.parent_id=%d"
		. " AND (c.access='0'"
		. ($this->diafan->_user->role_id ? " OR c.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		. ")"
		. " ORDER BY c.sort ASC, c.id ASC LIMIT 1", $this->diafan->cid, $sort, $sort, $this->diafan->cat, $parent_id);
		if ($next)
		{
			$this->result["next"]["text"] = $next["name"];
			$this->result["next"]["id"] = $next["id"];
			$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "photo", $next["id"]);
		}
	}

	/**
	 * Генерирует данные для страницы фотографии
	 * 
	 * @return array|boolean false
	 */
	public function id()
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "show",
				"cat_id"   => $this->diafan->cat,
				"show"     => $this->diafan->show,
				"lang_id" => _LANG,
				"site_id"  => $this->diafan->cid,
				"role_id" => $this->diafan->_user->role_id ? $this->diafan->_user->role_id : 0,
				"time"     => $time
			);
		if (! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
		{
			$row = $this->id_query($time);
			if (empty($row))
			{
				include ABSOLUTE_PATH.'includes/404.php';
				return false;
			}

			if (! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
			  || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
			{
				include ABSOLUTE_PATH.'includes/404.php';
				return false;
			}
			if (!empty($row['access']) && !$this->access($row['id']))
			{
				include ABSOLUTE_PATH . 'includes/403.php';
			}
			$this->result = $row;
			if(! $this->diafan->configmodules("cat"))
			{
				$this->result["cat_id"] = 0;
			}
			$this->diafan->cat = $this->result["cat_id"];

			$images = $this->diafan->_images->get(
					'large', $row["id"], $this->diafan->module,
					$this->diafan->cid, $row["name"], 0, false, 1
				);
			$this->result["img"]    = $images[0];

			$this->id_previous_next($row["sort"], $time);

			$this->meta($row);
			$this->theme_view_element($row);
			
			$this->result["path"] = $this->get_path('photo');

			if($row["act"])
			{
				//сохранение кеша
				$this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
			}
		}
		$this->diafan->cat = $this->result["cat_id"];

		$this->prepare_data_element($this->result);
		$this->format_data_element($this->result);

		if($this->result["anons_plus"])
		{
			$this->result["text"] = $this->result["anons"].$this->result["text"];
			$this->result["anons"] = '';
		}

		if (! empty($this->result["previous"]["text"]))
		{
			$this->result["previous"]["text"] =
				$this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'photo', _LANG);
		}
		if (! empty($this->result["next"]["text"]))
		{
			$this->result["next"]["text"] =
				$this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'photo', _LANG);
		}
		foreach ($this->result["path"] as $k => $path)
		{
			if ($k == 0)
				continue;
			$this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'photo_category', _LANG);
		}

		$this->counter_view();

		$this->result["comments"] = $this->diafan->_comments->get();

		if (! empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && ! empty($_POST["module_ajax"]) && $_POST["module_ajax"] == "photo")
		{
			header('Content-Type: text/html; charset=utf-8');
			$this->result["ajax"] = true;
			$this->diafan->_tpl->get('id', 'photo', $this->result);
			exit;
		}

		return $this->result;
	}

	/**
	 * Получает из базы данных данные о текущем элементе для страницы элемента
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @return array
	 */
	private function id_query($time)
	{
		$fields = '';
		foreach($this->diafan->languages as $l)
		{
			$fields .= ', act'.$l["id"];
		}
		$row = DB::query_fetch_array("SELECT id, [name], [anons], [anons_plus], [text], timeedit, cat_id, [keywords], [descr], sort,"
		." [title_meta], access, theme, view, [act]".$fields." FROM {photo}"
		." WHERE id=%d AND trash='0' AND site_id=%d"
		.(! $this->is_admin() ? " AND [act]='1' AND date_start<=%d AND (date_finish=0 OR date_finish>=%d)" : "")
		." LIMIT 1",
		$this->diafan->show, $this->diafan->cid, $time, $time);
		return $row;
	}

	/**
	 * Формирует ссылки на предыдущий и следующий элемент
	 * 
	 * @param integer $sort номер для сортировки текущего элемента
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @return void
	 */
	private function id_previous_next($sort, $time)
	{
		$previous = DB::query_fetch_array(
		"SELECT e.[name], e.id, e.cat_id FROM {photo} AS e"
		. ($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
		. " WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
		. ($this->diafan->configmodules("cat") ? " AND e.cat_id='".$this->diafan->cat."'" : '')
		. " AND (e.sort>%d OR e.sort=%d AND e.id>%d)"
		." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
		. " AND (e.access='0'"
		. ($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		. ")"
		. " ORDER BY e.sort ASC, e.id ASC LIMIT 1",
		$this->diafan->cid, $sort, $sort, $this->diafan->show, $time, $time
		);
		$next = DB::query_fetch_array(
		"SELECT e.[name], e.id, e.cat_id FROM {photo} AS e"
		. ($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
		. " WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
		. ($this->diafan->configmodules("cat") ? " AND e.cat_id='".$this->diafan->cat."'" : '')
		. " AND (e.sort<%d OR e.sort=%d AND e.id<%d)"
		." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
		. " AND (e.access='0'"
		. ($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		. ")"
		. " ORDER BY e.sort DESC, e.id DESC LIMIT 1",
		$this->diafan->cid, $sort, $sort, $this->diafan->show, $time, $time
		);
		if ($previous)
		{
			$this->result["previous"]["text"] = $previous["name"];
			if ( ! $this->diafan->configmodules("cat"))
			{
				$previous["cat_id"] = 0;
			}
			$this->result["previous"]["id"] = $previous["id"];
			$this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "photo", $previous["cat_id"], $previous["id"]);
		}
		if ($next)
		{
			$this->result["next"]["text"] = $next["name"];
			if ( ! $this->diafan->configmodules("cat"))
			{
				$next["cat_id"] = 0;
			}
			$this->result["next"]["id"] = $next["id"];
			$this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "photo", $next["cat_id"], $next["id"]);
		}
	}

	/**
	 * Генерирует данные для шаблонной функции: блок фотографий
	 *
	 * @param integer $count количество фотографий
	 * @param array $cat_ids категории
	 * @param array $site_ids страницы сайта
	 * @param string $sort сортировка date - по дате, rand - случайно
	 * @param string $images_variation размер изображений
	 * @return array
	 */
	public function show_block($count, $cat_ids, $site_ids, $sort, $images_variation)
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
				"name"     => "block",
				"cat_ids" => $cat_ids,
				"site_ids" => $site_ids,
				"count"    => $count,
				"lang_id" => _LANG,
				"images_variation" => $images_variation,
				"role_id" => $this->diafan->_user->role_id ? $this->diafan->_user->role_id : 0,
				"time"     => $time
			);

		if ($sort == "rand" || ! $this->result = $this->diafan->_cache->get($cache_meta, "photo"))
		{
			$minus = array();
			$one_cat_id = count($cat_ids) == 1 ? $cat_ids[0] : false;
			if(! $this->validate_attribute_site_cat('photo', $site_ids, $cat_ids, $minus))
			{
				return false;
			}
			$inner = "";
			$where = '';
			if($cat_ids)
			{
				$inner = " INNER JOIN {photo_category_rel} as r ON r.element_id=e.id"
				." AND r.cat_id IN (" . implode(',', $cat_ids) . ")";
			}
			elseif(! empty($minus["cat_ids"]))
			{
				$inner = " INNER JOIN {photo_category_rel} as r ON r.element_id=e.id"
				." AND r.cat_id NOT IN (" . implode(',', $minus["cat_ids"]) . ")";
			}
			if($site_ids)
			{
				$where .= " AND e.site_id IN (".implode(",", $site_ids).")";
			}
			elseif(! empty($minus["site_ids"]))
			{
				$where .= " AND e.site_id NOT IN (".implode(",", $minus["site_ids"]).")";	
			}

			if ($sort == "rand")
			{
				$max_count = DB::query_result(
				"SELECT COUNT(DISTINCT e.id) FROM {photo} as e"
				.$inner
				.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
				." WHERE e.[act]='1' AND e.trash='0'"
				.$where
				." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
				.($this->diafan->module == 'photo' && $this->diafan->show ? " AND e.id<>".$this->diafan->show : '')
				." AND (e.access='0'"
				.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
				.")", $time, $time
				);
				$rands = array();
				for ($i = 1; $i <= min($max_count, $count); $i++)
				{
					do
					{
						$rand = mt_rand(0, $max_count - 1);
					}
					while (in_array($rand, $rands));
					$rands[] = $rand;
				}
			}
			else
			{
				$rands[0] = 1;
			}
			$this->result["rows"] = array();

			switch($sort)
			{
				case 'date':
					$order = ' ORDER BY e.id DESC';
					break;

				case 'rand':
					$order = '';
					break;

				default:
					$order = ' ORDER BY e.sort DESC';
			}

			foreach ($rands as $rand)
			{
				$result = DB::query_range(
				"SELECT e.id, e.[name],e.[anons], e.cat_id, e.timeedit, e.site_id FROM {photo} AS e"
				.$inner
				.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
				." WHERE e.[act]='1' AND e.trash='0'"
				.($this->diafan->module == 'photo' && $this->diafan->show ? " AND e.id<>".$this->diafan->show : '')
				.$where
				." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
				." AND (e.access='0'"
				.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
				.")"
				." GROUP BY e.id"
				.$order,
				$time, $time,
				$sort == "rand" ? $rand : 0,
				$sort == "rand" ? 1     : $count
				);
				$this->result["rows"] = array_merge($this->result["rows"], $this->get_elements($result, $images_variation));
			}

			// если категория только одна, задаем ссылку на нее
			if (!empty($this->result["rows"]) && count($cat_ids) == 1)
			{
				$cat = DB::query_fetch_array("SELECT [name], site_id, id FROM {photo_category} WHERE id=%d LIMIT 1", $cat_ids[0]);
				
				$this->result["name"] = $cat["name"];
				$this->result["link_all"] = $this->diafan->_route->link($cat["site_id"], 'photo', $cat["id"]);
				$this->result["category"] = true;
			}
			// если раздел сайта только один, то задаем ссылку на него
			elseif (!empty($this->result["rows"]) && count($site_ids) == 1)
			{
				$this->result["name"] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d LIMIT 1", $site_ids[0]);
				$this->result["link_all"] = $this->diafan->_route->link($site_ids[0]);
				$this->result["category"] = false;
			}
			//сохранение кеша
			if ($sort != "rand")
			{
				$this->diafan->_cache->save($this->result, $cache_meta, "photo");
			}
		}

		for($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->prepare_data_element($this->result["rows"][$i]);
		}
		for($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->format_data_element($this->result["rows"][$i]);
		}

		return $this->result;
	}

	/**
	 * Генерирует данные для шаблонной функции: блок связанных фотографий
	 * 
	 * @param integer $count количество фотографий
	 * @param string $images_variation размер изображений
	 * @return array
	 */
	public function show_block_rel($count, $images_variation)
	{
		$time = mktime(23, 59, 0, date("m"), date("d"), date("Y"));

		//кеширование
		$cache_meta = array(
			"name" => "block_rel",
			"count" => $count,
			"lang_id" => _LANG,
			"element_id" => $this->diafan->show,
			"images_variation" => $images_variation,
			"role_id" => $this->diafan->_user->role_id ? $this->diafan->_user->role_id : 0,
			"time" => $time
		);

		if (! $this->result = $this->diafan->_cache->get($cache_meta, "photo"))
		{
			$this->result["rows"] = array();

			$result = DB::query_range(
			"SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id FROM {photo} AS e"
			. " INNER JOIN {photo_rel} AS r ON e.id=r.rel_element_id AND r.element_id=%d"
			.($this->diafan->configmodules("rel_two_sided") ? " OR e.id=r.element_id AND r.rel_element_id=".$this->diafan->show : '')
			. ($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
			. " WHERE e.[act]='1' AND e.trash='0'"
			." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
			. " AND (e.access='0'"
			. ($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
			. ")"
			. " GROUP BY e.id"
			. ' ORDER BY e.sort DESC',
			$this->diafan->show, $time, $time, 0, $count
			);
			$this->result["rows"] = $this->get_elements($result, $images_variation);
			$this->diafan->_cache->save($this->result, $cache_meta, "photo");
		}

		for($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->prepare_data_element($this->result["rows"][$i]);
		}
		for($i = 0; $i < count($this->result["rows"]); $i++)
		{
			$this->format_data_element($this->result["rows"][$i]);
		}
		return $this->result;
	}

	/**
	 * Форматирует данные о фотографии для списка фотографий
	 *
	 * @param resource $result результат выполнения SQL-запроса
	 * @param string $images_variation размер изображений
	 * @return array
	 */
	public function get_elements($result, $images_variation = 'medium')
	{
		if (empty($this->result["timeedit"]))
		{
			$this->result["timeedit"] = '';
		}
		$rows = array();
		while ($row = DB::fetch_array($result))
		{
			if ($this->diafan->configmodules('page_show', 'photo', $row["site_id"]))
			{
				$this->diafan->_route->prepare("photo", $row["id"]);
			}
			$this->diafan->_images->prepare($row["id"], "photo");
			$rows[] = $row;
		}
		DB::free_result($result);
		if($rows)
		{
			foreach($rows as $i => $row)
			{
				if (! $this->diafan->configmodules("cat", "photo", $row["site_id"]))
				{
					$row["cat_id"] = 0;
				}
				if ($row["timeedit"] < $this->result["timeedit"])
				{
					$this->result["timeedit"] = $row["timeedit"];
				}
				unset($row["timeedit"]);
				if ($this->diafan->configmodules('page_show', 'photo', $row["site_id"]))
				{
					$row["link"] = $this->diafan->_route->link($row["site_id"], "photo", $row["cat_id"], $row["id"]);
				}
				else
				{
					$row["link"] = 'large';
				}
				$images  = $this->diafan->_images->get(
						$images_variation, $row["id"], 'photo',
						$row["site_id"], $row["name"], 0, false,
						1,
						$row["link"]
					);

				unset($row["cat_id"]);
	
				$row["img"] = $images ? $images[0] : '';
	
				if (! $this->diafan->configmodules('page_show', 'photo', $row["site_id"]))
				{
					$row["link"] = '';
				}

				$rows[$i] = $row;
			}
		}
		return $rows;
	}
	
	/**
	 * Формирует данные о вложенных категориях
	 *
	 * @param integer $parent_id номер категории-родителя
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @return array
	 */
	private function get_children_category($parent_id, $time)
	{
		$children = array();
		$result_children = DB::query(
		"SELECT c.id, c.[name], c.[anons], c.site_id FROM {photo_category} AS c"
		.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='photo'" : "")
		." WHERE c.[act]='1' AND c.parent_id=%d AND c.trash='0' AND c.site_id=%d"
		." AND (c.access='0'"
		.($this->diafan->_user->role_id ? " OR c.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		.")"
		." GROUP BY c.id ORDER BY c.sort ASC, c.id ASC", $parent_id, $this->diafan->cid
		);

		while ($child = DB::fetch_array($result_children))
		{
			if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
			{
				$this->diafan->_images->prepare($child["id"], "photo", 0, true);
			}
			$this->diafan->_route->prepare("photo", $child["id"], "cat");
			$children[] = $child;
		}
		DB::free_result($result_children);
		foreach($children as $i => $child)
		{
			$child["link"] = $this->diafan->_route->link($child["site_id"], 'photo', $child["id"]);
			if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
			{
				$child["img"] = $this->diafan->_images->get(
						'medium', $child["id"], $this->diafan->module, $child["site_id"],
						$child["name"], 0, true, $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0,
						$child["link"]);
			}
			$child["rows"] = array();
			if($this->diafan->configmodules("count_child_list"))
			{
				if ($this->diafan->configmodules("children_elements"))
				{
					$cat_ids = $this->diafan->get_children($child["id"], "photo_category");
					$cat_ids[] = $child["id"];
				}
				else
				{
					$cat_ids = array($child["id"]);
				}
				$result_elements = $this->get_children_category_elements_query($time, $cat_ids);
				$child["rows"] = $this->get_elements($result_elements);
			}
			unset($child["site_id"]);
			$children[$i] = $child;
		}
		return $children;
	}

	/**
	 * Получает из базы данных элементы вложенных категорий
	 * 
	 * @param integer $time текущее время, округленное до минут, в формате UNIX
	 * @param array $cat_ids номера категорий, элементы из которых выбираются
	 * @return resource
	 */
	private function get_children_category_elements_query($time, $cat_ids)
	{
		$result = DB::query_range(
		"SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id FROM {photo} AS e"
		." INNER JOIN {photo_category_rel} AS r ON e.id=r.element_id"
		.($this->diafan->_user->role_id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='photo'" : "")
		." WHERE r.cat_id IN (%s) AND e.[act]='1' AND e.trash='0'"
		." AND e.date_start<=%d AND (e.date_finish=0 OR e.date_finish>=%d)"
		." AND (e.access='0'"
		.($this->diafan->_user->role_id ? " OR e.access='1' AND a.role_id=".$this->diafan->_user->role_id : '')
		.")"
		." GROUP BY e.id ORDER BY e.sort DESC, e.id DESC",
		implode(',', $cat_ids), $time, $time,
		0, $this->diafan->configmodules("count_child_list")
		);
		return $result;
	}

	/**
	 * Подготовка к форматированию данных о категории для шаблона вне зоны кэша
	 *
	 * @return void
	 */
	private function prepare_data_category(&$row)
	{
		$this->diafan->_tags->prepare($row["id"], 'photo', true);
		if(! empty($row["children"]))
		{
			for($i = 0; $i < count($row["children"]); $i++)
			{
				$this->prepare_data_category($row["children"][$i]);
			}
		}
		if(! empty($row["rows"]))
		{
			for($i = 0; $i < count($row["rows"]); $i++)
			{
				$this->prepare_data_element($row["rows"][$i]);
			}
		}
	}

	/**
	 * Форматирование данных о категории для шаблона вне зоны кэша
	 *
	 * @return void
	 */
	private function format_data_category(&$row)
	{
		$row["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'photo_category', _LANG);
		if(! empty($row["anons"]))
		{
			$row["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'photo_category', _LANG);
		}
		if(! empty($row["text"]))
		{
			$row["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'photo_category', _LANG);
		}
		$row["rating"] = $this->diafan->_rating->get($row["id"], 'photo', ! empty($row["site_id"]) ? $row["site_id"] : 0, true);
		if(! empty($row["children"]))
		{
			for($i = 0; $i < count($row["children"]); $i++)
			{
				$this->format_data_category($row["children"][$i]);
			}
		}
		if(! empty($row["rows"]))
		{
			for($i = 0; $i < count($row["rows"]); $i++)
			{
				$this->format_data_element($row["rows"][$i]);
			}
		}
	}

	/**
	 * Подготовка к форматированию данных о элементе для шаблона вне зоны кэша
	 *
	 * @return void
	 */
	private function prepare_data_element(&$row)
	{
		$this->diafan->_tags->prepare($row["id"], 'photo');
		$this->diafan->_rating->prepare($row["id"], 'photo');
	}

	/**
	 * Форматирование данных о элементе для шаблона вне зоны кэша
	 *
	 * @return void
	 */
	private function format_data_element(&$row)
	{
		if (! empty($row["name"]))
		{
			$row["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'photo', _LANG);
		}
		if (! empty($row["text"]))
		{
			$row["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'photo', _LANG);
		}
		if (! empty($row["anons"]))
		{
			$row["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'photo', _LANG);
		}

		$row["tags"] =  $this->diafan->_tags->get($row["id"], 'photo', ! empty($row["site_id"]) ? $row["site_id"] : 0);
		$row["rating"] = $this->diafan->_rating->get($row["id"], 'photo', ! empty($row["site_id"]) ? $row["site_id"] : 0);
	}
}