<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Photo
 */
class Photo extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'cat');
		if ($this->diafan->configmodules('page_show'))
		{
			$this->rewrite_variable_names[] = 'show';
		}
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;

		if ($this->diafan->show && $this->diafan->configmodules('page_show'))
		{
			$this->result = $this->model->id();
		}
		elseif (! $this->diafan->configmodules("cat"))
		{
			$this->result = $this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->result = $this->model->first_page();
		}
		else
		{
			$this->result = $this->model->list_category();
		}
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		$this->diafan->_tpl->get($this->view, 'photo', $this->result);
	}

	/**
	 * Шаблонная функция: выводит несколько фотографий.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых фотографий (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - альбомы фотографий, если в настройках модуля отмечено «Использовать альбомы». Идентификаторы альбомов перечисляются через запятую. По умолчанию альбом не учитывается, выводятся все фотографии
	 * sort - сортировка фотографий: по умолчанию как на странице модуля, **date** – по дате, **rand** – в случайном порядке
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/photo/views/photo.view.show_block_**template**.php; по умолчанию шаблон modules/photo/views/photo.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'cat_id', 'sort', 'site_id', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);
		$site_ids = explode(",", $attributes["site_id"]);
		$sort    = $attributes["sort"] == "date" || $attributes["sort"] == "rand" ? $attributes["sort"] : "";
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		$result = $this->model->show_block($count, $cat_ids, $site_ids, $sort, $images_variation);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'photo', $result))
		{
			$this->diafan->_tpl->get('show_block', 'photo', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице фотографии выводит похожие фотографии. По умолчанию связи между фотографиями являются односторонними, это можно изменить, отметив опцию «В блоке похожих фотографий связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых фотографий (по умолчанию 3)
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/photo/views/photo.view.show_block_rel_**template**.php; по умолчанию шаблон modules/photo/views/photo.view.show_block_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "photo" || ! $this->diafan->show)
			return;

		$result = $this->model->show_block_rel($count, $images_variation);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'photo', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'photo', $result);
		}
	}
}