<?php
/**
 * Обработка запроса при регистрации пользователя
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Registration_ajax
 */
class Registration_ajax extends Ajax
{
	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (empty($_POST['module']) || $_POST['module'] != 'registration' || empty($_POST["action"]) || $this->diafan->_user->id)
		{
			return false;
		}

		$this->module = 'users';
		$this->tag = 'registration';

		switch($_POST["action"])
		{
			case 'addnew':
				return $this->addnew();

			case 'fast_validate':
				return $this->fast_validate();

			case 'upload_image':
				return $this->upload_image();

			case 'delete_image':
				return $this->delete_image();
		}
		return false;
	}

	/*
	 * Добавляет нового пользователя
	 * 
	 * @return boolean
	 */
	private function addnew()
	{
		$this->check_captcha();
		$this->validate();

		$role_id = $this->get_role_id();
		$where_param_role_rel = $this->get_where_param_role_rel($role_id);

		$params = $this->get_params(array("module" => "users", "where" => "show_in_form_no_auth='1'".$where_param_role_rel));

		$this->empty_required_field(array("params" => $params));

		if ($this->send_errors())
			return true;

		if($this->diafan->configmodules("mail_as_login", "users"))
		{
			list($_POST["name"],) = explode('@', $_POST["mail"]);
			while(DB::query_result("SELECT name FROM {users} WHERE name='%h' AND trash='0'",  $_POST["name"]))
			{
				$_POST["name"] = $_POST["name"].mt_rand(1, 99999);
			}
		}

		$save_id = DB::query("INSERT INTO {users} (name, password, mail, phone, created, lang_id, fio, act, role_id)"
			. " VALUES ('%h', '%h', '%h', '%h', %d, %d, '%h', '%d', %d)",
			$_POST["name"], encrypt($_POST["password"]), $_POST["mail"], $_POST["phone"], time(),
			_LANG, $_POST["fio"], $this->diafan->configmodules("act", "users") ? 0 : 1,
			$role_id
		);

		$this->insert_values(array("id" => $save_id, "table" => "users", "params" => $params));

		if(! empty($_POST["tmpcode"]))
		{
			DB::query("UPDATE {images} SET element_id=%d, tmpcode='' WHERE module_name='users' AND element_id=0 AND tmpcode='%s'", $save_id, $_POST["tmpcode"]);
		}

		if ($this->send_errors())
			return true;
		
		if(in_array('subscribtion', $this->diafan->installed_modules))
		{
			if(! empty($_POST['subscribe']) || ! $this->diafan->configmodules('subscribe_in_registration', 'subscribtion'))
			{
				$email_id = DB::query_result("SELECT id FROM {subscribtion_emails} WHERE mail='%s' LIMIT 1", $_POST['mail']);
				if($email_id)
				{
					DB::query("UPDATE {subscribtion_emails} SET act='1', trash='0' WHERE id=%d LIMIT 1", $email_id);
				}
				else
				{
					$code = md5(rand(111, 99999));
					DB::query("INSERT INTO {subscribtion_emails} (created, mail, name, code, act) VALUES (%d, '%s', '%h', '%s', '1')", time(), $_POST['mail'], $_POST["fio"], $code);
				}
			}
			if($_POST["phone"])
			{
				$phone = preg_replace('/[^0-9]+/', '', $_POST["phone"]);
				if(! DB::query_result("SELECT id FROM {subscribtion_phones} WHERE phone='%s' AND trash='0'", $phone))
				{
					DB::query("INSERT INTO {subscribtion_phones} (phone, created, name, act) VALUES ('%s', %d, '%h', '1')", $phone, $_POST["fio"], time());
				}
			}
		}

		$this->send_mails($save_id);

		$this->upload_avatar();

		if (! $this->diafan->configmodules("act", "users"))
		{
			$this->diafan->_user->id = $save_id;
			if ($_POST["url"])
			{
				$this->result["redirect"] = $_POST["url"];
				return $this->send_errors();
			}
		}
		if($this->diafan->configmodules("hide_register_form", "users"))
		{
			$this->result["form_hide"] = true;
		}

		$this->result["data"] = $this->diafan->configmodules('mes', "users");
		$this->result["target"] = ".registration_message";
		$this->result["success"] = true;

		return $this->send_errors();
	}

	/**
	 * Валидация данных "на лету"
	 * 
	 * @return void
	 */
	private function fast_validate()
	{
		Customization::inc('includes/validate.php');
		header('Content-Type: text/html; charset=utf-8');
		switch($_POST["name"])
		{
			case "name":
				if(! $this->diafan->configmodules("mail_as_login", "users") && $mes = Validate::login($_POST["value"]))
				{
					echo $this->diafan->_($mes, false);
				}
				break;
			case "mail":
				if(! $mes = Validate::mail($_POST["value"]))
				{
					$mes = Validate::mail_user($_POST["value"]);
				}
				if($mes)
				{
					echo $this->diafan->_($mes, false);
				}
				break;
			case "password":
				if($mes = Validate::password($_POST["value"], true))
				{
					echo $this->diafan->_($mes, false);
				}
				break;
			case "password2":
				if($_POST["value"] != $_POST["value2"])
				{
					echo $this->diafan->_('Пароли не совпадают', false);
				}
				break;
		}
		return true;
	}

	/**
	 * Загружает изображение
	 *
	 * @return boolean
	 */
	private function upload_image()
	{
		if(empty($_POST["tmpcode"]))
		{
			return false;
		}
		$prefix = '';
		if(! empty($_POST["images_prefix"]))
		{
			$prefix = $this->diafan->get_param($_POST, "images_prefix", '', 1);
		}
		if(empty($_POST["images_param_id"]))
		{
			return false;
		}
		$param_id = $this->diafan->get_param($_POST, "images_param_id", '', 2);
		$this->result["success"] = true;
		if (! empty($_FILES[$prefix.'images'.$param_id]) && $_FILES[$prefix.'images'.$param_id]['tmp_name'] != '' && $_FILES[$prefix.'images'.$param_id]['name'] != '')
		{
			try
			{
				$this->diafan->_images->upload(0, "users", 0, $_FILES[$prefix.'images'.$param_id]['tmp_name'], $_FILES[$prefix.'images'.$param_id]['name'], false, false, $param_id, $_POST["tmpcode"]);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = $prefix.'p'.$param_id;
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
			$images = $this->diafan->_images->get('large', 0, "users", 0, '', $param_id, false, 0, '', $_POST["tmpcode"]);
			$this->result["data"] = $this->diafan->_tpl->get('images', "users", $images);
		}
		return $this->send_errors();
	}

	/**
	 * Удаляет изображение
	 *
	 * @return boolean
	 */
	private function delete_image()
	{
		if(empty($_POST["id"]))
		{
			return false;
		}
		if(empty($_POST["tmpcode"]))
		{
			return false;
		}
		$row = DB::query_fetch_array("SELECT * FROM {images} WHERE module_name='users' AND id=%d AND tmpcode='%s'", $_POST["id"], $_POST["tmpcode"]);
		if(! $row)
		{
			return false;
		}
		$this->diafan->_images->delete(0, "users", $row["id"]);
		return true;
	}

	/**
	 * Проверяет валидность введенных при регистрации данных
	 * 
	 * @return void
	 */
	private function validate()
	{
		Customization::inc('includes/validate.php');
		if(! $this->diafan->configmodules("mail_as_login", "users"))
		{
			$mes = Validate::login($_POST["name"]);
			if ($mes)
			{
				$this->result["errors"]["name"] = $this->diafan->_($mes, false);
			}
		}
		$mes = Validate::mail($_POST["mail"]);
		if ($mes)
		{
			$this->result["errors"]["mail"] = $this->diafan->_($mes, false);
		}
		else
		{
			$mes = Validate::mail_user($_POST["mail"]);
			if ($mes)
			{
				$this->result["errors"]["mail"] = $this->diafan->_($mes, false);
			}
		}
		if (! empty($_POST["phone"]))
		{
			$mes = Validate::phone($_POST["phone"]);
			if ($mes)
			{
				$this->result["errors"]["phone"] = $this->diafan->_($mes);
			}
		}
		$mes = Validate::password($_POST["password"]);
		if ($mes)
		{
			$this->result["errors"]["password"] = $this->diafan->_($mes, false);
		}
		elseif ($_POST["password"] != $_POST["password2"])
		{
			$this->result["errors"]["password"] = $this->diafan->_('Пароли не совпадают', false);
		}

		if (!$_POST["fio"])
		{
			$this->result["errors"]["fio"] = $this->diafan->_('Заполните поле ФИО или название компании', false);
		}
	}

	/**
	 * Проверяет валидность заполнения роли пользователя, определяет роль для нового пользователя
	 * 
	 * @return boolean
	 */
	private function get_role_id()
	{
		$roles = array();
		$result = DB::query("SELECT id FROM {users_role} WHERE registration='1' AND trash='0'");
		while ($row = DB::fetch_array($result))
		{
			$roles[] = $row["id"];
		}
		DB::free_result($result);
		if(! count($roles))
		{
			return 0;
		}
		if(count($roles) == 1)
		{
			return $roles[0];
		}

		if (empty($_POST["role_id"]) || !in_array($_POST["role_id"], $roles))
		{
			$this->result["errors"]["role_id"] = 'ERROR_ROLE_ID';
		}

		return $_POST["role_id"];
	}

	/**
	 * Получает условие для SQL-запроса: выбор полей с учетом роли пользователя
	 *
	 * @param integer $role_id номер роли пользователя
	 * @return string
	 */
	private function get_where_param_role_rel($role_id)
	{
		$param_ids = array();
		$param_role_rels = array();
		$result = DB::query("SELECT role_id, element_id FROM {users_param_role_rel} WHERE trash='0' AND role_id>0");
		while ($row = DB::fetch_array($result))
		{
			$param_role_rels[$row["element_id"]][] = $row["role_id"];
		}
		DB::free_result($result);
		foreach($param_role_rels as $param_id => $roles)
		{
			if(! in_array($role_id, $roles))
			{
				$param_ids[] = $param_id;
			}
		}
		if($param_ids)
		{
			return " AND id NOT IN (".implode(",", $param_ids).")";
		}
		return '';
	}

	/**
	 * Отправляет письмо новому пользователю и администратору сайта
	 * 
	 * @return boolean true
	 */
	private function send_mails($save)
	{
		include_once ABSOLUTE_PATH.'includes/mail.php';
		if($this->diafan->configmodules("mail_as_login", "users"))
		{
			$login = $this->diafan->get_param($_POST, "mail", '', 1);
		}
		else
		{
			$login = $this->diafan->get_param($_POST, "name", '', 1);
		}
		if ($this->diafan->configmodules("sendmailadmin", "users"))
		{
			$subject = str_replace(
					array('%title', '%url'), array(TITLE, BASE_URL), $this->diafan->configmodules('subject_admin', "users")
			);
			$message = str_replace(
					array('%login', '%title', '%url', '%fio', '%email', '%params'), array(
						$login,
						TITLE,
						BASE_URL,
						$this->diafan->get_param($_POST, "fio", '', 1),
						$this->diafan->get_param($_POST, "mail", '', 1),
						$this->message_admin_param,
					), $this->diafan->configmodules('message_admin', "users")
			);

			if ($message && $subject)
			{
				send_mail(
						$this->diafan->configmodules("emailconfadmin", "users") ? $this->diafan->configmodules("email_admin", "users") : EMAIL_CONFIG, $subject, $message, $this->diafan->configmodules("emailconf", "users") ? $this->diafan->configmodules("email", "users") : ''
				);
			}
		}

		//send mail user
		$subject = str_replace(
			array('%title', '%url'),
			array(TITLE, BASE_URL),
			$this->diafan->configmodules('subject', "users")
		);

		$actlink = '';
		if ($this->diafan->configmodules("act", "users") == 1)
		{
			$actcode = md5(rand(111, 99999));
			DB::query("INSERT INTO {users_actlink} (link, user_id, created) VALUES ('%s', '%d', '%d')", $actcode, $save, time() + 86400);
			$link = $this->diafan->_route->link($this->diafan->cid).'?action=act&user_id='.$save.'&code='.$actcode;
			$actlink = '<a href="'.BASE_PATH_HREF.$link.'">http://'.BASE_URL.'/'.$link.'</a>';
		}

		$message = str_replace(
				array('%login', '%title', '%url', '%fio', '%email', '%password', '%params', '%actlink'), array(
					$login,
					TITLE,
					BASE_URL,
					$this->diafan->get_param($_POST, "fio", '', 1),
					$this->diafan->get_param($_POST, "mail", '', 1),
					$this->diafan->get_param($_POST, "password", '', 1),
					$this->message_param,
					$actlink
				), $this->diafan->configmodules('message', "users")
		);
		if ($message && $subject)
		{
			send_mail(
					$_POST["mail"], $subject, $message, $this->diafan->configmodules("emailconf", "users") ? $this->diafan->configmodules("email", "users") : ''
			);
		}
		return true;
	}

	/**
	 * Загружает аватар
	 * 
	 * @return void
	 */
	private function upload_avatar()
	{
		if (isset($_FILES["avatar"]) && is_array($_FILES["avatar"]) && $_FILES["avatar"]['name'] != '')
		{
			try
			{
				$this->diafan->_user->create_avatar($_POST["name"], $_FILES["avatar"]['tmp_name']);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = "avatar";
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
		}
	}

}