<?php
/**
 * Обработка Ajax-запросов в административной части модуля
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if ( ! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_admin_ajax
 */
class Shop_admin_ajax extends Frame_admin
{
	/**
	 * @var array результаты, передаваемы Ajaxом
	 */
	private $result;

	/**
	 * Вызывает обработку Ajax-запросов
	 * 
	 * @return void
	 */
	public function ajax()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if ($_POST["action"] != "show_rel_goods")
		{
			if ( ! $this->diafan->_user->checked)
			{
				$this->result["error"] = 'ERROR_HASH';
				$this->send_json();
			}
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();

		if ( ! empty($_POST["action"]))
		{
			switch ($_POST["action"])
			{
				case 'show_discount_goods':
					$this->show_discount_goods();
					break;

				case 'discount_good':
					$this->discount_good();
					break;

				case 'delete_discount_good':
					$this->delete_discount_good();
					break;

				case 'clone_good':
					$this->clone_good();
					break;

				case 'show_order_goods':
					$this->show_order_goods();
					break;

				case 'add_order_good':
					$this->add_order_good();
					break;

				case 'new_order':
					$this->new_order();
					break;

				case 'list_site_id':
					$this->list_site_id();
					break;

				case 'param_category_rel':
				case 'param_category_unrel':
					$this->param_category();
					break;

				case 'group_no_buy':
				case 'group_not_no_buy':
				case 'group_hit':
				case 'group_not_hit':
				case 'group_action':
				case 'group_not_action':
				case 'group_new':
				case 'group_not_new':
					$this->group_option();
					break;
			}
		}
	}

	/**
	 * Подгружает список товаров
	 * 
	 * @return void
	 */
	private function show_discount_goods()
	{
		if (empty($_POST["discount_id"]))
		{
			$_POST["discount_id"] = 0;
		}
		$nastr = 16;
		$list = '';
		if (empty($_POST["page"]))
		{
			$start = 0;
			$page = 1;
			if ( ! isset($_POST["search"]))
			{
				$list = $this->diafan->_('Поиск').': <input type="text" size="30" class="rel_module_search">
				<div class="rel_all_elements_container">';
			}
		}
		else
		{
			$page = intval($_POST["page"]);
			$start = ($page - 1) * $nastr;
		}
		$discount_goods = array();
		if ($_POST["discount_id"])
		{
			$result = DB::query("SELECT good_id FROM {shop_discount_object} WHERE discount_id=%d", $_POST["discount_id"]);
			while ($row = DB::fetch_array($result))
			{
				$discount_goods[] = $row["good_id"];
			}
			DB::free_result($result);
		}
		if ( ! empty($_POST["search"]))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0' AND LOWER([name]) LIKE LOWER('%%%h%%')", $_POST["search"]);
			$result = DB::query_range("SELECT id, [name] FROM {shop} WHERE trash='0' AND LOWER([name]) LIKE LOWER('%%%h%%')", $_POST["search"], $start, $nastr);
		}
		else
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0'");
			$result = DB::query_range("SELECT id, [name] FROM {shop} WHERE trash='0'", $start, $nastr);
		}
		while ($row = DB::fetch_array($result))
		{
			$img = DB::query_fetch_array("SELECT name, folder_num FROM {images} WHERE element_id=%d AND module_name='shop' AND trash='0' ORDER BY sort DESC LIMIT 1", $row["id"]);
			$list .= '<div class="rel_module" element_id="'.$row["id"].'">
			<div'.(in_array($row["id"], $discount_goods) ? ' class="rel_module_selected"' : '').'>
			'.($img ? '<a href="#"><img src="'.BASE_PATH.USERFILES.'/small/'.($img["folder_num"] ? $img["folder_num"].'/' : '').$img["name"].'"></a><br>' : '').'
			<a href="#">'.$row["name"].'</a>
			</div>
			</div>';
		}
		DB::free_result($result);
		$list .= '<div class="clear rel_module_navig">';
		for ($i = 1; $i <= ceil($count / $nastr); $i ++ )
		{
			if ($i != $page)
			{
				$list .= '<a href="#" page="'.$i.'">'.$i.'</a> ';
			}
			else
			{
				$list .= '['.$i.'] ';
			}
		}
		$list .= '</div>';
		if (empty($_POST["page"]) && ! isset($_POST["search"]))
		{
			$list .= '</div>';
		}

		$this->result["data"] = $list;

		$this->send_json();
	}

	/**
	 * Прикрепляет скидку к товару
	 * 
	 * @return void
	 */
	private function discount_good()
	{
		if ( ! $_POST["discount_id"])
		{
			$_POST["discount_id"] = DB::query("INSERT INTO {shop_discount} () VALUES ()");
			$this->result["id"] = $_POST["discount_id"];
		}
		Customization::inc('modules/shop/admin/shop.admin.view.php');
		if ( ! DB::query_result("SELECT id FROM {shop_discount_object} WHERE good_id=%d AND discount_id=%d LIMIT 1", $_POST["good_id"], $_POST["discount_id"]))
		{
			DB::query("INSERT INTO {shop_discount_object} (good_id, discount_id) VALUES (%d, %d)", $_POST["good_id"], $_POST["discount_id"]);
		}

		$shop_admin_view = new Shop_admin_view($this->diafan);
		$this->result["data"] = $shop_admin_view->discount_goods($_POST["discount_id"]);

		$this->send_json();
	}

	/**
	 * Удаляет скидку на товар
	 * 
	 * @return void
	 */
	private function delete_discount_good()
	{
		DB::query("DELETE FROM {shop_discount_object} WHERE good_id=%d AND discount_id=%d", $_POST['good_id'], $_POST['discount_id']);

		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->send_json();
	}

	/**
	 * Клонирует товар
	 * 
	 * @return void
	 */
	private function clone_good()
	{
		// Проверяет права на редактирование
		if (! $this->diafan->_user->roles('edit', 'shop'))
		{
			return;
		}
		$id = intval($_POST['id']);
		if (empty($id))
			return;

		$row = DB::query_fetch_array("SELECT * FROM {shop} WHERE id=%d LIMIT 1", $id);

		foreach ($row as $k => $v)
		{
			if ($k == 'name'.$this->diafan->language_base_site)
			{
				$v = $this->diafan->_('КОПИЯ').' '.$v;
			}
			$row[$k] = "'".str_replace("'", "\\'", $v)."'";
		}
		unset($row['id']);
		unset($row['counter_buy']);

		$n_id = DB::query('INSERT INTO {shop} ('.implode(',', array_keys($row)).') VALUES ('.implode(',', $row).')');

		$site_id = $row['site_id'];

		$result = DB::query("SELECT cat_id, trash FROM {shop_category_rel} WHERE element_id='%d'", $id);
		while ($row = DB::fetch_array($result))
		{
			DB::query("INSERT INTO {shop_category_rel} (element_id, cat_id, trash) VALUES (%d, %d, '%s')", $n_id, $row['cat_id'], $row['trash']);
		}
		DB::free_result($result);
		
		$prices = array();
		$result = DB::query("SELECT * FROM {shop_price} WHERE good_id=%d AND trash='0'", $id);
		while ($row = DB::fetch_array($result))
		{
			$row['good_id'] = $n_id;
			foreach ($row as $k => $v)
			{
				if($k != "id")
				{
					$row_param[$k] = "'".str_replace("'", "\\'", $v)."'";
				}
			}
			$price_id = DB::query('INSERT INTO {shop_price} ('.implode(',', array_keys($row_param)).') VALUES ('.implode(',', $row_param).')');
			if($row["id"] == $row["price_id"])
			{
				$prices[$row["price_id"]] = $price_id;

				$result_param = DB::query("SELECT param_id, param_value FROM {shop_price_param} WHERE price_id=%d", $row["price_id"]);
				while ($row_param = DB::fetch_array($result_param))
				{
					DB::query("INSERT INTO {shop_price_param} (price_id, param_id, param_value) VALUES (%d, %d, %d)", $price_id, $row_param["param_id"], $row_param["param_value"]);
				}
				DB::free_result($result_param);
			}
		}
		DB::free_result($result);
		foreach($prices as $old => $new)
		{
			DB::query("UPDATE {shop_price} SET price_id=%d WHERE price_id=%d AND good_id=%d", $new, $old, $n_id);
		}

		$result = DB::query("SELECT * FROM {shop_param_element} WHERE element_id='%d' AND trash='0'", $id);
		while ($row = DB::fetch_array($result))
		{
			unset($row["id"]);
			$row['element_id'] = $n_id;
			foreach ($row as $k => &$v)
			{
				$v = "'".str_replace("'", "\\'", $v)."'";
			}
			DB::query('INSERT INTO {shop_param_element} ('.implode(',', array_keys($row)).') VALUES ('.implode(',', $row).')');
		}
		DB::free_result($result);
		$this->send_json();
	}

	/**
	 * Подгружает список товаров для добавления в заказ
	 * 
	 * @return void
	 */
	private function show_order_goods()
	{
		if (empty($_POST["order_id"]))
		{
			$_POST["order_id"] = 0;
		}
		$nastr = 18;
		$list = '';
		if (empty($_POST["page"]))
		{
			$start = 0;
			$page = 1;
			if ( ! isset($_POST["search"]) && ! isset($_POST["cat_id"]))
			{
				$list = '<form>'.$this->diafan->_('Поиск').': <input type="text" size="30" class="order_goods_search">';
				if($this->diafan->configmodules("cat", "shop"))
				{
					$result = DB::query("SELECT id, [name], parent_id FROM {shop_category} WHERE trash='0' ORDER BY sort ASC");
					while ($row = DB::fetch_array($result))
					{
						$cats[$row["parent_id"]][] = $row;
					}
					DB::free_result($result);
					$vals = array();
					if(! empty($_POST["cat_id"]))
					{
						$vals[] = $this->diafan->get_param($_POST, "cat_id", 2);
					}
					$list.= ' <select name="cat_id" class="order_goods_cat_id"><option value="">'.$this->diafan->_('Все').'</option>'.$this->diafan->get_options($cats, $cats[0], $vals).'</select>';
				}
				$list.= '</form><div class="order_all_goods_container">';
			}
		}
		else
		{
			$page = intval($_POST["page"]);
			$start = ($page - 1) * $nastr;
		}

		$where = '';
		if(! empty($_POST["cat_id"]))
		{
			$where = " AND cat_id=".$this->diafan->get_param($_POST, "cat_id", 2);
		}

		if ( ! empty($_POST["search"]))
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0' AND LOWER([name]) LIKE LOWER('%%%h%%')".$where, $_POST["search"]);
			$result = DB::query_range("SELECT id, [name] FROM {shop} WHERE trash='0' AND LOWER([name]) LIKE LOWER('%%%h%%')".$where, $_POST["search"], $start, $nastr);
		}
		else
		{
			$count = DB::query_result("SELECT COUNT(*) FROM {shop} WHERE trash='0'".$where);
			$result = DB::query_range("SELECT id, [name] FROM {shop} WHERE trash='0'".$where, $start, $nastr);
		}
		$user_id = DB::query_result("SELECT user_id FROM {shop_order} WHERE id=%d LIMIT 1", $_POST["order_id"]);
		while ($row = DB::fetch_array($result))
		{
			$img = DB::query_fetch_array("SELECT name, folder_num FROM {images} WHERE element_id=%d AND module_name='shop' AND trash='0' ORDER BY sort DESC LIMIT 1", $row["id"]);
			$prices = $this->diafan->_shop->price_get_all($row["id"], $user_id);
			$list .= '<div class="order_good">
			<div>
			'.($img ? '<img src="'.BASE_PATH.USERFILES.'/small/'.($img["folder_num"] ? $img["folder_num"].'/' : '').$img["name"].'"><br>' : '').'
			'.$row["name"];
			foreach($prices as $price)
			{
				$list .= '<br><a href="#" price_id="'.$price["id"].'">'.$this->diafan->_shop->price_format($price["price"]).' '.$this->diafan->configmodules("currency", "shop").'</a>';
			}
			$list .= '</div>
			</div>';
		}
		DB::free_result($result);
		$list .= '<div class="clear order_goods_navig">';
		for ($i = 1; $i <= ceil($count / $nastr); $i ++ )
		{
			if ($i != $page)
			{
				$list .= '<a href="#" page="'.$i.'">'.$i.'</a> ';
			}
			else
			{
				$list .= '['.$i.'] ';
			}
		}
		$list .= '</div>';
		if (empty($_POST["page"]) && ! isset($_POST["search"]))
		{
			$list .= '</div>';
		}

		$this->result["data"] = $list;

		$this->send_json();
	}

	/**
	 * Добавляет выбранный товар в заказ
	 * 
	 * @return void
	 */
	private function add_order_good()
	{
		if (empty($_POST["price_id"]))
		{
			return;
		}
		if(empty($_POST["order_id"]))
		{
			$_POST["order_id"] = DB::query("INSERT INTO {shop_order} () VALUES ()");
			$this->result["order_id"] = $_POST["order_id"];
		}
		$format_price = intval($this->diafan->configmodules("format_price_1", "shop"));
		$price = DB::query_fetch_array("SELECT price_id, price, old_price, good_id, discount_id FROM {shop_price} WHERE id=%d LIMIT 1", $_POST["price_id"]);
		$where = array();
		$params = array();
		$result = DB::query("SELECT param_id, param_value FROM {shop_price_param} WHERE price_id=%d AND trash='0'", $price["price_id"]);
		while ($row = DB::fetch_array($result))
		{
			$params[$row["param_id"]] = $row["param_value"];
			$where[] = "s.param_id=".intval($row["param_id"])." AND s.value=".intval($row["param_value"]);
		}
		DB::free_result($result);
		if(! DB::query_result("SELECT g.id FROM {shop_order_goods} AS g WHERE g.order_id=%d AND g.good_id=%d"
				.($where ? " AND (SELECT COUNT(*) FROM {shop_order_goods_param} AS s WHERE g.id=s.order_good_id AND (".implode(" OR ", $where).")) = ".count($where) : ""), 
				$_POST["order_id"], $price["good_id"]))
		{
			$order_good_id = DB::query("INSERT INTO {shop_order_goods} (order_id, good_id, count_goods, price, discount_id) VALUES (%d, %d, 1, %f, %d)", $_POST["order_id"], $price["good_id"], $price["price"], $price["discount_id"]);
			$depend = '';
			if($params)
			{
				foreach($params as $id => $value)
				{
					DB::query("INSERT INTO {shop_order_goods_param} (value, param_id, order_good_id) VALUES (%d, %d, %d)", $value, $id, $order_good_id);
					if(! $value)
						continue;
					$param_name  = DB::query_result("SELECT [name] FROM {shop_param} WHERE id=%d LIMIT 1", $id);
					$param_value = DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d LIMIT 1", $value);
					$depend .= ($depend ? ', ' : '').$param_name.': '.$param_value;
				}
			}
			$good = DB::query_fetch_array("SELECT [name], article FROM {shop} WHERE id=%d LIMIT 1", $price["good_id"]);
			$img = DB::query_fetch_array("SELECT i.name, i.folder_num FROM {images} AS i
			LEFT JOIN {shop_price_image_rel} AS r ON r.image_id=i.id AND r.price_id=%d
			WHERE i.element_id=%d AND i.module_name='shop' AND i.trash='0'
			ORDER BY r.image_id DESC LIMIT 1",
			$price["price_id"], $price["good_id"]);

			$old_price = $price["old_price"] ? $price["old_price"] : $price["price"];
			if($price["discount_id"])
			{
				$d = DB::query_fetch_array("SELECT discount, deduction FROM {shop_discount} WHERE id=%d LIMIT 1", $price["discount_id"]);
				$discount = $d["discount"] ? $d["discount"].'%' : $d["deduction"].' '.$this->diafan->configmodules("currency", "shop");
			}

			$this->result["data"] = '
			<tr class="tr_good">
				<td id="first">'.($img ? '<img src="'.BASE_PATH.USERFILES.'/small/'.($img["folder_num"] ? $img["folder_num"].'/' : '').$img["name"].'">' : '').'</td>
				<td><a href="'.BASE_PATH_HREF.'shop/edit'.$price["good_id"].'/">'.$good["name"].' '.$depend.' '.$good["article"].'</a></td>
				<td align="right"><input type="text" name="count_goods'.$order_good_id.'" value="1" size="2"></td>
				<td align="right">'.$this->diafan->_shop->price_format($old_price).'</td>
				<td align="right">'.($price["discount_id"] ? '<a href="'.BASE_PATH_HREF.'shop/discount/edit'.$price["discount_id"].'/">'.$discount.'</a>' : '').'</td>
				<td align="right"><input type="text" name="price_goods'.$order_good_id.'" value="'.number_format($price["price"], $format_price, ".", "").'" size="4"></td>
				<td align="right">'.$this->diafan->_shop->price_format($price["price"]).'</td>
				<td><a href="javascript:void(0)" confirm="'.$this->diafan->_('Вы действительно хотите удалить запись?').'" class="delete_order_good" title="'.$this->diafan->_('Удалить').'">
				<img src="'.BASE_PATH.'adm/img/delete.png" width="13" height="13" alt="'.$this->diafan->_('Удалить').'"></a></td>
</tr>';
		}

		$this->send_json();
	}

	/**
	 * Проверяет наличие новых заказов
	 * 
	 * @return void
	 */
	private function new_order()
	{
		$last_order_id = $this->diafan->get_param($_POST, "last_order_id", 0, 2);

		$this->result["next_order_id"] = DB::query_result("SELECT id FROM {shop_order} WHERE id>%d AND trash='0' LIMIT 1", $last_order_id);

		$this->send_json();
	}

	/**
	 * Подгружает список модулей
	 * 
	 * @return void
	 */
	private function list_site_id()
	{
		if (! $_POST["parent_id"])
		{
			$list = '<div class="menu_list menu_list_first"><div class="block_header">'.$this->diafan->_('Страницы сайта').'</div>';
		}
		else
		{
			$list = '<div class="menu_list">';
		}
		
		$result = DB::query("SELECT id, [name], module_name, count_children FROM {site} WHERE [act]='1' AND trash='0' AND parent_id=%d ORDER BY sort DESC", $_POST["parent_id"]);
		while ($row = DB::fetch_array($result))
		{
			$list .= '<p site_id="'.$row["id"].'" module_name="site" element_id="" cat_id="">';
			if ($row["count_children"])
			{
				$list .= '<a href="#" class="plus menu_plus">+</a>';
			}
			else
			{
				$list .= '&nbsp;&nbsp;';
			}
			$list .= '&nbsp;<a href="'.BASE_PATH_HREF.'site/edit'.$row["id"].'/" class="menu_select">'.$row["name"].'</a></p>';
		}
		DB::free_result($result);
		$list .= '</div>';

		$this->result["data"] = $list;

		$this->send_json();
	}

	/**
	 * Прикрепляет/открепляет характеристику к категории
	 * 
	 * @return void
	 */
	private function param_category()
	{
		if(! empty($_POST["cat"]) || ! empty($_POST["ids"]))
		{
			$ids = array();
			foreach($_POST["ids"] as $id)
			{
				$id = intval($id);
				if($id)
				{
					$ids[] = $id;
				}
			}
			if($ids)
			{
				if($_POST["action"] == 'param_category_rel')
				{
					DB::query("DELETE FROM {shop_param_category_rel} WHERE element_id IN(%s) AND cat_id IN(%d, 0)", implode(",", $ids), $_POST["cat"]);
				}
				else
				{
					DB::query("DELETE FROM {shop_param_category_rel} WHERE element_id IN(%s) AND cat_id=%d", implode(",", $ids), $_POST["cat"]);
				}
			}
			if($_POST["action"] == 'param_category_rel')
			{
				foreach($ids as $id)
				{
					DB::query("INSERT INTO {shop_param_category_rel} (element_id, cat_id) VALUES (%d, %d)", $id, $_POST["cat"]);
				}
			}
			else
			{
				// выбираем все характеристики из выделенных, которые прикреплены к каким-нибудь категориям
				$cats_rel = array();
				$rows = DB::query_fetch_all("SELECT DISTINCT(element_id) FROM {shop_param_category_rel} WHERE element_id IN (%s)", implode(",", $ids));
				foreach($rows as $row)
				{
					$cats_rel[] = $row["element_id"];
				}
				// если характеристика не прикреплена ни к одной категории, делаем ее общей
				foreach($ids as $id)
				{
					if(! in_array($id, $cats_rel))
					{
						DB::query("INSERT INTO {shop_param_category_rel} (element_id) VALUES (%d)", $id);
					}
				}
			}
		}

		$this->result["redirect"] = URL;
		$this->send_json();
	}

	/**
	 * Групповая операция "Товар временно отсутствует", "Акция" и др.
	 * 
	 * @return void
	 */
	private function group_option()
	{
		if(! empty($_POST["ids"]))
		{
			$ids = array();
			foreach($_POST["ids"] as $id)
			{
				$id = intval($id);
				if($id)
				{
					$ids[] = $id;
				}
			}
		}
		elseif(! empty($_POST["id"]))
		{
			$ids = array(intval($_POST["id"]));
		}
		if(! empty($ids))
		{
			switch ($_POST["action"])
			{
				case 'group_no_buy':
					$set = "no_buy='1'";
					$this->result["img"] = BASE_PATH.'adm/img/no_buy.png';
					$this->result["action"] = 'group_not_no_buy';
					break;

				case 'group_not_no_buy':
					$set = "no_buy='0'";
					$this->result["img"] = BASE_PATH.'adm/img/no_buy_noact.png';
					$this->result["action"] = 'group_no_buy';
					break;

				case 'group_hit':
					$set = "hit='1'";
					$this->result["img"] = BASE_PATH.'adm/img/hit.png';
					$this->result["action"] = 'group_not_hit';
					break;

				case 'group_not_hit':
					$set = "hit='0'";
					$this->result["img"] = BASE_PATH.'adm/img/hit_noact.png';
					$this->result["action"] = 'group_hit';
					break;

				case 'group_action':
					$set = "action='1'";
					$this->result["img"] = BASE_PATH.'adm/img/action.png';
					$this->result["action"] = 'group_not_action';
					break;

				case 'group_not_action':
					$set = "action='0'";
					$this->result["img"] = BASE_PATH.'adm/img/action_noact.png';
					$this->result["action"] = 'group_action';
					break;

				case 'group_new':
					$set = "new='1'";
					$this->result["img"] = BASE_PATH.'adm/img/new.png';
					$this->result["action"] = 'group_not_new';
					break;

				case 'group_not_new':
					$set = "new='0'";
					$this->result["img"] = BASE_PATH.'adm/img/new_noact.png';
					$this->result["action"] = 'group_new';
					break;
			}
			if($ids)
			{
				DB::query("UPDATE {shop} SET ".$set." WHERE id IN (%s)", implode(",", $ids));
				$this->diafan->_cache->delete("", "shop");
			}
		}

		$this->result["redirect"] = URL;
		$this->send_json();
	}

	/**
	 * Отдает ответ Ajax
	 * 
	 * @return void
	 */
	private function send_json()
	{
		if ($this->result)
		{
			include_once ABSOLUTE_PATH.'plugins/json.php';
			echo to_json($this->result);
			exit;
		}
	}
}
