<?php
/**
 * Редактирование скидок
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_admin_discount
 */
class Shop_admin_discount extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'shop_discount';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'info' => array(
				'type' => 'function',
				'name' => 'Информационное поле',
				'no_save' => true,
				'hide' => true,
			),
			'discount' => array(
				'type' => 'floattext',
				'name' => 'Скидка в процентах (%)',
				'help' => 'Если заполнено это поле, скидка будет считаться в процентах от исходной цены.',
			),
			'deduction' => array(
				'type' => 'floattext',
				'name' => 'Скидка в виде фиксированной суммы',
				'help' => 'Если заполнено это поле, скидка будет вычитаться от исходной цены в виде фиксированной суммы.',
			),
		),
		'goods' => array (
			'amount' => array(
				'type' => 'floattext',
				'name' => 'Cкидка действует на товары дороже',
				'help' => 'Cкидка будет применяться только к тем товарам, которые дороже указанной суммы.',
			),
			'object' => array(
				'type' => 'function',
				'name' => 'Объект',
				'help' => 'Товары и категории, на которые распространяется скидка. Если не указаны, то скидка считается общей.',
			),
		),
		'order' => array (
			'threshold' => array(
				'type' => 'numtext',
				'name' => 'Скидка действует от общей суммы заказа',
				'help' => 'Скидка начнет действовать когда пользователь наберет в корзину товаров на указанную сумму. Если заполнено, то скидка применяется только в корзине товаров на общую сумму.',
			),   
			'threshold_cumulative' => array(
				'type' => 'numtext',
				'name' => 'Накопительная скидка от суммы ранее оплаченных заказов',
				'help' => 'Скидка начнет действовать когда пользователь оплатит товаров на указанную сумму. Если заполнено, то скидка применяется только в корзине товаров на общую сумму.',
			),
		),
		'coupon' => array (
			'coupon' => array(
				'type' => 'function',
				'name' => 'Код купона',
				'help' => 'Пользователь должен активировать на сайте этот код, чтобы получить скидку.',
			),
		),
		'person' => array (
			'role_id' => array(
				'type' => 'select',
				'name' => 'Группы покупателей',
				'help' => 'Скидка будет применяться ко всей группе пользователей.',
			),
			'person' => array(
				'type' => 'function',
				'name' => 'ID',
				'help' => 'Если есть пользователи, использующие скидку, то скидка считается персонализированной и другим пользователям не применяется.',
			),
		),
		'actdis' => array (
			'date_start' => array(
				'type' => 'datetime',
				'name' => 'Период действия скидки',
				'help' => 'Если выбрать период действия скидки, она будет применяться только в указанное время.',
			),
			'date_finish' => array(
				'type' => 'datetime',
				'hide' => true,
			),
			'act' => array(
				'type' => 'checkbox',
				'name' => 'Активировать скидку',
				'help' => 'Если отметить, скидка будет опубликована на сайте и примениться ко всем товарам, отвечающим условиям выше.',
				'default' => true,
			),
		),
	);

	/**
	 * @var array названия табов
	 */
	public $tabs_name = array(
		'main' => 'Размер скидки',
		'goods' => 'На отдельные категории и товары',
		'order' => 'На весь заказ',
		'coupon' => 'Купоны',
		'person' => 'Для отдельных пользователей',
		'actdis' => 'Активировать скидку',
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'act', // показать/скрыть
		'del', // удалить
		'trash', // использовать корзину
	);

	/**
	 * @var array списки из таблицы
	 */
	public $select = array(
		'role_id' => array(
			'users_role',
			'id',
			'nameLANG',
			'',
			'Все',
			"trash='0'",
		),
	);

	/**
	 * @var array текст для ссылки на редактирование в списке
	 */
	public $text_for_base_link = array(
		'variable' => 'discount, e.deduction',
	);

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить скидку');
	}

	/**
	 * Выводит список заказов
	 * @return void
	 */
	public function show()
	{
		$this->diafan->list_row();
	}

	/**
	 * Определяет строку с GET переменными
	 *
	 * @return void
	 */
	public function set_get_nav()
	{
		$get_nav_params = $this->diafan->get_nav_params;
		$get_nav_params["search_date_start"] = 0;
		$get_nav_params["search_date_finish"] = 0;

		if (! empty($_GET["search_date_start"]))
		{
			$get_nav_params["search_date_start"] = $this->diafan->unixdate($_GET["search_date_start"]);
			$this->diafan->get_nav .= ( $this->diafan->get_nav ? '&amp;' : '?' ) . 'search_date_start='. $get_nav_params["search_date_start"];
			$get_nav_params["search_date_start"] = date('d.m.Y H:i', $date_start);
		}
		if (! empty($_GET["search_date_finish"]))
		{
			$get_nav_params["search_date_finish"] = $this->diafan->unixdate($_GET["search_date_finish"]);
			$this->diafan->get_nav .= ( $this->diafan->get_nav ? '&amp;' : '?' ) . 'search_date_finish=' . $get_nav_params["search_date_finish"];
			$get_nav_params["search_date_finish"] = date('d.m.Y H:i', $date_finish);
		}
		$this->diafan->get_nav_params = $get_nav_params;

		if ($this->diafan->get_nav_params["search_date_start"] && $this->diafan->get_nav_params["search_date_finish"])
		{
			$this->diafan->where .= " AND ((date_start <= '".$this->diafan->get_nav_params["search_date_start"]."' OR date_start = '0') AND (date_finish >= '".$this->diafan->get_nav_params["search_date_start"]."' OR date_finish = '0')"
			." OR date_start >= '".$this->diafan->get_nav_params["search_date_start"]."' AND date_start <= '".$this->diafan->get_nav_params["search_date_finish"]."')";
		}
		elseif ($this->diafan->get_nav_params["search_date_start"])
		{
			$this->diafan->where = " AND (date_start<='".$this->diafan->get_nav_params["search_date_start"]."' OR date_start='0') AND (date_finish>='".$this->diafan->get_nav_params["search_date_start"]."' OR date_finish='0')";
		}
		elseif ($this->diafan->get_nav_params["search_date_finish"])
		{
			$this->diafan->where = " AND (date_start<='".$this->diafan->get_nav_params["search_date_finish"]."' OR date_start='0') AND (date_finish>='".$this->diafan->get_nav_params["search_date_finish"]."' OR date_finish='0')";
		}
	}

	/**
	 * Поиск
	 *
	 * @return string
	 */
	public function show_search()
	{
		if($this->diafan->edit)
			return '';

		$html = '<td>
		'.$this->diafan->_('Дата').': <input type="text" name="date_start" value="'.($this->diafan->get_nav_params["search_date_start"] ? date('d.m.Y H:i', $this->diafan->get_nav_params["search_date_start"]) : "").'" class="timecalendar"> - 
		<input type="text" name="date_finish" value="'.($this->diafan->get_nav_params["search_date_finish"] ? date('d.m.Y H:i', $this->diafan->get_nav_params["search_date_finish"]) : "").'" class="timecalendar">
		<input type="submit" value="'.$this->diafan->_('Поиск').'"  class="button">
		</td>';

		return $html;
	}

	/**
	 * Заглушка для вывода даты окончания действия скидки в списке
	 * @return string
	 */
	public function other_row_date_finish($row)
	{
		return '';
	}

	/**
	 * Формирует основную ссылку для элемента в списке
	 *
	 * @param array $row информация о текущем элменте списка
	 * @param integer $level уровень вложенности
	 * @return string
	 */
	public function get_base_link($row, $level)
	{
		if(! empty($row["deduction"]))
		{
			$name = $row["deduction"].' '.$this->diafan->configmodules("currency", "shop");
		}
		else
		{
			$name = $row["discount"].' %';
		}

		$link = '<td class="name"><a href="';
		if ($this->diafan->_user->roles('init', $this->diafan->rewrite))
		{

			$link .= $this->diafan->_route->current_admin_link().'edit'.$row["id"].'/'.$this->diafan->get_nav.'" title="'.$this->diafan->_('Редактировать').' ('.$row["id"].')';
		}
		else
		{
			$link .= '#';
		}
		$link .= '"';
		if ($this->diafan->config("act") && !$row["act"])
		{
			$link .= ' class="noplus"';
		}
		$link .= '>'.$name.'</a>';
		return $link;
	}

	/**
	 * Редактирование поля "Информационное поле"
	 * @return void
	 */
	public function edit_variable_info()
	{
		echo '<div class="discount_info" style="color: green;text-align:center"></div>';
	}

	/**
	 * Редактирование поля "Объект"
	 * @return void
	 */
	public function edit_variable_object()
	{
		$marker = "&nbsp;&nbsp;";
		$result = DB::query("SELECT id, [name], parent_id FROM {shop_category} WHERE trash='0' ORDER BY sort ASC, id ASC");
		while ($row = DB::fetch_array($result))
		{
			$cats[$row["parent_id"]][] = $row;
		}
		DB::free_result($result);

		$values = array();
		if (! $this->diafan->addnew)
		{
			$result = DB::query("SELECT cat_id FROM {shop_discount_object} WHERE discount_id='%d' AND cat_id>0", $this->diafan->edit);
			while ($row = DB::fetch_array($result))
			{
				$values[] = $row["cat_id"];
			}
			DB::free_result($result);
		}
		Customization::inc('modules/shop/admin/shop.admin.view.php');

		echo '
		<tr valign="top">
			<td class="td_first">
				'.$this->diafan->_('Категории').'
			</td>
			<td>
				<select name="cat_ids[]" multiple="multiple" size="11">
				<option value="all"'.(empty($values) ? ' selected' : '').'>'.$this->diafan->_('Все').'</option>'
				.$this->diafan->get_options($cats, $cats[0], $values).'
				</select>
			</td>
		</tr>
		<script type="text/javascript" src="'.BASE_PATH.'modules/shop/admin/shop.admin.discount.js"></script>
		<tr valign="top">
			<td class="td_first">
				'.$this->diafan->_('Отдельные товары').'
				<span class="help_img"><img src="'.BASE_PATH.'adm/img/quest.gif" title="'.$this->diafan->_('Вы можете назначить скидку толко на некоторые конкретные товары.').'" alt=""></span>
			</td>
			<td>
				<div class="rel_elements">';
				if (! $this->diafan->addnew)
				{
					$view = new Shop_admin_view($this->diafan);
					echo $view->discount_goods($this->diafan->edit);
				}
				echo '</div>
				<a href="javascript:void(0)" class="rel_module_plus" title="'.$this->diafan->_('Добавить').'"><img src="'.BASE_PATH.'adm/img/add.png" width="16" height="16" alt="'.$this->diafan->_('Добавить').'"></a>
				<div class="hide" id="rel_module_container"></div>
			</td>
		</tr>';
	}

	/**
	 * Редактирование поля "Пользователь"
	 * @return void
	 */
	public function edit_variable_person()
	{
		$persons = array();
		if(! $this->diafan->addnew)
		{
			$result = DB::query("SELECT p.*, CONCAT(u.fio,' (', u.name, ')') AS user FROM {shop_discount_person} AS p"
			." LEFT JOIN {users} AS u ON u.id=p.user_id"
			." WHERE p.discount_id=%d", $this->diafan->edit);
			while($row  = DB::fetch_array($result))
			{
				$persons[] = $row;
			}
			DB::free_result($result);
		}
		echo '
		<tr>
			<td></td><td class="param_container"><table>';
		foreach($persons as $row)
		{
			echo '<tr class="param"><td>
				<input type="hidden" name="person_id[]" value="'.$row["id"].'">
				ID:';
			if($row["user"])
			{
				echo '<a href="'.BASE_PATH_HREF.'users/edit'.$row["user_id"].'/">'.$row["user"].'</a> ';
			}
				echo '<input type="text" name="person_user_id[]" size="4" value="'.($row["user_id"] ? $row["user_id"] : '').'">
				'.($row["session_id"] ? 'session_id: '.$row["session_id"] : '').'
				'.($row["coupon_id"] && ! empty($this->diafan->values["coupon"][$row["coupon_id"]]) ? ' '.$this->diafan->_('Добавлен по купону').' '.$this->diafan->values["coupon"][$row["coupon_id"]] : '').'
				<input type="hidden" name="person_session_id[]" value="'.$row["session_id"].'">
				<span class="param_actions">
					<a href="javascript:void(0)" action="delete_param" confirm="'.$this->diafan->_('Вы действительно хотите удалить запись?').'"><img src="'.BASE_PATH.'adm/img/delete.png" width="13" height="13" alt="'.$this->diafan->_('Удалить').'"></a>
				</span>
			</td></tr>';
			
		}
		echo '<tr class="param"><td>
					ID: <input type="text" name="person_user_id[]" size="4" value="">
					<span class="param_actions">
						<a href="javascript:void(0)" action="delete_param" confirm="'.$this->diafan->_('Вы действительно хотите удалить запись?').'" style="display:none"><img src="'.BASE_PATH.'adm/img/delete.png" width="13" height="13" alt="'.$this->diafan->_('Удалить').'"></a>
					</span>
				</td></tr>
				</table>
				<a href="javascript:void(0)" class="param_plus" title="'.$this->diafan->_('Добавить').'"><img src="'.BASE_PATH.'adm/img/add.png" width="16" height="16" alt="'.$this->diafan->_('Добавить').'"></a>
				'.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Редактирование поля "Код купона"
	 * @return void
	 */
	public function edit_variable_coupon()
	{
		$coupons = array();
		if(! $this->diafan->addnew)
		{
			$result = DB::query("SELECT * FROM {shop_discount_coupon} WHERE discount_id=%d", $this->diafan->edit);
			while($row  = DB::fetch_array($result))
			{
				$coupons[] = $row;
				$this->diafan->values["coupon"][$row["id"]] = $row["coupon"];
			}
			DB::free_result($result);
		}
		echo '
		<tr>
			<td class="td_first"></td><td class="param_container"><table>';
		foreach($coupons as $row)
		{
			echo '<tr class="param"><td>
				'.$this->diafan->_('Код купона').'
				<input type="hidden" name="coupon_id[]" value="'.$row["id"].'">
				<input type="text" name="coupon[]" size="20" value="'.$row["coupon"].'">
				<a href="#" id="coupon_generate">'.$this->diafan->_('сгенерировать').'</a><br>
				'.$this->diafan->_('Сколько раз можно использовать купон').'
				'.$this->diafan->help('Скидка становится неактивной, если она исчерпала этот лимит. Если поле не заполнено, ограничение по количеству раз не действует.').'
				<input type="text" name="coupon_count_use[]" size="4" value="'.($row["count_use"] ? $row["count_use"] : '').'">
				'.($row["used"] ? ' '.$this->diafan->_('Использован').': '.$row["used"] : '').'
				<span class="param_actions">
					<a href="javascript:void(0)" action="delete_param" confirm="'.$this->diafan->_('Вы действительно хотите удалить запись?').'"><img src="'.BASE_PATH.'adm/img/delete.png" width="13" height="13" alt="'.$this->diafan->_('Удалить').'"></a>
				</span>
				<hr>
			</td></tr>';
			
		}
		echo '<tr class="param"><td>
				'.$this->diafan->_('Код купона').'
				<input type="text" name="coupon[]" size="20" value="">
				<a href="#" id="coupon_generate">'.$this->diafan->_('сгенерировать').'</a><br>
				'.$this->diafan->_('Сколько раз можно использовать купон').'
				'.$this->diafan->help('Скидка становится неактивной, если она исчерпала этот лимит. Если поле не заполнено, ограничение по количеству раз не действует.').'
				<input type="text" name="coupon_count_use[]" size="4" value="">
				<span class="param_actions">
					<a href="javascript:void(0)" action="delete_param" confirm="'.$this->diafan->_('Вы действительно хотите удалить запись?').'" style="display:none"><img src="'.BASE_PATH.'adm/img/delete.png" width="13" height="13" alt="'.$this->diafan->_('Удалить').'"></a>
				</span>
				<hr>
			</td></tr>
			</table>
			<a href="javascript:void(0)" class="param_plus" title="'.$this->diafan->_('Добавить').'"><img src="'.BASE_PATH.'adm/img/add.png" width="16" height="16" alt="'.$this->diafan->_('Добавить').'"></a>
			'.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Сохранение поля "Объект"
	 * @return void
	 */
	public function save_variable_object()
	{
		DB::query("DELETE FROM {shop_discount_object} WHERE discount_id=%d AND good_id=0", $this->diafan->save);
		if(! empty($_POST["cat_ids"]) && in_array("all", $_POST["cat_ids"]))
		{
			$_POST["cat_ids"] = array();
		}

		if (! empty($_POST["cat_ids"]))
		{
			foreach ($_POST["cat_ids"] as $id)
			{
				if($id)
				{
					DB::query("DEV INSERT INTO {shop_discount_object} (discount_id, cat_id) VALUES (%d, %d)", $this->diafan->save, $id);
				}
			}
		}
		elseif (! DB::query_result("SELECT id FROM {shop_discount_object} WHERE discount_id=%d LIMIT 1", $this->diafan->save))
		{
			DB::query("INSERT INTO {shop_discount_object} (discount_id) VALUES (%d)", $this->diafan->save);
		}
		DB::query("UPDATE {shop_discount} SET act='%d' WHERE id=%d", ! empty($_POST["act"]) ? '1' : '0', $this->diafan->save);
	}

	/**
	 * Сохранение поля "Пользователь"
	 * @return void
	 */
	public function save_variable_person()
	{
		$person = 0;
		if(! empty($_POST["person_user_id"]))
		{
			foreach($_POST["person_user_id"] as $i => $user_id)
			{
				$user_id = intval($user_id);
				if(! empty($_POST["person_id"][$i]))
				{
					if(! empty($user_id) || ! empty($_POST["person_session_id"][$i]))
					{
						DB::query("UPDATE {shop_discount_person} SET user_id=%d, session_id='%s' WHERE id=%d AND discount_id=%d", $user_id,$_POST["person_session_id"][$i], $_POST["person_id"][$i], $this->diafan->save);
						$id = intval($_POST["person_id"][$i]);
						if($id)
						{
							$ids[] = $id;
						}
					}
				}
				else
				{
					if($user_id)
					{
						$ids[] = DB::query("INSERT INTO {shop_discount_person} (user_id, discount_id) VALUES (%d, %d)", $user_id,$this->diafan->save);
					}
				}
			}
			if(! empty($ids))
			{
				$person = 1;
			}
		}
		DB::query("DELETE FROM {shop_discount_person} WHERE ".(! empty($ids) ? "id NOT IN(".implode(",", $ids).") AND" : "")." discount_id=%d", $this->diafan->save);
		if(! $person && ! empty($_POST["coupon"]))
		{
			foreach($_POST["coupon"] as $c)
			{
				if(! empty($c))
				{
					$person = 1;
				}
			}
		}
		$this->diafan->set_query("person='%d'");
		$this->diafan->set_value($person);
	}

	/**
	 * Сохранение поля "Купон"
	 * @return void
	 */
	public function save_variable_coupon()
	{
		if(! empty($_POST["coupon"]))
		{
			foreach($_POST["coupon"] as $i => $coupon)
			{
				if(! empty($_POST["coupon_id"][$i]))
				{
					if(! empty($coupon))
					{
						DB::query("UPDATE {shop_discount_coupon} SET coupon='%h', count_use=%d WHERE id=%d AND discount_id=%d", $coupon, $_POST["coupon_count_use"][$i], $_POST["coupon_id"][$i], $this->diafan->save);
						$id = intval($_POST["coupon_id"][$i]);
						if($id)
						{
							$ids[] = $id;
						}
					}
				}
				else
				{
					if($coupon)
					{
						$ids[] = DB::query("INSERT INTO {shop_discount_coupon} (coupon, count_use, discount_id) VALUES ('%h', %d, %d)", $coupon, $_POST["coupon_count_use"][$i], $this->diafan->save);
					}
				}
			}
		}
		DB::query("DELETE FROM {shop_discount_coupon} WHERE ".(! empty($ids) ? "id NOT IN(".implode(",", $ids).") AND" : "")." discount_id=%d", $this->diafan->save);
	}

	/**
	 * Пользовательская функция, выполняемая перед редиректом при сохранении скидки
	 *
	 * @return void
	 */
	public function save_redirect()
	{
		$this->diafan->_shop->price_calc(0, $this->diafan->save);
		parent::__call('save_redirect', array());
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return void
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->_shop->price_calc(0, $del_id);
		$this->diafan->del_or_trash_where("shop_discount_object", "discount_id=".$del_id, $trash_id);
		$this->diafan->del_or_trash_where("shop_discount_person", "discount_id=".$del_id, $trash_id);
		$this->diafan->del_or_trash_where("shop_discount_coupon", "discount_id=".$del_id, $trash_id);
		$this->diafan->del_or_trash_where("shop_price", "discount_id=".$del_id, $trash_id);
	}
}