<?php
/**
 * Список описанных файлов
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop_admin_importexport_category
 */
class Shop_admin_importexport_category extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'shop_import_category';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'name' => array(
				'type' => 'text',
				'name' => 'Название',
				'help' => 'Краткое описание файла импорта (например, «Импорт товаров», «Импорт цен» и т. д.).',
			),
			'type' => array(
				'type' => 'select',
				'name' => 'Тип',
			),
			'delete_items' => array(
				'type' => 'checkbox',
				'name' => 'Удалять неописанные в файле импорта записи',
				'help' => 'Если список товаров или категорий в файле является исчерпывающим, то следует отметить эту опцию. Тогда при импорте удалятся ранее занесенные товары или категории, кроме тех, что будут обновлены (определяется по идентификатору).',
			),
			'site_id' => array(
				'type' => 'function',
				'name' => 'Раздел сайта',
				'help' => 'Страница сайта с прикрепленным модулем «Магазин», для которой будет производится импорт.',
			),
			'cat_id' => array(
				'type' => 'function',
				'name' => 'Категория товаров',
				'help' => 'Возможность ограничить импорт/экспорт одной категорией магазина.',
			),
			'count_part' => array(
				'type' => 'numtext',
				'name' => 'Количество строк, выгружаемых за один проход скрипта',
				'help' => 'Время работы скрипта ограничено, из-за чего скрипт может не успеть обработать весь файл за одну итерацию. Поэтому файл обрабатывается частями, величину итерации можно задать этим параметром.',
				'default' => 200
			),
			'delimiter' => array(
				'type' => 'text',
				'name' => 'Разделитель данных в строке',
				'help' => 'Разделитель полей в строке.',
				'default' => ";"
			),
			'end_string' => array(
				'type' => 'text',
				'name' => 'Обозначать конец строки символом',
				'help' => 'Если в строке содержатся символы перевода строки (например, в описании товара), то конец строки должен быть обозначен отдельным символом. Например, КОНЕЦ_СТРОКИ. Не обязательный параметр.'
			),
			'encoding' => array(
				'type' => 'text',
				'name' => 'Кодировка',
				'help' => 'Кодировка данных в файле CSV. Обычно cp1251 или utf8',
				'default' => 'cp1251',
			),
			'sub_delimiter' => array(
				'type' => 'text',
				'name' => 'Разделитель данных внутри поля',
				'help' => 'Некоторые поля содержат несколько данных в одной ячейки (например, значение характеристики с типом «список с выбором нескольких значений» или изображения), в этом случае данные разделены этим разделителем.',
				'default' => '|',
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'del', // удалить
		'category', // часть модуля - категории
		'link_to_element', // основная ссылка ведет к списку элементов, принадлежащих категории
		'trash', // использовать корзину
	);

	/**
	 * @var integer количество строк, выводимых на странице
	 */
	public $nastr = 100;

	/**
	 * @var array значения списков
	 */
	public $select_arr = array(
		'type' => array(
			'good' => 'Товары',
			'category' => 'Категории',
		),
	);

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить описание файла импорта/экспорта');
	}

	/**
	 * Выводит список категорий
	 * @return void
	 */
	public function show()
	{
		$this->diafan->list_row();
	}

	/**
	 * Редактирование поля "Категория"
	 * 
	 * @return void
	 */
	public function edit_variable_cat_id()
	{
		if(! $this->diafan->configmodules("cat", "shop"))
		{
			return;
		}
		echo '
		<tr valign="top">
			<td align="right">'.$this->diafan->variable_name() . '</td>
			<td>';

		$result = DB::query("SELECT id, [name], parent_id FROM {shop_category} WHERE trash='0' ORDER BY sort ASC");
		while ($row = DB::fetch_array($result))
		{
			$cats[$row["parent_id"]][] = $row;
		}
		DB::free_result($result);

		echo ' <select name="cat_id">
		<option value="">'.$this->diafan->_('Все').'</option>';
		echo $this->diafan->get_options($cats, $cats[0], array($this->diafan->value));
		echo '</select>';

		echo $this->diafan->help() . '
			</td>
		</tr>';
	}
}