<?php
/**
 * Контроллер модуля «Магазин»
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop
 */
class Shop extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat', 'sort', 'param');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;

		if ($this->diafan->show)
		{
			$this->model->id();
		}
		elseif ($this->diafan->param)
		{
			$this->model->list_param();
		}
		elseif(isset($_GET["action"]) && $_GET["action"] === 'search')
		{
			$this->model->list_search();
		}
		elseif(isset($_GET["action"]) && $_GET["action"] === 'compare' && ! $this->diafan->configmodules('hide_compare', "shop"))
		{
			$this->model->compare();
		}
		elseif(isset($_GET["action"]) && $_GET["action"] === 'file' && isset($_GET["code"]))
		{
			$this->model->file_get();
		}
		elseif (! $this->diafan->configmodules("cat"))
		{
			$this->model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$this->model->first_page();
		}
		else
		{
			$this->model->list_category();
		}
		$this->result = $this->model->get_result();
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return void
	 */
	public function show_module()
	{
		if (! empty($this->result["err"]))
		{
			echo $this->result["err"];
			return;
		}
		$this->diafan->_tpl->get($this->view, 'shop', $this->result);
	}

	/**
	 * Шаблонная функция: выводит несколько товаров из каталога.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых товаров (по умолчанию 3)
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы
	 * cat_id - категории товаров, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. По умолчанию категория не учитывается, выводятся все товары
	 * images - количество изображений, прикрепленных к товару
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * sort - сортировка товаров: по умолчанию как на странице модуля, **date** – по дате, **rand** – в случайном порядке, **price** - по цене, **sale** – по количеству продаж
	 * only_shop - выводить блок только на странице, к которой прикреплен модуль «Магазин»
	 * hits_only - выводить только товары с пометкой «Хит»
	 * action_only - выводить только товары с пометкой «Акция»
	 * new_only - выводить только товары с пометкой «Новинка»
	 * discount_only - выводить только товары, на которые действует скидка
	 * param - значения дополнительных характеристике
	 * template - шаблон тега (файл modules/shop/views/shop.view.show_block_**template**.php; по умолчанию шаблон modules/shop/views/shop.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_shop', 'count', 'cat_id', 'site_id', 'images', 'images_variation', 'sort', 'param', 'template',
		'hits_only', 'action_only', 'new_only', 'discount_only');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_ids  = explode(",", $attributes["cat_id"]);
		$site_ids = explode(",", $attributes["site_id"]);
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';
		$param   = $attributes["param"];
		$sort    = in_array($attributes["sort"], array("date", "rand", "price", "sale")) ? $attributes["sort"] : "";
		$only_shop = $attributes["only_shop"];
		$hits_only = (bool) $attributes["hits_only"];
		$action_only  = (bool) $attributes["action_only"];
		$new_only   = (bool) $attributes["new_only"];
		$discount_only   = (bool) $attributes["discount_only"];

		if ($only_shop && ($this->diafan->module != "shop" || in_array($this->diafan->cid, $site_ids)))
			return;

		$this->model->show_block($count, $cat_ids, $site_ids, $sort, $images, $images_variation, $param, $hits_only,
		$action_only, $new_only, $discount_only);
		$result = $this->model->get_result();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_block', 'shop', $result);
		}
	}

	/**
	 * Шаблонная функция: на странице товара выводит похожие товары. По умолчанию связи между товарами являются односторонними, это можно изменить, отметив опцию «В блоке похожих товаров связь двусторонняя» в настройках модуля.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых товаров (по умолчанию 3)
	 * images - количество изображений, прикрепленных к товару
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/shop/views/shop.view.show_block_rel_**template**.php; по умолчанию шаблон modules/shop/views/shop.view.show_block_rel.php)
	 * @return void
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "shop" || ! $this->diafan->show)
			return false;

		$this->model->show_block_rel($count, $images, $images_variation);
		$result = $this->model->get_result();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'shop', $result);
		}
	}

	/**
	 * Шаблонная функция: товары, которые обычно покупают с текущим товаром.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * count - количество выводимых товаров (по умолчанию 3)
	 * images - количество изображений, прикрепленных к товару
	 * images_variation - тег размера изображений, задается в настроках модуля
	 * template - шаблон тега (файл modules/shop/views/shop.view.show_block_order_rel_**template**.php; по умолчанию шаблон modules/shop/views/shop.view.show_block_order_rel.php)
	 * 
	 * @return void
	 */
	public function show_block_order_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'images_variation', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);
		$images_variation = $attributes["images_variation"] ? strval($attributes["images_variation"]) : 'medium';

		if ($this->diafan->module != "shop" || ! $this->diafan->show)
			return false;

		$this->model->show_block_order_rel($count, $images, $images_variation);
		$result = $this->model->get_result();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_order_rel_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_block_order_rel', 'shop', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит форму поиска товаров. Если для категорий прикреплены дополнительные характеристики, то поиск по ним производится только на странице категории. Поиск по обязательным полям подключается в настройках модуля (опции «Искать по цене», «Искать по описанию», «Искать по названию», «Искать по артикулу», «Искать товары по акции», «Искать новинки», «Искать хиты»). Если в форму поиска выведены характеристики с типом «выпадающий список» и «список с выбором нескольких значений», то значения характеристик, которые не найдут ни один товар, в форме поиска не выведутся.
	 *
	 * @param array $attributes атрибуты шаблонного тега
	 * only_shop - выводить форму поиска только на странице магазина
	 * site_id - страницы, к которым прикреплен модуль. Идентификаторы страниц перечисляются через запятую. По умолчанию выбираются все страницы. Если выбрано несколько страниц сайта, то в форме поиска появляется выпадающих список по выбранным страницам
	 * cat_id - категории товаров, если в настройках модуля отмечено «Использовать категории». Идентификаторы категорий перечисляются через запятую. Можно указать значение **current**, тогда поиск будет осуществляться по текущей (открытой) категории магазина или по всем категориям, если ни одна категория не открыта. Если выбрано несколько категорий, то в форме поиска появится выпадающий список категорий магазина, который будет подгружать прикрепленные к категориям характеристики. Можно указать значение **all**, тогда поиск будет осуществлятся по всем категориям товаров и в форме будут участвовать только общие характеристики
	 * ajax - подгружать результаты поиска без перезагрузки страницы. Результаты подгружаются только если открыта страница со списком товаром, иначе поиск работает обычным образом
	 * template - шаблон тега (файл modules/shop/views/shop.view.show_search_**template**.php; по умолчанию шаблон modules/shop/views/shop.view.show_search.php)
	 * 
	 * @return void
	 */
	public function show_search($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_shop', 'cat_id', 'site_id', 'template', 'ajax');

		$only_shop = $attributes["only_shop"];
		$cat_ids   = $attributes["cat_id"] === 'current' || $attributes["cat_id"] === 'all' ? $attributes["cat_id"] : explode(",", $attributes["cat_id"]);
		$site_ids  = explode(",", $attributes["site_id"]);
		$ajax      = $attributes["ajax"] == "true" ? true : false;

		if ($cat_ids === 'current')
		{
			if($this->diafan->cat && $this->diafan->module == "shop" && (count($site_ids) == 1 && $site_ids[0] == 0 || in_array($this->diafan->cid, $site_ids)))
			{
				$cat_ids  = array($this->diafan->cat);
				$site_ids = array($this->diafan->cid);
			}
			else
			{
				$cat_ids = array();
			}
		}

		if ($only_shop && ($this->diafan->module != "shop" || $site_ids && ! in_array($this->diafan->cid, $site_ids)))
			return;

		$result = $this->model->show_search($cat_ids, $site_ids, $ajax);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_search_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_search', 'shop', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит форму активирования купона на скидку, если неактивированный купон есть в системе, пользователь авторизован и у него не активирован другой купон.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * template - шаблон тега (файл modules/shop/views/shop.view.show_add_coupon_**template**.php; по умолчанию шаблон modules/shop/views/shop.view.show_add_coupon.php)
	 * 
	 * @return void
	 */
	public function show_add_coupon($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'template');
		$result = $this->model->show_add_coupon();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_add_coupon_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_add_coupon', 'shop', $result);
		}
	}
}