<?php
/**
 * Редактирование блоков на сайте
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Site_admin_blocks
 */
class Site_admin_blocks extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'site_blocks';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main'       => array (
			'name'     => array(
				'type' => 'text',
				'name' => 'Название блока',
				'help' => 'Название блока, выводится перед содержимым блока, если не отмечена галка «Не выводить название блока».',
				'multilang' => true
			),
			'act'     => array(
				'type' => 'checkbox',
				'name' => 'Опубликовать на сайте',
				'help' => 'Отображение блока на сайте. Если не отмечена, блок на сайте не будет показываться.',
				'multilang' => true
			),
			'site_ids'         => array(
				'type' => 'function',
				'name' => 'Отображать на страницах',
				'help' => 'Выбор отдельных страниц, где будет показываться блок. Удерживайте CTLR, чтобы выбрать несколько страниц.'
			),
			'title_no_show' => array(
				'type' => 'checkbox',
				'name' => 'Не выводить название блока',
				'help' => 'Если отмечено, заголовок перед содержимым блока автоматически выводиться не будет.'
			),
			'text'     => array(
				'type' => 'editor',
				'name' => 'Контент блока',
				'help' => 'Основное содержимое блока. Если отметить «Применить типограф», контент будет отформатирован согласно правилам экранной типографики с помощью [веб-сервиса «Типограф»](http://www.artlebedev.ru/tools/typograf/webservice/).',
				'multilang' => true
			),
		),
		'other_rows' => array (
			'number'        => array(
				'type' => 'function',
				'name' => 'Номер',
				'help' => 'Номер элемента в БД (веб-мастеру и программисту).'
			),
			'admin_id' => array(
				'type' => 'function',
				'name' => 'Редактор',
				'help' => 'Изменяется после первого сохранения. Показывает, кто из администраторов сайта первый правил текущий блок.'
			),
			'timeedit' => array(
				'type' => 'text',
				'name' => 'Время последнего изменения',
				'help' => 'Изменяется после сохранения элемента. Отдается в заголовке *Last Modify*.',
			),
			'access'        => array(
				'type' => 'function',
				'name' => 'Доступ',
				'help' => 'Если отметить опцию «Доступ только», блок увидят только авторизованные на сайте пользователи, отмеченных типов (администратору сайта).',
			),
			'date_start' => array(
				'type' => 'date',
				'name' => 'Период показа',
				'help' => 'Если выставить, текущий блок будет опубликован на сайте в указанный период. В иное время пользователи сайта блок не будут видеть (администратору сайта).'
			),
			'date_finish' => array(
				'type' => 'date',
				'hide' => true,
			),
			'hr_period' => 'hr',
			'sort' => array(
				'type' => 'function',
				'name' => 'Сортировка: установить перед',
				'help' => 'Изменить положение текущего блока среди других блоков. Используется для удобство администрирования блоков (администратору сайта).'
			)
		)
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'order',  //сортируется
		'act',    //показать/скрыть
		'del',    //удалить
		'trash',   //использовать корзину
		'only_self', // показывать только материалы редактора, если это задано в правах пользователя
	);

	/**
	 * @var integer количество строк, выводимых на странице
	 */
	public $nastr = 100;

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить блок');
	}

	/**
	 * Выводит список страниц сайта
	 * @return void
	 */
	public function show()
	{
		$this->diafan->list_row();
	}

	/**
	 * Редактирование поля "Расположение"
	 *
	 * @return void
	 */
	public function edit_variable_site_ids()
	{
		$show_in_site_id = array();
		if(! $this->diafan->addnew)
		{
			$result = DB::query("SELECT site_id FROM {site_block_rel} WHERE element_id=%d AND site_id>0", $this->diafan->edit);
			while ($row = DB::fetch_array($result))
			{
				$show_in_site_id[] = $row["site_id"];
			}
			DB::free_result($result);
		}
		echo '
		<tr valign="top" id="site_ids">
		<td align="right">'.$this->diafan->variable_name().'</td>
		<td>
		<select multiple="multiple" name="'.$this->diafan->key.'[]" size="11">
		<option value="all"'.(empty($show_in_site_id) ? ' selected' : '').'>'.$this->diafan->_('Все').'</option>';

		$result = DB::query("SELECT id, [name], parent_id FROM {site} WHERE trash='0' AND [act]='1' ORDER BY sort DESC, id DESC");
		while ($row = DB::fetch_array($result))
		{
			$cats[$row["parent_id"]][] = $row;
		}
		DB::free_result($result);
		echo $this->diafan->get_options($cats, $cats[0], $show_in_site_id).'
				</select>
				'.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Редактирование поля "Доступ"
	 *
	 * @return void
	 */
	public function edit_variable_access()
	{
		echo '
		<tr valign="top" id="access">
			<td class="td_first">'.$this->diafan->variable_name().'</td>
			    <td>';

		$checked = array ();
		if ($this->diafan->value == '1')
		{
			$element_id = $this->diafan->edit;

			$res = DB::query("SELECT role_id FROM {access} WHERE module_name='site_blocks' AND element_id=%d", $element_id);
			while ($row = DB::fetch_array($res))
			{
				$checked[] = $row['role_id'];
			}
			DB::free_result($res);
		}
		echo '<input type="checkbox" name="access" '.($this->diafan->value=='1'?' checked':'').'> '.$this->diafan->_('Доступ только').':<br><div style="margin-left:25px">';
		$result = DB::query("SELECT id, [name] FROM {users_role} WHERE trash='0'");
		while ($row = DB::fetch_array($result))
		{
			echo '<input type="checkbox" name="access_role[]" value="'.$row['id'].'"'.( !$this->diafan->value || in_array($row['id'], $checked) ? ' checked' : '' ).'> '.$row['name'].'<br>';
		}
		DB::free_result($result);

		echo '</div>'.$this->diafan->help().'</td></tr>';
	}

	/**
	 * Сохранение поля "Расположение"
	 * @return void
	 */
	public function save_variable_site_ids()
	{
		$this->diafan->update_table_rel("site_block_rel", "element_id", "site_id", ! empty($_POST['site_ids']) ? $_POST['site_ids'] : array(), $this->diafan->save, $this->diafan->savenew);
	}

	/**
	 * Сохранение поля "Доступ"
	 *
	 * @return void
	 */
	public function save_variable_access()
	{
		$cat_id = 0;
		$element_id = $this->diafan->save;

		$roles = array ();
		$old_roles = array ();
		$new_roles = array ();

		$result = DB::query("SELECT id FROM {users_role} WHERE trash='0'");
		while ($row = DB::fetch_array($result))
		{
			$roles[] = $row['id'];
		}
		DB::free_result($result);
		if ($this->diafan->oldrow['access'])
		{
			$result = DB::query("SELECT role_id FROM {access} WHERE element_id=%d AND cat_id=%d AND module_name='site_blocks'", $element_id, $cat_id);
			while ($row = DB::fetch_array($result))
			{
				$old_roles[] = $row['role_id'];
			}
			DB::free_result($result);
		}

		foreach ($_POST['access_role'] as $role_id)
		{
			$new_roles[] = intval($role_id);
		}
		// отмечены все роли
		if (empty( $_POST["access"]))
		{
			$new_roles = array ();
		}

		// изменений нет
		if (!array_diff($new_roles, $old_roles) && !array_diff($old_roles, $new_roles))
		{
			return true;
		}
		$this->diafan->set_query("access='%d'");
		$this->diafan->set_value($new_roles ? 1 : 0);

		// Роли, которым дан доступ
		$diff_new_roles = array_diff($new_roles, $old_roles);
		foreach ($diff_new_roles as $role_id)
		{
			DB::query("INSERT INTO {access} (element_id, module_name, role_id) VALUES (%d, 'site_blocks', %d)", $element_id, $role_id);
		}
		// Роли, для которых теперь нет доступа
		$diff_old_roles = array_diff($old_roles, $new_roles);
		if ($diff_old_roles)
		{
			DB::query("DELETE FROM {access} WHERE element_id=%d AND module_name='site_blocks' AND role_id IN (%s)", $element_id, implode(",", $diff_old_roles));
		}
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return void
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->del_or_trash_where("site_block_rel", "element_id=".$del_id, $trash_id);
	}
}