<?php
/**
 * Редактирование страниц сайта
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Site_admin
 */
class Site_admin extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'site';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main'       => array (
			'name'     => array(
				'type' => 'text',
				'name' => 'Название',
				'help' => 'Название страницы, отображается в ссылках на страницу, используется для автоматической генерации пункта меню, заголовка страницы и её адреса (ЧПУ).',
				'multilang' => true
			),
			'act'     => array(
				'type' => 'checkbox',
				'name' => 'Опубликовать на сайте',
				'help' => 'Отображение страницы на сайте. Если не отмечена, страница на сайте не будет доступна пользователям и выдаст «404 Страница не найдена». Страница всегда доступна для просмотра администратору.',
				'multilang' => true
			),
			'parent_id'     => array(
				'type' => 'select',
				'name' => 'Корневая страница',
				'help' => 'Перемещение текущей страницы и всех её подстраниц в принадлежность другой страницы (администратору сайта).'
			),
			'menu'     => array(
				'type' => 'module',
				'name' => 'Создать пункт в меню',
				'help' => 'Если отметить, в [модуле «Меню на сайте»](http://cms.diafan.ru/dokument/full-manual/sysmodules/menu/) будет создан пункт со ссылкой на текущую страницу.'
			),
			'tags'     => array(
				'type' => 'module',
				'name' => 'Теги',
				'help' => 'Добавление тегов к странице. Можно добавить либо новый тег, либо открыть и выбрать из уже существующих тегов. Параметр выводится, если в настройках модуля включен параметр «Подключить теги».',
			),
			'images'     => array(
				'type' => 'module',
				'name' => 'Изображения',
				'help' => 'Изображения будут загружены автоматически после выбора. После загрузки изображения будут обработаны автоматически, согласно настройкам модуля. Параметр выводится, если в настройках модуля отмечена опция «Использовать изображения».',
			),
			'comments'     => array(
				'type' => 'module',
				'name' => 'Комментарии',
				'help' => 'Комментарии, которые оставили пользователи к текущей странице. Параметр выводится, если в настройках модуля включен параметр «Показывать комментарии к новостям».',
			),
			'text'     => array(
				'type' => 'editor',
				'name' => 'Контент страницы',
				'help' => 'Основное содержимое страницы. Если отметить «Применить типограф», контент будет отформатирован согласно правилам экранной типографики с помощью [веб-сервиса «Типограф»](http://www.artlebedev.ru/tools/typograf/webservice/).',
				'multilang' => true
			),
			'search' => array(
				'type' => 'module',
				'name' => 'Индексирование для поиска',
				'help' => 'Страница автоматически индексируется для модуля «Поиск по сайту» при внесении изменений.',
			),
			'map' => array(
				'type' => 'module',
				'name' => 'Индексирование для карты сайта',
				'help' => 'Страница автоматически индексируется для карты сайта sitemap.xml.',
			),
		),
		'other_rows' => array (
			'number'        => array(
				'type' => 'function',
				'name' => 'Номер',
				'help' => 'Номер страницы в БД (веб-мастеру и программисту).'
			),
			'admin_id' => array(
				'type' => 'function',
				'name' => 'Редактор',
				'help' => 'Изменяется после первого сохранения. Показывает, кто из администраторов сайта первый правил текущую страницу.'
			),
			'timeedit' => array(
				'type' => 'text',
				'name' => 'Время последнего изменения',
				'help' => 'Изменяется после сохранения элемента. Отдается в заголовке *Last Modify*.',
			),
			'hr_block'           => 'hr',
			'title_meta'    => array(
				'type' => 'text',
				'name' => 'Заголовок окна в браузере, тег Title',
				'help' => 'Если не заполнен, тег *Title* будет автоматически сформирован как «Название страницы – Название сайта», либо согласно шаблонам автоформирования из настроек модуля (SEO-специалисту).',
				'multilang' => true
			),
			'keywords' => array(
				'type' => 'textarea',
				'name' => 'Ключевые слова, тег Keywords',
				'help' => 'Если не заполнен, тег *Keywords* будет автоматически сформирован согласно шаблонам автоформирования из настроек модуля (SEO-специалисту).',
				'multilang' => true
			),
			'descr' => array(
				'type' => 'textarea',
				'name' => 'Описание страницы, тэг Description',
				'help' => 'Если не заполнен, тег *Description* будет автоматически сформирован согласно шаблонам автоформирования из настроек модуля (SEO-специалисту).',
				'multilang' => true
			),
			'title_no_show' => array(
				'type' => 'checkbox',
				'name' => 'Не копировать автоматически название в заголовок Н1',
				'help' => 'Если отмечено, заголовок *Н1* перед текстом страницы автоматически выводиться не будет. Тогда его можно вписать в визуальный редактор в свободном виде (SEO-специалисту).'
			),
			'rewrite'       => array(
				'type' => 'function',
				'name' => 'Псевдоссылка',
				'help' => 'ЧПУ, т.е. адрес страницы вида: *http://site.ru/psewdossylka/*. Смотрите параметры сайта (SEO-специалисту).'
			),
			'noindex'   => array(
				'type' => 'checkbox',
				'name' => 'Не индексировать',
				'help' => 'Запрет индексации текущей страницы, если отметить, у страницы выведется тег: `<meta name="robots" content="noindex">` (SEO-специалисту).'
			),
			'changefreq'   => array(
				'type' => 'function',
				'name' => 'Changefreq',
				'help' => 'Вероятная частота изменения этой страницы. Это значение используется для генерирования файла sitemap.xml. Подробнее читайте в описании [XML-формата файла Sitemap](http://www.sitemaps.org/ru/protocol.html) (SEO-специалисту).',
			),
			'priority'   => array(
				'type' => 'floattext',
				'name' => 'Priority',
				'help' => 'Приоритетность URL относительно других URL на Вашем сайте. Это значение используется для генерирования файла sitemap.xml. Подробнее читайте в описании [XML-формата файла Sitemap](http://www.sitemaps.org/ru/protocol.html) (SEO-специалисту).',
			),
			'hr_map2' => 'hr',
			'date_start' => array(
				'type' => 'date',
				'name' => 'Период показа',
				'help' => 'Если выставить, текущая страница будет опубликована на сайте в указанный период. В иное время пользователи сайта страницу не будут видеть, получая ошибку 404 «Страница не найдена» (администратору сайта).'
			),
			'date_finish' => array(
				'type' => 'date',
				'hide' => true,
			),
			'access'        => array(
				'type' => 'function',
				'name' => 'Доступ',
				'help' => 'Если отметить опцию «Доступ только», страницу увидят только авторизованные на сайте пользователи, отмеченных типов. Не авторизованные, в том числе поисковые роботы, увидят «404 Страница не найдена» (администратору сайта).',
			),
			'sort' => array(
				'type' => 'function',
				'name' => 'Сортировка: установить перед',
				'help' => 'Изменить положение текущей страницы среди других страниц (администратору сайта).'
			),
			'map_no_show'   => array(
				'type' => 'checkbox',
				'name' => 'Не показывать на карте сайта',
				'help' => 'Если отмечено, страница не будет показываться на карте сайта (администратору сайта).'
			),
			'search_no_show'   => array(
				'type' => 'checkbox',
				'name' => 'Не показывать в результатах поиска по сайту',
				'help' => 'Страница не участвует в поисковой выдаче внутреннего поиска по сайту.'
			),
			'module_name'     => array(
				'type' => 'select',
				'name' => 'Прикрепить модуль',
				'help' => 'Прикрепление модуля к текущей странице. Содержимое модуля выведется после контента страницы (администратору сайта).'
			),
			'hr2'           => 'hr',
			'js'            => array(
				'type' => 'textarea',
				'name' => 'JavaScript',
				'help' => 'Поле для ввода JavaScript на текущей странице. Например, для кода Яндекс.Карт. ВНИМАНИЕ, не вставляйте JS-код в визуальный редактор, он может его обрезать. Вставляйте его в это поле.  (Веб-мастеру и программисту).'
			),
			'theme' => array(
				'type' => 'text',
				'name' => 'Дизайн страницы',
				'help' => 'Возможность подключить для страницы шаблон сайта отличный от основного (themes/site.php). Все шаблоны для сайта должны храниться в папке *themes* с расширением *.php* (например, themes/dizain_so_slajdom.php). Подробнее в [разделе «Шаблоны сайта»](http://cms.diafan.ru/dokument/full-manual/templates/site/). (веб-мастеру и программисту, не меняйте этот параметр, если не уверены в результате!).',
			),
		)
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'parent', //содержит вложенности
		'order',  //сортируется
		'act',    //показать/скрыть
		'view',   //просмотр на сайте
		'del',    //удалить
		'menu',   //используется в меню
		'nopage', //не разбивать на страницы
		'trash',   //использовать корзину
		'only_self', // показывать только материалы редактора, если это задано в правах пользователя
	);

	/**
	 * @var integer количество строк, выводимых на странице
	 */
	public $nastr = 100;

	/**
	 * @var array поля для выбора в SQL-запросе
	 */
	public $fields = array('e.module_name');

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить страницу сайта');
	}

	/**
	 * Выводит список страниц сайта
	 * @return void
	 */
	public function show()
	{
		echo '<ul class="list">
		<li class="level_0">';


		//Главная страница
		if ($row = DB::query_fetch_array("SELECT [name] FROM {".$this->diafan->table."} WHERE id=1 LIMIT 1"))
		{
			echo '<div class="table_wrap"><table width="100%"><tbody><tr>
			
			<td class="name" style="padding-left:8px"><a title="'.$this->diafan->_('Редактировать').'" href="'.URL.'edit1/">'.( $row["name"] ? $row["name"] : 1 ).'</a>';

			if (DB::query_result("SELECT id FROM {menu} WHERE module_name='site' AND site_id=1 AND trash='0' AND [act]='1' LIMIT 1"))
			{
				echo ' <span title="'.$this->diafan->_('Отображается в меню').'">('.$this->diafan->_('м').')</span>';
			}

			echo '</td><td class="action_separator"></td><td class="action_icon"><a href="'.URL.'parent0/addnew1/" title="'.$this->diafan->_('Создать страницу сайта').'"><img src="'.BASE_PATH.'adm/img/add.png" alt="'.$this->diafan->_('Создать страницу сайта').'"></a></td>
		     </tr></tbody></table></div>';
		}
		echo '</li></ul>';

		//весь список
		$this->diafan->where = " AND e.id<>1";
		$this->diafan->list_row();
	}

	/**
	 * Подготавливает конфигурацию модуля
	 * @return void
	 */
	public function prepare_config()
	{
		//нельзя редактировать поля Принадлежит, Псевдоссылка, Сторонняя ссылка, Активность для Главной страницы
		if ($this->diafan->edit == 1 && ! $this->diafan->addnew || $this->diafan->save == 1 && !$this->diafan->savenew)
		{
			$this->diafan->variable_unset("act");
			$this->diafan->variable_unset("parent_id");
			$this->diafan->variable_unset("rewrite");
			$this->diafan->variable_unset("hr3");
			$this->diafan->variable_unset("site_ids");
			$this->diafan->variable_unset("map_no_show");
			$this->diafan->variable_unset("date_start");
			$this->diafan->variable_unset("date_finish");
		}
	}

	/**
	 * Редактирование поля "Прикрепленный модуль"
	 * @return void
	 */
	public function edit_variable_module_name()
	{
		echo '<tr>
			<td class="td_first">
				'.$this->diafan->variable_name().'
			</td>
			<td>
				<select name="'.$this->diafan->key.'">
					<option value="">'.$this->diafan->_('нет').'</option>';
					$result = DB::query("SELECT DISTINCT(name), title FROM {modules} WHERE site_page='1' ORDER BY id ASC");
					while ($row = DB::fetch_array($result))
					{
						echo '<option value="'.$row["name"].'"'.( $this->diafan->value == $row["name"] ? ' selected' : '' ).'>'.$this->diafan->_($row["title"]).'</option>';
					}
					DB::free_result($result);

		echo '
				</select>
				'.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Сохранение псевдоссылки
	 *
	 * @return void
	 */
	public function get_rewrite()
	{
		$this->diafan->is_save_rewrite = true;

		if($this->diafan->save == 1)
		{
			return;
		}
		// сохранение редиректа
		if (! $this->diafan->savenew)
		{
			$row = DB::query_fetch_array("SELECT * FROM {redirect} WHERE module_name='site' AND site_id=%d LIMIT 1", $this->diafan->save);
		}
		if (! empty($row["id"]))
		{
			if (! $_POST["rewrite_redirect"])
			{
				DB::query("DELETE FROM {redirect} WHERE id=%d", $row["id"]);
			}
			elseif($_POST["rewrite_redirect"] != $row["redirect"] || $_POST["rewrite_code"] != $row["code"])
			{
				DB::query("UPDATE {redirect} SET redirect='%s', code=%d WHERE id=%d", $_POST["rewrite_redirect"], $_POST["rewrite_code"], $row["id"]);
			}
		}
		elseif($_POST["rewrite_redirect"])
		{
			DB::query("INSERT INTO {redirect} (redirect, code, site_id, module_name) VALUES ('%s', %d, %d, 'site')",
				$_POST["rewrite_redirect"], $_POST["rewrite_code"], $this->diafan->save);
		}
		//----------

		DB::query("DELETE FROM {rewrite} WHERE module_name='%s' AND site_id=%d", $this->diafan->module, $this->diafan->save);
		if (!empty( $_POST["rewrite"] ))
		{
			$rewrite = $_POST["rewrite"];
			//если такая псевдоссылка принадлежит другому элементу, добавляем номер id
			if (DB::query_result("SELECT id FROM {rewrite} WHERE rewrite='%s' AND trash='0' LIMIT 1", $rewrite))
			{
				$rewrite .= $this->diafan->save;
			}
			DB::query("INSERT INTO {rewrite} (module_name, site_id, rewrite) VALUES ('site', %d, '%h')", $this->diafan->save, $rewrite);

			return;
		}
		if (!empty( $rewrite ))
		{
			// в готовой псевдоссылке берется только последняя часть
			if (strpos($rewrite, '/') !== false)
			{
				$rewrite_array = explode("/", $rewrite);
				$rewrite = $rewrite_array[count($rewrite_array) - 1];
			}
		}
		else
		{
			$rewrite = $this->diafan->generate_rewrite();
		}

		//если сохраняемый элемент имеет родителя, то в начало псевдоссылки подставляется псевдоссылка родителя
		$rewrite = ( $_POST["parent_id"] ? DB::query_result("SELECT rewrite FROM {rewrite} WHERE module_name='%s' AND site_id=%d LIMIT 1", $this->diafan->table, $_POST["parent_id"]).'/' : '' ).$rewrite;

		//если такая псевдоссылка принадлежит другому элементу, добавляем номер id
		if (DB::query_result("SELECT id FROM {rewrite} WHERE rewrite='%s' AND id<>%d AND trash='0' LIMIT 1", $this->diafan->save, $rewrite))
		{
			$rewrite .= $this->diafan->save;
		}

		//изменяем псевдоссылку у дочерних страниц
		$this->save_children_rewrite($this->diafan->save, $rewrite);

		DB::query("INSERT INTO {rewrite} (module_name, site_id, rewrite) VALUES ('site', %d, '%h')", $this->diafan->save, $rewrite);
	}

	/**
	 * Изменяет псевдоссылки вложенных страниц
	 * @return void
	 */
	private function save_children_rewrite($parent_id, $rewrite)
	{
		$result = DB::query("SELECT r.rewrite, s.id FROM {site} AS s"
		                    ." INNER JOIN {rewrite} AS r ON r.site_id=s.id AND r.module_name='site'"
		                    ." WHERE s.parent_id=%d AND s.id<>%d", $parent_id, $parent_id);
		while ($row = DB::fetch_array($result))
		{
			if (empty( $row["rewrite"] ))
			{
				$rewrite1 = $rewrite.'/'.$row["id"];
			}
			else
			{
				$rew = explode("/", $row["rewrite"]);
				if (empty( $rew[count($rew) - 1] ))
				{
					$rewrite1 = $rewrite.'/'.$row["id"];
				}
				else
				{
					$rewrite1 = $rewrite.'/'.$rew[count($rew) - 1];
				}
			}
			DB::query("UPDATE {rewrite} SET rewrite='%h' WHERE module_name='%s', site_id=%d", $rewrite1, $row["id"]);
			$this->save_children_rewrite($row["id"], $rewrite1);
		}
		DB::free_result($result);
	}

	/**
	 * Сохранение поля "JavaScript"
	 * @return void
	 */
	public function save_variable_js()
	{
		$this->diafan->set_query("js='%s'");
		$this->diafan->set_value($_POST["js"]);
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return void
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->del_or_trash_where("site_block_rel", "site_id=".$del_id, $trash_id);
	}
}