<?php
/**
 * Контроллер
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Site
 */
class Site extends Controller
{
	/**
	 * Шаблонная функция: выводит содержимое страницы сайта, номер которой передан в виде атрибута id. Страница должна быть помечена опцией «Блок на сайте» и быть активной.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * id - идентификатор блока
	 * template - шаблон тега (файл modules/site/views/site.view.show_block_**template**.php; по умолчанию шаблон modules/site/views/site.view.show_block.php)
	 * 
	 * @return void
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'id', 'template');

		$attributes["id"] = intval($attributes["id"]);
		$result = $this->model->show_block($attributes["id"]);
		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'site', $result))
		{
			$this->diafan->_tpl->get('show_block', 'site', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит ссылки на страницы нижнего уровня, принадлежащие текущей странице.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * template - шаблон тега (файл modules/site/views/site.view.show_links_**template**.php; по умолчанию шаблон modules/site/views/site.view.show_links.php)
	 * 
	 * @return void
	 */
	public function show_links($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'template');
		$result = $this->model->show_links();
		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_links_'.$attributes["template"], 'site', $result))
		{
			$this->diafan->_tpl->get('show_links', 'site', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит ссылки на предыдущую и последующую страницы.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * template - шаблон тега (файл modules/site/views/site.view.show_previous_next_**template**.php; по умолчанию шаблон modules/site/views/site.view.show_previous_next.php)
	 * 
	 * @return void
	 */
	public function show_previous_next($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'template');

		$result = $this->model->show_previous_next();
		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_previous_next_'.$attributes["template"], 'site', $result))
		{
			$this->diafan->_tpl->get('show_previous_next', 'site', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит изображения, прикрепленные к странице сайта, если в конфигурации модуля «Страницы сайта» включен параметры «Использовать изображения».
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * template - шаблон тега (файл modules/site/views/site.view.show_images_**template**.php; по умолчанию шаблон modules/site/views/site.view.show_images.php)
	 * 
	 * @return void
	 */
	public function show_images($attributes)
	{
		if (! $this->diafan->configmodules('images', 'site'))
		{
			return;
		}
		$attributes = $this->get_attributes($attributes, 'template');

		$result = $this->model->show_images();
		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_images_'.$attributes["template"], 'site', $result))
		{
			$this->diafan->_tpl->get('show_images', 'site', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит комментарии, прикрепленные к странице сайта, если в конфигурации модуля «Страницы сайты» подключены комментарии.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * template - шаблон тега (файл modules/site/views/site.view.show_comments_**template**.php; по умолчанию шаблон modules/site/views/site.view.show_comments.php)
	 * 
	 * @return void
	 */
	public function show_comments($attributes)
	{
		if ($this->diafan->module)
		{
			return;
		}
		$attributes = $this->get_attributes($attributes, 'template');

		$result = $this->diafan->_comments->get($this->diafan->cid, 'site');
		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_comments_'.$attributes["template"], 'site', $result))
		{
			$this->diafan->_tpl->get('show_comments', 'site', $result);
		}
	}

	/**
	 * Шаблонная функция: выводит теги (слова-якори), прикрепленные к странице сайта, если в конфигурации модуля «Страницы сайты» подключены теги.
	 * 
	 * @param array $attributes атрибуты шаблонного тега
	 * template - шаблон тега (файл modules/site/views/site.view.show_tags_**template**.php; по умолчанию шаблон modules/site/views/site.view.show_tags.php)
	 * 
	 * @return void
	 */
	public function show_tags($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'template');

		$result = $this->diafan->_tags->get($this->diafan->cid, "site");
		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_tags_'.$attributes["template"], 'site', $result))
		{
			$this->diafan->_tpl->get('show_tags', 'site', $result);
		}
	}
}