<?php
/**
 * База электронных ящиков для рассылок
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Subscribtion_admin_emails
 */
class Subscribtion_admin_emails extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'subscribtion_emails';
	
	/**
	 * @var string категории рассылок
	 */
	public $subscribtion = '';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'mail' => array(
				'type' => 'email',
				'name' => 'Почтовый ящик',
				'help' => 'Электронный адрес подписчика.',
			),
			'name' => array(
				'type' => 'text',
				'name' => 'Имя получателя',
				'help' => 'В рассылке не участвует.',
			),
			'created' => array(
				'type' => 'datetime',
				'name' => 'Дата добавления',
				'help' => 'Дата добавления подписчика. Вводится в формате дд.мм.гггг чч:мм.',
			),
			'act' => array(
				'type' => 'checkbox',
				'name' => 'Получает рассылку',
				'help' => 'Позволяет отключить подписчика от рассылки. Пользователи, отписаннавшиеся от рассылки на сайте из базы данных не удаляются, только снимается отметка «Получает рассылку».',
			),
			'code' => array(
				'type' => 'none',
				'name' => 'Код управления рассылкой',
				'help' => 'Код, указанный пользователю в ссылке на управление рассылкой.',
			),
			'category' => array(
				'type' => 'function',
				'help' => 'Категории рассылок, на которые подписан пользователь. Параметр появляется, если отмечена опция «Использовать категории».',
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'del', // удалить
		'trash', // использовать корзину
		'act', // показать/скрыть
	);
	
	/**
	 * @var array текст для ссылки на редактирование в списке
	 */
	public $text_for_base_link = array(
		'variable' => 'mail'
	);

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
		$this->diafan->addnew_init('Добавить подписчика');
	}

	/**
	 * Выводит список рассылок
	 * @return void
	 */
	public function show()
	{
		$this->upload();

		$this->diafan->list_row();

		$this->form_upload();
	}
	
	/**
	 * Выводит список категорий рассылок
	 * @return void
	 */
	public function edit_variable_category()
	{
		if(!$this->diafan->configmodules("cat", "subscribtion"))
		{
			return TRUE;
		}
		else if($this->diafan->addnew)
		{
			$cat_unrel = array();
			$this->parent_id_subscribtion(0, '', array(), $cat_unrel);
			echo '<tr id="category"><td class="td_first">'.$this->diafan->_('Категории рассылок').'</td><td>'.$this->subscribtion.'</td></tr>';
		}
		else
		{
			$row = DB::query_fetch_array("SELECT * FROM {subscribtion_emails} WHERE id=%d AND trash='0' LIMIT 1", $this->diafan->edit);
			$cat_unrel = array();
			$cat_unrel_result = DB::query("SELECT cat_id FROM {subscribtion_emails_cat_unrel} WHERE element_id=%d AND trash='0'", $row['id']);
			while($cat_id = DB::fetch_array($cat_unrel_result))
			{
				$cat_unrel[] = $cat_id['cat_id'];
			}
			DB::free_result($cat_unrel_result);
			$this->parent_id_subscribtion(0, '', array(), $cat_unrel);
			echo '<tr id="category"><td class="td_first">'.$this->diafan->_('Категории рассылок').'</td><td>'.$this->subscribtion.'</td></tr>';
		}
	}
	
	/**
	 * Сохраняет список категорий рассылок
	 * @return void
	 */
	public function save_variable_category()
	{
	    DB::query("DELETE FROM {subscribtion_emails_cat_unrel} WHERE element_id=%d", $this->diafan->save);
	    
	    $result = DB::query("SELECT id FROM {subscribtion_category} WHERE trash='0'");
	    while ($row = DB::fetch_array($result))
	    {
		    if(empty($_POST['subscribtion_category']) || !in_array($row['id'], $_POST['subscribtion_category']))
		    {
			DB::query("INSERT INTO {subscribtion_emails_cat_unrel} (element_id, cat_id) VALUES (%d, %d)", $this->diafan->save, $row['id']);
		    }
	    }
		DB::free_result($result);
	}
	
	/**
	 * Формирует список рассылок
	 * @return array
	 */
	private function parent_id_subscribtion($parent_id, $rew, $array, $cat_unrel)
	{
		$result = DB::query("SELECT [name], id FROM {subscribtion_category} WHERE parent_id=%d AND trash='0' ORDER BY sort ASC", $parent_id);		
		while ($row  = DB::fetch_array($result))
		{
			$this->subscribtion .=$rew . '<input type="checkbox" name="subscribtion_category[]" value="' . $row["id"] . '"'
					. (!in_array($row['id'], $cat_unrel) ? ' checked' : '')
					. '> ' . $row["name"] . '<br>';
			if (in_array($row["id"], $array))
			{
				return $array;
			}
			$array[] = $row["id"];
			$array   = $this->parent_id_subscribtion($row["id"], '&nbsp;&nbsp;&nbsp;' . $rew, $array, $cat_unrel);
		}
		DB::free_result($result);
		return $array;
	}
	
	/**
	 * Сохраняет код управления рассылкой
	 * @return void
	 */
	public function save_variable_code()
	{
		if(! $this->diafan->oldrow["code"])
		{
			$this->diafan->set_query("code='%s'");
			$this->diafan->set_value(md5(rand(0, 9999999)));
		}
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return void
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->del_or_trash_where("subscribtion_emails_cat_unrel", "element_id=".$del_id, $trash_id);
	}

	/**
	 * Выводит форму импорт/экспорт ключевиков
	 * 
	 * @return void
	 */
	private function form_upload()
	{
		echo '<div class="block">';
		echo '<table width="100%"><tr><td valign="top" width="70%"><h2>'.$this->diafan->_('Импорт').'</h2>';
		echo '<form action="" method="post" enctype="multipart/form-data">
		<input type="hidden" name="upload" value="1">
		<input type="checkbox" name="delete_old" value="1"> '.$this->diafan->_('удалить неописанные в файле строки').'<br><br>
		<input type="file" name="file">
		<input type="submit" class="button" value="'.$this->diafan->_('Загрузить').'"><br>
		'.$this->diafan->_('*файл .txt, каждый подписчик с новой строки в формате «Имя;mail@site.ru»').'
		</form></td><td valign="top" width="30%"><h2>'.$this->diafan->_('Экспорт').'</h2>
		<a href="'.BASE_PATH.'subscribtion/export/emails/">'.$this->diafan->_('Скачать emails.txt').'</a>
		</td></tr></table>
		</div>';
	}

	/**
	 * Загружает файл перевода
	 * 
	 * @return void
	 */
	private function upload()
	{
		if(! empty($_POST["delete_old"]))
		{
			DB::query("TRUNCATE TABLE {subscribtion_emails}");
		}
		if (! isset($_FILES["file"]) || ! is_array($_FILES["file"]) || $_FILES["file"]['name'] == '')
		{
			return;
		}
		$oldmails  = array();
		if(empty($_POST["delete_old"]))
		{
			$result = DB::query("SELECT * FROM {subscribtion_emails} WHERE trash='0'");
			while($row = DB::fetch_array($result))
			{
				$oldmails[$row["mail"]] = $row["id"];
			}
		}

		$file = file_get_contents($_FILES["file"]['tmp_name']);

		$newmails = explode("\n", $file);
		foreach($newmails as $s)
		{
			if(! trim($s))
			{
				continue;
			}
			list($name, $mail) = explode(';', $s);
			if(! empty($oldmails[$mail]))
			{
				DB::query("UPDATE {subscribtion_emails} SET act='1', name='%s' WHERE id=%d", $name, $oldmails[$mail]);
			}
			else
			{
				DB::query("INSERT INTO {subscribtion_emails} (act, name, mail, created) VALUES ('1', '%s', '%s', %d)", $name, $mail, time());
			}
		}
		unlink($_FILES["file"]['tmp_name']);

		$this->diafan->redirect(URL.'success1/');
	}

	/**
	 * Выводит системное сообщение
	 *
	 * @return void
	 */
	public function show_error_message()
	{
		if ($this->diafan->error)
		{
			echo '<div class="error">'.$this->diafan->_('Файл не верного формата.').'</div>';
		}

		if ($this->diafan->success)
		{
			echo '<div class="ok">'.$this->diafan->_('Изменения сохранены!').'</div>';
		}
	}
}