<?php
/**
 * Восстановление базы данных
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

// 0 - выключить лог восстановления
// 1 - включить лог восстановления
// 2 - включить полный лог
define('REPAIR_LOG', 1);

// 0 - не добавлять страницы административного интерфейса и не обновлять информацию о них
// 1 - только добавлять новые страницы административного интерфейса
// 2 - добавлять страницы административного интерфейса и обновлять информацию о них
define('REPAIR_ADMINSITE', 1);

/**
 * Update_admin_repair
 */
class Update_admin_repair extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'repair';

	/**
	 * Выводит контент модуля
	 * @return void
	 */
	public function show()
	{
		if (! empty($_GET["repair"]))
		{
			$this->repair();
		}
		else
		{
			echo '<p>'.$this->diafan->_('Данный сервис предназначен для проверки правильности структуры базы данных diafan.CMS и восстановления поврежденных таблиц до рабочего состояния и создания недостающих. Рекомендуется запускать после обновления CMS с предыдущих версий, либо после программных доработок системы.').'</p>';
			echo '<p><a href="'.URL.'?repair=1"><b>'.$this->diafan->_('Начать проверку и восстановление базы данных').'</b></a></p>';
		}
	}

	/**
	 * Восстановление базы данных
	 * @return void
	 */
	public function repair()
	{
		Customization::inc("includes/install.php");

		$url = parse_url(DB_URL);
		$result = DB::query("SHOW TABLES FROM ".substr($url['path'], 1));
		while ($row = DB::fetch_array($result))
		{
			foreach($row as $k => $v)
			{
				$this->cache["tables"][] = preg_replace('/^'.DB_PREFIX.'/', '', $v);
				break;
			}
		}
		DB::free_result($result);

		if(REPAIR_ADMINSITE)
		{
			$result = DB::query("SELECT * FROM {adminsite}");
			while ($row = DB::fetch_array($result))
			{
				if(! empty($this->cache["adminsites"][$row["rewrite"].($row["parent_id"] ? '_child' : '')]))
				{
					DB::query("DELETE FROM {adminsite} WHERE id=%d", $row["id"]);
					DB::query("DELETE FROM {adminsite_parents} WHERE parent_id=%d", $row["id"]);
					DB::query("DELETE FROM {adminsite} WHERE parent_id=%d", $row["id"]);
					if(REPAIR_LOG)
					{
						echo '<br>Удалена дублирующая страница административного интерфейса '.$row["name"]. ' '.$row["rewrite"];
					}
				}
				else
				{
					$this->cache["adminsites"][$row["rewrite"].($row["parent_id"] ? '_child' : '')] = $row;
				}
			}
			DB::free_result($result);
		}

		if (file_exists(ABSOLUTE_PATH.'installation/core.php'))
		{
			$db = false;
			include_once ABSOLUTE_PATH.'installation/core.php';
			$this->check_file($db);
			$error = false;
		}
		else
		{
			throw new Exception('Файл installation/core.php не существует');
		}

		$result = DB::query("SELECT DISTINCT(module_name) FROM {modules} WHERE module_name=name");
		while ($row = DB::fetch_array($result))
		{
			if (file_exists(ABSOLUTE_PATH."modules/".$row["module_name"]."/".$row["module_name"].".install.php"))
			{
				$db = false;
				include_once ABSOLUTE_PATH."modules/".$row["module_name"]."/".$row["module_name"].".install.php";
				$this->check_file($db);
			}
		}
		DB::free_result($result);

		echo '<div class="ok">'.$this->diafan->_('Базы данных успешно обновлена.').'</div>';
	}

	/**
	 * Восстановление базы данных из файла
	 * @return void
	 */
	private function check_file($db)
	{
		if(! empty($db["tables"]))
		{
			foreach($db["tables"] as $row)
			{
				if (REPAIR_LOG == 2)
				{
					echo '<p>Таблица: <b>'.DB_PREFIX.$row["name"].'</b>';
				}
				if (in_array($row["name"], $this->cache["tables"]))
				{
					$table_fileds = array();
					$result = DB::query("DESCRIBE {".$row["name"]."}");
					while ($r = DB::fetch_array($result))
					{
						$table_fileds[] = $r["Field"];
						if(strpos($r["Field"], ')') !== false)
						{
							list($table_fileds_type[$r["Field"]]) = explode(")", $r["Type"]);
							$table_fileds_type[$r["Field"]] .= ')';
						}
						else
						{
							list($table_fileds_type[$r["Field"]]) = explode(" ", $r["Type"]);
						}
						$table_fileds_type[$r["Field"]] = str_replace(" ", '', $table_fileds_type[$r["Field"]]);
					}
					DB::free_result($result);
					$fields = array();
					foreach ($row["fields"] as $f)
					{
						if(! empty($f["multilang"]))
						{
							foreach($this->diafan->languages as $l)
							{
								$fields[] = array(
									"name" => $f["name"].$l["id"],
									"type" => $f["type"]
								);
							}
						}
						else
						{
							$fields[] = $f;
						}
					}
					foreach ($fields as $f)
					{
						if (! in_array($f["name"], $table_fileds))
						{
							$query = 'ALTER TABLE {'.$row["name"]."} ADD `".$f["name"]."` ".$f["type"];
							if (REPAIR_LOG)
							{
								echo '<br>добавлено поле <b>'.$f["type"].'</b>:<pre>'.$query.'</pre>';
							}
							DB::query($query);
						}
						else
						{
							if(strpos($f["type"], ')') !== false)
							{
								list($type) = explode(")", $f["type"]);
								$type .= ')';
							}
							else
							{
								list($type) = explode(" ", $f["type"]);
							}
							$type = str_replace(" ", '', strtolower($type));
							if($table_fileds_type[$f["name"]] != $type)
							{
								$query = 'ALTER TABLE {'.$row["name"]."} CHANGE `".$f["name"]."` `".$f["name"]."` ".$f["type"];
								if (REPAIR_LOG)
								{
									echo '<br>изменено поле <b>'.$f["type"].'</b>:<pre>'.$query.'</pre>';
								}
								DB::query($query);
							}
							elseif (REPAIR_LOG == 2)
							{
								echo '<br>поле <b>'.$f["name"].'</b> существует';
							}
						}
					}
					$this->update_table_parents($row["name"]);
				}
				else
				{
					$query = "CREATE TABLE {".$row["name"]."} (";
					foreach($row["fields"] as $field)
					{
						if(empty($field["multilang"]))
						{
							$query .= "\n".'`'.$field["name"].'` '.$field["type"].',';
						}
						else
						{
							foreach($this->diafan->languages as $lang_id)
							{
								$query .= "\n".'`'.$field["name"].$lang_id.'` '.$field["type"].',';
							}
						}
					}
					if(! empty($row["keys"]))
					{
						$query .= "\n".implode(',', $row["keys"]);
					}
					$query .= "\n) CHARSET=utf8";
					if (REPAIR_LOG)
					{
						echo '<p>Таблица: <b>'.DB_PREFIX.$row["name"].'</b> добавлена:<br><pre>'.$query.'</pre></p><hr>';
					}
					DB::query($query);
				}
			}
		}
		if(! empty($db["sql"]))
		{
			foreach ($db["sql"] as $query)
			{
				if (preg_match('/CREATE TABLE \{(.*)\} \(([\w\W]*)PRIMARY KEY/', $query, $m))
				{
					$table = $m[1];
					if (REPAIR_LOG == 2)
					{
						echo '<p>Таблица: <b>'.DB_PREFIX.$table.'</b>';
					}
					if (in_array($table, $this->cache["tables"]))
					{
						if (REPAIR_LOG == 2)
						{
							echo ' существует';
						}
						$fields = explode('
		', $m[2]);
						$table_fileds = array();
						$result = DB::query("DESCRIBE {".$table."}");
						while ($row = DB::fetch_array($result))
						{
							$table_fileds[] = $row["Field"];
						}
						DB::free_result($result);
						foreach ($fields as $f)
						{
							$f = trim($f);
							if (preg_match('/([^ ]*) (.*),/', $f, $fm))
							{
								$fm[1] = str_replace('`', '', $fm[1]);
								if (! in_array($fm[1], $table_fileds))
								{
									if (REPAIR_LOG == 2)
									{
										echo '<br>добавлено поле <b>'.$fm[1].'</b>:<pre>ALTER TABLE {'.$table."} ADD `".$fm[1]."` ".$fm[2].'</pre>';
									}
									elseif (REPAIR_LOG)
									{
										echo '<p>Добавлено поле <b>'.$fm[1].'</b> в таблицу <b>'.DB_PREFIX.$table.'</b>:
										<pre>ALTER TABLE {'.$table."} ADD `".$fm[1]."` ".$fm[2].'</pre></p><hr>';
									}
									DB::query("ALTER TABLE {".$table."} ADD `".$fm[1]."` ".$fm[2]);
								}
								else
								{
									if (REPAIR_LOG == 2)
									{
										echo '<br>поле <b>'.$fm[1].'</b> существует.';
									}
								}
							}
							elseif ($f && REPAIR_LOG == 2)
							{
								echo '<br>не обработано поле <b>'.$fm[1].'</b>.';
							}
						}
						$this->update_table_parents($table);
					}
					else
					{
						if (REPAIR_LOG == 2)
						{
							echo ' добавлена:<br><pre>'.$query.'</pre>';
						}
						elseif (REPAIR_LOG)
						{
							echo '<p>Таблица: <b>'.DB_PREFIX.$table.'</b> добавлена:<br><pre>'.$query.'</pre></p><hr>';
						}
						DB::query($query);
					}
					if (REPAIR_LOG == 2)
					{
						echo '</p><hr>';
					}
				}
			}
		}
		if(REPAIR_ADMINSITE && ! empty($db["adminsites"]))
		{
			foreach ($db["adminsites"] as $row)
			{
				if(empty($row["sort"]))
				{
					$row["sort"] = 0;
				}
				if(empty($row["docs"]))
				{
					$row["docs"] = '';
				}
				if(empty($row["group_id"]))
				{
					$row["group_id"] = '';
				}
				if(empty($this->cache["adminsites"][$row["rewrite"]]))
				{
					$row["id"] = DB::query("INSERT INTO {adminsite} (group_id, name, rewrite, act, sort, docs) VALUES ('%s', '%s', '%s', '%s', %d, '%s')", $row['group_id'], $row['name'], $row['rewrite'], $row['act'], $row['sort'], $row["docs"]);
					if (REPAIR_LOG)
					{
						echo '<br>Добавлена страница административного интерфейса '.$row["name"].' '.$row["rewrite"];
					}
					$this->cache["adminsites"][$row["rewrite"]] = $row;
				}
				else
				{
					$v = $this->cache["adminsites"][$row["rewrite"]];
					if(REPAIR_ADMINSITE == 2 && ($v["group_id"] != $row["group_id"] || $v["name"] != $row["name"] || $v["docs"] != $row["docs"]))
					{
						DB::query("UPDATE {adminsite} SET group_id='%s', name='%s', docs='%s' WHERE id=%d", $row['group_id'], $row['name'], $row["docs"], $v["id"]);

						DB::query("UPDATE {adminsite} SET group_id='%s', docs='%s' WHERE parent_id=%d", $row['group_id'], $row["docs"], $v["id"]);

						if (REPAIR_LOG)
						{
							echo '<br>Обновлена страница административного интерфейса '.$row["name"].' '.$row["rewrite"];
						}
						$row["id"] = $v["id"];
						$this->cache["adminsites"][$row["rewrite"]] = $row;
					}
					elseif(REPAIR_LOG == 2)
					{
						echo '<br>Страница административного интерфейса '.$v["name"].' '.$v["rewrite"].' cуществует.';
					}
					$this->cache["adminsites"][$row["rewrite"]]["children"] = (! empty($row["children"]) ? $row["children"] : array());
				}
				$row = $this->cache["adminsites"][$row["rewrite"]];
				if(! empty($row["children"]))
				{
					foreach($row["children"] as $ch)
					{
						if(empty($this->cache["adminsites"][$ch["rewrite"].'_child']))
						{
							$ch["id"] = DB::query("INSERT INTO {adminsite} (group_id, name, rewrite, act, sort, docs, parent_id) VALUES ('%s', '%s', '%s', '%s', %d, '%s', %d)", $row['group_id'], $ch['name'], $ch['rewrite'], (! empty($ch['act']) ? 1 : 0), $ch['sort'], $row["docs"], $row["id"]);
							DB::query("INSERT INTO {adminsite_parents} (element_id, parent_id) VALUES (%d, %d)", $ch["id"], $row["id"]);
							DB::query("UPDATE {adminsite} SET count_children=count_children+1 WHERE id=%d", $row["id"]);
							if (REPAIR_LOG)
							{
								echo '<br>Добавлена страница административного интерфейса '.$row["name"].'/'.$ch["name"].' '.$ch["rewrite"].'.';
							}
						}
						else
						{
							$v = $this->cache["adminsites"][$ch["rewrite"].'_child'];
							if(REPAIR_ADMINSITE == 2 && $v["name"] != $ch["name"])
							{
								DB::query("UPDATE {adminsite} SET name='%s' WHERE id=%d", $ch["name"], $v["id"]);
								if (REPAIR_LOG)
								{
									echo '<br>Обновлена страница административного интерфейса '.$row["name"].'/'.$ch["name"].' '.$ch["rewrite"].'.';
								}
							}
							elseif(REPAIR_LOG == 2)
							{
								echo '<br>Страница административного интерфейса '.$row["name"].'/'.$v["name"].' '.$v["rewrite"].' cуществует.';
							}
							$ch["id"] = $v["id"];
						}
						$this->cache["adminsites"][$ch["rewrite"].'_child'] = $ch;
					}
				}
			}
		}
	}

	/**
	 * Обновление таблицы родительских связей
	 * @return void
	 */
	private function update_table_parents($table)
	{
		if (strpos($table, '_parents') !== false)
		{
			DB::query("TRUNCATE TABLE {".$table."}");
			$table_parent = str_replace('_parents', '', $table);
			
			$result = DB::query("SELECT parent_id, id FROM {".$table_parent."}");
			while ($row = DB::fetch_array($result))
			{
				if ($row["parent_id"])
				{
					$parents = array();
					while ($row["parent_id"] > 0 && ! in_array($row["parent_id"], $parents))
					{
						$parents[] = $row["parent_id"];
						DB::query("INSERT INTO {".$table."} (`element_id`, `parent_id`) VALUES (%d, %d)", $row["id"], $row["parent_id"]);
						$row["parent_id"] = DB::query_result("SELECT parent_id FROM {".$table_parent."} WHERE id=%d LIMIT 1", $row["parent_id"]);
					}
				}
			}
			DB::free_result($result);
			$result = DB::query("SELECT id FROM {".$table_parent."}"); 
			while ($row = DB::fetch_array($result))
			{
				$count = DB::query_result("SELECT COUNT(*) FROM  {".$table."} WHERE parent_id=%d", $row["id"]);
				DB::query("UPDATE {".$table_parent."} SET count_children=%d WHERE id=%d", $count, $row["id"]);
			}
			DB::free_result($result);
		}
	}
}