<?php
/**
 * Редактирование пользователей сайта
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Users_admin
 */
class Users_admin extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'users';

	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'main' => array (
			'fio' => array(
				'type' => 'text',
				'name' => 'ФИО',
				'help' => 'Свободное информационное текстовое поле.',
			),
			'created' => array(
				'type' => 'datetime',
				'name' => 'Дата регистрации',
				'help' => 'В формате дд.мм.гггг чч:мм, при регистрации устанавливается текущая.',
			),
			'act' => array(
				'type' => 'checkbox',
				'name' => 'Активен',
				'help' => 'Если не отмечена, пользователь не сможет авторизоваться на сайте',
				'default' => true,
			),
			'hr1' => 'hr',
			'name' => array(
				'type' => 'text',
				'name' => 'Логин',
				'help' => 'Имя пользователя для авторизации на сайте и входа в систему администрирования (если установлен соответствующий тип пользователя). Только цифры и латинские буквы.',
			),
			'password' => array(
				'type' => 'password',
				'name' => 'Пароль',
				'help' => 'Пароль для входа в систему администрирования. Только цифры и латинские буквы. Если не указан, пароль не перезаписывается.',
			),
			'mail' => array(
				'type' => 'email',
				'name' => 'Email',
				'help' => 'Почтовый адрес пользователя в формате mail@site.ru.',
			),
			'phone' => array(
				'type' => 'text',
				'name' => 'Телефон',
				'help' => 'Телефон для SMS, в федеральном формате.',
			),
			'role_id' => array(
				'type' => 'select',
				'name' => 'Тип пользователя',
				'help' => 'Тип прав пользователя. Уровень доступа настраивается в модуле «Права доступа».',
			),
			'hr3' => 'hr',
			'start_adminsite' => array(
				'type' => 'select',
				'name' => 'Стартовая страница административной части',
				'help' => 'Первая страница, загружаемая при авторизации администратора. По умолчанию «Страницы сайта». Возможно установить любую другую, согласно потребностям и тематике сайта, например, «Заказы» или «Вопрос-Ответ».',
			),
			'useradmin' => array(
				'type' => 'checkbox',
				'name' => 'Подключить панель быстрого редактирования',
				'help' => 'Если отмечено, в пользовательской части сайта в шапке будет выводиться панель, с помошью которой можно редактировать контент прямо на сайте.',
			),
			'htmleditor' => array(
				'type' => 'checkbox',
				'name' => 'Использовать визуальный редактор',
				'help' => 'Если отмечено, при редактировании контента во всех модулях будет использоваться визуальный редактор.',
			),
			'copy_files' => array(
				'type' => 'checkbox',
				'name' => 'Сохранять картинки с внешних сайтов, при вставке контента в визуальный редактор',
				'help' => 'Если отмечено, при вставке контента в визуальный редактор во всех модулях, будут определяться используемые в контенте изображения (как тег img src), сохраняться на сервере и вставляться как локальные. Например, если выделить и скопировать текст с изображениями с новостного сайта, а затем вставить на Ваш сайт, изображения будут автоматически сохранены на Ваш сайт, а ссылки заменены на локальные.',
			),
			'hr2' => 'hr',
			'identity' => array(
				'type' => 'text',
				'name' => 'URL на страницу в соц. сети',
				'help' => 'Заполняется при авторизации через соц. сеть. Служит идентификатором, с помощью которого пользователь может зайти в аккаунт.',
			),
			'lang_id' => array(
				'type' => 'select',
				'name' => 'Язык интерфейса',
				'help' => 'Если предусмотрено несколько языковых версий сайта, то этой опцией можно задать какую версию сайта открыть при авторизации пользователя. При регистрации система запоминает текущую языковую версию.',
			),
			'avatar' => array(
				'type' => 'function',
				'name' => 'Аватар',
				'help' => 'Небольшое изображение для форума, личного кабинета пользователя. Параметр появляется, если в настройках модуля отмечена опция «Использовать аватар».',
			),
			'param' => array(
				'type' => 'function',
				'name' => 'Дополнительные поля',
				'help' => 'Группа полей, определенных в части «Конструктор формы регистрации». Поля выводятся соответственно выбранному типу пользователя.',
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'del', // удалить
		'act', // показать/скрыть
		'trash', // использовать корзину
	);

	/**
	 * @var array списки из таблицы
	 */
	public $select = array(
		'role_id' => array(
			'users_role',
			'id',
			'nameLANG',
			'',
			'',
			"trash='0'",
		),
		'start_adminsite' => array(
			'adminsite',
			'rewrite',
			'name',
			'',
			'-',
			"parent_id=0 AND act='1'",
		),
	);

	/**
	 * @var array выводить в списке содержание полей:
	 */
	public $config_other_row = array(
		'name' => 'function',
		'role_id' => 'function',
	);

	/**
	 * @var array текст для ссылки на редактирование в списке
	 */
	public $text_for_base_link = array(
		'variable' => 'fio'
	);

	/**
	 * Подготавливает конфигурацию модуля
	 * @return void
	 */
	public function prepare_config()
	{
		if(count($this->diafan->languages) > 1)
		{
			foreach ($this->diafan->languages as $language)
			{
				$this->diafan->select_arr("lang_id", $language["id"], $language["name"]);
			}
		}
		else
		{
			$this->diafan->variable_unset("lang_id");
		}
		if (! $this->diafan->addnew && $this->diafan->edit == $this->diafan->_user->id
			|| ! $this->diafan->savenew && $this->diafan->save == $this->diafan->_user->id)
		{
			$this->diafan->variable_unset('role_id');
			$this->diafan->variable_unset('act');
		}
	}

	/**
	 * Выводит ссылку на добавление
	 * @return void
	 */
	public function show_add()
	{
        if($this->diafan->error == 2)
        {
            echo '<div class="error">'.$this->diafan->_('Извините, пользователь с таким логином уже существует.').'</div>';
        }

		$this->diafan->addnew_init('Добавить пользователя');
	}

	/**
	 * Выводит список пользователей
	 * @return void
	 */
	public function show()
	{
		$this->diafan->list_row();
	}

	/**
	 * Выводит форму редактирования
	 * @return void
	 */
	public function edit()
	{
		$result = DB::query("SELECT DISTINCT(role_id) FROM {users_role_perm} WHERE type='admin'");
		while($row = DB::fetch_array($result))
		{
			$admin_roles[] = $row["role_id"];
		}
		DB::free_result($result);
		if(! $this->diafan->addnew && $this->diafan->edit == $this->diafan->_user->id)
		{
			if(in_array($this->diafan->_user->role_id, $admin_roles))
			{
				echo '<script type="text/javascript">$("#hr3, #start_adminsite, #useradmin, #htmleditor, #copy_files").hide();</script>';
			}
		}
		else
		{
			echo '<script type="text/javascript">
			var admin_roles = ["'.implode('", "', $admin_roles).'"];
			var admin_fields = "#hr3, #start_adminsite, #useradmin, #htmleditor, #copy_files";
			$(document).ready(function () {
				if(admin_roles.indexOf($("select[name=role_id]").val()) == -1)
				{
					$(admin_fields).hide();
				}
				else
				{
					$(admin_fields).show();
				}
				$("select[name=role_id]").change(function(){
					if(admin_roles.indexOf($("select[name=role_id]").val()) == -1)
					{
						$(admin_fields).hide();
					}
					else
					{
						$(admin_fields).show();
					}
				});
			});
			</script>';
		}
		parent::__call('edit', array());
	}

	/**
	 * Выводит статус пользователя (на сайте) в списке
	 * 
	 * @return string
	 */
	public function other_row_name($row)
	{
		return '<td class="nick">'.$row["name"]
		.($this->diafan->_user->id == $row["id"]
		  || DB::query_result("SELECT user_id FROM {sessions} WHERE timestamp>=%d AND user_id=%d LIMIT 1", (time() - 900), $row["id"])
		 ? ' <font style="color: #ffffff; background-color: red;">'.$this->diafan->_('на сайте').'</font>'
		 : ''
		 );
	}

	/**
	 * Выводит тип пользователя в списке
	 * 
	 * @return string
	 */
	public function other_row_role_id($row)
	{
		global $roles;
		if (! $roles)
		{
			$roles[0] = '';
			$result = DB::query("SELECT [name], id FROM {users_role}");
			while ($role = DB::fetch_array($result))
			{
				$roles[$role["id"]] = $role["name"];
			}
			DB::free_result($result);
		}
		return '</td><td class="role_id">'.(! empty($roles[$row["role_id"]]) ? $roles[$row["role_id"]] : '');
	}

	/**
	 * Проверяет можно ли удалить текущий элемент строки
	 *
	 * @param array $row информация о текущем элменте списка
	 * @return boolean
	 */
	public function check_delete($row)
	{
		// пользователь не может удалить самого себя
		if($this->diafan->_user->id == $row["id"])
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/**
	 * Определяет строку с GET переменными
	 *
	 * @return void
	 */
	public function set_get_nav()
	{
		$get_nav_params = $this->diafan->get_nav_params;
		$get_nav_params["search_fio"] = '';

		if (!empty( $_GET["search_fio"] ))
		{
			$get_nav_params["search_fio"] = htmlspecialchars($_GET["search_fio"]);
			$this->diafan->where .= " AND fio LIKE '%%".addslashes(str_replace("%", "%%", $_GET["search_fio"]))."%%'";
			$this->diafan->get_nav .= ($this->diafan->get_nav ? '&amp;' : '?').'search_fio='.urlencode($_GET["search_fio"]);
		}
		$get_nav_params["role_id"] = '';

		if (!empty( $_GET["role_id"] ))
		{
			$get_nav_params["role_id"] = $this->diafan->get_param($_GET, "role_id", 0, 2);
			$this->diafan->get_nav .= ( $this->diafan->get_nav ? '&amp;' : '?' ).'role_id='.$get_nav_params["role_id"];
			$this->diafan->where .= " AND role_id='".$get_nav_params["role_id"]."'";
		}
		$this->diafan->get_nav_params = $get_nav_params;
	}

	/**
	 * Поиск
	 *
	 * @return string
	 */
	public function show_search()
	{
		if($this->diafan->edit)
			return '';

		global $roles;
		if (! $roles)
		{
			$roles[0] = '';
			$result = DB::query("SELECT [name], id FROM {users_role}");
			while ($role = DB::fetch_array($result))
			{
				$roles[$role["id"]] = $role["name"];
			}
			DB::free_result($result);
		}

		$html = '
		<td class="name">
		<input type="text" name="search_fio" value="'.$this->diafan->get_nav_params["search_fio"].'" size="20">
		</td>
		<td class="nick"></td>
		<td class="role_id"><select name="role_id">';
		foreach($roles as $id => $role)
		{
			$html .= '<option value="'.$id.'"'.($this->diafan->get_nav_params["role_id"] == $id ? ' selected' : '').'>'.($id ? $role : $this->diafan->_('Все')).'</option>';
		}
		$html .= '</select>
		<input type="submit" class="button" value="'.$this->diafan->_('Ок').'"></td>';

		return $html;
	}
	
	/**
	 * Редактирование поля "Аватар"
	 * @return void
	 */
	public function edit_variable_avatar()
	{
		if (! $this->diafan->configmodules("avatar", "users"))
			return;

		echo '
		<tr valign="top">
			<td align="right">
				'.$this->diafan->variable_name().'
			</td>
			<td>';
		if (! empty($this->diafan->values["name"]) && file_exists(ABSOLUTE_PATH.USERFILES.'/avatar/'.$this->diafan->values["name"].'.png'))
		{
			echo '<img src="'.BASE_PATH.USERFILES.'/avatar/'.$this->diafan->values["name"].'.png?'.rand(0, 99).'" width="'
			.$this->diafan->configmodules("avatar_width", "users").'" height="'
			.$this->diafan->configmodules("avatar_height", "users").'" alt="'.$this->diafan->values["fio"].' ('.$this->diafan->values["name"].')">'
			.'<input type="checkbox" name="delete_avatar" value="1"> '.$this->diafan->_('Удалить')
			.'<br><br>';
		}
		echo '
		<input type="file" name="avatar" size="40" class="inptext">'
		.$this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Редактирование поля "Дополнительные параметры"
	 * 
	 * @return void
	 */
	public function edit_variable_param()
	{
		$param_role_rels = array();
		$params = array();
		$result = DB::query("SELECT role_id, element_id FROM {users_param_role_rel} WHERE trash='0' AND role_id>0");
		while ($row = DB::fetch_array($result))
		{
		    $param_role_rels[$row["role_id"]][] = $row["element_id"];
			$params[] = $row["element_id"];
		}
		DB::free_result($result);
		array_unique($params);
		
		if (! $this->diafan->addnew && $this->diafan->edit == $this->diafan->_user->id)
		{
			?><script language="javascript">
			$(document).ready(function(){
				$("#param<?php echo implode(',#param', $params)?>").hide();
				<?php
				if(! empty($param_role_rels[$this->diafan->_user->role_id]))
				{
				echo '$("#param'.implode(',#param', $param_role_rels[$this->diafan->_user->role_id]).'").show();';
				}
				?>
			});
			</script><?php
		}
		else
		{
			?><script language="javascript">
			$(document).ready(function(){
				$(document).on('change', "select[name=role_id]", function(){
					show_param_rele_rel(this);
				});
				show_param_rele_rel("select[name=role_id]");
			});
			function show_param_rele_rel(th)
			{
				$("#param<?php echo implode(',#param', $params)?>").hide();
				<?php
				foreach($param_role_rels as $role_id => $params)
				{
					echo '
					if($(th).val() == '.$role_id.'){
						$("#param'.implode(',#param', $params).'").show();
					}
					';
				} ?>
			}
			</script><?php
		}

		parent::__call('edit_variable_param', array());
	}
	
	/**
	 * Редактирование поля "Пароль"
	 * @return void
	 */
	public function edit_variable_password()
	{
		echo '
		<tr id="password">
			<td align="right">
				'.$this->diafan->variable_name().'
			</td>
			<td>
				<input type="password" name="password" size="40" value="">';
				echo $this->diafan->help().'
			</td>
		</tr>';
	}

	/**
	 * Валидация поля "Логин"
	 * 
	 * @return void
	 */
	public function validate_variable_name()
	{
		$this->diafan->set_error("name", Validate::login($_POST["name"], $_POST["id"]));
	}

	/**
	 * Валидация поля "E-mail"
	 * 
	 * @return void
	 */
	public function validate_variable_mail()
	{
		$mes = Validate::mail($_POST["mail"]);
		if($mes)
		{
			$this->diafan->set_error("mail", $mes);
		}
		else
		{
			$this->diafan->set_error("mail", Validate::mail_user($_POST["mail"], $_POST["id"]));
		}
	}

	/**
	 * Валидация поля "Пароля"
	 * 
	 * @return void
	 */
	public function validate_variable_password()
	{
		$access_admin = $_POST["id"] == $this->diafan->_user->id || DB::query_result("SELECT id FROM {users_role_perm} WHERE role_id=%d AND type='admin'", $_POST["role_id"]);
		if($_POST["password"] && $access_admin)
		{
			$this->diafan->set_error("password", Validate::password($_POST["password"], true));
		}
	}

	/**
	 * Сохранение поля "Аватар"
	 * 
	 * @return void
	 */
	public function save_variable_avatar()
	{
		if (! $this->diafan->configmodules("avatar", "users"))
		{
			return false;
		}
		if(! $this->diafan->savenew)
		{
			if (! empty($_POST["delete_avatar"]))
			{
				Files_inc::delete_file(USERFILES.'/avatar/'.$this->diafan->oldrow["name"].'.png');
			}
			elseif(file_exists(ABSOLUTE_PATH.USERFILES.'/avatar/'.$this->diafan->oldrow["name"].'.png') && $this->diafan->oldrow["name"] != $_POST["name"])
			{
				Files_inc::rename_file($_POST["name"].'.png', $this->diafan->oldrow["name"].'.png', USERFILES.'/avatar');
			}
		}
		if (isset($_FILES["avatar"]) && is_array($_FILES["avatar"]) && $_FILES["avatar"]['name'] != '')
		{
			$this->diafan->_user->create_avatar($_POST["name"], $_FILES["avatar"]['tmp_name']);
		}
	}
	
	/**
	 * Сохранение поля "Дополнительные параметры"
	 * 
	 * @return void
	 */
	public function save_variable_param()
	{
		parent::__call('save_variable_param', array($this->get_where_param_role_rel()));
	}

	/**
	 * Получает условие для SQL-запроса: выбор полей с учетом роли пользователя
	 *
	 * @return string
	 */
	private function get_where_param_role_rel()
	{
		$param_ids = array();
		$param_role_rels = array();
		$result_roles = array();
		$result = DB::query("SELECT element_id FROM {users_param_role_rel} WHERE role_id=%d OR role_id=0", $_POST["role_id"]);
		while ($row = DB::fetch_array($result))
		{
			$param_ids[] = $row["element_id"];
		}
		DB::free_result($result);
		if($param_ids)
		{
			return " AND id IN (".implode(",", $param_ids).")";
		}
		return '';
	}
	
	/**
	 * Сохранение поля "Пароль"
	 * @return void
	 */
	public function save_variable_password()
	{
		if (!empty( $_POST["password"]))
		{
			$this->diafan->set_query("password='%s'");
			$this->diafan->set_value(encrypt($_POST["password"]));
		}
	}

	/**
	 * Сохранение поля "Телефон"
	 * @return void
	 */
	public function save_variable_phone()
	{
		$phone = preg_replace('/[^0-9]+/', '', $_POST["phone"]);
		if(! DB::query_result("SELECT id FROM {subscribtion_phones} WHERE phone='%s' AND trash='0'", $phone))
		{
			DB::query("INSERT INTO {subscribtion_phones} (phone, created, name, act) VALUES ('%s', %d, '%h', '1')", $phone, $_POST["fio"], time());
		}
		$this->diafan->set_query("phone='%h'");
		$this->diafan->set_value($_POST["phone"]);
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return void
	 */
	public function delete($del_id, $trash_id)
	{
		$this->diafan->del_or_trash_where("shop_order_param_user", "user_id=".$del_id.")", $trash_id);
		$this->diafan->del_or_trash_where("shop_cart", "user_id=".$del_id.")", $trash_id);
		$this->diafan->del_or_trash_where("shop_wishlist", "user_id=".$del_id.")", $trash_id);
		$this->diafan->del_or_trash_where("shop_waitlist", "user_id=".$del_id.")", $trash_id);
	}
}
