<?php
/**
 * Обработка запроса при изменении данных о пользователе
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Usersettings_ajax
 */
class Usersettings_ajax extends Ajax
{
	/**
	 * Обрабатывает полученные данные из формы
	 * 
	 * @return boolean
	 */
	public function ajax_request()
	{
		if (empty($_POST['module']) || $_POST['module'] != 'usersettings' || empty($_POST["action"]) || $this->diafan->module != 'usersettings' || ! $this->diafan->_user->id)
		{
			return false;
		}

		$this->module = 'users';
		$this->tag = 'usersettings';

		switch($_POST["action"])
		{
			case 'edit':
				return $this->edit();

			case 'delete_avatar':
				return $this->delete_avatar();

			case 'upload_image':
				return $this->upload_image();

			case 'delete_image':
				return $this->delete_image();
		}
		return false;
	}

	/*
	 * Редактирует данные пользователя
	 * 
	 * @return boolean
	 */
	private function edit()
	{
		if (!$this->diafan->_user->checked)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();

		if ($this->validate())
			return true;

		$where_param_role_rel = $this->get_where_param_role_rel($this->diafan->_user->role_id);
		$params = $this->get_params(array("module" => "users", "where" => "show_in_form_auth='1'".$where_param_role_rel));
		$this->empty_required_field(array("params" => $params));

		if($this->diafan->_route->id_module('cart'))
		{
			$order_params = $this->get_params(array("module" => "shop", "table" => "shop_order", "where" => "show_in_form_register='1'"));
			//$this->empty_required_field(array("params" => $order_params, "prefix" => "dop_"));
		}

		if ($this->send_errors())
			return true;

		if (!empty($_POST["lang_id"]) && !DB::query_result("SELECT id FROM {languages} WHERE id=%d LIMIT 1", $_POST["lang_id"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		$this->update_values(array("id" => $this->diafan->_user->id, "table" => "users", "params" => $params));

		if ($this->send_errors())
			return true;

		if(! empty($order_params))
		{
			$this->update_values(array("id" => $this->diafan->_user->id, "table" => "shop_order", "params" => $order_params, "prefix" => "dop_", "rel" => "user"));
		}

		if ($this->send_errors())
			return true;

		if($this->diafan->configmodules("mail_as_login", "users"))
		{
			if($this->diafan->_user->name)
			{
				$_POST["name"] = $this->diafan->_user->name;
			}
			else
			{
				list($_POST["name"],) = explode('@', $_POST["mail"]);
				while(DB::query_result("SELECT name FROM {users} WHERE name='%h' AND trash='0'",  $_POST["name"]))
				{
					$_POST["name"] = $_POST["name"].mt_rand(1, 99999);
				}
			}
		}

		DB::query("UPDATE {users} SET name='%h', mail='%h', phone='%h', lang_id=%d, fio='%h' WHERE id=%d",
			  $_POST["name"], $_POST["mail"], $_POST["phone"], !empty($_POST["lang_id"]) ? $_POST["lang_id"] : 0,
			  $_POST["fio"], $this->diafan->_user->id
		);

		if(in_array('subscribtion', $this->diafan->installed_modules))
		{
			if(! $this->diafan->_route->id_module('subscribtion') && $this->diafan->configmodules('subscribe_in_registration', 'subscribtion'))
			{
				DB::query("UPDATE {subscribtion_emails} SET act='%d' WHERE mail='%s'", (empty($_POST['subscribe']) ? 0 : 1), $this->diafan->_user->mail);
			}
			// при смене e-mail, меняем его в списке рассылки
			if($this->diafan->_user->mail != $_POST["mail"])
			{
				DB::query("UPDATE {subscribtion_emails} SET mail='%s' WHERE mail='%s'", $_POST['mail'], $this->diafan->_user->mail);
			}
			if($_POST["phone"])
			{
				$phone = preg_replace('/[^0-9]+/', '', $_POST["phone"]);
				if(! DB::query_result("SELECT id FROM {subscribtion_phones} WHERE phone='%s' AND trash='0'", $phone))
				{
					DB::query("INSERT INTO {subscribtion_phones} (phone, name, created, act) VALUES ('%s', '%h', %d, '1')", $phone, $_POST["fio"], time());
				}
			}
		}

		if ($_POST["password"])
		{
			DB::query("UPDATE {users} SET password='%h' WHERE id=%d", encrypt($_POST["password"]), $this->diafan->_user->id);
		}

		$this->upload_avatar();

		$this->result["errors"][0] = $this->diafan->_('Изменения сохранены!', false);

		return $this->send_errors();
	}

	/*
	 * Удаляет аватар пользователя
	 * 
	 * @return boolean
	 */
	private function delete_avatar()
	{
		if ($this->diafan->configmodules("avatar", "users"))
		{
			Files_inc::delete_file(USERFILES.'/avatar/'.$this->diafan->_user->name.'.png');
			return true;
		}
		return false;
	}

	/**
	 * Загружает изображение
	 *
	 * @return boolean
	 */
	private function upload_image()
	{
		if(! $this->diafan->_user->id)
		{
			return false;
		}
		$prefix = '';
		if(! empty($_POST["images_prefix"]))
		{
			$prefix = $this->diafan->get_param($_POST, "images_prefix", '', 1);
		}
		if(empty($_POST["images_param_id"]))
		{
			return false;
		}

		if (! $this->diafan->_user->checked)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();

		$param_id = $this->diafan->get_param($_POST, "images_param_id", '', 2);
		$this->result["success"] = true;
		if (! empty($_FILES[$prefix.'images'.$param_id]) && $_FILES[$prefix.'images'.$param_id]['tmp_name'] != '' && $_FILES[$prefix.'images'.$param_id]['name'] != '')
		{
			try
			{
				$this->diafan->_images->upload($this->diafan->_user->id, "users", 0, $_FILES[$prefix.'images'.$param_id]['tmp_name'], $_FILES[$prefix.'images'.$param_id]['name'], false, false, $param_id);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = $prefix.'p'.$param_id;
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
			$images = $this->diafan->_images->get('large', $this->diafan->_user->id, "users", 0, '', $param_id);
			$this->result["data"] = $this->diafan->_tpl->get('images', "usersettings", $images);
		}
		return $this->send_errors();
	}

	/**
	 * Удаляет изображение
	 *
	 * @return boolean
	 */
	private function delete_image()
	{
		if(! $this->diafan->_user->id)
		{
			return false;
		}
		if(empty($_POST["id"]))
		{
			return false;
		}
		$row = DB::query_fetch_array("SELECT * FROM {images} WHERE module_name='users' AND id=%d AND element_id=%d", $_POST["id"], $this->diafan->_user->id);
		if(! $row)
		{
			return false;
		}

		if (! $this->diafan->_user->checked)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();
		$this->result["success"] = true;

		$this->diafan->_images->delete($this->diafan->_user->id, "users", $row["id"]);
		return $this->send_errors();
	}

	/**
	 * Проверяет валидность введенных данных
	 * 
	 * @return boolean
	 */
	private function validate()
	{
		Customization::inc('includes/validate.php');
		if (! $this->diafan->configmodules("mail_as_login", "users") &&  $_POST["name"] != $this->diafan->_user->name)
		{
			$mes = Validate::login($_POST["name"]);
			if ($mes)
			{
				$this->result["errors"]["name"] = $this->diafan->_($mes);
			}
		}
		$mes = Validate::mail($_POST["mail"]);
		if ($mes)
		{
			$this->result["errors"]["mail"] = $this->diafan->_($mes);
		}
		if ($_POST["mail"] != $this->diafan->_user->mail)
		{
			$mes = Validate::mail_user($_POST["mail"]);
			if ($mes)
			{
				$this->result["errors"]["mail"] = $this->diafan->_($mes);
			}
		}
		if (! empty($_POST["phone"]))
		{
			$mes = Validate::phone($_POST["phone"]);
			if ($mes)
			{
				$this->result["errors"]["phone"] = $this->diafan->_($mes);
			}
		}
		if ($_POST["password"])
		{
			$mes = Validate::password($_POST["password"]);
			if ($mes)
			{
				$this->result["errors"]["password"] = $this->diafan->_($mes);
			}
			elseif ($_POST["password"] != $_POST["password2"])
			{
				$this->result["errors"]["password"] = $this->diafan->_('Пароли не совпадают', false);
			}
		}

		if (empty($_POST["fio"]))
		{
			$this->result["errors"]["fio"] = $this->diafan->_('Заполните поле ФИО или название компании', false);
		}

		return $this->send_errors();
	}

	/**
	 * Получает условие для SQL-запроса: выбор полей с учетом роли пользователя
	 *
	 * @param integer $role_id номер роли пользователя
	 * @return string
	 */
	private function get_where_param_role_rel($role_id)
	{
		$param_ids = array();
		$param_role_rels = array();
		$result = DB::query("SELECT role_id, element_id FROM {users_param_role_rel} WHERE trash='0' AND role_id>0");
		while ($row = DB::fetch_array($result))
		{
			$param_role_rels[$row["element_id"]][] = $row["role_id"];
		}
		DB::free_result($result);
		foreach($param_role_rels as $param_id => $roles)
		{
			if(! in_array($role_id, $roles))
			{
				$param_ids[] = $param_id;
			}
		}
		if($param_ids)
		{
			return " AND id NOT IN (".implode(",", $param_ids).")";
		}
		return '';
	}

	/**
	 * Загружает аватар
	 * 
	 * @return void
	 */
	private function upload_avatar()
	{
		if (isset($_FILES["avatar"]) && is_array($_FILES["avatar"]) && $_FILES["avatar"]['name'] != '')
		{
			try
			{
				$this->diafan->_user->create_avatar($_POST["name"], $_FILES["avatar"]['tmp_name']);
			}
			catch(Exception $e)
			{
				Dev::$exception_field = "avatar";
				Dev::$exception_result = $this->result;
				throw new Exception($e->getMessage());
			}
			$result["name"]          = $this->diafan->_user->name;
			$result["fio"]           = $this->diafan->_user->fio;
			$result["avatar_width"]  = $this->diafan->configmodules("avatar_width", "users");
			$result["avatar_height"] = $this->diafan->configmodules("avatar_height", "users");

			$this->result["data"] = $this->diafan->_tpl->get('avatar', 'usersettings', $result);
			$this->result["target"] = '.usersettings_avatar';
		}
	}
}