<?php
/**
 * Модель модуля «Настройки аккаунта»
 * 
 * @package    Diafan.CMS
 * @author     diafan.ru
 * @version    5.3
 * @license    http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2013 OOO «Диафан» (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Usersettings_model
 */
class Usersettings_model extends Model
{
	/**
	 * Генерирует данные для формы регистрации
	 * 
	 * @return array
	 */
	public function form()
	{
		$this->result["captcha"] = '';
		if ($this->diafan->configmodules('captcha', "users") && !$this->diafan->_user->id)
		{
			$this->result["captcha"] = $this->diafan->_captcha->get("usersettings", $this->get_error("usersettings", "captcha"));
		}

		$this->result["fio"] = $this->diafan->_user->fio;
		$this->result["hash"] = $this->diafan->_user->get_hash();
		$this->result["mail"] = $this->diafan->_user->mail;
		$this->result["name"] = $this->diafan->_user->name;
		$this->result["phone"] = $this->diafan->_user->phone;
		$this->result["use_name"] = ! $this->diafan->configmodules("mail_as_login", "users");

		if (count($this->diafan->languages) > 1)
		{
			foreach ($this->diafan->languages as $language)
			{
				$this->result["languages"][] = array(
					"value"    => $language["id"],
					"selected" => ($language["id"] == $this->diafan->_user->lang_id ? ' selected' : ''),
					"name"     => $language["name"]
				);
			}
		}

		$this->get_shop_order_param();

		$this->result["action"] = BASE_PATH_HREF.$this->diafan->_route->current_link();
		$this->result["error"] = $this->get_error("usersettings");
		$this->result["error_name"] = $this->get_error("usersettings", 'name');
		$this->result["error_fio"] = $this->get_error("usersettings", 'fio');
		$this->result["error_phone"] = $this->get_error("usersettings", 'phone');
		$this->result["error_password"] = $this->get_error("usersettings", 'password');
		$this->result["error_password2"] = $this->get_error("usersettings", 'password2');
		$this->result["error_mail"] = $this->get_error("usersettings", 'mail');
		$this->result["user_id"] = $this->diafan->_user->id;
		$this->result["url"] = '';
		$this->result["use_avatar"] = $this->diafan->configmodules("avatar", "users");
		if ($this->result["use_avatar"])
		{
			$this->result["avatar"] = file_exists(ABSOLUTE_PATH.USERFILES.'/avatar/'.$this->diafan->_user->name.'.png');
			$this->result["avatar_width"] = $this->diafan->configmodules("avatar_width", "users");
			$this->result["avatar_height"] = $this->diafan->configmodules("avatar_height", "users");
			$this->result["error_avatar"] = $this->get_error("usersettings", 'avatar');
		}

		if (! $this->diafan->configmodules("act", "users"))
		{
			$this->result["url"] = $this->result["action"].'?action=success';
		}
		$where_param_role_rel = $this->get_where_param_role_rel();
		$where = "show_in_form_auth='1'".$where_param_role_rel;
		$this->result["rows_param"] = $this->get_params(array("module" => "users", "where" => $where));

		$param_types_array = array();
		foreach ($this->result["rows_param"] as $row)
		{
			$this->result['error_p'.$row["id"]] = $this->get_error("usersettings", 'p'.$row["id"]);
			$param_types_array[$row["id"]] = $row["type"];
			if($row["type"] == 'attachments' && ! $row["attachments_access_admin"])
			{
				$this->result['attachments'][$row["id"]] = $this->diafan->_attachments->get($this->diafan->_user->id, 'users', $row["id"]);
			}
			if($row["type"] == 'images')
			{
				$this->result['images'][$row["id"]] = $this->diafan->_images->get('large', $this->diafan->_user->id, 'users', 0, '', $row["id"]);
			}
		}

		$user_param = array();
		$result = DB::query("SELECT value, param_id FROM {users_param_element} WHERE trash='0' AND element_id=%d", $this->diafan->_user->id);
		while ($row = DB::fetch_array($result))
		{
			if(empty($param_types_array[$row["param_id"]]))
				continue;

			switch ($param_types_array[$row["param_id"]])
			{
				case 'multiple':
					$user_param[$row["param_id"]][] = $row["value"];
					break;

				case 'date':
					$user_param[$row["param_id"]] = $this->diafan->formate_from_date($row["value"]);
					break;

				case 'datetime':
					$user_param[$row["param_id"]] = $this->diafan->formate_from_datetime($row["value"]);
					break;

				default:
					$user_param[$row["param_id"]] = $row["value"];
			}
		}
		DB::free_result($result);
		foreach ($this->result["rows_param"] as $i => $row)
		{
			$row["value"] = '';
			if(! empty($user_param[$row["id"]]))
			{
				$row["value"] = $user_param[$row["id"]];
			}
			$this->result["rows_param"][$i] = $row;
		}

		if($this->diafan->_route->id_module('subscribtion'))
		{
			$this->result['link_subscribtion'] = BASE_PATH_HREF.$this->diafan->_route->module("subscribtion", true).'?action=edit&mail='.$this->diafan->_user->mail;
			if($code = DB::query_result("SELECT code FROM {subscribtion_emails} WHERE mail='%s' AND trash='0'", $this->diafan->_user->mail))
			{
				$this->result['link_subscribtion'] .= '&code='.$code;
			}
		}
		else
		{
			if(DB::query_result("SELECT id FROM {modules} WHERE module_name='subscribtion' LIMIT 1"))
			{
				$this->result['use_subscribtion'] = $this->diafan->configmodules('subscribe_in_usersettings', 'subscribtion');
				$this->result["is_subscribe"] = DB::query_result("SELECT id FROM {subscribtion_emails} WHERE mail='%s' AND trash='0' AND act='1' LIMIT 1", $this->diafan->_user->mail);
			}
		}

		return $this->result;
	}

	/**
	 * Получает условие для SQL-запроса: выбор полей с учетом роли пользователя
	 *
	 * @param integer $role_id номер роли пользователя
	 * @return string
	 */
	private function get_where_param_role_rel()
	{
		$param_ids = array();
		$param_role_rels = array();
		$result_roles = array();
		$result = DB::query("SELECT role_id, element_id FROM {users_param_role_rel} WHERE trash='0' AND role_id>0");
		while ($row = DB::fetch_array($result))
		{
			$param_role_rels[$row["element_id"]][] = $row["role_id"];
		}
		DB::free_result($result);
		$roles = array($this->diafan->_user->role_id);
		if(count($result_roles) > 1)
		{
			$this->result["roles"] = $result_roles;
			$this->result["param_role_rels"] = $param_role_rels;
		}
		foreach($param_role_rels as $param_id => $rel_roles)
		{
			$in = false;
			foreach($roles as $role_id)
			{
				if(in_array($role_id, $rel_roles))
				{
					$in = true;
				}
			}
			if(! $in)
			{
				$param_ids[] = $param_id;
			}
		}
		if($param_ids)
		{
			return " AND id NOT IN (".implode(",", $param_ids).")";
		}
		return '';
	}

	/**
	 * Формирует поля формы "Оформление заказа", доступные для внешнего редактирования
	 * @return boolean
	 */
	private function get_shop_order_param()
	{
		if(! $this->diafan->_route->id_module('cart'))
			return false;

		if (empty($this->diafan->_user->id))
			return false;

		$result = DB::query("SELECT value, param_id FROM {shop_order_param_user} WHERE trash='0' AND user_id=%d", $this->diafan->_user->id);
		while ($row = DB::fetch_array($result))
		{
			$values[$row["param_id"]] = $row["value"];
		}
		DB::free_result($result);

		$this->result["dop_rows_param"] = $this->get_params(array("module" => "shop", "table" => "shop_order", "where" => "show_in_form_register='1'"));
		foreach($this->result["dop_rows_param"] as $i => $row)
		{
			if($row["type"] == "attachments" || $row["type"] == "images")
			{
				unset($this->result["dop_rows_param"][$i]);
				continue;
			}
			$row["value"] = '';
			if(! empty($values[$row["id"]]))
			{
				$row["value"] = $values[$row["id"]];
			}
			$row["required"] = false;
			$this->result["dop_rows_param"][$i] = $row;
			$this->result['error_dop_p'.$row["id"]] = $this->get_error("registartion", 'dop_p'.$row["id"]);
		}
		return true;
	}
}